@@ -72,10 +72,16 @@ static XtResource resources[] = {
 
 #undef offset
 
-static void Initialize(), Destroy(), Redisplay(), MoveChart(), SetPoints();
+/* Added argument types to these to shut picky compilers up. PH */
+
+static void CreateGC(StripChartWidget, unsigned int);
+static void DestroyGC(StripChartWidget, unsigned int);
+static void Initialize(), Destroy(), Redisplay();
+static void MoveChart(StripChartWidget, Boolean);
+static void SetPoints(StripChartWidget);
 static Boolean SetValues();
 
-int repaint_window();            /* PH hack */
+int repaint_window(StripChartWidget, int, int);     /* PH hack */
 /* static int repaint_window(); */
 
 StripChartClassRec stripChartClassRec = {
@@ -100,7 +106,7 @@ StripChartClassRec stripChartClassRec = {
     /* compress_enterleave	*/	TRUE,
     /* visible_interest		*/	FALSE,
     /* destroy			*/	Destroy,
-    /* resize			*/	SetPoints,
+    /* resize			*/	(void (*)(Widget))SetPoints,
     /* expose			*/	Redisplay,
     /* set_values		*/	SetValues,
     /* set_values_hook		*/	NULL,
