@@ -30,6 +30,8 @@ SOFTWARE.
 * file" features of the search facility have been removed.  Also took out   *
 * the declaration of sys_errlist, as it isn't used and causes trouble on    *
 * some systems that declare it differently. September 1996.                 *
+* Added the arguments into the static functions declared at the head, to    *
+* stop some compiler warnings. August 1999.                                 *
 ****************************************************************************/
 
 
@@ -80,14 +82,23 @@ extern int errno, sys_nerr;
 
 #define R_OFFSET      1
 
-static void CenterWidgetOnPoint(), PopdownSearch();
-static void InitializeSearchWidget(), SetResource(), SetSearchLabels();
-static Widget CreateDialog(), GetShell();
-static void SetWMProtocolTranslations();
-static Boolean DoSearch();
-static String GetString();
+/* Argument types added by PH August 1999 */
 
-static void AddSearchChildren();
+static void CenterWidgetOnPoint(Widget, XEvent *);
+static void PopdownSearch(Widget, XtPointer, XtPointer);
+static void InitializeSearchWidget(struct SearchAndReplace *,
+  XawTextScanDirection, Boolean);
+static void  SetResource(Widget, char *, XtArgVal);
+static void  SetSearchLabels(struct SearchAndReplace *, String, String,
+  Boolean);
+static Widget CreateDialog(Widget, String, String,
+  void (*)(Widget, char *, Widget));
+static Widget  GetShell(Widget);
+static void SetWMProtocolTranslations(Widget w);
+static Boolean DoSearch(struct SearchAndReplace *);
+static String GetString(Widget);
+
+static void AddSearchChildren(Widget, char *, Widget);
 
 static char radio_trans_string[] =
     "<Btn1Down>,<Btn1Up>:   set() notify()";
@@ -303,6 +314,8 @@ struct SearchAndReplace * search;
 XawTextScanDirection dir;
 Boolean replace_active;
 {
+replace_active = replace_active; /* PH - shuts compilers up */
+
   switch (dir) {
   case XawsdLeft:
     SetResource(search->left_toggle, XtNstate, (XtArgVal) TRUE);
@@ -495,6 +508,7 @@ struct SearchAndReplace * search;
 String msg1, msg2;
 Boolean bell;
 {
+msg2 = msg2; /* PH - shuts compilers up */
   (void) SetResource( search->label1, XtNlabel, (XtArgVal) msg1);
   /* (void) SetResource( search->label2, XtNlabel, (XtArgVal) msg2); */
   if (bell)
@@ -667,6 +681,10 @@ Widget w;
     return (w);
 }
 
+/* Add proper prototype to keep IRIX 6 compiler happy. PH */
+
+static Boolean InParams(String, String *, Cardinal);
+
 static Boolean InParams(str, p, n)
     String str;
     String *p;
