@@ -2,7 +2,7 @@
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -29,7 +29,7 @@ XtTranslations text_trans;
 
 Widget dialog_ref_widget;
 Widget toplevel_widget;
-Widget log_widget;
+Widget log_widget = NULL;
 Widget queue_widget;
 Widget unhide_widget = NULL;
 
@@ -77,6 +77,7 @@ pcre  *yyyymmdd_regex;
 char  *size_stripchart = NULL;
 char  *size_stripchart_name = NULL;
 int    spool_is_split = FALSE;
+int    start_small = FALSE;
 int    stripchart_height = 90;
 int    stripchart_number = 1;
 pcre **stripchart_regex;
@@ -99,6 +100,9 @@ char  *window_title = "exim monitor";
 /* These ones are used by Exim modules included in Eximon. Not all are
 actually relevant to the operation of Eximon. */
 
+char  *authenticated_id       = NULL;
+char  *authenticated_sender   = NULL;
+
 char  *big_buffer             = NULL;
 int    big_buffer_size        = BIG_BUFFER_SIZE;
 int    body_linecount         = 0;
@@ -106,7 +110,7 @@ int    body_linecount         = 0;
 off_t  data_start_offset      = MESSAGE_ID_LENGTH + 3;
 FILE  *debug_file             = NULL;
 int    debug_level            = 0;
-BOOL   debug_trace_memory     = FALSE;
+int    debug_trace_memory     = 0;
 int    deliver_datafile       = -1;
 BOOL   deliver_firsttime      = FALSE;
 BOOL   deliver_freeze         = FALSE;
@@ -125,14 +129,19 @@ header_name *header_names     = NULL;
 /* These are never used, but some compilers insist on there being some
 data, as otherwise they complain about not knowing the size. */
 
-header_name header_names_normal[] = { "dummy", 5 };
-header_name header_names_resent[] = { "dummy", 5 };
+header_name header_names_normal[] = {{ "dummy", 5 }};
+header_name header_names_resent[] = {{ "dummy", 5 }};
+
+BOOL   host_lookup_failed     = FALSE;
+char  *interface_address      = NULL;
 
 BOOL   local_error_message    = FALSE;
+int    message_age            = 0;
 char  *message_id;
 char   message_id_option[MESSAGE_ID_LENGTH + 3];
 char  *message_id_external;
 
+int    message_body_size      = 0;
 int    message_linecount      = 0;
 char  *message_precedence     = NULL;
 int    message_size           = 0;
@@ -155,6 +164,7 @@ char  *sender_address         = NULL;
 char  *sender_fullhost        = NULL;
 char  *sender_helo_name       = NULL;
 char  *sender_host_address    = NULL;
+char  *sender_host_authenticated = NULL;
 char  *sender_host_name       = NULL;
 char  *sender_ident           = NULL;
 BOOL   sender_local           = FALSE;
@@ -163,7 +173,6 @@ volatile BOOL sigalrm_seen;
 BOOL   split_spool_directory  = FALSE;
 char  *spool_directory        = SPOOL_DIRECTORY
 			  "\0<--------------Space to patch spool_directory->";
-int    store_max_malloc       = 0;
 int    store_pool             = POOL_MAIN;
 
 tree_node *tree_duplicates    = NULL;
