@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -92,7 +92,7 @@ anything. */
 /* The sys/resource.h header on SunOS 4 causes trouble with the gcc
 compiler. Just stuff the bit we want in here; pragmatic easy way out. */
 
-#ifdef NO_SYS_RESOURCE
+#ifdef NO_SYS_RESOURCE_H
 #define RLIMIT_NOFILE   6               /* maximum descriptor index + 1 */
 struct rlimit {
         int     rlim_cur;               /* current (soft) limit */
@@ -109,6 +109,7 @@ struct rlimit {
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
+#include <X11/keysym.h>
 #include <X11/Shell.h>
 #include <X11/Xaw/AsciiText.h>
 #include <X11/Xaw/Command.h>
@@ -263,6 +264,7 @@ extern char  *size_stripchart;     /* path for size monitoring */
 extern char  *size_stripchart_name; /* name for size stripchart */
 extern char  *spool_directory;     /* Name of exim spool directory */
 extern int    spool_is_split;      /* True if detected split spool */
+extern int    start_small;         /* True to start with small window */
 extern int    stripchart_height;   /* height of stripcharts */
 extern int    stripchart_number;   /* number of stripcharts */
 extern pcre **stripchart_regex;  /* vector of regexps */
