@@ -2,7 +2,7 @@
 *                  Exim monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code to initialize things from the
@@ -71,7 +71,7 @@ for (i = 0; i <= 1; i++)
         int offset;
         const char *error;
         stripchart_regex[indx] = pcre_compile(buffer, PCRE_COPT, &error,
-          &offset);
+          &offset, NULL);
         if (stripchart_regex[indx] == NULL)
           {
           printf("regular expression error: %s at offset %d "
@@ -194,6 +194,9 @@ if (s != NULL && (x = atoi(s)) != 0) queue_width = x;
 s = getenv("SPOOL_DIRECTORY");
 if (s != NULL) spool_directory = string_copy(s);
 
+s = getenv("START_SMALL");
+if (s != NULL && strcmp(s, "yes") == 0) start_small = 1;
+
 s = getenv("TEXT_DEPTH");
 if (s != NULL && (x = atoi(s)) != 0) text_depth = x;
 
@@ -226,7 +229,7 @@ queue_stripchart_name = (s != NULL)? string_copy(s) : "queue";
 /* Compile the regex for matching yyyy-mm-dd at the start of a string. */
 
 yyyymmdd_regex = pcre_compile("^\\d{4}-\\d\\d-\\d\\d\\s", PCRE_COPT,
-  &error, &erroroffset);
+  &error, &erroroffset, NULL);
 }
 
 /* End of em_init.c */
