@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for scanning the smaill log,
@@ -30,6 +30,10 @@ va_list ap;
 XawTextBlock b;
 char buffer[log_buffer_len + 24];
 
+/* Do nothing if not tailing a log */
+
+if (log_widget == NULL) return;
+
 /* Initialize the text block structure */
 
 b.firstPos = 0;
@@ -152,7 +156,7 @@ if (LOG != NULL)
 
     for (i = stripchart_varstart; i < stripchart_number; i++)
       {
-      if (pcre_exec(stripchart_regex[i], NULL, buffer, length, PCRE_EOPT,
+      if (pcre_exec(stripchart_regex[i], NULL, buffer, length, 0, PCRE_EOPT,
             NULL, 0) >= 0)
         stripchart_total[i]++;
       }
@@ -160,7 +164,7 @@ if (LOG != NULL)
     /* Munge the log entry and display shortened form on one line.
     We omit the date and show only the time. */
 
-    if (pcre_exec(yyyymmdd_regex,NULL,buffer,length,PCRE_EOPT,NULL,0) >= 0)
+    if (pcre_exec(yyyymmdd_regex,NULL,buffer,length,0,PCRE_EOPT,NULL,0) >= 0)
       {
       id = string_copyn(buffer + 20, MESSAGE_ID_LENGTH);
       show_log("%s", buffer+11);
