@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -123,6 +123,7 @@ static Dimension minposx, minposy;
 static Dimension maxwidth, maxheight;
 static Widget outer_form_widget;
 static Widget hide_widget;
+static Widget above_queue_widget;
 
 
 
@@ -192,6 +193,9 @@ if (action_queue_update) tick_queue_accumulator = 999999;
 
 void updateAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;       /* Keep picky compilers happy */
+client_data = client_data;
+call_data = call_data;
 scan_spool_input(TRUE);
 queue_display();
 tick_queue_accumulator = 0;
@@ -199,6 +203,9 @@ tick_queue_accumulator = 0;
 
 void hideAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;       /* Keep picky compilers happy */
+client_data = client_data;
+call_data = call_data;
 actioned_message[0] = 0;
 dialog_ref_widget = w;
 dialog_action = da_hide;
@@ -208,6 +215,11 @@ create_dialog("Hide addresses ending with", "");
 void unhideAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 skip_item *sk = queue_skip;
+
+w = w;       /* Keep picky compilers happy */
+client_data = client_data;
+call_data = call_data;
+
 while (sk != NULL)
   {
   skip_item *next = sk->next;
@@ -226,6 +238,9 @@ tick_queue_accumulator = 0;
 
 void quitAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;       /* Keep picky compilers happy */
+client_data = client_data;
+call_data = call_data;
 exit(0);
 }
 
@@ -256,6 +271,10 @@ Dimension width, height;
 XWindowAttributes a;
 Window w = XtWindow(toplevel_widget);
 
+button = button;    /* Keep picky compilers happy */
+client_data = client_data;
+call_data = call_data;
+
 /* Get the position and size of the top level widget. */
 
 sizepos_args[0].value = (XtArgVal)(&width);
@@ -368,7 +387,7 @@ switch(dialog_action)
 
   if (unhide_widget == NULL)
     {
-    unhide_args[0].value = (XtArgVal) log_widget;
+    unhide_args[0].value = (XtArgVal) above_queue_widget;
     unhide_args[1].value = (XtArgVal) hide_widget;
     unhide_widget = XtCreateManagedWidget("unhide", commandWidgetClass,
       outer_form_widget, unhide_args, XtNumber(unhide_args));
@@ -407,6 +426,9 @@ tick_queue_accumulator += tick_interval;
 tick_stripchart_accumulator += tick_interval;
 read_log();
 
+pt = pt;    /* Keep picky compilers happy */
+i = i;
+
 /* If we have passed the queue update time, we must do a full
 scan of the queue, checking for new arrivals, etc. This will
 as a by-product set the count of items for use by the stripchart
@@ -468,6 +490,55 @@ XtAppAddTimeOut(X_appcon, tick_interval * 1000, ticker, 0);
 
 
 
+/*************************************************
+*             Find Num Lock modifiers            *
+*************************************************/
+
+/* Return a string with the modifiers generated by XK_Num_Lock, or return
+NULL if XK_Num_Lock doesn't generate any modifiers. This is needed because Num
+Lock isn't always the same modifier on all servers.
+
+Arguments:
+  display   the Display
+  buf       a buffer in which to put the answers (long enough to hold 5)
+
+Returns:    points to the buffer, or NULL
+*/
+
+static char *
+numlock_modifiers(Display *display, char *buf)
+{
+XModifierKeymap *m;
+int i, j;
+char *ret = NULL;
+
+m = XGetModifierMapping(display);
+if (m == NULL)
+  {
+  printf("Not enough memory\n");
+  exit (EXIT_FAILURE);
+  }
+
+/* Look at Mod1 through Mod5, and fill in the buffer as necessary. */
+
+buf[0] = 0;
+for (i = 3; i < 8; i++)
+  {
+  for (j = 0; j < m->max_keypermod; j++)
+    {
+    if (XKeycodeToKeysym(display, m->modifiermap [i*m->max_keypermod + j], 0)
+        == XK_Num_Lock)
+      {
+      sprintf(buf+strlen(buf), " Mod%d", i-2);
+      ret = buf;
+      }
+    }
+  }
+
+XFreeModifiermap(m);
+return ret;
+}
+
 
 
 /*************************************************
@@ -478,6 +549,8 @@ int main(int argc, char **argv)
 {
 int i;
 struct stat statdata;
+char modbuf[] = " Mod1 Mod2 Mod3 Mod4 Mod5";
+char *numlock;
 Widget stripchart_form_widget,
        update_widget,
        quit_widget,
@@ -519,25 +592,32 @@ log_display_buffer[0] = 0;
 
 stripchart_init();
 
-/* Open the log file and position to the end of it. Save its inode
-so that we can detect when the file is switched to another one. However,
-allow the monitor to start up without a log file (can happen if no
-messages have been sent today.) */
+/* If log_file contains the empty string, then Exim is running using syslog
+only, and we can't tail the log. If not, open the log file and position to the
+end of it. Save its inode so that we can detect when the file is switched to
+another one. However, allow the monitor to start up without a log file (can
+happen if no messages have been sent today.) */
 
-LOG = fopen(log_file, "r");
-if (LOG == NULL)
+if (log_file[0] != 0)
   {
-  printf("*** eximon warning: can't open log file\n");
+  LOG = fopen(log_file, "r");
+  if (LOG == NULL)
+    {
+    printf("*** eximon warning: can't open log file - will try periodically\n");
+    }
+  else
+    {
+    fseek(LOG, 0, SEEK_END);
+    log_position = ftell(LOG);
+    fstat(fileno(LOG), &statdata);
+    log_inode = statdata.st_ino;
+    }
   }
 else
   {
-  fseek(LOG, 0, SEEK_END);
-  log_position = ftell(LOG);
-  fstat(fileno(LOG), &statdata);
-  log_inode = statdata.st_ino;
+  printf("*** eximon warning: no log file available to tail\n");
   }
 
-
 /* Now initialize the X world and create the top-level widget */
 
 toplevel_widget = XtAppInitialize(&X_appcon, "Eximon", NULL, 0, &argc, argv,
@@ -570,10 +650,29 @@ otherwise it isn't noticed. */
    <FocusOut>:     display-caret(off)\n\
 */
 
+/* The translation manager sets up passive grabs for the menu popups as a
+result of MenuPopup(), but the grabs match only the exact modifiers listed,
+hence combinations with and without caps-lock and num-lock must be given,
+rather than just one "Shift<Btn1Down>" (or whatever menu_event is set to),
+despite the fact that that notation (without a leading !) should ignore the
+state of other modifiers. Thanks to Kevin Ryde for this information, and for
+the function above that discovers which modifier is Num Lock, because it turns
+out that it varies from server to server. */
 
 sprintf(big_buffer,
-   "%s:   menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
-   <Btn1Down>:     select-start()\n\
+  "!%s:            menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
+   !Lock %s:       menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
+  ", menu_event, menu_event);
+
+numlock = numlock_modifiers(X_display, modbuf); /* Get Num Lock modifier(s) */
+
+if (numlock != NULL) sprintf(big_buffer + (int)strlen(big_buffer),
+  "!%s %s:         menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
+   !Lock %s %s:    menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
+  ", numlock, menu_event, numlock, menu_event);
+
+sprintf(big_buffer + (int)strlen(big_buffer),
+  "<Btn1Down>:     select-start()\n\
    <Btn1Motion>:   extend-adjust()\n\
    <Btn1Up>:       extend-end(PRIMARY,CUT_BUFFER0)\n\
    <Btn3Down>:     extend-start()\n\
@@ -583,7 +682,7 @@ sprintf(big_buffer,
    <Key>Down:      scroll-one-line-up()\n\
    Ctrl<Key>R:     search(backward)\n\
    Ctrl<Key>S:     search(forward)\n\
-   ", menu_event);
+  ");
 
 queue_trans = XtParseTranslationTable(big_buffer);
 
@@ -660,37 +759,47 @@ resize_widget = XtCreateManagedWidget("resize", commandWidgetClass,
   outer_form_widget, resize_args, XtNumber(resize_args));
 XtAddCallback(resize_widget, "callback",  resizeAction, NULL);
 
-/* Create an Ascii text widget for the log tail display. */
+/* In the absence of log tailing, the quit widget is the one above the
+queue listing. */
 
-log_args[0].value = (XtArgVal) quit_widget;
-log_widget = XtCreateManagedWidget("log", asciiTextWidgetClass,
-  outer_form_widget, log_args, XtNumber(log_args));
-XawTextDisplayCaret(log_widget, TRUE);
-xs_SetValues(log_widget, 6,
-  "editType",  XawtextAppend,
-  "translations", text_trans,
-  "string",    log_display_buffer,
-  "length",    log_buffer_size,
-  "height",    log_depth,
-  "width",     log_width);
+above_queue_widget = quit_widget;
 
-if (log_font != NULL)
+/* Create an Ascii text widget for the log tail display if we are tailing a
+log. Skip it if not. */
+
+if (log_file[0] != 0)
   {
-  XFontStruct *f = XLoadQueryFont(X_display, log_font);
-  if (f != NULL) xs_SetValues(log_widget, 1, "font", f);
-  }
+  log_args[0].value = (XtArgVal) quit_widget;
+  log_widget = XtCreateManagedWidget("log", asciiTextWidgetClass,
+    outer_form_widget, log_args, XtNumber(log_args));
+  XawTextDisplayCaret(log_widget, TRUE);
+  xs_SetValues(log_widget, 6,
+    "editType",  XawtextAppend,
+    "translations", text_trans,
+    "string",    log_display_buffer,
+    "length",    log_buffer_size,
+    "height",    log_depth,
+    "width",     log_width);
 
+  if (log_font != NULL)
+    {
+    XFontStruct *f = XLoadQueryFont(X_display, log_font);
+    if (f != NULL) xs_SetValues(log_widget, 1, "font", f);
+    }
+
+  above_queue_widget = log_widget;
+  }
 
 /* The update button */
 
-update_args[0].value = (XtArgVal) log_widget;
+update_args[0].value = (XtArgVal) above_queue_widget;
 update_widget = XtCreateManagedWidget("update", commandWidgetClass,
   outer_form_widget, update_args, XtNumber(update_args));
 XtAddCallback(update_widget, "callback",  updateAction, NULL);
 
 /* The hide button */
 
-hide_args[0].value = (XtArgVal) log_widget;
+hide_args[0].value = (XtArgVal) above_queue_widget;
 hide_args[1].value = (XtArgVal) update_widget;
 hide_widget = XtCreateManagedWidget("hide", commandWidgetClass,
   outer_form_widget, hide_args, XtNumber(hide_args));
@@ -752,6 +861,10 @@ screenheight= XDisplayHeight(X_display,0);
 
 XtAppAddActions(X_appcon, actionTable, actionTableSize);
 
+/* Reduce the window to the small size if this is wanted */
+
+if (start_small) resizeAction(NULL, NULL, NULL);
+
 /* Enter the application loop which handles things from here
 onwards. The return statement is never obeyed, but is needed to
 keep pedantic ANSI compilers happy. */
