@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -117,6 +117,8 @@ static Arg item_14_arg[] = {
 
 static void popdownAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+client_data = client_data;    /* Keep picky compilers happy */
+call_data = call_data;
 if (highlighted_x >= 0)
   XawTextSinkDisplayText(queue_text_sink,
     highlighted_x, highlighted_y,
@@ -138,6 +140,9 @@ char buffer[256];
 Widget text = text_create((char *)client_data, text_depth);
 FILE *f = NULL;
 
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
+
 /* End up with the split version, so message looks right when non-exist */
 
 for (i = 0; i < (spool_is_split? 2:1); i++)
@@ -171,6 +176,9 @@ char buffer[256];
 Widget text = text_create((char *)client_data, text_depth);
 FILE *f = NULL;
 
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
+
 for (i = 0; i < (spool_is_split? 2:1); i++)
   {
   message_subdir[0] = (i != 0)? ((char *)client_data)[5] : 0;
@@ -414,6 +422,8 @@ if (pid < 0) text_showf(text, "Failed to fork: %s\n", strerror(pid)); else
 
 static void deliverAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-v -M", "");
 }
 
@@ -425,6 +435,8 @@ ActOnMessage((char *)client_data, "-v -M", "");
 
 static void freezeAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-Mf", "");
 }
 
@@ -436,6 +448,8 @@ ActOnMessage((char *)client_data, "-Mf", "");
 
 static void thawAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-Mt", "");
 }
 
@@ -451,8 +465,13 @@ start-up time. If the string is empty, do nothing. */
 
 XtActionProc dialogAction(Widget w, XEvent *event, String *ss, Cardinal *c)
 {
-char *s;
-s = XawDialogGetValueString(dialog_widget);
+char *s = XawDialogGetValueString(dialog_widget);
+
+w = w;      /* Keep picky compilers happy */
+event = event;
+ss = ss;
+c = c;
+
 XtPopdown((Widget)dialog_shell);
 XtDestroyWidget((Widget)dialog_shell);
 while (isspace(*s)) s++;
@@ -549,6 +568,8 @@ filled in. */
 
 static void addrecipAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 strcpy(actioned_message, (char *)client_data);
 action_required = "-Mar";
 dialog_ref_widget = menushell;
@@ -563,6 +584,8 @@ create_dialog("Recipient address to add?", "");
 
 static void markdelAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 strcpy(actioned_message, (char *)client_data);
 action_required = "-Mmd";
 dialog_ref_widget = menushell;
@@ -576,6 +599,8 @@ create_dialog("Recipient address to mark delivered?", "");
 
 static void markalldelAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-Mmad", "");
 }
 
@@ -589,6 +614,8 @@ static void editsenderAction(Widget w, XtPointer client_data,
 {
 queue_item *q;
 char *sender;
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 strcpy(actioned_message, (char *)client_data);
 q = find_queue(actioned_message, queue_noop, 0);
 sender = (q == NULL)? "" : q->sender;
@@ -605,6 +632,8 @@ create_dialog("New sender address?", sender);
 
 static void editbodyAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-Meb", "");
 }
 
@@ -616,6 +645,8 @@ ActOnMessage((char *)client_data, "-Meb", "");
 
 static void giveupAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-v -Mg", "");
 }
 
@@ -627,6 +658,8 @@ ActOnMessage((char *)client_data, "-v -Mg", "");
 
 static void removeAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 ActOnMessage((char *)client_data, "-Mrm", "");
 }
 
@@ -641,11 +674,15 @@ static void headersAction(Widget w, XtPointer client_data, XtPointer call_data)
 char buffer[256];
 header_line *h, *next;
 Widget text = text_create((char *)client_data, text_depth);
+void *reset_point;
+
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
 
 /* Remember the point in the dynamic store so we can recover to it afterwards.
 Then use Exim's function to read the header. */
 
-void *reset_point = store_get(0);
+reset_point = store_get(0);
 
 sprintf(buffer, "%s-H", (char *)client_data);
 if (spool_read_header(buffer, TRUE, FALSE) != spool_read_OK)
@@ -704,6 +741,9 @@ static void dismissAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 pipe_item *p = pipe_chain;
 
+w = w;      /* Keep picky compilers happy */
+call_data = call_data;
+
 XtPopdown((Widget)client_data);
 XtDestroyWidget((Widget)client_data);
 
@@ -807,6 +847,9 @@ XtTranslations menu_trans = XtParseTranslationTable(
    <BtnUp>:         MenuPopdown()notify()unhighlight()\n\
   ");
 
+actargs = actargs;   /* Keep picky compilers happy */
+count = count;
+
 /* Get the sink and source and the current text pointer */
 
 queue_get_arg[0].value = (XtArgVal)(&queue_text_sink);
