@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -586,6 +586,7 @@ sprintf(buffer, "%s-H", p->name);
 if (spool_read_header(buffer, FALSE, TRUE) != spool_read_OK)
   {
   store_reset(reset_point);
+  if (jread != NULL) fclose(jread);
   return;
   }
 
@@ -742,7 +743,7 @@ while (p != NULL)
     }
 
 
-  if (ddd != NULL)
+  if (ddd != NULL || dd == NULL)
     {
     text_showf(queue_widget, "%c%2d%c %s %s %-8s %s%s%s",
       (p->frozen)? '*' : ' ',
