@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include <stdio.h>
@@ -18,13 +18,17 @@
 #include "structs.h"
 #include "globals.h"
 
+/* Don't include functions.h, because it needs types that aren't available.
+Instead, just give a prototype for the one function we need. */
+
+extern void *store_get_3(int, char *, int);
 
 void version_init(void)
 {
 int i = 0;
 char today[20];
 
-version_string = "1.51";
+version_string = "1.61";
 
 strcpy(today, __DATE__);
 if (today[4] == ' ') i = 1;
