@@ -6,7 +6,7 @@ scripts=../scripts
 
 # Get the OS type, and check that there is a make file for it.
 
-os=`$scripts/os-type` || exit 1
+os=`$scripts/os-type -generic` || exit 1
 
 if	test ! -r ../OS/Makefile-$os
 then    echo ""
@@ -32,7 +32,7 @@ ln -s ../OS/os.h-$os os.h || exit 1
 # Special-purpose fudge for older versions of Linux (pre 2.1.15) that
 # use the structure name "options" instead of "ip_options".
 
-if [ "$os" != "Linux" ] ; then exit 0; fi
+if [ "$os" != "Linux" -a "$os" != "Linux-libc5" ] ; then exit 0; fi
 
 grep ip_options /usr/include/linux/ip.h >/dev/null
 if [ $? = 0 ] ; then exit 0; fi
