@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -37,6 +37,7 @@ Arguments:
   outfd       a fd for the stdout and stderr of the new process; if -1 is given
                 fds to /dev/null are set up
   wd          if not NULL, a path to be handed to chdir() in the new process
+  make_leader if TRUE, make the new process a process group leader
 
 Returns:      the pid of the created process or -1 if anything has gone wrong;
               (via fdptr) the fd of the standard input of the new process
@@ -44,7 +45,7 @@ Returns:      the pid of the created process or -1 if anything has gone wrong;
 
 pid_t
 child_open(char **argv, char **envp, int newumask, uid_t *newuid, gid_t *newgid,
-  int *fdptr, int outfd, char *wd)
+  int *fdptr, int outfd, char *wd, BOOL make_leader)
 {
 int pfd[2];
 
@@ -67,6 +68,7 @@ on the wait. */
 if (pipe(pfd) == 0)
   {
   pid_t pid;
+  int save_errno;
 
   oldsignal = signal(SIGCHLD, SIG_DFL);
   pid = fork();
@@ -79,14 +81,16 @@ if (pipe(pfd) == 0)
     {
     int fd;
 
-    close(pfd[pipe_write]);     /* Be absolutely sure this one is closed */
+    if (make_leader) setpgid(0,0); /* Process group leader if required */
+
+    close(pfd[pipe_write]);        /* Be absolutely sure this one is closed */
 
     for (fd = mac_maxfd; fd >= 0; fd--)
       if (fd != pfd[pipe_read] && fd != outfd) close(fd);
 
     if (pfd[pipe_read] != 0)
       {
-      dup2(pfd[pipe_read], 0);  /* Make the standard input */
+      dup2(pfd[pipe_read], 0);     /* Make the standard input */
       close(pfd[pipe_read]);
       }
 
@@ -130,6 +134,8 @@ if (pipe(pfd) == 0)
     _exit(EX_UNAVAILABLE);      /* Note: must be _exit(), NOT exit() */
     }
 
+  save_errno = errno;
+
   /* Parent */
 
   close(pfd[pipe_read]);
@@ -140,7 +146,6 @@ if (pipe(pfd) == 0)
     }
   else                          /* fork failed */
     {
-    int save_errno = errno;
     close(pfd[pipe_write]);
     errno = save_errno;
     return (pid_t)(-1);
