@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with running Exim as a daemon */
@@ -12,87 +12,6 @@
 
 
 
-/*************************************************
-*          Host and net setup tables             *
-*************************************************/
-
-/* Setup work for lists of hosts, nets and addresses is done once at the start
-of the daemon, to save doing it for every call. These list the options that are
-so processed. */
-
-typedef struct {
-  char **text;
-  host_item **data;
-} host_setup;
-
-typedef struct {
-  char **text;
-  ip_net_item **data;
-} net_setup;
-
-typedef struct {
-  char **text;
-  re_block **data;
-} address_setup;
-
-host_setup host_setup_list[] = {
-  { &helo_accept_junk_hosts,     &helo_accept_junk_hostlist },
-  { &helo_verify_except_hosts,   &helo_verify_except_hostlist },
-  { &sender_verify_except_hosts, &sender_verify_except_hostlist },
-  { &rfc1413_except_hosts,       &rfc1413_except_hostlist },
-  { &sender_host_accept,         &sender_host_accept_hosts },
-  { &sender_host_accept_relay,   &sender_host_accept_relay_hosts },
-  { &sender_host_reject,         &sender_host_reject_hosts },
-  { &sender_host_reject_except,  &sender_host_reject_except_hosts },
-  { &sender_host_reject_recipients, &sender_host_reject_recipients_hosts },
-  { &sender_host_reject_relay,   &sender_host_reject_relay_hosts },
-  { &sender_host_reject_relay_except, &sender_host_reject_relay_except_hosts },
-  { &smtp_etrn_hosts,            &smtp_etrn_hostlist },
-  { &smtp_expn_hosts,            &smtp_expn_hostlist },
-  { &smtp_reserve_hosts,         &smtp_reserve_hostlist },
-  { &receiver_unqualified_hosts, &receiver_unqualified_hostlist },
-  { &receiver_verify_except_hosts, &receiver_verify_except_hostlist },
-  { &sender_unqualified_hosts,   &sender_unqualified_hostlist },
-  { NULL, NULL}
-};
-
-net_setup net_setup_list[] = {
-  { &helo_accept_junk_nets,      &helo_accept_junk_netlist },
-  { &helo_verify_except_nets,    &helo_verify_except_netlist },
-  { &host_lookup_nets,           &host_lookup_netlist },
-  { &rbl_except_nets,            &rbl_except_netlist },
-  { &sender_verify_except_nets,  &sender_verify_except_netlist },
-  { &rfc1413_except_nets,        &rfc1413_except_netlist },
-  { &sender_net_accept,          &sender_net_accept_nets },
-  { &sender_net_accept_relay,    &sender_net_accept_relay_nets },
-  { &sender_net_reject,          &sender_net_reject_nets },
-  { &sender_net_reject_except,   &sender_net_reject_except_nets },
-  { &sender_net_reject_recipients, &sender_net_reject_recipients_nets },
-  { &sender_net_reject_relay,    &sender_net_reject_relay_nets },
-  { &sender_net_reject_relay_except, &sender_net_reject_relay_except_nets },
-  { &smtp_etrn_nets,             &smtp_etrn_netlist },
-  { &smtp_reserve_nets,          &smtp_reserve_netlist },
-  { &receiver_unqualified_nets,  &receiver_unqualified_netlist },
-  { &receiver_verify_except_nets, &receiver_verify_except_netlist },
-  { &sender_unqualified_nets,    &sender_unqualified_netlist },
-  { NULL, NULL}
-};
-
-address_setup address_setup_list[] = {
-  { &queue_remote_except,        &re_queue_remote_except },
-  { &receiver_verify_addresses,  &re_receiver_verify_addresses },
-  { &receiver_verify_senders,    &re_receiver_verify_senders },
-  { &receiver_verify_senders_except, &re_receiver_verify_senders_except },
-  { &recipients_reject_except,   &re_recipients_reject_except },
-  { &sender_accept,              &re_sender_accept },
-  { &sender_accept_recipients,   &re_sender_accept_recipients },
-  { &sender_reject,              &re_sender_reject },
-  { &sender_reject_recipients,   &re_sender_reject_recipients },
-  { NULL, NULL}
-};
-
-
-
 /*************************************************
 *               Local static variables           *
 *************************************************/
@@ -106,11 +25,10 @@ static int   accept_retry_errno;
 static BOOL  accept_retry_select_failed;
 
 static int   queue_run_count = 0;
-static int   queue_pid_slotcount;
 static pid_t *queue_pid_slots;
-
-static int   smtp_pid_slotcount;
 static pid_t *smtp_pid_slots;
+static char  **smtp_host_address_slots;
+
 
 
 
@@ -128,6 +46,7 @@ Returns:  nothing
 static void
 daemon_sigalrm_handler(int sig)
 {
+sig = sig;    /* Keep picky compilers happy */
 daemon_sigalrm_seen = TRUE;
 os_non_restarting_signal(SIGALRM, daemon_sigalrm_handler);
 }
@@ -147,6 +66,7 @@ Returns:  nothing
 static void
 sighup_handler(int sig)
 {
+sig = sig;    /* Keep picky compilers happy */
 sighup_seen = TRUE;
 signal(SIGHUP, sighup_handler);
 }
@@ -170,6 +90,7 @@ Returns:  nothing
 static void
 main_sigchld_handler(int sig)
 {
+sig = sig;    /* Keep picky compilers happy */
 sigchld_seen = TRUE;
 signal(SIGCHLD, SIG_DFL);
 }
@@ -250,8 +171,7 @@ subprocess because it might take time. */
 if (smtp_load_reserve >= 0)
   {
   load_average = os_getloadavg();
-  if (smtp_reserve_hosts == NULL && smtp_reserve_nets == NULL &&
-      load_average > smtp_load_reserve)
+  if (smtp_reserve_hosts == NULL && load_average > smtp_load_reserve)
     {
     DEBUG(1) debug_printf("rejecting SMTP connection: load average = %.2f\n",
       (double)load_average/1000.0);
@@ -265,7 +185,56 @@ if (smtp_load_reserve >= 0)
     }
   }
 
-/* Now fork the accepting process */
+/* Check that one specific host (strictly, IP address) is not hogging
+resources. This is done here to prevent a denial of service attack by someone
+forcing you to fork lots of times to then deny him service. If we have fewer
+connections than smtp_accept_max_per_host, we can skip the tedious per
+host_address checks. Note that at this stage smtp_accept_count contains
+the count of *other* connections, not including this one. */
+
+if ((smtp_accept_max_per_host > 0) &&
+    (smtp_accept_count >= smtp_accept_max_per_host))
+  {
+  int i;
+  int host_accept_count = 0;
+  int other_host_count = 0;    /* keep a count of non matches to optomise */
+
+  for (i = 0; i < smtp_accept_max; ++i)
+    {
+    if (smtp_host_address_slots[i] != NULL)
+      {
+      if (strcmp(sender_host_address, smtp_host_address_slots[i]) == 0)
+       host_accept_count++;
+      else
+       other_host_count++;
+
+      /* Testing all these strings is expensive - see if we can drop out
+      early, either by hitting the target, or finding there are not enough
+      connections left to make the target. */
+
+      if ((host_accept_count >= smtp_accept_max_per_host) ||
+	 ((smtp_accept_count - other_host_count) < smtp_accept_max_per_host))
+       break;
+      }
+    }
+
+  if (host_accept_count >= smtp_accept_max_per_host)
+    {
+    DEBUG(1) debug_printf("rejecting SMTP connection (too many from one "
+      "address: count=%d max=%d\n",
+      host_accept_count, smtp_accept_max_per_host);
+    DEBUG(1) debug_printf("421 %s: Too many concurrent SMTP connections "
+      "from one IP address; please try again later.\n", primary_hostname);
+    fprintf(smtp_out, "421 %s: Too many concurrent SMTP connections "
+      "from one IP address; please try again later.\r\n", primary_hostname);
+    log_write(4, LOG_MAIN, "Connection from %s refused: too many connections "
+      "from that address", sender_host_address);
+    goto ERROR_RETURN;
+    }
+  }
+
+/* OK, the connection count checks have been passed. Now we can fork the
+accepting process */
 
 pid = fork();
 
@@ -275,7 +244,25 @@ if (pid == 0)
   {
   int i;
   int message_count = 0;
-  BOOL local_queue_only = queue_only;
+  int queue_only_reason = 0;
+  BOOL local_queue_only;
+  union sockaddr_46 interface_sockaddr;
+
+  /* Get the local interface address into some permanent store */
+
+  int size = sizeof(interface_sockaddr);
+  if (getsockname(accept_socket, (struct sockaddr *)(&interface_sockaddr),
+       &size) == 0)
+    {
+    char *buffer = store_malloc(sizeof(ip_address_item));
+    interface_address = host_ntoa(-1, &interface_sockaddr, buffer);
+    DEBUG(9) debug_printf("interface address = %s\n", interface_address);
+    }
+
+  /* Initialize the queueing flags */
+
+  queue_check_only();
+  local_queue_only = queue_only;
 
   /* Close the listening sockets, and set the SIGCHLD handler to SIG_IGN.
   This is not the same as SIG_DFL, despite the fact that documentation often
@@ -305,14 +292,21 @@ if (pid == 0)
   till later so it will have a message id attached. */
 
   if (smtp_accept_queue > 0 && smtp_accept_count >= smtp_accept_queue)
+    {
     local_queue_only = TRUE;
+    queue_only_reason = 1;
+    }
 
   /* Handle the start of the SMTP session, then loop, accepting incoming
   messages from the SMTP connection. The end will come at the QUIT command,
-  when accept_msg() returns FALSE. A break in the connection causes the
+  when smtp_setup_msg() returns 0. A break in the connection causes the
   process to die (see accept.c). */
 
-  if (!smtp_start_session()) _exit(EXIT_SUCCESS);
+  if (!smtp_start_session())
+    {
+    fflush(smtp_out);
+    _exit(EXIT_SUCCESS);
+    }
 
   for (;;)
     {
@@ -333,10 +327,14 @@ if (pid == 0)
 
     if ((rc = smtp_setup_msg()) > 0)
       {
-      (void) accept_msg(smtp_in, smtp_out, FALSE);
+      (void) accept_msg(smtp_in, FALSE);
       if (message_id[0] == 0) continue;
       }
-    else exit((rc == 0)? EXIT_SUCCESS : EXIT_FAILURE);
+    else
+      {
+      fflush(smtp_out);
+      _exit((rc == 0)? EXIT_SUCCESS : EXIT_FAILURE);
+      }
 
     /* Show the recipients when debugging */
 
@@ -383,30 +381,41 @@ if (pid == 0)
           ++message_count > smtp_accept_queue_per_connection)
         {
         local_queue_only = TRUE;
-        log_write(4, LOG_MAIN, "no immediate delivery: more than %d messages "
-          "received in one connection\n", smtp_accept_queue_per_connection);
+        queue_only_reason = 2;
         }
       else if (queue_only_load >= 0)
         {
         local_queue_only = (load_average = os_getloadavg()) > queue_only_load;
-        if (local_queue_only)
-          log_write(4, LOG_MAIN, "no immediate delivery: load average = %.2f\n",
-            (double)load_average/1000.0);
+        if (local_queue_only) queue_only_reason = 3;
         }
       }
 
     /* Log the queueing here, when it will get a message id attached, but
-    not if queue_only is set. */
+    not if queue_only is set (case 0). */
 
-    else if (!queue_only) log_write(4, LOG_MAIN,
-      "no immediate delivery: too many connections (%d, max %d)\n",
-        smtp_accept_count, smtp_accept_queue);
+    if (local_queue_only) switch(queue_only_reason)
+      {
+      case 1:
+      log_write(4, LOG_MAIN, "no immediate delivery: too many connections "
+        "(%d, max %d)", smtp_accept_count, smtp_accept_queue);
+      break;
+
+      case 2:
+      log_write(4, LOG_MAIN, "no immediate delivery: more than %d messages "
+        "received in one connection", smtp_accept_queue_per_connection);
+      break;
+
+      case 3:
+      log_write(4, LOG_MAIN, "no immediate delivery: load average %.2f",
+        (double)load_average/1000.0);
+      break;
+      }
 
     /* If a delivery attempt is required, spin off a new process to handle it.
     If we are not root, we have to re-exec exim unless root can be regained by
     the use of seteuid. */
 
-    if (!local_queue_only && (pid = fork()) == 0)
+    else if ((pid = fork()) == 0)
       {
       fclose(smtp_in);
       fclose(smtp_out);
@@ -414,7 +423,7 @@ if (pid == 0)
       if (security_level >= 2)
         {
         int i = 0;
-        char *argv[9 + MAX_CLMACROS];
+        char *argv[11 + MAX_CLMACROS];
         argv[i++] = exim_path;
         if (clmacro_count > 0)
           {
@@ -423,8 +432,10 @@ if (pid == 0)
           }
         if (debug_level > 0)
           argv[i++] = string_sprintf("-d%d", debug_level);
-        if (debug_trace_memory) argv[i++] = "-dm";
+        if (debug_trace_memory != 0) argv[i++] = "-dm";
         if (dont_deliver) argv[i++] = "-N";
+        if (queue_smtp) argv[i++] = "-odqs";
+        if (queue_remote) argv[i++] = "-odqr";
         if (config_changed)
           {
           argv[i++] = "-C";
@@ -434,11 +445,12 @@ if (pid == 0)
         argv[i++] = message_id;
         argv[i++] = (char *)0;
         execv(argv[0], argv);
-        log_write(0, LOG_PANIC_DIE, "exec of exim -Mc failed");
+        log_write(0, LOG_PANIC_DIE, "exec of exim -Mc failed: %s",
+          strerror(errno));
         }
 
       /* No need to re-exec */
-      (void)deliver_message(message_id, FALSE, FALSE, FALSE);
+      (void)deliver_message(message_id, FALSE, FALSE);
       _exit(EXIT_SUCCESS);
       }
 
@@ -461,11 +473,13 @@ if (pid < 0)
 else
   {
   int i;
-  for (i = 0; i < smtp_pid_slotcount; ++i)
+  for (i = 0; i < smtp_accept_max; ++i)
     {
     if (smtp_pid_slots[i] <= 0)
       {
       smtp_pid_slots[i] = pid;
+      if (smtp_accept_max_per_host > 0)
+	smtp_host_address_slots[i] = string_copy_malloc(sender_host_address);
       smtp_accept_count++;
       break;
       }
@@ -576,7 +590,7 @@ terminal, if we are not debugging. Most modern Unixes seem to have setsid() for
 getting rid of the controlling terminal. For any OS that doesn't, setsid() can
 be #defined as a no-op, or as something else. */
 
-if (debug_level <= 0 && !debug_trace_memory)
+if (debug_level <= 0)
   {
   pid_t pid;
   int fd;
@@ -622,41 +636,12 @@ if (daemon_listen)
   int sk;
   int on = 1;
   ip_address_item *addresses, *ipa;
+  union sockaddr_46 sin;
 
   #if HAVE_IPV6
-  struct sockaddr_in6 sin;
   struct in6_addr anyaddr6 = { IN6ADDR_ANY_INIT };
-  #else
-  struct sockaddr_in sin;
   #endif
 
-  host_setup *hset;
-  net_setup *nset;
-  address_setup *aset;
-
-  /* If there are lists of nets and hosts + idents and addresses for various
-  checks, pre-process them now so that this work is not repeated for each
-  message. For a sender_{accept,reject} list, just do a match that won't work
-  so as to get any regular expressions compiled. */
-
-  for (hset = host_setup_list; hset->text != NULL; hset++)
-    if (*(hset->text) != NULL)
-      verify_setup_hostlist(*(hset->text), hset->data);
-
-  for (nset = net_setup_list; nset->text != NULL; nset++)
-    if (*(nset->text) != NULL)
-      verify_setup_netlist(*(nset->text), nset->data);
-
-  for (aset = address_setup_list; aset->text != NULL; aset++)
-    if (*(aset->text) != NULL)
-      (void)match_address_list("?@?", 2, *(aset->text), aset->data, -1, ':');
-
-  /* The above may have caused databases to get opened and cached; in
-  particular, the sender_{accept,reject} options may do this. So make
-  sure that any cached open files are closed. */
-
-  search_tidyup();
-
   /* Find the standard SMTP port if no port number given; otherwise
   convert the given port to network order. */
 
@@ -715,13 +700,36 @@ if (daemon_listen)
     int i;
 
     #if HAVE_IPV6
-    listen_sockets[sk] = socket(AF_INET6, SOCK_STREAM, 0);
-    #else
-    listen_sockets[sk] = socket(AF_INET, SOCK_STREAM, 0);
+    BOOL ipv6_socket, ipv6_address;
     #endif
 
+    /* A system which has all the IPv6 libraries etc. may nevertheless not
+    support IPv6 in the kernel, so if the socket creation fails, fall back
+    on creating an IPv4 socket, except when we have an explicit IPv6 address,
+    for which we really do need an IPv6 socket. */
+
+    #if HAVE_IPV6
+    ipv6_address = strchr(ipa->address, ':') != NULL;
+    listen_sockets[sk] = socket(AF_INET6, SOCK_STREAM, 0);
     if (listen_sockets[sk] < 0)
-      log_write(0, LOG_PANIC_DIE, "socket creation failed: %s", strerror(errno));
+      {
+      if (!ipv6_address)
+        {
+        listen_sockets[sk] = socket(AF_INET, SOCK_STREAM, 0);
+        ipv6_socket = FALSE;
+        }
+      }
+    else ipv6_socket = TRUE;
+
+    /* The IPv4 case */
+
+    #else    /* HAVE_IPV6 */
+    listen_sockets[sk] = socket(AF_INET, SOCK_STREAM, 0);
+    #endif   /* HAVE_IPV6 */
+
+    if (listen_sockets[sk] < 0)
+      log_write(0, LOG_PANIC_DIE, "socket creation failed: %s",
+        strerror(errno));
 
     /* Set SO_REUSEADDR so that the daemon can be restarted while a connection
     is being handled.  Without this, a connection will prevent reuse of the
@@ -739,48 +747,73 @@ if (daemon_listen)
     memset(&sin, 0, sizeof(sin));
 
     /* This code has to be entirely different for IPv6 and IPv4 because
-    of the different struct layouts. Sigh. */
+    of the different struct layouts. Sigh. Under IPv6, however, we may have
+    ended up with an IPv4 socket because the kernel may not support IPv6 even
+    if the libraries etc. are available. In this case, revert to IPv4 style
+    code (this will happen only when the the address is not an IPv6 one).
+    Although the code is identical, I've chosen to replicate it for ease of
+    reading the two cases. */
 
     #if HAVE_IPV6
-    sin.sin6_family = AF_INET6;
-    sin.sin6_port = net_port;
+    if (ipv6_socket)
+      {
+      sin.v6.sin6_family = AF_INET6;
+      sin.v6.sin6_port = net_port;
 
-    if (ipa->address[0] == 0)
-      {
-      sin.sin6_addr = anyaddr6;
-      DEBUG(9) debug_printf("listening on all interfaces\n");
-      }
-    else
-      {
-      int rc;
-      if (strchr(ipa->address, ':') != NULL)
-        rc = inet_pton(AF_INET6, ipa->address, &sin.sin6_addr);
+      if (ipa->address[0] == 0)
+        {
+        sin.v6.sin6_addr = anyaddr6;
+        DEBUG(9) debug_printf("listening on all interfaces (IPv6)\n");
+        }
       else
         {
-        rc = inet_pton(AF_INET, ipa->address, &sin.sin6_flowinfo);
-        sin.sin6_family = AF_INET;
+        int rc;
+        if (ipv6_address)
+          rc = inet_pton(AF_INET6, ipa->address, &sin.v6.sin6_addr);
+        else
+          {
+          rc = inet_pton(AF_INET, ipa->address, &sin.v4.sin_addr);
+          sin.v4.sin_family = AF_INET;
+          }
+        if (rc != 1)
+          log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", ipa->address);
+        DEBUG(9) debug_printf("listening on %s\n", ipa->address);
         }
-      if (rc != 1)
-        log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", ipa->address);
-      DEBUG(9) debug_printf("listening on %s\n", ipa->address);
       }
-    #else
+    else   /* Revert to IPv4 code */
+      {
+      sin.v4.sin_family = AF_INET;
+      sin.v4.sin_port = net_port;
+      if (ipa->address[0] == 0)
+        {
+        sin.v4.sin_addr.s_addr = (S_ADDR_TYPE)INADDR_ANY;
+        DEBUG(9) debug_printf("listening on all interfaces (IPv4)\n");
+        }
+      else
+        {
+        sin.v4.sin_addr.s_addr = (S_ADDR_TYPE)inet_addr(ipa->address);
+        DEBUG(9) debug_printf("listening on %s\n", ipa->address);
+        }
+      }
+    #else   /* HAVE_IPV6 */
 
-    /* The IPv4 case */
 
-    sin.sin_family = AF_INET;
-    sin.sin_port = net_port;
+    /* The IPv4 case is simple. */
+
+    sin.v4.sin_family = AF_INET;
+    sin.v4.sin_port = net_port;
     if (ipa->address[0] == 0)
       {
-      sin.sin_addr.s_addr = (S_ADDR_TYPE)INADDR_ANY;
+      sin.v4.sin_addr.s_addr = (S_ADDR_TYPE)INADDR_ANY;
       DEBUG(9) debug_printf("listening on all interfaces\n");
       }
     else
       {
-      sin.sin_addr.s_addr = (S_ADDR_TYPE)inet_addr(ipa->address);
+      sin.v4.sin_addr.s_addr = (S_ADDR_TYPE)inet_addr(ipa->address);
       DEBUG(9) debug_printf("listening on %s\n", ipa->address);
       }
-    #endif
+    #endif  /* HAVE_IPV6 */
+
 
     /* Retry loop */
 
@@ -802,30 +835,30 @@ if (daemon_listen)
       }
     }
 
-  /* Do a sanity check on the max connects value; zero means no limit,
-  in which case we don't need to keep a list of them. */
+  /* Do a sanity check on the max connects value just to save us from getting
+  a huge amount of store. */
 
-  if (smtp_accept_max < 0 || smtp_accept_max > 4095) smtp_accept_max = 0;
+  if (smtp_accept_max > 4095) smtp_accept_max = 4096;
 
-  /* Do a sanity check on the "max connects until we queue only" (no delivery)
-  value. Again, zero means no limit, and there's no point setting it unless it
-  is less than the max connects limit. */
+  /* There's no point setting smtp_accept_queue unless it is less than the max
+  connects limit. The configuration reader ensures that the max is set if the
+  queue-only option is set. */
 
-  if (smtp_accept_queue < 0 || smtp_accept_queue > 4095) smtp_accept_queue = 0;
-  if (smtp_accept_max > 0 && smtp_accept_queue > smtp_accept_max)
-    smtp_accept_queue = 0;
+  if (smtp_accept_queue > smtp_accept_max) smtp_accept_queue = 0;
 
   /* Get somewhere to keep the list of SMTP accepting pids if we are keeping
-  track of them, either for total number, or for queue-only handling. */
+  track of them for total number and queue/host limits. */
 
-  smtp_pid_slotcount = (smtp_accept_queue > smtp_accept_max)?
-    smtp_accept_queue : smtp_accept_max;
-
-  if (smtp_pid_slotcount > 0)
+  if (smtp_accept_max > 0)
     {
     int i;
-    smtp_pid_slots = store_get(smtp_pid_slotcount * sizeof(pid_t));
-    for (i = 0; i < smtp_pid_slotcount; i++) smtp_pid_slots[i] = 0;
+    smtp_pid_slots = store_get(smtp_accept_max * sizeof(pid_t));
+    smtp_host_address_slots = store_get(smtp_accept_max * sizeof(char *));
+    for (i = 0; i < smtp_accept_max; i++)
+      {
+      smtp_pid_slots[i] = 0;
+      smtp_host_address_slots[i] = NULL;
+      }
     }
 
   /* Start listening on the bound sockets, establishing the maximum backlog of
@@ -867,22 +900,16 @@ if (exim_uid_set)
     mac_setegid(exim_gid);
     mac_seteuid(exim_uid);
     }
-  DEBUG(2) debug_printf("uid=%d gid=%d euid=%d egid=%d\n",
-    getuid(), getgid(), geteuid(), getegid());
   }
 
 /* Get somewhere to keep the list of queue-runner pids if we are keeping track
 of them (and also if we are doing queue runs). */
 
-if (queue_interval > 0)
+if (queue_interval > 0 && queue_run_max > 0)
   {
-  queue_pid_slotcount = queue_run_max;
-  if (queue_pid_slotcount > 0)
-    {
-    int i;
-    queue_pid_slots = store_get(queue_pid_slotcount * sizeof(pid_t));
-    for (i = 0; i < queue_pid_slotcount; i++) queue_pid_slots[i] = 0;
-    }
+  int i;
+  queue_pid_slots = store_get(queue_run_max * sizeof(pid_t));
+  for (i = 0; i < queue_run_max; i++) queue_pid_slots[i] = 0;
   }
 
 /* Set up the handler for termination of child processes. */
@@ -937,8 +964,7 @@ it closes the log first. */
 
 log_close_all();
 
-DEBUG(2) debug_printf("daemon running with uid=%d gid=%d euid=%d egid=%d\n",
-  (int)getuid(), (int)getgid(), (int)geteuid(), (int)getegid());
+DEBUG(2) debug_print_ids("daemon running with");
 
 /* Any messages accepted via this route are going to be SMTP. */
 
@@ -955,6 +981,7 @@ for (;;)
   #endif
 
   int len = sizeof(accepted);
+  int status;
   pid_t pid;
 
   /* This code is placed first in the loop, so that it gets obeyed at the
@@ -1016,7 +1043,8 @@ for (;;)
           *p = 0;
           argv[i++] = (char *)0;
           execv(argv[0], argv);
-          log_write(0, LOG_PANIC_DIE, "exec of exim %s failed", opt);
+          log_write(0, LOG_PANIC_DIE, "exec of exim %s failed: %s", opt,
+            strerror(errno));
           }
 
         /* No need to re-exec */
@@ -1032,7 +1060,7 @@ for (;;)
       else
         {
         int i;
-        for (i = 0; i < queue_pid_slotcount; ++i)
+        for (i = 0; i < queue_run_max; ++i)
           {
           if (queue_pid_slots[i] <= 0)
             {
@@ -1185,36 +1213,49 @@ for (;;)
   lose SIGCHLD signals at busy times, so to be on the safe side, just
   do it each time round. It shouldn't be too expensive. */
 
-  while ((pid = waitpid(-1, NULL, WNOHANG)) > 0)
+  while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
     {
     int i;
-    DEBUG(2) debug_printf("child %d ended\n", pid);
+    DEBUG(2) debug_printf("child %d ended: status=0x%x\n", pid, status);
 
-    /* Deal with an accepting process. */
+    /* If it's a listening daemon, deal with an accepting process. */
 
-    for (i = 0; i < smtp_pid_slotcount; ++i)
+    if (daemon_listen)
       {
-      if (smtp_pid_slots[i] == pid)
+      for (i = 0; i < smtp_accept_max; i++)
         {
-        smtp_pid_slots[i] = 0;
-        if (--smtp_accept_count < 0) smtp_accept_count = 0;
-        DEBUG(2) debug_printf("%d SMTP accept process%s now running\n",
-          smtp_accept_count, (smtp_accept_count == 1)? "" : "es");
-        break;
+        if (smtp_pid_slots[i] == pid)
+          {
+          smtp_pid_slots[i] = 0;
+          if (smtp_host_address_slots[i] != NULL)
+            {
+	    store_free(smtp_host_address_slots[i]);
+	    smtp_host_address_slots[i] = NULL;
+            }
+          if (--smtp_accept_count < 0) smtp_accept_count = 0;
+          DEBUG(2) debug_printf("%d SMTP accept process%s now running\n",
+            smtp_accept_count, (smtp_accept_count == 1)? "" : "es");
+          break;
+          }
         }
+      if (i < smtp_accept_max) continue;  /* Found an accepting process */
       }
 
-    /* Deal with a queue-runner process. */
+    /* If it wasn't an accepting process, see if it was a queue-runner
+    process, if we are keeping track of them. */
 
-    for (i = 0; i < queue_pid_slotcount; ++i)
+    if (queue_interval > 0)
       {
-      if (queue_pid_slots[i] == pid)
+      for (i = 0; i < queue_run_max; i++)
         {
-        queue_pid_slots[i] = 0;
-        if (--queue_run_count < 0) queue_run_count = 0;
-        DEBUG(2) debug_printf("%d queue-runner process%s now running\n",
-          queue_run_count, (queue_run_count == 1)? "" : "es");
-        break;
+        if (queue_pid_slots[i] == pid)
+          {
+          queue_pid_slots[i] = 0;
+          if (--queue_run_count < 0) queue_run_count = 0;
+          DEBUG(2) debug_printf("%d queue-runner process%s now running\n",
+            queue_run_count, (queue_run_count == 1)? "" : "es");
+          break;
+          }
         }
       }
     }
@@ -1241,8 +1282,8 @@ for (;;)
     for (sk = 0; sk < listen_socket_count; sk++) close(listen_sockets[sk]);
     sighup_argv[0] = exim_path;
     execv(exim_path, sighup_argv);
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed",
-      getpid(), exim_path);
+    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed: %s",
+      getpid(), exim_path, strerror(errno));
     }
 
   }   /* End of main loop */
