@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -50,6 +50,7 @@ on a separate file, this is believed not to be likely to happen any more. */
 static void
 segv_handler(int signal)
 {
+signal = signal;       /* Keep picky compilers happy */
 longjmp(read_env, 1);
 }
 
@@ -90,7 +91,7 @@ created, and we also try to make the directory if the initial open fails. */
 sprintf(buffer, "%s/db/%s.lockfile", spool_directory, name);
 if ((dbblock->lockfd = open(buffer, O_RDWR|O_CREAT, DB_LOCKFILE_MODE)) < 0)
   {
-  directory_make(spool_directory, "db", DB_DIRECTORY_MODE, TRUE);
+  (void)directory_make(spool_directory, "db", DB_DIRECTORY_MODE, TRUE);
   dbblock->lockfd = open(buffer, O_RDWR|O_CREAT, DB_LOCKFILE_MODE);
   }
 
@@ -194,7 +195,7 @@ close(dbblock->lockfd);
 *************************************************/
 
 /* Passing back the pointer unchanged is useless, because there is
-no guarantee of alignment. Since all the records used by exim need
+no guarantee of alignment. Since all the records used by Exim need
 to be properly aligned to pick out the timestamps, etc., we might as
 well do the copying centrally here.
 
@@ -249,10 +250,16 @@ if (setjmp(read_env) == 0)
   return yield;
   }
 
+/* Caught a SEGV. Returning NULL from here is often interperted as "no retry
+data for this domain", and that might cause frequent subsequent attempts to
+read the same data. That in turn could cause a heck of a lot of data to get
+written to the logs. Therefore, insert a short delay. */
+
 else
   {
   log_write(0, LOG_MAIN|LOG_PANIC, "SEGV while reading \"%s\" from "
     "dbm file: record assumed not to exist", key);
+  sleep(1);
   return NULL;
   }
 }
@@ -395,8 +402,9 @@ if (stat(name, &statbuf) != 0)
   printf("Failed to stat %s: %s\n", name, strerror(errno));
   return;
   }
-printf("%s: uid = %d gid = %d mode = %04o size=%d blksize=%d\n",
-  name, (int)statbuf.st_uid, (int)statbuf.st_gid, (int)statbuf.st_mode & 07777,
+printf("%s: uid = %ld gid = %ld mode = %04o size=%d blksize=%d\n",
+  name, (long int)(statbuf.st_uid), (long int)(statbuf.st_gid),
+  (int)statbuf.st_mode & 07777,
   (int)statbuf.st_size, (int)statbuf.st_blksize);
 }
 
