@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -63,12 +63,31 @@ of DB_VERSION_STRING, which is present in versions 2.x. */
 #define EXIM_DBPUT(db, key, data)      \
        (db)->put(db, NULL, &key, &data, 0)
 
+/* EXIM_DBPUTB - non-overwriting for use by dbmbuild */
+#define EXIM_DBPUTB(db, key, data)      \
+       (db)->put(db, NULL, &key, &data, DB_NOOVERWRITE)
+
+/* Return values from EXIM_DBPUTB */
+
+#define EXIM_DBPUTB_OK  0
+#define EXIM_DBPUTB_DUP DB_KEYEXIST
+
 /* EXIM_DBDEL */
 #define EXIM_DBDEL(db, key)     (db)->del(db, NULL, &key, 0)
 
 /* EXIM_DBCREATE_CURSOR - initialize for scanning operation */
+
+/* The API of this function was changed between releases 2.4.14 and 2.7.3. I do
+not know exactly where the change happened, but the Change Log for 2.5.9 lists
+the new option that is available, so I guess that it happened at 2.5.x. */
+
+#if DB_VERSION_MINOR >= 5
+#define EXIM_DBCREATE_CURSOR(db, cursor) \
+       (db)->cursor(db, NULL, cursor, 0)
+#else
 #define EXIM_DBCREATE_CURSOR(db, cursor) \
        (db)->cursor(db, NULL, cursor)
+#endif
 
 /* EXIM_DBSCAN - returns TRUE if data is returned, FALSE at end */
 #define EXIM_DBSCAN(db, key, data, first, cursor)      \
@@ -120,6 +139,15 @@ datum structure contains other fields that must be cleared. */
 #define EXIM_DBPUT(db, key, data)      \
        (db)->put(db, &key, &data, 0)
 
+/* EXIM_DBPUTB - non-overwriting for use by dbmbuild */
+#define EXIM_DBPUTB(db, key, data)      \
+       (db)->put(db, &key, &data, R_NOOVERWRITE)
+
+/* Returns from EXIM_DBPUTB */
+
+#define EXIM_DBPUTB_OK  0
+#define EXIM_DBPUTB_DUP 1
+
 /* EXIM_DBDEL */
 #define EXIM_DBDEL(db, key)     (db)->del(db, &key, 0)
 
@@ -130,8 +158,9 @@ datum structure contains other fields that must be cleared. */
 #define EXIM_DBSCAN(db, key, data, first, cursor)      \
        ((db)->seq(db, &key, &data, (first? R_FIRST : 0)) == 0)
 
-/* EXIM_DBDELETE_CURSOR - terminate scanning operation (null) */
-#define EXIM_DBDELETE_CURSOR(cursor) {}
+/* EXIM_DBDELETE_CURSOR - terminate scanning operation (null). Make it
+refer to cursor, to keep picky compilers happy. */
+#define EXIM_DBDELETE_CURSOR(cursor) { cursor = cursor; }
 
 /* EXIM_DBCLOSE */
 #define EXIM_DBCLOSE(db)        (db)->close(db)
@@ -176,6 +205,15 @@ datum structure contains other fields that must be cleared. */
 #define EXIM_DBPUT(db, key, data)      \
        dbm_store(db, key, data, DBM_REPLACE)
 
+/* EXIM_DBPUTB - non-overwriting for use by dbmbuild */
+#define EXIM_DBPUTB(db, key, data)      \
+       dbm_store(db, key, data, DBM_INSERT)
+
+/* Returns from EXIM_DBPUTB */
+
+#define EXIM_DBPUTB_OK  0
+#define EXIM_DBPUTB_DUP 1
+
 /* EXIM_DBDEL */
 #define EXIM_DBDEL(db, key) dbm_delete(db, key)
 
@@ -186,8 +224,9 @@ datum structure contains other fields that must be cleared. */
 #define EXIM_DBSCAN(db, key, data, first, cursor)      \
        (key = (first? dbm_firstkey(db) : dbm_nextkey(db)), key.dptr != NULL)
 
-/* EXIM_DBDELETE_CURSOR - terminate scanning operation (null) */
-#define EXIM_DBDELETE_CURSOR(cursor) {}
+/* EXIM_DBDELETE_CURSOR - terminate scanning operation (null). Make it
+refer to cursor, to keep picky compilers happy. */
+#define EXIM_DBDELETE_CURSOR(cursor) { cursor = cursor; }
 
 /* EXIM_DBCLOSE */
 #define EXIM_DBCLOSE(db) dbm_close(db)
