@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -37,11 +37,34 @@ if (debug_level >= 1 && debug_string != NULL)
 
 
 
+/*************************************************
+*      Print current uids and gids               *
+*************************************************/
+
+/*
+Argument:   an introductory string
+Returns:    nothing
+*/
+
+void
+debug_print_ids(char *s)
+{
+debug_printf("%s uid=%ld gid=%ld euid=%ld egid=%ld\n", s,
+  (long int)getuid(), (long int)getgid(), (long int)geteuid(),
+  (long int)getegid());
+}
+
+
+
+
 /*************************************************
 *           Print debugging message              *
 *************************************************/
 
-/* If debug_pid is nonzero, print the pid at the start of each message and add
+/* There are two entries, one for use when being called directly from a
+function with a variable argument list.
+
+If debug_pid is nonzero, print the pid at the start of each message and add
 a newline if there isn't one. This is for tidier output when running parallel
 remote deliveries with debugging turned on. Must do the whole thing with a
 single printf and flush, as otherwise output may get interleaved. */
@@ -50,11 +73,18 @@ void
 debug_printf(char *format, ...)
 {
 va_list ap;
+va_start(ap, format);
+debug_vprintf(format, ap);
+va_end(ap);
+}
+
+void
+debug_vprintf(char *format, va_list ap)
+{
 char *ptr;
 char buffer[1024];
 
 if (debug_file == NULL) return;
-va_start(ap, format);
 
 ptr = buffer;
 
@@ -95,7 +125,6 @@ if (debug_pid > 0)
 
 fprintf(debug_file, "%s", buffer);
 fflush(debug_file);
-va_end(ap);
 }
 
 /* End of debug.c */
