@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -29,6 +29,8 @@ optionlist optionlist_directors[] = {
                  (void *)(offsetof(director_instance, uid_set)) },
   { "condition",          opt_stringptr|opt_public,
                  (void *)offsetof(director_instance, condition) },
+  { "current_directory",  opt_stringptr|opt_public,
+                 (void *)(offsetof(director_instance, current_directory)) },
   { "debug_print",        opt_stringptr | opt_public,
                  (void *)offsetof(director_instance, debug_string) },
   { "domains",            opt_stringptr|opt_public,
@@ -37,12 +39,6 @@ optionlist optionlist_directors[] = {
                  (void *)offsetof(director_instance, driver_name) },
   { "errors_to",          opt_stringptr|opt_public,
                  (void *)(offsetof(director_instance, errors_to)) },
-  { "except_domains",     opt_stringptr|opt_public,
-                 (void *)offsetof(director_instance, except_domains) },
-  { "except_local_parts", opt_stringptr|opt_public,
-                 (void *)offsetof(director_instance, except_local_parts) },
-  { "except_senders",     opt_stringptr|opt_public,
-                 (void *)offsetof(director_instance, except_senders) },
   { "expn",               opt_bool|opt_public,
                  (void *)offsetof(director_instance, expn) },
   { "fail_verify",        opt_bool_verify|opt_hidden|opt_public,
@@ -59,6 +55,8 @@ optionlist optionlist_directors[] = {
                  (void *)offsetof(director_instance, extra_headers) },
   { "headers_remove",     opt_stringptr|opt_public,
                  (void *)offsetof(director_instance, remove_headers) },
+  { "home_directory",     opt_stringptr|opt_public,
+                 (void *)(offsetof(director_instance, home_directory)) },
   { "initgroups",         opt_bool | opt_public,
                  (void *)(offsetof(director_instance, initgroups)) },
   { "local_parts",        opt_stringptr|opt_public,
@@ -128,8 +126,9 @@ if (ugid->gid_set)
   {
   addr->gid = ugid->gid;
   addr->gid_set = TRUE;
-  addr->initgroups = ugid->initgroups;
   }
+
+addr->initgroups = ugid->initgroups;
 }
 
 
@@ -250,6 +249,7 @@ char *s;
 if (dblock->errors_to == NULL) return OK;
 
 s = expand_string(dblock->errors_to);
+
 if (s == NULL)
   {
   addr->message = string_sprintf("%s %s failed to expand \"%s\": %s",
@@ -259,13 +259,19 @@ if (s == NULL)
   return ERROR;
   }
 
-/* While verifying, set the sender address to null, because that's what
+/* If we are already verifying, or if we are just testing an address, do not
+check the errors address. Apart from saving effort, this ensures that the saved
+values of the per-address expansion variables for the original address don't
+get clobbered by a verify-within-a-verify.
+
+If we do need to verify, set the sender address to null, because that's what
 it will be when sending an error message, and there are now configuration
 options that control the running of directors and routers by checking
 the sender address. When testing an address, there may not be a sender
-address. */
+address. We also need to save and restore the expansion values associated with
+an address. */
 
-if (verify) *errors_to = s; else
+if (verify || address_test_mode) *errors_to = s; else
   {
   char *snew;
   int save1 = 0;
@@ -274,10 +280,12 @@ if (verify) *errors_to = s; else
     save1 = sender_address[0];
     sender_address[0] = 0;
     }
+  deliver_save_expansions();
   DEBUG(2) debug_printf("------ Verifying errors address %s ------\n", s);
   if (verify_address(s, NULL, NULL, &snew, vopt_is_recipient | vopt_local)
     == OK) *errors_to = snew;
   DEBUG(2) debug_printf("------ End verifying errors address %s ------\n", s);
+  deliver_restore_expansions();
   if (sender_address != NULL) sender_address[0] = save1;
   }
 
@@ -311,9 +319,11 @@ int
 direct_get_munge_headers(address_item *addr, director_instance *dblock,
   char *type, header_line **extra_headers, char **remove_headers)
 {
-if (dblock->extra_headers == NULL)
-  *extra_headers = addr->extra_headers;
-else
+/* Default is to retain existing headers */
+
+*extra_headers = addr->extra_headers;
+
+if (dblock->extra_headers != NULL)
   {
   header_line *h;
   char *s = expand_string(dblock->extra_headers);
@@ -353,9 +363,11 @@ else
     }
   }
 
-if (dblock->remove_headers == NULL)
-  *remove_headers = addr->remove_headers;
-else
+/* Default is to retain existing removes */
+
+*remove_headers = addr->remove_headers;
+
+if (dblock->remove_headers != NULL)
   {
   char *s = expand_string(dblock->remove_headers);
   if (s == NULL)
@@ -384,6 +396,82 @@ return OK;
 
 
 
+/*************************************************
+*    Check home directory for a director         *
+*************************************************/
+
+/* This is a utility function called by directors to check that a home
+directory matches a specific pattern. On catastrophic errors, a message
+is attached to the address
+
+Arguments:
+  addr        the address being directed
+  home_dir    the home directory path
+  pattern     the pattern to be matched
+  dname       the director name, for error messages
+
+Returns:      OK, FAIL, DEFER, or ERROR
+*/
+
+int
+direct_match_directory(address_item *addr, char *home_dir, char *pattern,
+  char *dname)
+{
+BOOL match;
+char *expat = pattern;
+
+/* The pattern gets expanded if it contains any expansion items. In this case,
+we mustn't use the regex anchor block that is passed in, because we have to
+re-compile the regex each time. */
+
+if (strchr(expat, '$') != NULL)
+  {
+  expat = expand_string(expat);
+
+  if (expat == NULL)
+    {
+    if (expand_string_forcedfail)
+      {
+      DEBUG(2) debug_printf("%s director expansion of \"%s\" forced "
+        "failure\n", dname, pattern);
+      return FAIL;
+      }
+    addr->message = string_sprintf("%s director: failed to expand "
+      "match_directory string \"%s\": %s", dname, pattern,
+      expand_string_message);
+    if (search_find_defer) return DEFER;
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
+  }
+
+match = match_check_string(home_dir, expat, -1, TRUE, FALSE, NULL);
+
+if (match) return OK;
+
+if (search_find_defer)
+  {
+  addr->message = string_sprintf("%s director: lookup defer while matching "
+    "directory %s against %s", dname, home_dir, pattern);
+  return DEFER;
+  }
+
+DEBUG(2)
+  {
+  debug_printf("%s director failed to match home directory for %s\n  "
+    "directory = \"%s\" pattern = \"%s\"\n", dname, addr->local_part,
+    home_dir, pattern);
+  if (strcmp(expat, pattern) != 0)
+    debug_printf("  expanded string was \"%s\"\n", expat);
+  }
+
+return FAIL;
+}
+
+
+
+
+
 /*************************************************
 *  Check files, sender, condition for a director *
 *************************************************/
@@ -392,7 +480,7 @@ return OK;
 them. It carries out file existence, sender checking, and general condition
 tests, in that order, because the latter may be using the files mentioned in
 the former. There are two failure yields, partly for historical reasons. FAIL
-causes the directing not to continue of the director has no_more set;
+causes the directing not to continue if the director has no_more set;
 FAIL_CONDITION does not. Retaining FAIL is for backwards compatibility, but it
 may be that changing them all to FAIL_CONDITION one day might be reasonable.
 
@@ -426,21 +514,22 @@ switch (match_exists(dblock->require_files))
   return DEFER;
   }
 
-if ((dblock->except_senders != NULL &&
-     (rc = match_sender(dblock->except_senders, &(dblock->re_except_senders)))
-       == OK) ||
-    (dblock->senders != NULL &&
-     (rc = match_sender(dblock->senders, &(dblock->re_senders))) != OK))
+if (dblock->senders != NULL)
   {
-  if (rc == DEFER)
+  char *senders = expand_string_panic(dblock->senders, dblock->name,
+    "director");
+  if ((rc = match_sender(senders)) != OK)
     {
-    addr->message = string_sprintf("sender check defer in %s director",
+    if (rc == DEFER)
+      {
+      addr->message = string_sprintf("sender check defer in %s director",
+        dblock->name);
+      return DEFER;
+      }
+    DEBUG(9) debug_printf("%s director: sender match failure\n",
       dblock->name);
-    return DEFER;
+    return FAIL;
     }
-  DEBUG(9) debug_printf("%s director: sender match failure\n",
-    dblock->name);
-  return FAIL;
   }
 
 if (dblock->condition != NULL &&
@@ -507,7 +596,18 @@ if (strcmp(lastname, s) != 0)
   strncpy(lastname, s, sizeof(lastname));
   lastname[sizeof(lastname)-1] = 0;          /* just in case */
 
-  for (;;)
+  /* Force failure if string length is greater than given maximum */
+
+  if (max_username_length > 0 && (int)strlen(lastname) > max_username_length)
+    {
+    DEBUG(9) debug_printf("forced failure of finduser(): string "
+      "length of %s is greater than %d\n", lastname, max_username_length);
+    lastpw = NULL;
+    }
+
+  /* Try a few times if so configured; this handles delays in NIS etc. */
+
+  else for (;;)
     {
     if ((lastpw = getpwnam(s)) != NULL) break;
     if (++i > finduser_retries) break;
@@ -836,7 +936,9 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
   /* Check the conditions that are common to both routers and directors. The
   relevant items in the instance block are carefully put in the same position,
   which is why we can coerce the director instance pointer to a router instance
-  pointer here. */
+  pointer here. We need the expansions set up before doing this. */
+
+  deliver_set_expansions(addr);
 
   if (route_skip_driver((router_instance *)d, addr->local_part, addr->domain,
     verify, &more, "director")) continue;
@@ -863,9 +965,8 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
     /* Loop for possible list of prefixes. If any one begins with '*',
     special wildcard checking is done. */
 
-    for (prefix = string_nextinlist(&listptr, ':', prebuf, sizeof(prebuf));
-         prefix != NULL;
-         prefix = string_nextinlist(&listptr, ':', prebuf, sizeof(prebuf)))
+    while ((prefix = string_nextinlist(&listptr, ':', prebuf, sizeof(prebuf)))
+           != NULL)
       {
       int plen = (int)strlen(prefix);
       if (prefix[0] == '*')
@@ -914,9 +1015,8 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
     char *listptr = d->suffix;
     char sufbuf[64];
 
-    for (suffix = string_nextinlist(&listptr, ':', sufbuf, sizeof(sufbuf));
-         suffix != NULL;
-         suffix = string_nextinlist(&listptr, ':', sufbuf, sizeof(sufbuf)))
+    while ((suffix = string_nextinlist(&listptr, ':', sufbuf, sizeof(sufbuf)))
+         != NULL)
       {
       int slen = (int)strlen(suffix);
       int alen = (int)strlen(addr->local_part);
@@ -1010,25 +1110,13 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
     }
 
   /* Prefix and suffix (if any) have been stripped; ready to call the
-  director, first setting up values that may be used in string expansions.
-  Reset them afterwards. Also, we must arrange for any dynamic transport to get
-  set up. The code in route_set_transport() handles this for both routers and
-  directors, using items in the data blocks that are at the same offsets. */
+  director. Set the expansions again, in case prefix/suffix have
+  changed things. Reset them afterwards. */
 
   deliver_set_expansions(addr);
-
-  if (route_set_transport((router_instance *)d, "director", &(addr->message)))
-    {
-    DEBUG(9) debug_printf("calling %s director\n", d->name);
-    yield = (d->info->code)(d, addr, addr_local, addr_remote, addr_new,
-      addr_succeed, verify != v_none && verify != v_expn);
-    }
-  else
-    {
-    DEBUG(9) debug_printf("%s", addr->message);
-    yield = DEFER;
-    }
-
+  DEBUG(9) debug_printf("calling %s director\n", d->name);
+  yield = (d->info->code)(d, addr, addr_local, addr_remote, addr_new,
+    addr_succeed, verify != v_none && verify != v_expn);
   deliver_set_expansions(NULL);
 
   /* If yield = FAIL_CONDITION it means that a condition that has to be tested
@@ -1054,7 +1142,8 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
   hand on to the subsequent directors. Actually, we can't pass it on directly;
   we have to put it on the new queue, but since it has the same address as
   the current one, it will be passed by all previous directors and also the
-  one that has just succeeded, by the loop-avoidance code. */
+  one that has just succeeded, by the loop-avoidance code. This is all handled
+  by a function common to both directors and routers. */
 
   if (check_yield == OK || check_yield == DEFER || check_yield == ERROR)
     {
@@ -1068,29 +1157,19 @@ for (d = (addr->start_director == NULL)? directors : addr->start_director;
         yield = check_yield = FAIL;
         addr->message = string_sprintf("%s director forced verify failure",
           d->name);
+        if (*addr_remote == addr) *addr_remote = addr->next;
+        if (*addr_local == addr) *addr_local = addr->next;
         }
       else if (d->unseen)
-        {
-        address_item *new = deliver_make_addr(addr->orig, TRUE);
-        new->parent = addr;
-        new->ignore_error |= addr->ignore_error;
-        new->errors_address = addr->errors_address;
-        addr->child_count++;
-        new->next = *addr_new;
-        *addr_new = new;
-        }
+        route_unseen(d->name, addr, addr_local, addr_remote, addr_new);
       }
 
     DEBUG(2)
       {
       if (check_yield == OK)
         {
-        debug_printf("%s director succeeded for %s\n", d->name,
-          addr->local_part);
-        debug_printf("  transport: %s\n", (addr->transport == NULL)?
-          "<none>" : addr->transport->name);
-        if (d->unseen)
-          debug_printf("\"unseen\" set: replicated %s\n", addr->orig);
+        debug_printf("%s director succeeded for %s%s\n", d->name,
+          addr->local_part, (d->unseen)? " (unseen)" : "");
         }
       else if (check_yield == DEFER)
         {
@@ -1128,6 +1207,8 @@ if there is nothing already set. */
 if (addr->message == NULL)
   addr->message = string_sprintf("unknown local-part \"%s\" in domain \"%s\"",
     addr->local_part, addr->domain);
+
+deliver_set_expansions(NULL);
 return FAIL;
 }
 
