@@ -2,11 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 #include "../exim.h"
+#include "dfunctions.h"
 #include "aliasfile.h"
 
 
@@ -14,14 +15,18 @@
 /* Options specific to the aliasfile director. */
 
 optionlist aliasfile_director_options[] = {
+  { "*expand_directory2_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(aliasfile_director_options_block, expand_directory2_transport) },
+  { "*expand_directory_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(aliasfile_director_options_block, expand_directory_transport) },
+  { "*expand_file_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(aliasfile_director_options_block, expand_file_transport) },
+  { "*expand_pipe_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(aliasfile_director_options_block, expand_pipe_transport) },
   { "*partial_match", opt_int | opt_hidden,
       (void *)(offsetof(aliasfile_director_options_block, partial_match)) },
   { "check_ancestor",     opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, check_ancestor)) },
-  { "current_directory",     opt_stringptr,
-      (void *)(offsetof(aliasfile_director_options_block, current_dir)) },
-  { "directory",          opt_stringptr | opt_hidden,
-      (void *)(offsetof(aliasfile_director_options_block, home_dir)) },
   { "directory2_transport",opt_transportptr,
       (void *)(offsetof(aliasfile_director_options_block, directory2_transport)) },
   { "directory_transport",opt_transportptr,
@@ -38,8 +43,6 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, forbid_pipe)) },
   { "freeze_missing_include", opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, freeze_missing_include)) },
-  { "home_directory",     opt_stringptr,
-      (void *)(offsetof(aliasfile_director_options_block, home_dir)) },
   { "include_domain",     opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, include_domain)) },
   { "modemask",           opt_octint,
@@ -66,6 +69,8 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, search_type)) },
   { "skip_syntax_errors", opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, skip_syntax_errors)) },
+  { "syntax_errors_text", opt_stringptr,
+      (void *)(offsetof(aliasfile_director_options_block, syntax_errors_text)) },
   { "syntax_errors_to",   opt_stringptr,
       (void *)(offsetof(aliasfile_director_options_block, syntax_errors_to)) }
 };
@@ -79,32 +84,41 @@ int aliasfile_director_options_count =
 /* Default private options block for the aliasfile director. */
 
 aliasfile_director_options_block aliasfile_director_option_defaults = {
+  /************** Common block ****************/
+  NULL,     /* directory_transport */
+  NULL,     /* expand_directory_transport */
+  NULL,     /* directory2_transport */
+  NULL,     /* expand_directory2_transport */
+  NULL,     /* file_transport */
+  NULL,     /* expand_file_transport */
+  NULL,     /* pipe_transport */
+  NULL,     /* expand_pipe_transport */
+  NULL,     /* reply_transport */
+  NULL,     /* expand_reply_transport */
+  FALSE,    /* check_ancestor */
+  FALSE,    /* check_local_user */
+  FALSE,    /* forbid_file */
+  FALSE,    /* forbid_pipe */
+  FALSE,    /* forbid_reply */
+  FALSE,    /* one_time */
+  /********************************************/
   NULL,     /* file */
   NULL,     /* queries */
   NULL,     /* query */
-  NULL,     /* home_dir */
-  NULL,     /* current_dir */
   NULL,     /* syntax_errors_to */
+  NULL,     /* syntax_errors_text */
   022,      /* modemask */
   -1,       /* partial_match */
   -1,       /* search_type */
   NULL,     /* owners */
   NULL,     /* owngroups */
   FALSE,    /* include_domain */
-  FALSE,    /* check_ancestor */
   FALSE,    /* expand */
   FALSE,    /* optional */
   FALSE,    /* qualify_preserve_domain */
-  FALSE,    /* forbid_file */
-  FALSE,    /* forbid_pipe */
   TRUE,     /* freeze_missing_include */
-  FALSE,    /* one_time */
   TRUE,     /* rewrite */
-  FALSE,    /* skip_syntax_errors */
-  NULL,     /* directory_transport */
-  NULL,     /* directory2_transport */
-  NULL,     /* file_transport */
-  NULL      /* pipe_transport */
+  FALSE     /* skip_syntax_errors */
 };
 
 
@@ -215,6 +229,7 @@ void *handle;
 header_line *extra_headers;
 char *remove_headers;
 error_block *eblock = NULL;
+transport_instance *transport;
 ugid_block ugid;
 void *reset_point;
 int yield = OK;
@@ -237,8 +252,8 @@ if (!mac_islookup(ob->search_type, lookup_querystyle))
   filename = expand_string(ob->file);
   if (filename == NULL)
     {
-    addr->message = string_sprintf("%s director failed to expand \"%s\": %s",
-      dblock->name, ob->file, expand_string_message);
+    addr->message = string_sprintf("failed to expand \"%s\": %s",
+      ob->file, expand_string_message);
     if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
@@ -254,8 +269,26 @@ if (!mac_islookup(ob->search_type, lookup_querystyle))
     return ERROR;
     }
 
-  DEBUG(2) debug_printf("%s director: %s key=%s\n  file=\"%s\"\n",
-    dblock->name, lookup_list[ob->search_type].name, query, filename);
+  DEBUG(2)
+    {
+    char *pmstar = "";
+    char *pmat = "";
+    int pm = ob->partial_match;
+    if (pm > 2000)
+      {
+      pmstar = "*@";
+      pm -= 2048;
+      }
+    if (pm > 1000)
+      {
+      pmstar = "*";
+      pm -= 1024;
+      }
+    debug_printf("%s director: ", dblock->name);
+    if (pm >= 0) debug_printf("partial%d-", pm);
+    debug_printf("%s%s%s key=%s\n  file=\"%s\"\n",
+      lookup_list[ob->search_type].name, pmstar, pmat, query, filename);
+    }
   }
 
 /* For query-style lookups, expand the query and set the filename NULL.
@@ -268,9 +301,8 @@ else
   query = expand_string((ob->query == NULL)? ob->queries : ob->query);
   if (query == NULL)
     {
-    addr->message = string_sprintf("%s director failed to expand \"%s\": %s",
-      dblock->name, (ob->query == NULL)? ob->queries : ob->query,
-      expand_string_message);
+    addr->message = string_sprintf("failed to expand \"%s\": %s",
+      (ob->query == NULL)? ob->queries : ob->query, expand_string_message);
     if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
@@ -298,6 +330,10 @@ if (handle == NULL)
     }
   addr->basic_errno = ERRNO_BADALIAS;
   addr->message = error;
+
+  if (errno == ERRNO_BADMODE)
+    error = string_sprintf("%s (check the modemask option)", error);
+
   log_write(0, LOG_MAIN|LOG_PANIC, "%s director: %s", dblock->name, error);
   return ERROR;
   }
@@ -309,17 +345,17 @@ a list of queries to try. */
 if (ob->queries != NULL)
   {
   char *onequery;
-  for (onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE);
-       onequery != NULL;
-       onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE))
+  while ((onequery = string_nextinlist(&query, ':', big_buffer,
+          BIG_BUFFER_SIZE)) != NULL)
     {
-    aliastext = search_find(handle, filename, onequery, ob->search_type,
-      ob->partial_match, NULL, &error);
+    aliastext = search_find(handle, filename, onequery, ob->partial_match,
+      NULL, &error);
     if (aliastext != NULL || search_find_defer) break;
     }
   }
-else aliastext = search_find(handle, filename, query, ob->search_type,
-  ob->partial_match, NULL, &error);
+else
+  aliastext = search_find(handle, filename, query, ob->partial_match, NULL,
+    &error);
 
 /* The lookup did not succeed. Defer if it deferred; otherwise this director
 fails to handle the address. */
@@ -328,8 +364,7 @@ if (aliastext == NULL)
   {
   if (search_find_defer)
     {
-    addr->message = string_sprintf("%s director: search_find deferred: %s",
-      dblock->name, error);
+    addr->message = string_sprintf("search_find deferred: %s", error);
     return DEFER;
     }
 
@@ -345,8 +380,8 @@ if (ob->expand)
   char *newtext = expand_string(aliastext);
   if (newtext == NULL)
     {
-    addr->message = string_sprintf("%s director failed to expand \"%s\" "
-      "(generated from local part %s): %s", dblock->name, aliastext,
+    addr->message = string_sprintf("failed to expand \"%s\" "
+      "(generated from local part %s): %s", aliastext,
       addr->local_part, expand_string_message);
     if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
@@ -371,23 +406,27 @@ rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
   &remove_headers);
 if (rc != OK) return rc;
 
+/* Get the transport for this director, if any. It might be expanded, or it
+might be unset. */
+
+if (!route_get_transport(dblock->transport, dblock->expand_transport,
+  addr, &transport, dblock->name, NULL)) return ERROR;
+
 /* If there is a transport specified for the director, then set up this
 address to use that transport. Ignore the alias text. */
 
-if (dblock->transport != NULL)
+if (transport != NULL)
   {
-  addr->transport = dblock->transport;
-
+  addr->transport = transport;
   addr->errors_address = errors_to;
-  addr->director = dblock;
-  addr->home_dir = ob->home_dir;
-  addr->current_dir = ob->current_dir;
+  addr->home_dir = dblock->home_directory;
+  addr->current_dir = dblock->current_directory;
   addr->extra_headers = extra_headers;
   addr->remove_headers = remove_headers;
 
   direct_set_ugid(addr, &ugid);
 
-  if (addr->transport->info->local)
+  if (transport->info->local)
     {
     addr->next = *addr_local;
     *addr_local = addr;
@@ -402,10 +441,10 @@ if (dblock->transport != NULL)
 
   DEBUG(2)
     {
-    debug_printf("  queued for %s transport ", dblock->transport->name);
-    if (addr->uid_set) debug_printf("uid=%d ", (int)(addr->uid));
+    debug_printf("  queued for %s transport ", transport->name);
+    if (addr->uid_set) debug_printf("uid=%ld ", (long int)(addr->uid));
       else debug_printf("uid=unset ");
-    if (addr->gid_set) debug_printf("gid=%d\n", (int)(addr->gid));
+    if (addr->gid_set) debug_printf("gid=%ld\n", (long int)(addr->gid));
       else debug_printf("gid=unset\n");
     }
 
@@ -419,12 +458,8 @@ addresses get qualified with the same domain as the incoming address. */
 if (ob->qualify_preserve_domain) qualify_domain_recipient = addr->domain;
 
 /* There is a common function for use by forwarding and aliasing directors that
-extracts a list of addresses from a text string. Setting the fourth-last
-argument FALSE makes generating no addresses an error. However, setting the
-third-last TRUE allows an address that is ":blackhole:" in the alias file to
-generate nothing without causing an error. It also permits :defer: and :fail:
-to occur. Setting the last argument NULL causes no checks to be made on
-:include: files.
+extracts a list of addresses from a text string. If the last argument is NULL,
+syntax errors are hard; otherwise they just get listed.
 
 It is possible that a fair amount of store might be used, especially if there
 is an :include: file. Therefore, switch to the temporary pool for the duration.
@@ -434,7 +469,11 @@ store_pool = POOL_TEMP;
 reset_point = store_get(0);
 
 extracted = parse_extract_addresses(aliastext, addr->domain, &generated,
-  &error, FALSE, TRUE, ob->rewrite, NULL,
+  &error,
+  FALSE,           /* No addresses is an error */
+  TRUE,            /* Allow :blackhole: etc. */
+  ob->rewrite,     /* Rewrite if required */
+  NULL,            /* No restriction on :include: files */
   ob->skip_syntax_errors? &eblock : NULL);
 
 /* Restore the qualify domain, and reset to the main pool for the generation of
@@ -443,37 +482,11 @@ any error texts. */
 qualify_domain_recipient = save_qualify_domain_recipient;
 store_pool = POOL_MAIN;
 
-/* If extraction failed, return error and freeze, unless it was a missing
-include file and no_freeze_missing_include is set, or unless it was an explicit
-:defer: or :fail: that was encountered. If verifying, set a flag that causes
-the messages from :defer: and :fail: to be passed back to the caller - in
-particular, this gets them into the response to an SMTP command. */
+/* A common function handles cases of extraction failure. */
 
-if (extracted != 0) switch (extracted)
-  {
-  case EXTRACTED_DEFER:
-  addr->message = string_sprintf("forced defer: %s", error);
-  if (verify && *error != 0) verify_forced_errmsg = error;
-  yield = DEFER;
-  break;
-
-  case EXTRACTED_FAIL:
-  addr->message = string_sprintf("forced failure: %s", error);
-  if (verify && *error != 0) verify_forced_errmsg = error;
-  yield =  FORCEFAIL;
-  break;
-
-  default:
-  addr->basic_errno = ERRNO_BADALIAS;
-  addr->message = (filename != NULL)?
-    string_sprintf("<%s> - error in alias file %s: %s", addr->orig, filename,
-      error) :
-    string_sprintf("<%s> - error in alias lookup: %s", addr->orig, error);
-  if (extracted > 0 && !ob->freeze_missing_include) return DEFER;
-  addr->special_action = SPECIAL_FREEZE;
-  yield = ERROR;
-  break;
-  }
+if (extracted != 0) yield = df_extracted(extracted, dblock, addr, error,
+  verify, ob->freeze_missing_include, (filename != NULL)?
+    string_sprintf("alias file %s", filename) : "alias lookup");
 
 /* If skip_syntax_errors was set and there were syntax errors in the list,
 error messages will be present in eblock. Log them, and sent them off in
@@ -481,8 +494,14 @@ a mail message if so configured. A common function is used by aliasfile
 and forwardfile. */
 
 if (yield == OK && eblock != NULL &&
-    !moan_skipped_syntax_errors(dblock->name, filename,
-      eblock, verify? NULL : ob->syntax_errors_to))
+    !moan_skipped_syntax_errors(
+      dblock->name,                          /* For message content */
+      filename,                              /* Ditto */
+      eblock,                                /* Ditto */
+      (verify || address_test_mode)?
+        NULL : ob->syntax_errors_to,         /* Who to mail */
+      generated != NULL,                     /* True if not all failed */
+      ob->syntax_errors_text))               /* Custom message */
   {
   addr->special_action = SPECIAL_FREEZE;
   yield = ERROR;
@@ -497,111 +516,11 @@ store_pool = POOL_MAIN;
 
 if (yield != OK) return yield;
 
+/* Set up the new addresses and add them to the addr_new chain. A common
+function handles this. */
 
-/* Add the new addresses to the list of new addresses, copying in the
-uid, gid and permission flags for use by pipes and files, setting
-the parent, and or-ing its ignore_error flag. Also record the setting
-for any starting director.
-
-If the generated address is the same as one of its ancestors, and the
-check_ancestor flag is set, do not use this generated address, but replace it
-with a copy of the input address. This is to cope with cases where A is aliased
-to B and B has a .forward file pointing to A, though it is usually set on the
-forwardfile rather than the aliasfile. We can't just pass on the old
-address by returning FAIL, because it must act as a general parent for
-generated addresses, and only get marked "done" when all its children are
-delivered. */
-
-while (generated != NULL)
-  {
-  address_item *parent;
-  address_item *next = generated;
-
-  generated = next->next;
-  next->parent = addr;
-  next->ignore_error |= addr->ignore_error;
-  next->start_director = dblock->new;
-  addr->child_count++;
-
-  if (ob->one_time)
-    {
-    for (parent = addr; parent->parent != NULL; parent = parent->parent);
-    next->onetime_parent = parent->orig;
-    }
-
-  next->next = *addr_new;
-  *addr_new = next;
-
-  if (ob->check_ancestor)
-    {
-    for (parent = addr; parent != NULL; parent = parent->parent)
-      {
-      if (strcmp(next->orig, parent->orig) == 0)
-        {
-        DEBUG(2) debug_printf("generated parent replaced by child\n");
-        next->orig = string_copy(addr->orig);
-        break;
-        }
-      }
-    }
-
-  next->errors_address = errors_to;
-  next->extra_headers = extra_headers;
-  next->remove_headers = remove_headers;
-
-  if (next->pfr)
-    {
-    next->director = dblock;
-    next->home_dir = ob->home_dir;
-    next->home_dir = ob->current_dir;
-    next->allow_pipe = !ob->forbid_pipe;
-    next->allow_file = !ob->forbid_file;
-    direct_set_ugid(next, &ugid);
-
-    /* Aliasfile can produce only pipes or files; if the transport
-    setting is null, the global setting will get used later. */
-
-    if (next->orig[0] == '|') next->transport = ob->pipe_transport; else
-      {
-      int len = (int)strlen(next->orig);
-      if (next->orig[len-1] == '/')
-        {
-        next->transport = ob->directory_transport;
-        if (len > 1 && next->orig[len-2] == '/' &&
-            ob->directory2_transport != NULL)
-          next->transport = ob->directory2_transport;
-        }
-      else next->transport = ob->file_transport;
-      }
-    }
-
-  DEBUG(2)
-    {
-    debug_printf("%s director generated %s\n%s%s%s%s%s%s%s",
-      dblock->name,
-      next->orig,
-      next->pfr? "  pipe, file, or autoreply\n" : "",
-      (errors_to != NULL)? "  errors to " : "",
-      (errors_to != NULL)? errors_to : "",
-      (errors_to != NULL)? "\n" : "",
-      (next->transport == NULL)? "" : "  transport=",
-      (next->transport == NULL)? "" : (next->transport)->name,
-      (next->transport == NULL)? "" : "\n");
-
-    if (next->uid_set)
-      debug_printf("  uid=%d ", (int)(next->uid));
-    else
-      debug_printf("  uid=unset ");
-
-    if (next->gid_set)
-      debug_printf("gid=%d ", (int)(next->gid));
-    else
-      debug_printf("gid=unset ");
-
-    debug_printf("home=%s\n", (next->home_dir == NULL)? "null" :
-      next->home_dir);
-    }
-  }
+df_generated(dblock, addr_new, addr, generated, errors_to, extra_headers,
+  remove_headers, &ugid, NULL, NULL);
 
 /* If no children were generated for this address, but no error was given,
 it means that there was just a :blackhole: entry in the alias file. Log
