@@ -2,40 +2,33 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the aliasfile director */
 
-/* Structure for the private options. */
+/* Structure for the private options. The first block of them must be the
+common director fields for all directors that use shared functions. */
 
 typedef struct {
+  COMMON_DIRECTOR_FIELDS;
   char *file;
   char *queries;
   char *query;
-  char *home_dir;
-  char *current_dir;
   char *syntax_errors_to;
+  char *syntax_errors_text;
   int   modemask;
   int   partial_match;
   int   search_type;
   int  *owners;
   int  *owngroups;
   BOOL  include_domain;
-  BOOL  check_ancestor;
   BOOL  expand;
   BOOL  optional;
   BOOL  qualify_preserve_domain;
-  BOOL  forbid_file;
-  BOOL  forbid_pipe;
   BOOL  freeze_missing_include;
-  BOOL  one_time;
   BOOL  rewrite;
   BOOL  skip_syntax_errors;
-  transport_instance *directory_transport;
-  transport_instance *directory2_transport;
-  transport_instance *file_transport;
-  transport_instance *pipe_transport;
 } aliasfile_director_options_block;
 
 /* Data for reading the private options. */
