@@ -2,11 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 #include "../exim.h"
+#include "dfunctions.h"
 #include "forwardfile.h"
 
 
@@ -14,6 +15,16 @@
 /* Options specific to the forwardfile director. */
 
 optionlist forwardfile_director_options[] = {
+  { "*expand_directory2_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(forwardfile_director_options_block, expand_directory2_transport) },
+  { "*expand_directory_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(forwardfile_director_options_block, expand_directory_transport) },
+  { "*expand_file_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(forwardfile_director_options_block, expand_file_transport) },
+  { "*expand_pipe_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(forwardfile_director_options_block, expand_pipe_transport) },
+  { "*expand_reply_transport",  opt_stringptr|opt_hidden,
+      (void *)offsetof(forwardfile_director_options_block, expand_reply_transport) },
   { "allow_system_actions", opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, allow_system_actions)) },
   { "check_ancestor",     opt_bool,
@@ -22,10 +33,6 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, check_group)) },
   { "check_local_user",   opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, check_local_user)) },
-  { "current_directory",  opt_stringptr,
-      (void *)(offsetof(forwardfile_director_options_block, current_dir)) },
-  { "directory",          opt_stringptr | opt_hidden,
-      (void *)(offsetof(forwardfile_director_options_block, file_dir)) },
   { "directory2_transport",opt_transportptr,
       (void *)(offsetof(forwardfile_director_options_block, directory2_transport)) },
   { "directory_transport",opt_transportptr,
@@ -40,10 +47,16 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, filter)) },
   { "forbid_file",        opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_file)) },
-  { "forbid_filter_log",  opt_bool|opt_hidden,
-      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_logwrite)) },
+  { "forbid_filter_existstest",  opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_exists)) },
   { "forbid_filter_logwrite",opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_filter_logwrite)) },
+  { "forbid_filter_lookup",      opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_lookup)) },
+  #ifdef EXIM_PERL
+  { "forbid_filter_perl",        opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_perl)) },
+  #endif
   { "forbid_include",     opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_include)) },
   { "forbid_pipe",        opt_bool,
@@ -52,12 +65,12 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, forbid_reply)) },
   { "freeze_missing_include", opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, freeze_missing_include)) },
-  { "home_directory",     opt_stringptr,
-      (void *)(offsetof(forwardfile_director_options_block, home_dir)) },
-  { "ignore_eacces",     opt_bool,
+  { "ignore_eacces",      opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, ignore_eacces)) },
   { "ignore_enotdir",     opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, ignore_enotdir)) },
+  { "match_directory",    opt_stringptr,
+      (void *)(offsetof(forwardfile_director_options_block, pw_dir)) },
   { "modemask",           opt_octint,
       (void *)(offsetof(forwardfile_director_options_block, modemask)) },
   { "one_time",           opt_bool,
@@ -78,6 +91,8 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, use_seteuid)) },
   { "skip_syntax_errors", opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, skip_syntax_errors)) },
+  { "syntax_errors_text", opt_stringptr,
+      (void *)(offsetof(forwardfile_director_options_block, syntax_errors_text)) },
   { "syntax_errors_to",   opt_stringptr,
       (void *)(offsetof(forwardfile_director_options_block, syntax_errors_to)) }
 };
@@ -91,37 +106,47 @@ int forwardfile_director_options_count =
 /* Default private options block for the forwardfile director. */
 
 forwardfile_director_options_block forwardfile_director_option_defaults = {
+  /************** Common block ****************/
+  NULL,     /* directory_transport */
+  NULL,     /* expand_directory_transport */
+  NULL,     /* directory2_transport */
+  NULL,     /* expand_directory2_transport */
+  NULL,     /* file_transport */
+  NULL,     /* expand_file_transport */
+  NULL,     /* pipe_transport */
+  NULL,     /* expand_pipe_transport */
+  NULL,     /* reply_transport */
+  NULL,     /* expand_reply_transport */
+  FALSE,    /* check_ancestor */
+  TRUE,     /* check_local_user */
+  FALSE,    /* forbid_file */
+  FALSE,    /* forbid_pipe */
+  FALSE,    /* forbid_reply */
+  FALSE,    /* one_time */
+  /********************************************/
   NULL,     /* file_dir */
-  NULL,     /* home_dir */
-  NULL,     /* current_dir */
+  NULL,     /* pw_dir */
   NULL,     /* file */
   NULL,     /* syntax_errors_to */
+  NULL,     /* syntax_errors_text */
   022,      /* modemask */
   NULL,     /* owners */
   NULL,     /* owngroups */
-  FALSE,    /* check_ancestor */
-  TRUE,     /* check_local_user */
   FALSE,    /* check_group */
   FALSE,    /* filter */
   FALSE,    /* allow_system_actions */
+  FALSE,    /* forbid_filter_exists */
   FALSE,    /* forbid_filter_logwrite */
-  FALSE,    /* forbid_file */
+  FALSE,    /* forbid_filter_lookup */
+  FALSE,    /* forbid_filter_perl */
   FALSE,    /* forbid_include */
-  FALSE,    /* forbid_pipe */
-  FALSE,    /* forbid_reply */
   FALSE,    /* ignore_eacces */
   FALSE,    /* ignore_enotdir */
-  FALSE,    /* one_time */
   FALSE,    /* qualify_preserve_domain */
   FALSE,    /* use_seteuid */
   TRUE,     /* freeze_missing_include */
   TRUE,     /* rewrite */
-  FALSE,    /* skip_syntax_errors */
-  NULL,     /* directory_transport */
-  NULL,     /* directory2_transport */
-  NULL,     /* file_transport */
-  NULL,     /* pipe_transport */
-  NULL,     /* reply_transport */
+  FALSE     /* skip_syntax_errors */
 };
 
 
@@ -151,16 +176,25 @@ though we can't check for certain until it has been expanded. */
 if (ob->file_dir != NULL && ob->file_dir[0] != '/' &&
     ob->file_dir[0] != '$')
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
-    "absolute path name required for the 'directory' option", dblock->name);
+    "absolute path name required for the 'file_directory' option",
+    dblock->name);
 
 /* Permit relative paths only if local user checking is set, or if the
-directory option (which must be absolute) is set. */
+directory option (which must be absolute) is set. Permit match_directory
+only if local user checking is set. */
 
-if (!ob->check_local_user && ob->file[0] != '/' && ob->file[0] != '$' &&
-     ob->file_dir == NULL)
-  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
-    "absolute file path required when check_local_user and directory are not set",
-    dblock->name);
+if (!ob->check_local_user)
+  {
+  if (ob->file[0] != '/' && ob->file[0] != '$' &&
+      ob->file_dir == NULL)
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
+      "absolute file path required when check_local_user and directory are not set",
+      dblock->name);
+  if (ob->pw_dir != NULL)
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
+      "check_local_user is required when match_directory is set",
+      dblock->name);
+  }
 
 /* Permit seteuid only if it is configured to be available, and either local
 user checking is set, or uid/gid are supplied. */
@@ -223,7 +257,7 @@ char *remove_headers;
 void *reset_point = NULL;
 header_line *extra_headers;
 struct stat statbuf;
-struct passwd *pw;
+struct passwd *pw = NULL;
 error_block *eblock = NULL;
 ugid_block ugid;
 FILE *fwd;
@@ -240,12 +274,15 @@ BOOL  delivered = FALSE;
 BOOL  is_filter = FALSE;
 BOOL  filter_header = FALSE;
 
+addr_local = addr_local;     /* Keep picky compilers happy */
+addr_remote = addr_remote;
+
 /* If the check_local_user option is set, check that the local_part is
 the login of a local user, and fail if not. Note: the third argument to
 direct_finduser() must be NULL here, to prevent a numeric string being
 taken as a numeric uid. If the user is found, set directory to the home
 directory, and the home expansion variable as well, so that it can be
-used while expanding ob->file_dir! */
+used while expanding match_directory and ob->file_dir. */
 
 if (ob->check_local_user)
   {
@@ -255,10 +292,16 @@ if (ob->check_local_user)
       dblock->name, addr->local_part);
     return FAIL;
     }
-  else
+
+  /* Set the home directory, and if there is a setting of match_directory,
+  check for a match. The resetting of deliver_home happens at the outer
+  level. */
+
+  deliver_home = directory = pw->pw_dir;
+  if (ob->pw_dir != NULL)
     {
-    directory = pw->pw_dir;
-    deliver_home = directory;
+    rc = direct_match_directory(addr, pw->pw_dir, ob->pw_dir, dblock->name);
+    if (rc != OK) return rc;
     }
   }
 
@@ -271,8 +314,8 @@ if (ob->file_dir != NULL)
   directory = expand_string(ob->file_dir);
   if (directory == NULL)
     {
-    addr->message = string_sprintf("%s director failed to expand %s: %s",
-      dblock->name, ob->file_dir, expand_string_message);
+    addr->message = string_sprintf("failed to expand \"%s\": %s",
+      ob->file_dir, expand_string_message);
     if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
@@ -293,15 +336,14 @@ filename = expand_string(ob->file);
 
 if (filename == NULL)
   {
-  addr->message = string_sprintf("%s director failed to expand %s: %s",
-    dblock->name, ob->file, expand_string_message);
+  addr->message = string_sprintf("failed to expand \"%s\": %s",
+    ob->file, expand_string_message);
   if (search_find_defer) return DEFER;
   addr->special_action = SPECIAL_FREEZE;
   return ERROR;
   }
 
-DEBUG(2) debug_printf("%s director: file = %s\n", dblock->name,
-  filename);
+DEBUG(2) debug_printf("%s director: file = %s\n", dblock->name, filename);
 
 /* If a directory is set and the file name is not absolute, construct the
 complete file name. Otherwise set a flag to prevent an attempt at statting the
@@ -365,8 +407,7 @@ if ((ugid.uid_set || ob->check_local_user) &&
     mac_seteuid(pw->pw_uid);
     }
 
-  DEBUG(2) debug_printf("%s director euid=%d egid=%d\n", dblock->name,
-    (int)geteuid(), (int)getegid());
+  DEBUG(2) debug_print_ids("set");
   restore_uid = TRUE;
   }
 
@@ -403,23 +444,30 @@ if (directory != NULL && stat_directory)
   alarm(30);
   sprintf(big_buffer, "%.*s/.", big_buffer_size, directory);
   rc = stat(big_buffer, &statbuf);
-  if (rc != 0 && errno == EACCES && !sigalrm_seen)
+  if (rc == 0)
+    {
+    DEBUG(2) debug_printf("successful stat of %s\n", big_buffer);
+    }
+  else if (errno == EACCES && !sigalrm_seen)
+    {
     rc = stat(directory, &statbuf);
+    if (rc == 0)
+      {
+      DEBUG(2) debug_printf("successful stat of %s\n", directory);
+      }
+    }
   saved_errno = errno;
   alarm(0);
   signal(SIGALRM, SIG_DFL);
 
   if (sigalrm_seen || rc != 0)
     {
-    addr->message = string_sprintf("%s director failed to stat %s (%s)",
-      dblock->name, directory,
+    addr->message = string_sprintf("failed to stat %s (%s)", directory,
       sigalrm_seen? "timeout" : strerror(saved_errno));
-    DEBUG(2) debug_printf("%s\n", addr->message);
     yield = DEFER;
     goto RESTORE_UID;             /* skip forward */
     }
 
-  DEBUG(2) debug_printf("successful stat of %s\n", directory);
   }
 
 /* Now try to open the file for reading. If this fails with a non-existence
@@ -442,15 +490,15 @@ if (fwd == NULL)
   switch(errno)
     {
     case ENOENT:          /* File does not exist */
-    DEBUG(2) debug_printf("%s director: no file found\n", dblock->name);
+    DEBUG(2) debug_printf("%s not found\n", filename);
     yield = FAIL;
     break;
 
     case ENOTDIR:         /* Something on the path isn't a directory */
     if (ob->ignore_enotdir)
       {
-      DEBUG(2) debug_printf("%s director: non-directory on path: file assumed "
-        "not to exist\n", dblock->name);
+      DEBUG(2) debug_printf("non-directory on path %s: file assumed not to "
+        "exist\n", filename);
       yield = FAIL;
       break;
       }
@@ -459,8 +507,8 @@ if (fwd == NULL)
     case EACCES:           /* Permission denied */
     if (ob->ignore_eacces)
       {
-      DEBUG(2) debug_printf("%s director: permission denied: file assumed not "
-        "to exist\n", dblock->name);
+      DEBUG(2) debug_printf("permission denied for %s: file assumed not to "
+        "exist\n", filename);
       yield = FAIL;
       break;
       }
@@ -468,9 +516,7 @@ if (fwd == NULL)
 
     DEFAULT_ERROR:
     default:
-    addr->message = string_open_failed(errno, "%s (%s director)",
-      filename, dblock->name);
-    DEBUG(2) debug_printf("%s\n", addr->message);
+    addr->message = string_open_failed(errno, "%s", filename);
     yield = ERROR;
     break;
     }
@@ -492,13 +538,20 @@ if (fstat(fileno(fwd), &statbuf) != 0)
   goto CLOSE_RESTORE_UID;         /* skip forward */
   }
 
-if ((statbuf.st_mode & S_IFMT) != S_IFREG ||
-    (statbuf.st_mode & ob->modemask) != 0)
+if ((statbuf.st_mode & S_IFMT) != S_IFREG)
+  {
+  addr->basic_errno = ERRNO_BADFORWARD;
+  addr->message = string_sprintf("%s is not a regular file", filename);
+  yield = DEFER;
+  goto CLOSE_RESTORE_UID;         /* skip forward */
+  }
+
+if ((statbuf.st_mode & ob->modemask) != 0)
   {
   addr->basic_errno = ERRNO_BADFORWARD;
   addr->message =
-    string_sprintf("<%s> - bad mode (%o) for %s (%s director)",
-      addr->orig, statbuf.st_mode, filename, dblock->name);
+    string_sprintf("bad mode (0%o) for %s: 0%o bit(s) unexpected",
+      statbuf.st_mode, filename, statbuf.st_mode & ob->modemask);
   yield = DEFER;
   goto CLOSE_RESTORE_UID;         /* skip forward */
   }
@@ -535,8 +588,7 @@ if (!uid_ok || !gid_ok)
   {
   addr->basic_errno = ERRNO_BADFORWARD;
   addr->message =
-    string_sprintf("<%s> - bad %s for %s (%s director)",
-      addr->orig, uid_ok? "group" : "owner", filename, dblock->name);
+    string_sprintf("bad %s for %s", uid_ok? "group" : "owner", filename);
   yield = DEFER;
   goto CLOSE_RESTORE_UID;         /* skip forward */
   }
@@ -558,9 +610,7 @@ if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
   {
   store_pool = POOL_MAIN;
   addr->basic_errno = errno;
-  addr->message =
-    string_sprintf("<%s> - error while reading forward file (%s director)\n",
-    addr->orig, dblock->name);
+  addr->message = "error while reading forward file";
   yield = DEFER;
   goto CLOSE_RESTORE_UID;         /* skip forward */
   }
@@ -602,25 +652,39 @@ qualified with the same domain as the incoming address. */
 if (ob->qualify_preserve_domain) qualify_domain_recipient = addr->domain;
 
 /* Filter interpretation is done by a general function that is also called from
-the filter testing option (-bf). The second-last argument specifies whether the
-log command should be locked out or skipped; this is done when verifying, or if
-we have not set the uid to a local user, or when explicitly configured. Set up
-the value of extracted to be the same as it is from parse_extract_addresses().
-*/
+the filter testing option (-bf). Several features of string expansion may be
+locked out at sites that don't trust users. This is done by setting global
+flags that the expander inspects. The second-last argument specifies whether
+the log command should be locked out or skipped; this is done when verifying or
+testing addresses, or if we have not set the uid to a local user, or when
+explicitly configured. */
 
 if (filter_header && ob->filter)
   {
   int action = SPECIAL_NONE;
+  BOOL oldforbid_exists = expand_forbid_exists;
+  BOOL oldforbid_lookup = expand_forbid_lookup;
+  BOOL oldforbid_perl = expand_forbid_perl;
+
   DEBUG(2) debug_printf("file is a filter file\n");
   is_filter = TRUE;
+
+  expand_forbid_exists = ob->forbid_filter_exists;
+  expand_forbid_lookup = ob->forbid_filter_lookup;
+  expand_forbid_perl = ob->forbid_filter_perl;
+
   extracted = filter_interpret(filebuf, &generated, &delivered,
     (ob->allow_system_actions)? &action : NULL,
     &error,
-    verify? 1 :                        /* skip logging if verifying */
+    (verify || address_test_mode)? 1 : /* skip logging if verifying/testing */
       ob->forbid_filter_logwrite? 2 :  /* lock out if forbidden */
       restore_uid? 0 : 3,              /* else permit only if uid set */
     ob->rewrite)? 0 : -1;
 
+  expand_forbid_exists = oldforbid_exists;
+  expand_forbid_lookup = oldforbid_lookup;
+  expand_forbid_perl = oldforbid_perl;
+
   store_pool = POOL_MAIN;              /* Reset for error strings */
 
   /* Handle special action (will be SPECIAL_NONE unless permitted otherwise).
@@ -641,16 +705,28 @@ if (filter_header && ob->filter)
     }
 
   /* If extraction from a filter file failed, it is a "probable user error", to
-  use a good old IBM term. Just defer delivery and let the user clean things
-  up. */
+  use a good old IBM term. By default, just defer delivery and let the user
+  clean things up in due course. However, users are very lax about this. The
+  skip_syntax_errors option now applies to filter files as well as to normal
+  forward files. It can be used to carry on delivering, while sending an
+  error message if syntax_errors_to is set. */
 
   if (extracted != 0)
     {
-    addr->basic_errno = ERRNO_BADFORWARD;
-    addr->message =
-      string_sprintf("<%s> - error in filter file: %s", addr->orig,
-        error);
-    yield = DEFER;
+    if (ob->skip_syntax_errors)
+      {
+      eblock = store_get(sizeof(error_block));
+      eblock->next = NULL;
+      eblock->text1 = error;
+      eblock->text2 = NULL;
+      yield = OK;
+      }
+    else
+      {
+      addr->basic_errno = ERRNO_BADFORWARD;
+      addr->message = string_sprintf("error in filter file: %s", error);
+      yield = DEFER;
+      }
     }
   }
 
@@ -699,7 +775,7 @@ else
     {
     addr->basic_errno = ERRNO_BADFORWARD;
     addr->message =
-      string_sprintf("<%s> - error in forward file%s: %s", addr->orig,
+      string_sprintf("error in forward file%s: %s",
         filter_header? " (filtering not enabled)" : "", error);
     if (extracted > 0 && ob->freeze_missing_include)
       {
@@ -730,8 +806,7 @@ if (restore_uid)
   if (dblock->initgroups) setgroups(0, NULL);
   mac_setegid(saved_egid);
   mac_seteuid(saved_euid);
-  DEBUG(2) debug_printf("%s director restored euid=%d egid=%d\n", dblock->name,
-    (int)geteuid(), (int)getegid());
+  DEBUG(2) debug_print_ids("restored");
   }
 
 
@@ -742,8 +817,14 @@ forwardfile. We cannot do this earlier, because the error message must be sent
 as Exim, not as the local user. */
 
 if (yield == OK && eblock != NULL &&
-    !moan_skipped_syntax_errors(dblock->name, filename,
-      eblock, verify? NULL : ob->syntax_errors_to))
+    !moan_skipped_syntax_errors(
+      dblock->name,                           /* For message content */
+      filename,                               /* Ditto */
+      eblock,                                 /* Ditto */
+      (verify || address_test_mode)?
+        NULL : ob->syntax_errors_to,          /* Who to mail */
+      generated != NULL,                      /* True if not all failed */
+      ob->syntax_errors_text))                /* Custom message */
   {
   addr->special_action = SPECIAL_FREEZE;
   yield = ERROR;
@@ -759,12 +840,29 @@ if (reset_point != NULL)
   store_pool = POOL_MAIN;
   }
 
+
 /* If there has been an error, return the error value now. Subsequently we
 can just return directly on error, since there is no further need to mess with
 the uid or close the file. */
 
 if (yield != OK) return yield;
 
+/* If a forward file generates no addresses, it is not an error. The director
+just fails. Compare aliasfile, which is different. However, if it is a filter
+file that has set "delivered", the director has succeeded. Log the fact,
+as otherwise messages may appear to disappear without trace. */
+
+if (generated == NULL)
+  {
+  if (is_filter && delivered)
+    {
+    if (!verify && !address_test_mode)
+      log_write(0, LOG_MAIN, "=> discarded <%s> D=%s", addr->orig,
+        dblock->name);
+    return OK;
+    }
+  else return FAIL;
+  }
 
 /* Set up the errors address for the children of this address. */
 
@@ -778,175 +876,51 @@ rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
   &remove_headers);
 if (rc != OK) return rc;
 
-/* Add the new addresses to the list of new addresses, copying in the uid, gid
-and permission flags for use by pipes and files and autoreplies, setting the
-parent, and or-ing its ignore_error flag.
+/* Set up the new addresses and add them to the addr_new chain. A common
+function handles this. */
 
-If the generated address is the same as one of its ancestors, and the
-check_ancestor flag is set, do not use this generated address, but replace it
-with a copy of the input address. This is to cope with cases where A is aliased
-to B and B has a .forward file pointing to A. We can't just pass on the old
-address by returning FAIL, because it must act as a general parent for
-generated addresses, and only get marked "done" when all its children are
-delivered. */
+df_generated(dblock, addr_new, addr, generated, errors_to, extra_headers,
+  remove_headers, &ugid, pw, directory);
 
-while (generated != NULL)
+/* If the filter interpreter returned not "delivered", we have to arrange for
+this address to be passed on to subsequent directors. Returning FAIL would
+appear to be the answer, but it isn't, because successful delivery of the base
+address gets it marked "done", so deferred generated addresses never get tried
+again. We have to generate a new version of the base address, as if there were
+a "deliver" command in the filter file, with the original address as parent. */
+
+if (is_filter && !delivered)
   {
-  address_item *parent;
-  address_item *next = generated;
+  address_item *next;
 
-  generated = next->next;
+  next = deliver_make_addr(addr->orig, FALSE);
   next->parent = addr;
   next->ignore_error |= addr->ignore_error;
-  next->start_director = dblock->new;
+  next->pfr = addr->pfr;
   addr->child_count++;
   next->next = *addr_new;
   *addr_new = next;
 
-  if (ob->one_time)
-    {
-    for (parent = addr; parent->parent != NULL; parent = parent->parent);
-    next->onetime_parent = parent->orig;
-    }
-
-  if (ob->check_ancestor)
-    {
-    for (parent = addr; parent != NULL; parent = parent->parent)
-      {
-      if (strcmp(next->orig, parent->orig) == 0)
-        {
-        DEBUG(2) debug_printf("generated parent replaced by child\n");
-        next->orig = string_copy(addr->orig);
-        break;
-        }
-      }
-    }
-
   next->errors_address = errors_to;
   next->extra_headers = extra_headers;
   next->remove_headers = remove_headers;
 
-  if (next->pfr)
-    {
-    next->director = dblock;
-
-    /* If there are no explicit uid/gid settings on the driver,
-    use those for the local user if we've looked one up. */
-
-    direct_set_ugid(next, &ugid);
-
-    if (ob->check_local_user)
-      {
-      if (!ugid.uid_set)
-        {
-        next->uid = pw->pw_uid;
-        next->uid_set = TRUE;
-        }
-      if (!ugid.gid_set)
-        {
-        next->gid = pw->pw_gid;
-        next->gid_set = TRUE;
-        }
-      }
-
-    /* Remaining options */
-
-    if (ob->home_dir != NULL) next->home_dir = ob->home_dir;
-      else if (directory != NULL) next->home_dir = string_copy(directory);
-    next->current_dir = ob->current_dir;
-    next->allow_pipe = !ob->forbid_pipe;
-    next->allow_file = !ob->forbid_file;
-    next->allow_reply = !ob->forbid_reply;
-
-    /* Forwardfile can produce opipes or files or autoreplies; if the transport
-    setting is null, the global setting will get used later. */
-
-    if (next->orig[0] == '|')
-      next->transport = ob->pipe_transport;
-    else
-      if (next->orig[0] == '>') next->transport = ob->reply_transport;
-    else
-      {
-      int len = (int)strlen(next->orig);
-      if (next->orig[len-1] == '/')
-        {
-        next->transport = ob->directory_transport;
-        if (len > 1 && next->orig[len-2] == '/' &&
-            ob->directory2_transport != NULL)
-          next->transport = ob->directory2_transport;
-        }
-      else next->transport = ob->file_transport;
-      }
-    }
-
-  DEBUG(2) debug_printf("%s director generated %s\n%s%s%s%s%s%s%s",
+  DEBUG(2) debug_printf("%s director generated %s\n%s%s%s%s",
     dblock->name,
     next->orig,
     next->pfr? "  pipe, file, or autoreply\n" : "",
     (errors_to != NULL)? "  errors to " : "",
     (errors_to != NULL)? errors_to : "",
-    (errors_to != NULL)? "\n" : "",
-    (next->transport == NULL)? "" : "  transport=",
-    (next->transport == NULL)? "" : (next->transport)->name,
-    (next->transport == NULL)? "" : "\n");
+    (errors_to != NULL)? "\n" : "");
   }
 
-/* If the filter interpreter returned "delivered" then we have succeeded
-in completely handling this address. Otherwise, we have to arrange for this
-address to be passed on to subsequent directors. Returning FAIL would appear to
-be the answer, but it isn't, because successful delivery of the base address
-gets it marked "done", so deferred generated addresses never get tried again.
-We have to generate a new version of the base address, as if there were a
-"deliver" command in the filter file, with the original address as parent.
-However, we don't need to do this if there were no generated addresses. */
+/* Put the original address onto the succeed queue so that any retry items that
+get attached to it get processed. */
 
-if (is_filter)
-  {
-  address_item *next;
+addr->next = *addr_succeed;
+*addr_succeed = addr;
 
-  if (!delivered)
-    {
-    if (addr->child_count <= 0) return FAIL;
-
-    next = deliver_make_addr(addr->orig, FALSE);
-    next->parent = addr;
-    next->ignore_error |= addr->ignore_error;
-    next->pfr = addr->pfr;
-    addr->child_count++;
-    next->next = *addr_new;
-    *addr_new = next;
-
-    next->errors_address = errors_to;
-    next->extra_headers = extra_headers;
-    next->remove_headers = remove_headers;
-
-    DEBUG(2) debug_printf("%s director generated %s\n%s%s%s%s",
-      dblock->name,
-      next->orig,
-      next->pfr? "  pipe, file, or autoreply\n" : "",
-      (errors_to != NULL)? "  errors to " : "",
-      (errors_to != NULL)? errors_to : "",
-      (errors_to != NULL)? "\n" : "");
-    }
-
-  yield = OK;
-  }
-
-/* If the forward file generated no addresses, it is not an error. The
-director just fails. Compare aliasfile, which is different. */
-
-else yield = (addr->child_count <= 0)? FAIL : OK;
-
-/* If the yield is OK, put the original address onto the succeed queue so
-that any retry items that get attached to it get processed. */
-
-if (yield == OK)
-  {
-  addr->next = *addr_succeed;
-  *addr_succeed = addr;
-  }
-
-return yield;
+return OK;
 }
 
 /* End of directors/forwardfile.c */
