@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,12 +14,6 @@
 /* Options specific to the localuser director. */
 
 optionlist localuser_director_options[] = {
-  { "current_directory", opt_stringptr,
-      (void *)(offsetof(localuser_director_options_block, current_dir)) },
-  { "directory",         opt_stringptr | opt_hidden,
-      (void *)(offsetof(localuser_director_options_block, pw_dir)) },
-  { "home_directory",    opt_stringptr,
-      (void *)(offsetof(localuser_director_options_block, home_dir)) },
   { "match_directory",   opt_stringptr,
       (void *)(offsetof(localuser_director_options_block, pw_dir)) }
 };
@@ -33,10 +27,7 @@ int localuser_director_options_count =
 /* Default private options block for the localuser director. */
 
 localuser_director_options_block localuser_director_option_defaults = {
-  NULL,        /* pw_dir */
-  NULL,        /* home_dir */
-  NULL,        /* current_dir */
-  NULL         /* re_pw_dir */
+  NULL        /* pw_dir */
 };
 
 
@@ -56,6 +47,8 @@ localuser_director_options_block *ob =
   (localuser_director_options_block *)(dblock->options_block);
 */
 
+/* There must be a transport unless verifying. */
+
 if (dblock->transport == NULL &&
     dblock->expand_transport == NULL &&
     !dblock->verify_only)
@@ -88,6 +81,9 @@ char *remove_headers;
 header_line *extra_headers;
 int rc;
 
+addr_new = addr_new;          /* Keep picky compilers happy */
+addr_succeed = addr_succeed;
+
 /* The third argument to direct_finduser() must be NULL here, to prevent a
 digit string being taken as a numeric uid. */
 
@@ -98,53 +94,14 @@ if (!direct_finduser(addr->local_part, &pw, NULL))
   return FAIL;
   }
 
-/* If there is a setting for pw_dir, check for a match. Expand the string
-if necessary, allowing the home directory to appear in the expansion,
-for exotic cases. The outer level resets deliver_home. */
+/* Set the home directory, and if there is a setting of match_directory, check
+for a match. The resetting of deliver_home happens at the outer level. */
 
 deliver_home = pw->pw_dir;
-
 if (ob->pw_dir != NULL)
   {
-  BOOL match;
-  char *dir = ob->pw_dir;
-  if (strchr(dir, '$') != NULL)
-    {
-    dir = expand_string(dir);
-    if (dir == NULL)
-      {
-      if (expand_string_forcedfail)
-        {
-        DEBUG(2) debug_printf("%s director expansion of \"%s\" forced "
-          "failure\n", dblock->name, ob->pw_dir);
-        return FAIL;
-        }
-      addr->message = string_sprintf("%s director: failed to expand "
-        "directory string \"%s\": %s", dblock->name, ob->pw_dir,
-        expand_string_message);
-      if (search_find_defer) return DEFER;
-      addr->special_action = SPECIAL_FREEZE;
-      return ERROR;
-      }
-    }
-  match = match_check_string(pw->pw_dir, dir, &(ob->re_pw_dir), -1, TRUE, NULL);
-  if (!match)
-    {
-    if (search_find_defer)
-      {
-      addr->message = "lookup defer for match_directory";
-      return DEFER;
-      }
-
-    DEBUG(2)
-      {
-      debug_printf("%s director failed to match pw_dir with \"%s\" "
-        "for %s\n", dblock->name, ob->pw_dir, addr->local_part);
-      if (strcmp(dir, ob->pw_dir) != 0)
-        debug_printf("expanded string was \"%s\"\n", dir);
-      }
-    return FAIL;
-    }
+  rc = direct_match_directory(addr, pw->pw_dir, ob->pw_dir, dblock->name);
+  if (rc != OK) return rc;
   }
 
 /* Perform file existence and sender verification checks now that we
@@ -164,10 +121,13 @@ rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
   &remove_headers);
 if (rc != OK) return rc;
 
-/* This local user is OK; accept the address; update its errors address if the
-and header munging if necessary. */
+/* This local user is OK; set the transport and accept the address; update its
+errors address and header munging. Initialization ensures that there is a
+transport except when verifying. */
+
+if (!route_get_transport(dblock->transport, dblock->expand_transport,
+  addr, &(addr->transport), dblock->name, NULL)) return ERROR;
 
-addr->transport = dblock->transport;
 addr->errors_address = errors_to;
 addr->extra_headers = extra_headers;
 addr->remove_headers = remove_headers;
@@ -180,9 +140,10 @@ addr->uid_set = TRUE;
 addr->gid = pw->pw_gid;
 addr->gid_set = TRUE;
 
-if (ob->home_dir != NULL) addr->home_dir = ob->home_dir;
+if (dblock->home_directory != NULL) addr->home_dir = dblock->home_directory;
   else addr->home_dir = string_copy(pw->pw_dir);
-addr->current_dir = ob->current_dir;
+
+addr->current_dir = dblock->current_directory;
 
 return route_queue(addr, addr_local, addr_remote, (router_instance *)dblock,
   "director")? OK : ERROR;
