@@ -2,16 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
   char *pw_dir;
-  char *home_dir;
-  char *current_dir;
-  re_block *re_pw_dir;
 } localuser_director_options_block;
 
 /* Data for reading the private options. */
