@@ -2,23 +2,17 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
-/* Private structure for the private options. */
+/* Structure for the private options. The first block of them must be the
+common director fields for all directors that use shared functions. */
 
 typedef struct {
+  COMMON_DIRECTOR_FIELDS;
   char *new_address;
-
-#ifdef ALLOW_EXPIRE
-  char *expire_pattern;
-  char *expire_subject;
-  char *expire_time;
-  pcre *expire_pattern_re;
-  BOOL  expire_force_fail;
-#endif
-
   BOOL  panic_expansion_fail;
+  BOOL  rewrite;
 } smartuser_director_options_block;
 
 /* Data for reading the private options. */
