@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "exim.h"
@@ -66,7 +66,7 @@ while (c != 0 && *p != 0)
     {
     if (mkdir(buffer, mode) < 0 && errno != EEXIST)
       {
-      if (*parent == 0) return FALSE;
+      if (!panic) return FALSE;
       log_write(0, LOG_PANIC_DIE,
         "Failed to create directory \"%s\": %s\n", buffer, strerror(errno));
       }
