@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for interfacing with the DNS. */
@@ -247,24 +247,33 @@ dns_basic_lookup(char *name, int type)
 /* If configured, check the hygene of the name passed to lookup. Otherwise,
 although DNS lookups may give REFUSED at the lower level, some resolvers
 turn this into TRY_AGAIN, which is silly. Give a NOMATCH return, since such
-domains cannot be in the DNS. */
+domains cannot be in the DNS. The check is now done by a regular expression;
+give it space for substring storage to save it having to get its own if the
+regex has substrings that are used - the default uses a conditional. */
+
+#ifndef STAND_ALONE   /* Omit this for stand-alone tests */
 
 if (check_dns_names)
   {
-  int c;
-  char *ptr = name;
+  int ovector[3*(EXPAND_MAXN+1)];
 
-  while ((c = *ptr) != 0 && (isalnum(c) || c == '.' || c == '-')) ptr++;
+  if (regex_check_dns_names == NULL)
+    regex_check_dns_names =
+      regex_must_compile(check_dns_names_pattern, FALSE, TRUE);
 
-  if (c != 0)
+  if (pcre_exec(regex_check_dns_names, NULL, name, (int)strlen(name), 0,
+      PCRE_EOPT, ovector, sizeof(ovector)/sizeof(int)) < 0)
     {
     DEBUG(3)
-      debug_printf("DNS name check: %s (%s) includes illegal character \"%c\"\n",
-	name, dns_text_type(type), c);
+      debug_printf("DNS name syntax check failed: %s (%s)\n", name,
+        dns_text_type(type));
+    host_find_failed_syntax = TRUE;
     return DNS_NOMATCH;
     }
   }
 
+#endif /* STAND_ALONE */
+
 /* Call the resolver */
 
 dns_answerlen = res_search(name, C_IN, type, dns_answer, MAXPACKET);
@@ -282,8 +291,8 @@ if (dns_answerlen < 0) switch (h_errno)
 
   /* Cut this out for various test programs */
   #ifndef STAND_ALONE
-  if (!match_isinlist(name, dns_again_means_nonexist,
-      &re_dns_again_means_nonexist, FALSE)) return DNS_AGAIN;
+  if (!match_isinlist(name, &dns_again_means_nonexist, FALSE, TRUE, NULL))
+    return DNS_AGAIN;
   DEBUG(8) debug_printf("%s is in dns_again_means_nonexist: returning "
     "DNS_NOMATCH\n", name);
   #endif
