@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -24,9 +24,10 @@ Warning: Linking some objects which contain exception information sections
 
 As this may cause people to worry needlessly, include a dummy function here
 to stop the message from appearing. Make it call itself to stop picky compilers
-complaining that it isn't used. */
+compilers complaining that it is unused, and put in a dummy argument to stop
+even pickier compilers complaining about infinite loops. */
 
-static void dummy(void) { dummy(); }
+static void dummy(int x) { dummy(x-1); }
 
 
 /* Table of information about all possible lookup methods. The entries are
@@ -54,6 +55,10 @@ be NULL for methods that don't need them. */
 #include "lookups/lsearch.h"
 #endif
 
+#ifdef LOOKUP_MYSQL
+#include "lookups/mysql.h"
+#endif
+
 #ifdef LOOKUP_NIS
 #include "lookups/nis.h"
 #endif
@@ -81,16 +86,17 @@ lookup_info lookup_list[] = {
 /* cdb lookup in single file */
 
   {
-  "cdb",		       /* lookup name */
-  lookup_absfile,	       /* uses absolute file name */
+  "cdb",			 /* lookup name */
+  lookup_absfile,		 /* uses absolute file name */
 #ifdef LOOKUP_CDB
-  cdb_open,		       /* open function */
-  cdb_check,		       /* check function */
-  cdb_find,		       /* find function */
-  cdb_close,		       /* close function */
-  NULL			       /* no tidy function */
+  cdb_open,			 /* open function */
+  cdb_check,			 /* check function */
+  cdb_find,			 /* find function */
+  cdb_close,			 /* close function */
+  NULL,				 /* no tidy function */
+  NULL                           /* no quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -105,9 +111,28 @@ but the code is called dbmdb to avoid name clashes. */
   dbmdb_check,                   /* check function */
   dbmdb_find,                    /* find function */
   dbmdb_close,                   /* close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL         /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* This variant of DBM does not include the binary zero on the end
+of the key strings. */
+
+  {
+  "dbmnz",                       /* lookup name */
+  lookup_absfile,                /* uses absolute file name */
+#ifdef LOOKUP_DBM
+  dbmdb_open,      /* sic */     /* open function */
+  dbmdb_check,     /* sic */     /* check function */
+  dbmnz_find,                    /* find function */
+  dbmdb_close,     /* sic */     /* close function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -121,9 +146,10 @@ but the code is called dbmdb to avoid name clashes. */
   NULL,                          /* check function */
   dnsdb_find,                    /* find function */
   NULL,                          /* no close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -137,9 +163,10 @@ but the code is called dbmdb to avoid name clashes. */
   NULL,                          /* check function */
   eldap_find,                    /* find function */
   NULL,                          /* no close function */
-  eldap_tidy                     /* tidy function */
+  eldap_tidy,                    /* tidy function */
+  eldap_quote                    /* quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -149,13 +176,14 @@ but the code is called dbmdb to avoid name clashes. */
   "ldapm",                       /* lookup name */
   lookup_querystyle,             /* query-style lookup */
 #ifdef LOOKUP_LDAP
-  eldapm_open,                   /* open function */
+  eldap_open,       /* sic */    /* open function */
   NULL,                          /* check function */
   eldapm_find,                   /* find function */
   NULL,                          /* no close function */
-  eldapm_tidy                    /* tidy function */
+  eldap_tidy,       /* sic */    /* tidy function */
+  eldap_quote       /* sic */    /* quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -169,9 +197,27 @@ but the code is called dbmdb to avoid name clashes. */
   lsearch_check,                 /* check function */
   lsearch_find,                  /* find function */
   lsearch_close,                 /* close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
+#endif
+  },
+
+/* MYSQL lookup */
+
+  {
+  "mysql",                       /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+#ifdef LOOKUP_MYSQL
+  mysql_open,                    /* open function */
+  NULL,                          /* no check function */
+  mysql_find,                    /* find function */
+  NULL,                          /* no close function */
+  mysql_tidy,                    /* tidy function */
+  mysql_quote                    /* quoting function */
+#else
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -185,9 +231,10 @@ but the code is called dbmdb to avoid name clashes. */
   NULL,                          /* check function */
   nis_find,                      /* find function */
   NULL,                          /* no close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -201,9 +248,10 @@ but the code is called dbmdb to avoid name clashes. */
   NULL,                          /* check function */
   nis0_find,                     /* find function */
   NULL,                          /* no close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -217,9 +265,10 @@ but the code is called dbmdb to avoid name clashes. */
   NULL,                          /* check function */
   nisplus_find,                  /* find function */
   NULL,                          /* no close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  nisplus_quote                  /* quoting function */
 #else
-  NULL, NULL, NULL, NULL, NULL   /* lookup not present */
+  NULL, NULL, NULL, NULL, NULL, NULL /* lookup not present */
 #endif
   },
 
@@ -236,16 +285,71 @@ of "lookup type not available". */
   NULL,                          /* check function */
   testdb_find,                   /* find function */
   NULL,                          /* no close function */
-  NULL                           /* no tidy function */
+  NULL,                          /* no tidy function */
+  NULL                           /* no quoting function */
   },
 #endif
 
 /* Terminating entry */
 
-{ "", 0, NULL, NULL, NULL, NULL, NULL }
+{ "", 0, NULL, NULL, NULL, NULL, NULL, NULL }
 };
 
 
+/* Table of information about all possible authentication mechamisms. All
+entries are always present if any mechanism is declared, but the functions are
+set to NULL for those that are not compiled into the binary. None of this is
+wanted when building the test program for expand.c, which just needs the tables
+for lookups. */
+
+#ifdef HAVE_AUTH
+#ifndef TEST_EXPAND
+
+#ifdef AUTH_CRAM_MD5
+#include "auths/cram_md5.h"
+#endif
+
+#ifdef AUTH_PLAINTEXT
+#include "auths/plaintext.h"
+#endif
+
+auth_info auths_available[] = {
+
+/* Checking by an expansion condition on plain text */
+
+#ifdef AUTH_CRAM_MD5
+  {
+  "cram_md5",				     /* lookup name */
+  auth_cram_md5_server,                      /* server function */
+  auth_cram_md5_init,                        /* init function */
+  auth_cram_md5_client,                      /* client function */
+  auth_cram_md5_options,
+  &auth_cram_md5_options_count,
+  &auth_cram_md5_option_defaults,
+  sizeof(auth_cram_md5_options_block)
+  },
+#endif
+
+#ifdef AUTH_PLAINTEXT
+  {
+  "plaintext",				     /* lookup name */
+  auth_plaintext_server,                     /* server function */
+  auth_plaintext_init,                       /* init function */
+  auth_plaintext_client,                     /* client function */
+  auth_plaintext_options,
+  &auth_plaintext_options_count,
+  &auth_plaintext_option_defaults,
+  sizeof(auth_plaintext_options_block)
+  },
+#endif
+
+{ "", NULL, NULL, NULL, NULL, 0, NULL, 0  }
+};
+
+#endif  /* TEST_EXPAND */
+#endif  /* HAVE_AUTH */
+
+
 /* Tables of information about which directors, routers and transports are
 included in the exim binary. None are wanted when building the test program for
 expand.c, which just needs the lookup tables above. */
@@ -254,6 +358,8 @@ expand.c, which just needs the lookup tables above. */
 
 /* Pull in the necessary header files */
 
+#include "directors/dfunctions.h"
+
 #ifdef DIRECTOR_ALIASFILE
 #include "directors/aliasfile.h"
 #endif
@@ -462,20 +568,6 @@ transport_info transports_available[] = {
   NULL                                         /* no closedown entry */
   },
 #endif
-#ifdef TRANSPORT_DEBUG
-  {
-  "debug",                                     /* driver name */
-  debug_transport_entry,                       /* main entry point */
-  debug_transport_init,                        /* init entry point */
-  NULL,                                        /* no tidyup entry */
-  debug_transport_options,                     /* local options table */
-  &debug_transport_options_count,              /* number of entries */
-  &debug_transport_option_defaults,            /* private options defaults */
-  sizeof(debug_transport_options_block),       /* size of private block */
-  TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */
-  },
-#endif
 #ifdef TRANSPORT_PIPE
   {
   "pipe",                                      /* driver name */
