@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file is not part of the main Exim code. The Exim monitor and some other
@@ -14,7 +14,12 @@ of such functions - well not really dummies, more like alternates. */
 #include <stdarg.h>
 #include <stdio.h>
 #include <errno.h>
+#include <string.h>
 
+/* We don't have the full Exim headers dragged in, but this function
+is used for debugging output. */
+
+extern int  string_vformat(char *, int, char *, va_list);
 
 
 /*************************************************
@@ -41,6 +46,8 @@ va_start(ap, format);
 vfprintf(stderr, format, ap);
 fprintf(stderr, "\n");
 va_end(ap);
+level = level;     /* Keep picky compilers happy */
+flags = flags;
 }
 
 
@@ -61,8 +68,22 @@ void
 debug_printf(char *format, ...)
 {
 va_list ap;
+char buffer[1024];
+
 va_start(ap, format);
-vfprintf(stderr, format, ap);
+
+if (!string_vformat(buffer, sizeof(buffer), format, ap))
+  {
+  char *s = "**** debug string overflowed buffer ****\n";
+  char *p = buffer + (int)strlen(buffer);
+  int maxlen = sizeof(buffer) - (int)strlen(s) - 3;
+  if (p > buffer + maxlen) p = buffer + maxlen;
+  if (p > buffer && p[-1] != '\n') *p++ = '\n';
+  strcpy(p, s);
+  }
+
+fprintf(stderr, "%s", buffer);
+fflush(stderr);
 va_end(ap);
 }
 
@@ -77,6 +98,7 @@ extern int sigalrm_seen;
 void
 sigalrm_handler(int sig)
 {
+sig = sig;            /* Keep picky compilers happy */
 sigalrm_seen = 1;
 }
 
@@ -90,10 +112,22 @@ sigalrm_seen = 1;
 invoked from a utility, or aren't relevant. */
 
 int
-header_checkname(void *h, char *name, int len) { return 0; }
+header_checkname(void *h, char *name, int len)
+{
+h = h;            /* Keep picky compilers happy */
+name = name;
+len = len;
+return 0;
+}
 
 void
-directory_make(char *parent, char *name, int mode, int panic) { }
+directory_make(char *parent, char *name, int mode, int panic)
+{
+parent = parent;  /* Keep picky compilers happy */
+name = name;
+mode = mode;
+panic = panic;
+}
 
 void
 host_build_sender_fullhost(void) { }
