@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -43,6 +43,12 @@ are needed by any of the other headers, including system headers. */
 #include <limits.h>
 #endif
 
+/* Just in case some aged system doesn't define it... */
+
+#ifndef INT_MAX
+#define INT_MAX 2147483647
+#endif
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <netdb.h>
@@ -65,6 +71,9 @@ are needed by any of the other headers, including system headers. */
 #ifndef EX_TEMPFAIL
 #define EX_TEMPFAIL    75        /* temp failure; user is invited to retry */
 #endif
+#ifndef EX_CONFIG
+#define EX_CONFIG      78        /* configuration error */
+#endif
 
 #include <sys/time.h>
 #include <sys/param.h>
@@ -74,6 +83,16 @@ are needed by any of the other headers, including system headers. */
 #endif
 
 #include <sys/socket.h>
+
+/* If we are on an IPv6 system, the macro AF_INET6 will have been defined in
+the sys/socket.h header. It is helpful to have this defined on an IPv4 system
+so that it can appear in the code, even if it is never actually used when
+the code is run. It saves some #ifdef occurrences. */
+
+#ifndef AF_INET6
+#define AF_INET6 24
+#endif
+
 #include <sys/ioctl.h>
 
 /* The new standard is statvfs; some OS have statfs. For statvfs the block
@@ -105,6 +124,19 @@ or a macro with entries f_frsize and f_bsize. */
     #include <sys/mount.h>
     #endif
   #endif
+
+  /* Macros for the fields for the available space for non-superusers; define
+  these only if the OS header has not. Not all OS have f_favail; those that
+  are known to have it define F_FAVAIL as f_favail. The default is to use
+  f_free. */
+
+  #ifndef F_BAVAIL
+  #define F_BAVAIL f_bavail
+  #endif
+
+  #ifndef F_FAVAIL
+  #define F_FAVAIL f_ffree
+  #endif
 #endif
 
 
@@ -136,7 +168,9 @@ present Exim doesn't actually use it. */
 #endif
 
 #include <utime.h>
+#ifndef NO_NET_IF_H
 #include <net/if.h>
+#endif
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
@@ -186,15 +220,6 @@ we arrange to undefine __P if and only if ident.h defines it. */
 #undef __P
 #endif
 
-/* If we are on an IPv6 system, the macro AF_INET6 will have been defined in
-the sys/socket.h header. It is helpful to have this defined on an IPv4 system
-so that it can appear in the code, even if it is never actually used when
-the code is run. It saves some #ifdef occurrences. */
-
-#ifndef AF_INET6
-#define AF_INET6 24
-#endif
-
 /* One operating system uses a different type for the 2nd argument of select().
 Its os.h file defines SELECT_ARG2_TYPE. For the rest, define a default here. */
 
@@ -221,6 +246,21 @@ use different values for specific OS if ever necessary. */
 #define S_ADDR_TYPE u_long
 #endif
 
+/* One operating system (SunOS4) defines getc, ungetc, feof, and ferror as
+macros and not as functions. Exim needs them to be assignable functions. This
+flag gets set to cause this to be sorted out here. */
+
+#ifdef FUDGE_GETC_AND_FRIENDS
+#undef getc
+extern int getc(FILE *);
+#undef ungetc
+extern int ungetc(int, FILE *);
+#undef feof
+extern int feof(FILE *);
+#undef ferror
+extern int ferror(FILE *);
+#endif
+
 /* The header from the PCRE regex package */
 
 #include "pcre/pcre.h"
@@ -235,6 +275,19 @@ use different values for specific OS if ever necessary. */
 #include "globals.h"
 #include "functions.h"
 
+/* The following stuff must follow the inclusion of config.h because it
+requires various things that are set therein. */
+
+/* Define a union to hold either an IPv4 or an IPv6 sockaddr structure; this
+simplifies some of the coding. */
+
+union sockaddr_46 {
+  struct sockaddr_in v4;
+  #if HAVE_IPV6
+  struct sockaddr_in6 v6;
+  #endif
+};
+
 /* If SPOOL_DIRECTORY, STDERR_FILE, LOG_FILE_PATH or PID_FILE_PATH have not
 been defined, set them to the null string. */
 
