@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -37,9 +37,13 @@ int main(int argc, char **argv)
 {
 int started;
 int count = 0;
+int dupcount = 0;
 int yield = 0;
 int arg = 1;
+int add_zero = 1;
 BOOL lowercase = TRUE;
+BOOL warn = TRUE;
+BOOL lastdup = FALSE;
 #ifndef USE_DB
 int is_db = 0;
 struct stat statbuf;
@@ -56,13 +60,13 @@ char *line = malloc(maxsize);
 
 while (argc > 1)
   {
-  if (strcmp(argv[arg], "-nolc") == 0)
-    {
-    lowercase = FALSE;
-    arg++;
-    argc--;
-    }
+  if      (strcmp(argv[arg], "-nolc") == 0)    lowercase = FALSE;
+  else if (strcmp(argv[arg], "-nowarn") == 0)  warn = FALSE;
+  else if (strcmp(argv[arg], "-lastdup") == 0) lastdup = TRUE;
+  else if (strcmp(argv[arg], "-nozero") == 0)  add_zero = 0;
   else break;
+  arg++;
+  argc--;
   }
 
 if (argc != 3)
@@ -155,42 +159,64 @@ while (fgets(line, maxsize, f) != NULL)
 
   else
     {
-    int i;
+    int i, rc;
     char *s = line;
+
     if (started)
       {
       #ifdef EXIM_CLEAR_DATUMSTRUCT
       memset(&content, 0, sizeof(content));
       #endif
       EXIM_DATUM_DATA(content) = buffer;
-      EXIM_DATUM_SIZE(content) = bptr - buffer + 1;
-      EXIM_DBPUT(d, key, content);
-      count++;
+      EXIM_DATUM_SIZE(content) = bptr - buffer + add_zero;
+
+      switch(rc = EXIM_DBPUTB(d, key, content))
+        {
+        case EXIM_DBPUTB_OK:
+        count++;
+        break;
+
+        case EXIM_DBPUTB_DUP:
+        if (warn) fprintf(stderr, "** Duplicate key \"%s\"\n",
+          keybuffer);
+        dupcount++;
+        yield = 1;
+        if (lastdup) EXIM_DBPUT(d, key, content);
+        break;
+
+        default:
+        fprintf(stderr, "Error %d while writing key %s: errno=%d\n", rc,
+          keybuffer, errno);
+        yield = 2;
+        goto TIDYUP;
+        }
+
       bptr = buffer;
       }
+
     while (*s != 0 && *s != ':' && !isspace(*s)) s++;
     #ifdef EXIM_CLEAR_DATUMSTRUCT
     memset(&key, 0, sizeof(key));
     #endif
     EXIM_DATUM_DATA(key) = keybuffer;
-    EXIM_DATUM_SIZE(key) = s - line + 1;
+    EXIM_DATUM_SIZE(key) = s - line + add_zero;
 
     if (EXIM_DATUM_SIZE(key) > 256)
       {
       printf("Keys longer than 255 characters cannot be handled\n");
       started = 0;
-      yield = 1;
-      break;
+      yield = 2;
+      goto TIDYUP;
       }
 
     if (lowercase)
       {
-      for (i = 0; i < EXIM_DATUM_SIZE(key) - 1; i++)
+      for (i = 0; i < EXIM_DATUM_SIZE(key) - add_zero; i++)
         keybuffer[i] = tolower(line[i]);
       }
     else
       {
-      for (i = 0; i < EXIM_DATUM_SIZE(key) - 1; i++)
+      for (i = 0; i < EXIM_DATUM_SIZE(key) - add_zero; i++)
         keybuffer[i] = line[i];
       }
 
@@ -214,24 +240,52 @@ while (fgets(line, maxsize, f) != NULL)
 
 if (started)
   {
+  int rc;
+
   #ifdef EXIM_CLEAR_DATUMSTRUCT
   memset(&content, 0, sizeof(content));
   #endif
   EXIM_DATUM_DATA(content) = buffer;
-  EXIM_DATUM_SIZE(content) = bptr - buffer + 1;
-  EXIM_DBPUT(d, key, content);
-  count++;
+  EXIM_DATUM_SIZE(content) = bptr - buffer + add_zero;
+
+  switch(rc = EXIM_DBPUTB(d, key, content))
+    {
+    case EXIM_DBPUTB_OK:
+    count++;
+    break;
+
+    case EXIM_DBPUTB_DUP:
+    if (warn) fprintf(stderr, "** Duplicate key \"%s\"\n", keybuffer);
+    dupcount++;
+    yield = 1;
+    if (lastdup) EXIM_DBPUT(d, key, content);
+    break;
+
+    default:
+    fprintf(stderr, "Error %d while writing key %s: errno=%d\n", rc,
+      keybuffer, errno);
+    yield = 2;
+    break;
+    }
   }
 
+/* Close files, rename or abandon the temporary files, and exit */
+
+TIDYUP:
+
 EXIM_DBCLOSE(d);
 fclose(f);
 
 /* If successful, output the number of entries and rename the temporary
 files. */
 
-if (yield == 0)
+if (yield == 0 || yield == 1)
   {
-  printf("%d entries written\n", count);
+  printf("%d entr%s written\n", count, (count == 1)? "y" : "ies");
+  if (dupcount > 0)
+    {
+    printf("%d duplicate key%s \n", dupcount, (dupcount > 1)? "s" : "");
+    }
 
   #ifdef USE_DB
   strcpy(real_dbmname, temp_dbmname);
