@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include <stdarg.h>
@@ -49,9 +49,6 @@ a pipe! */
 extern char *spool_directory;
 char *spool_directory;
 
-extern int store_max_malloc;
-int store_max_malloc = 0;
-
 extern int store_pool;
 int store_pool = POOL_MAIN;
 
@@ -61,8 +58,8 @@ volatile int sigalrm_seen;
 extern int debug_level;
 int debug_level = 0;
 
-extern BOOL debug_trace_memory;
-BOOL debug_trace_memory = 0;
+extern int debug_trace_memory;
+int debug_trace_memory = 0;
 
 extern BOOL print_topbitchars;
 BOOL print_topbitchars = FALSE;
