@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -10,8 +10,8 @@
 arguments are two optional flags, followed by the spool directory name and the
 database name. The flags are
 
-   -f         full tidy - for the waiting database, checks are made on the
-              existence of messages
+   -f         full tidy - for the waiting and retry databases, checks are made
+              on the existence of messages
 
    -t <time>  expiry time for old records - default 30 days
 
@@ -66,17 +66,14 @@ char *spool_directory;
 extern volatile int sigalrm_seen;
 volatile int sigalrm_seen;
 
-extern int store_max_malloc;
-int store_max_malloc = 0;
-
 extern int store_pool;
 int store_pool = POOL_MAIN;
 
 extern int debug_level;
 int debug_level = 0;
 
-extern BOOL debug_trace_memory;
-BOOL debug_trace_memory = 0;
+extern int debug_trace_memory;
+int debug_trace_memory = 0;
 
 extern BOOL print_topbitchars;
 BOOL print_topbitchars = FALSE;
@@ -228,7 +225,8 @@ while (keychain != NULL)
 
   if (!full) continue;
 
-  /* Otherwise do database-specific tidying for wait databases */
+  /* Otherwise do database-specific tidying for wait databases, and message-
+  specific tidying for the retry database. */
 
   if (dbdata_type == type_wait)
     {
@@ -327,6 +325,37 @@ while (keychain != NULL)
         wait->count * MESSAGE_ID_LENGTH);
       }
     }
+
+  /* If a retry record's key ends with a message-id, check that that message
+  still exists; if not, remove this record. */
+
+  else if (dbdata_type == type_retry)
+    {
+    char *id;
+    int len = (int)strlen(key);
+
+    if (len < MESSAGE_ID_LENGTH + 1) continue;
+    id = key + len - MESSAGE_ID_LENGTH - 1;
+    if (*id++ != ':') continue;
+
+    for (i = 0; i < MESSAGE_ID_LENGTH; i++)
+      {
+      if (i == 6 || i == 13)
+        { if (id[i] != '-') break; }
+      else
+        { if (!isalnum(id[i])) break; }
+      }
+    if (i < MESSAGE_ID_LENGTH) continue;
+
+    strncpy(buffer+path_len, id, MESSAGE_ID_LENGTH);
+    sprintf(buffer+path_len + MESSAGE_ID_LENGTH, "-D");
+
+    if (stat(buffer, &statbuf) != 0)
+      {
+      dbfn_delete(dbm, key);
+      printf("deleted %s (no message)\n", key);
+      }
+    }
   }
 
 dbfn_close(dbm);
