@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -48,7 +48,6 @@ static int  had_else_endif;
 static int  log_fd;
 static int  log_mode;
 static int  output_indent;
-static int  log_control;
 static BOOL seen_force;
 static BOOL seen_value;
 static BOOL noerror_force;
@@ -59,6 +58,69 @@ enum { had_neither, had_else, had_elif, had_endif };
 static BOOL read_command_list(char **, filter_cmd ***, BOOL);
 
 
+/* The string arguments for the mail command. The header line ones (that are
+permitted to include \n followed by white space) first, and then the body text
+one (it can have \n anywhere). Then the file names and once_repeat, which may
+not contain \n. */
+
+static char *mailargs[] = {  /* "to" must be first, and */
+  "to",                      /* "cc" and "bcc" must follow */
+  "cc",
+  "bcc",
+  "from",
+  "reply_to",
+  "subject",
+  "text",
+  "file",
+  "log",
+  "once",
+  "once_repeat"
+};
+
+/* The count of string arguments */
+
+#define mailargs_string_count (sizeof(mailargs)/sizeof(char *))
+
+/* The count of string arguments that are actually passed over as strings
+(once_repeat is converted to an int). */
+
+#define mailargs_string_passed (mailargs_string_count - 1)
+
+/* This defines the offsets for the arguments; first the string ones, and
+then the non-string ones. The order must be as above. */
+
+enum { mailarg_index_to,
+       mailarg_index_cc,
+       mailarg_index_bcc,
+       mailarg_index_from,
+       mailarg_index_reply_to,
+       mailarg_index_subject,
+       mailarg_index_text,         /* text is first after headers */
+       mailarg_index_file,         /* between text and expand are filenames */
+       mailarg_index_log,
+       mailarg_index_once,
+       mailarg_index_once_repeat,  /* a time string */
+       mailarg_index_expand,       /* first non-string argument */
+       mailarg_index_return,
+       mailargs_total              /* total number of arguments */
+       };
+
+/* Offsets in the data structure for the string arguments (note that
+once_repeat isn't a string argument at this point.) */
+
+static int reply_offsets[] = {  /* must be in same order as above */
+  offsetof(reply_item, to),
+  offsetof(reply_item, cc),
+  offsetof(reply_item, bcc),
+  offsetof(reply_item, from),
+  offsetof(reply_item, reply_to),
+  offsetof(reply_item, subject),
+  offsetof(reply_item, text),
+  offsetof(reply_item, file),
+  offsetof(reply_item, logfile),
+  offsetof(reply_item, oncelog),
+};
+
 /* Condition identities and names, with negated versions for some
 of them. */
 
@@ -117,9 +179,9 @@ static int cond_types[] = { cond_BEGINS, cond_BEGINS, cond_CONTAINS,
   cond_contains, cond_ends, cond_ends, cond_is, cond_matches, cond_matches };
 
 /* Command identities: must be kept in step with the list of command words
-which follows. Stupidly, there was a "log" command as well as a "log" option on
-the mail command. The main command is now "logwrite", but the old name must
-remain as a synonym. */
+and the list of expanded argument counts which follow. Stupidly, there was a
+"log" command as well as a "log" option on the mail command. The main command
+is now "logwrite", but the old name must remain as a synonym. */
 
 enum { add_command, deliver_command, elif_command, else_command, endif_command,
        finish_command, fail_command, freeze_command, headers_command,
@@ -128,81 +190,37 @@ enum { add_command, deliver_command, elif_command, else_command, endif_command,
        seen_command, testprint_command, unseen_command, vacation_command };
 
 static char *command_list[] = {
-  "add",
-  "deliver",
-  "elif",
-  "else",
-  "endif",
-  "finish",
-  "fail",
-  "freeze",
-  "headers",
-  "if",
-  "log",
-  "logfile",
-  "logwrite",
-  "mail",
-  "noerror",
-  "pipe",
-  "save",
-  "seen",
-  "testprint",
-  "unseen",
-  "vacation"
+  "add",     "deliver", "elif", "else", "endif",     "finish",   "fail",
+  "freeze",  "headers", "if",   "log",  "logfile",   "logwrite", "mail",
+  "noerror", "pipe",    "save", "seen", "testprint", "unseen",   "vacation"
 };
 
 static int command_list_count = sizeof(command_list)/sizeof(char *);
 
-/* The arguments for the mail command. All the text ones come first, with
-the header line ones (that are permitted to include \n followed by white space)
-first, and then the body text one last. It can have \n anywhere. The file names
-may not contain \n. */
-
-static char *mailargs[] = {  /* "to" must be first, and */
-  "to",                      /* "cc" and "bcc" must follow */
-  "cc",
-  "bcc",
-  "subject",
-  "text",
-  "file",
-  "log",
-  "once"
+static int command_exparg_count[] = {
+   2, /* add */
+   2, /* deliver */
+   0, /* elif */
+   0, /* else */
+   0, /* endif */
+   0, /* finish */
+   1, /* fail */
+   1, /* freeze */
+   1, /* headers */
+   0, /* if */
+   1, /* log */
+   1, /* logfile */
+   1, /* logwrite */
+   mailargs_string_count, /* mail */
+   0, /* noerror */
+   0, /* pipe */
+   1, /* save */
+   0, /* seen */
+   1, /* testprint */
+   0, /* unseen */
+   mailargs_string_count /* vacation */
 };
 
-/* This defines the offsets for the arguments; first the string ones, and
-then the non-string ones. The order must be as above. */
-
-enum { mailarg_index_to,
-       mailarg_index_cc,
-       mailarg_index_bcc,
-       mailarg_index_subject,
-       mailarg_index_text,         /* text is first after headers */
-       mailarg_index_file,         /* between text and expand are filenames */
-       mailarg_index_log,
-       mailarg_index_once,
-       mailarg_index_expand,       /* first non-string argument */
-       mailarg_index_return,
-       mailargs_total              /* total number of arguments */
-       };
-
-/* The count of string arguments */
-
-#define mailargs_string_count mailarg_index_expand
-
-/* Offsets in the data structure for the string arguments */
-
-static int reply_offsets[] = {  /* must be in same order as above */
-  offsetof(reply_item, to),
-  offsetof(reply_item, cc),
-  offsetof(reply_item, bcc),
-  offsetof(reply_item, subject),
-  offsetof(reply_item, text),
-  offsetof(reply_item, file),
-  offsetof(reply_item, logfile),
-  offsetof(reply_item, oncelog)
-};
-
-
 
 
 /*************************************************
@@ -870,9 +888,9 @@ switch (command)
     }
   /* Fall through */
 
-  /* Deliver, logwrite, logfile, pipe, and testprint all take a single
-  argument, and save and logfile can have an option second argument for
-  the mode. */
+  /* Logwrite, logfile, pipe, and testprint all take a single argument, save
+  and logfile can have an option second argument for the mode, and deliver can
+  have "errors_to <address>" in a system filter. */
 
   case deliver_command:
   case logfile_command:
@@ -888,8 +906,8 @@ switch (command)
 
   if (*error_pointer != NULL) yield = FALSE; else
     {
-    char *argument;
-    void *second_argument = (void *)(-1);
+    char *argument = NULL;
+    char *second_argument = NULL;
 
     if (command == add_command)
       {
@@ -940,7 +958,8 @@ switch (command)
       }
 
     /* The argument for the logwrite command must end in a newline, and the save
-    and logfile commands can have an optional mode argument. */
+    and logfile commands can have an optional mode argument. The deliver
+    command can have an optional "errors_to <address>" for a system filter. */
 
     else
       {
@@ -952,11 +971,26 @@ switch (command)
 
       argument = string_copy(buffer);
 
-      if ((command == save_command || command == logfile_command) &&
-          isdigit(*ptr))
+      if (command == save_command || command == logfile_command)
         {
+        if (isdigit(*ptr))
+          {
+          ptr = nextword(ptr, buffer, sizeof(buffer), FALSE);
+          second_argument = (void *)strtol(buffer, NULL, 8);
+          }
+        else second_argument = (void *)(-1);
+        }
+
+      else if (command == deliver_command && system_filtering)
+        {
+        char *save_ptr = ptr;
         ptr = nextword(ptr, buffer, sizeof(buffer), FALSE);
-        second_argument = (void *)strtol(buffer, NULL, 8);
+        if (strcmp(buffer, "errors_to") == 0)
+          {
+          ptr = nextword(ptr, buffer, sizeof(buffer), FALSE);
+          second_argument = string_copy(buffer);
+          }
+        else ptr = save_ptr;
         }
       }
 
@@ -1023,7 +1057,8 @@ switch (command)
       ptr = saveptr;
     else
       {
-      ptr = nextitem(ptr, buffer, sizeof(buffer), FALSE);
+      if (*saveptr != '\"')
+        ptr = nextitem(ptr, buffer, sizeof(buffer), FALSE);
       fmsg = string_copy(buffer);
       }
     }
@@ -1128,12 +1163,11 @@ switch (command)
   break;
 
 
-  /* The mail & vacation commands have a whole slew of keyworded arguments,
-  each of which must be expanded. The final argument values are the file expand
-  and return message booleans, whose offsets are defined in mailarg_index_
-  {expand,return}. Although they are logically booleans, because they are
-  stored in a char * value, we use NULL and not FALSE, to keep 64-bit
-  compilers happy. */
+  /* The mail & vacation commands have a whole slew of keyworded arguments.
+  The final argument values are the file expand and return message booleans,
+  whose offsets are defined in mailarg_index_{expand,return}. Although they
+  are logically booleans, because they are stored in a char * value, we use
+  NULL and not FALSE, to keep 64-bit compilers happy. */
 
   case mail_command:
   case vacation_command:
@@ -1142,9 +1176,7 @@ switch (command)
   new->command = command;
   new->seen = seen_force? seen_value : FALSE;
   new->noerror = noerror_force;
-  for (i = 0; i < mailargs_string_count; i++) new->args[i] = NULL;
-  new->args[mailarg_index_expand] = NULL;
-  new->args[mailarg_index_return] = NULL;
+  for (i = 0; i < mailargs_total; i++) new->args[i] = NULL;
 
   /* Read keyword/value pairs until we hit one that isn't. The data
   must contain only printing chars plus tab, though the "text" value
@@ -1231,6 +1263,8 @@ switch (command)
       new->args[mailarg_index_log] = string_copy(".vacation.log");
     if (new->args[mailarg_index_once] == NULL)
       new->args[mailarg_index_once] = string_copy(".vacation");
+    if (new->args[mailarg_index_once_repeat] == NULL)
+      new->args[mailarg_index_once_repeat] = string_copy("7d");
     }
 
   /* Join the address on to the chain of generated addresses */
@@ -1316,7 +1350,7 @@ if (conditional)
   expect_endif--;
   if (had_else_endif == had_neither)
     {
-    *error_pointer = "\"endif\" missing at end of filter file\n";
+    *error_pointer = "\"endif\" missing at end of filter file";
     return FALSE;
     }
   }
@@ -1399,7 +1433,7 @@ test_condition(condition_block *c, BOOL *delivered, BOOL toplevel)
 {
 BOOL yield;
 pcre *re;
-char *exp[2], *p, *pp, *ss;
+char *exp[2], *p, *pp;
 const char *regcomp_error = NULL;
 int regcomp_error_offset;
 int val[2];
@@ -1429,7 +1463,7 @@ switch (c->type)
     alias *aliases = (alias *)(c->left);
     char *self, *self_from, *self_to;
     char *psself = NULL;
-    char *psself_from, *psself_to;
+    char *psself_from = NULL, *psself_to = NULL;
     int to_count = 2;
     int from_count = 5;
 
@@ -1465,6 +1499,8 @@ switch (c->type)
               "daemon@", "root@", self, self_from, psself, psself_from) &&
             test_header("subject:", FALSE, NULL, 1, "circular") &&
             test_header("precedence:", FALSE, NULL, 1, "bulk") &&
+            test_header("precedence:", FALSE, NULL, 1, "list") &&
+            test_header("precedence:", FALSE, NULL, 1, "junk") &&
             (sender_address == NULL || sender_address[0] != 0);
     }
   break;
@@ -1499,7 +1535,7 @@ switch (c->type)
 
   case cond_foranyaddress:
   p = (char *)(c->left);
-  ss = pp = expand_string(p);
+  pp = expand_string(p);
   if (pp == NULL)
     {
     *error_pointer = string_sprintf("failed to expand \"%s\" in "
@@ -1513,6 +1549,7 @@ switch (c->type)
     char *error;
     int start, end, domain;
     int saveend;
+
     p = parse_find_address_end(pp, FALSE);
     saveend = *p;
 
@@ -1529,7 +1566,6 @@ switch (c->type)
         debug_printf("Extracted address %s\n", filter_thisaddress);
         }
       yield = test_condition(c->right, delivered, FALSE);
-      filter_thisaddress = NULL;
       }
 
     if (yield) break;
@@ -1601,9 +1637,11 @@ switch (c->type)
       debug_printf("  Subject = %s\n", exp[0]);
       debug_printf("  Pattern = %s\n", exp[1]);
       }
+
     re = pcre_compile(exp[1],
       PCRE_COPT | ((c->type == cond_matches)? PCRE_CASELESS : 0),
-      &regcomp_error, &regcomp_error_offset);
+      &regcomp_error, &regcomp_error_offset, NULL);
+
     if (re == NULL)
       {
       *error_pointer = string_sprintf("error while compiling "
@@ -1611,6 +1649,7 @@ switch (c->type)
         exp[1], regcomp_error, regcomp_error_offset);
       return FALSE;
       }
+
     yield = regex_match_and_setup(re, exp[0], PCRE_EOPT, -1);
     break;
 
@@ -1662,6 +1701,7 @@ Arguments:
   generated   where to hang newly-generated addresses
   delivered   points to the "delivered" state of the filtering
   rewrite     TRUE if "deliver" addresses are to be rewritten
+  log_access  control value for writing to a log file
 
 Returns:      FALSE if there was an error, or if "finish", "freeze" or
                 "fail" was obeyed
@@ -1669,7 +1709,7 @@ Returns:      FALSE if there was an error, or if "finish", "freeze" or
 
 static BOOL
 interpret_commands(filter_cmd *commands, address_item **generated,
-  BOOL *delivered, BOOL rewrite)
+  BOOL *delivered, BOOL rewrite, int log_access)
 {
 char *s;
 int mode;
@@ -1679,27 +1719,47 @@ BOOL condition_value;
 
 while (commands != NULL)
   {
+  char *expargs[mailargs_string_count];
+
   int i, n[2];
 
+  /* Expand the relevant number of arguments for the command that are
+  not NULL. */
+
+  for (i = 0; i < command_exparg_count[commands->command]; i++)
+    {
+    char *ss = (char *)(commands->args[i]);
+    if (ss == NULL)
+      {
+      expargs[i] = NULL;
+      }
+    else
+      {
+      expargs[i] = expand_string(ss);
+      if (expargs[i] == NULL)
+        {
+        *error_pointer = string_sprintf("failed to expand \"%s\" in "
+          "%s command: %s", ss, command_list[commands->command],
+          expand_string_message);
+        return FALSE;
+        }
+      }
+    }
+
+  /* Now switch for each command */
+
   switch(commands->command)
     {
     case add_command:
     for (i = 0; i < 2; i++)
       {
-      char *ss, *end;
-      s = expand_string((char *)(commands->args[i]));
-      if (s == NULL)
-        {
-        *error_pointer = string_sprintf("failed to expand \"%s\": %s",
-          (char *)(commands->args[i]), expand_string_message);
-        return FALSE;
-        }
+      char *ss = expargs[i];
+      char *end;
 
-      ss = s;
       if (i == 1 && (*ss++ != 'n' || ss[1] != 0))
         {
         *error_pointer = string_sprintf("unknown variable \"%s\" in \"add\" "
-          "command", s);
+          "command", expargs[i]);
         return FALSE;
         }
 
@@ -1720,90 +1780,110 @@ while (commands != NULL)
     break;
 
     case deliver_command:
-    case save_command:
     if (commands->seen) *delivered = TRUE;
-    s = expand_string((char *)(commands->args[0]));
-    mode = (int)(commands->args[1]);
-    if (s == NULL)
-      {
-      *error_pointer = string_sprintf("failed to expand \"%s\": %s",
-        (char *)(commands->args[0]), expand_string_message);
-      return FALSE;
-      }
 
-    /* A deliver command's argument must be a valid address */
+    /* A deliver command's argument must be a valid address. Its optional
+    second argument (system filter only) must also be a valid address. */
 
-    if (commands->command == deliver_command)
+    for (i = 0; i < 2; i++)
       {
-      int start, end, domain;
-      char *error;
-      char *ss = parse_extract_address(s, &error, &start, &end, &domain,
-        FALSE);
-      if (ss != NULL)
-        s = rewrite? rewrite_address(ss, TRUE, TRUE) :
-          rewrite_address_qualify(ss, TRUE);
-      else
+      s = expargs[i];
+      if (s != NULL)
         {
-        *error_pointer = string_sprintf("malformed address \"%s\" in "
-          "filter file: %s", s, error);
-        return FALSE;
+        int start, end, domain;
+        char *error;
+        char *ss = parse_extract_address(s, &error, &start, &end, &domain,
+          FALSE);
+        if (ss != NULL)
+          expargs[i] = rewrite? rewrite_address(ss, TRUE, FALSE) :
+            rewrite_address_qualify(ss, TRUE);
+        else
+          {
+          *error_pointer = string_sprintf("malformed address \"%s\" in "
+            "filter file: %s", s, error);
+          return FALSE;
+          }
         }
       }
 
+    /* Stick the errors address into a simple variable, as it will
+    be referenced a few times. */
+
+    s = expargs[1];
+
     /* Test case: report what would happen */
 
     if (filter_test != NULL)
       {
       indent();
-      switch(commands->command)
-        {
-        case deliver_command:
-        printf("%seliver message to: %s%s\n", (commands->seen)?
-          "D" : "Unseen d", s, commands->noerror? " (noerror)" : "");
-        break;
-        case save_command:
-        if (mode < 0)
-          printf("%save message to: %s%s\n", (commands->seen)?
-            "S" : "Unseen s", s, commands->noerror? " (noerror)" : "");
-        else
-          printf("%save message to: %s %04o%s\n", (commands->seen)?
-            "S" : "Unseen s", s, mode, commands->noerror? " (noerror)" : "");
-        break;
-        }
+      printf("%seliver message to: %s%s%s%s\n",
+        (commands->seen)? "D" : "Unseen d",
+        expargs[0],
+        commands->noerror? " (noerror)" : "",
+        (s != NULL)? " errors_to " : "",
+        (s != NULL)? s : "");
       }
 
-    /* Real case: Ensure save command starts with / and cause deliver
-    argument to be rewritten. */
+    /* Real case: cause deliver argument to be rewritten. */
 
     else
       {
-      BOOL pfr = FALSE;
-
-      switch(commands->command)
-        {
-        case deliver_command:
-        DEBUG(10) debug_printf("Filter: %sdeliver message to: %s%s\n",
-          (commands->seen)? "" : "unseen ", s,
-          commands->noerror? " (noerror)" : "");
-        break;
-
-        case save_command:
-        DEBUG(10) debug_printf("Filter: %ssave message to: %s%s\n",
-          (commands->seen)? "" : "unseen ", s,
-          commands->noerror? " (noerror)" : "");
-        if (s[0] != '/') s = string_sprintf("%s/%s", deliver_home, s);
-        pfr = TRUE;
-        break;
-        }
+      DEBUG(10) debug_printf("Filter: %sdeliver message to: %s%s%s%s\n",
+        (commands->seen)? "" : "unseen ",
+        expargs[0],
+        commands->noerror? " (noerror)" : "",
+        (s != NULL)? " errors_to " : "",
+        (s != NULL)? s : "");
 
       /* Create the new address and add it to the chain, setting the
-      ignore_error flag if necessary, the mode value for save, and an
-      appropriate pfr flag. Ensure that the address is placed in the
+      ignore_error flag if necessary, and the errors address, which can be
+      set in a system filter. Ensure that the address is placed in the
       MAIN storage pool. */
 
+      store_pool = POOL_MAIN;
+      addr = deliver_make_addr(expargs[0], TRUE);  /* TRUE => copy s */
+      addr->errors_address = expargs[1];           /* Default is NULL */
+      addr->ignore_error = commands->noerror;
+      addr->next = *generated;
+      *generated = addr;
+      store_pool = pool_reset;
+      }
+    break;
+
+    case save_command:
+    if (commands->seen) *delivered = TRUE;
+    s = expargs[0];
+    mode = (int)(commands->args[1]);
+
+    /* Test case: report what would happen */
+
+    if (filter_test != NULL)
+      {
+      indent();
+      if (mode < 0)
+        printf("%save message to: %s%s\n", (commands->seen)?
+          "S" : "Unseen s", s, commands->noerror? " (noerror)" : "");
+      else
+        printf("%save message to: %s %04o%s\n", (commands->seen)?
+          "S" : "Unseen s", s, mode, commands->noerror? " (noerror)" : "");
+      }
+
+    /* Real case: Ensure save command starts with /. */
+
+    else
+      {
+      DEBUG(10) debug_printf("Filter: %ssave message to: %s%s\n",
+        (commands->seen)? "" : "unseen ", s,
+        commands->noerror? " (noerror)" : "");
+      if (s[0] != '/') s = string_sprintf("%s/%s", deliver_home, s);
+
+      /* Create the new address and add it to the chain, setting the
+      pfr flag, the ignore_error flag if necessary, and the mode value.
+      Ensure that the address is placed in the MAIN storage pool. */
+
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(s, TRUE);  /* TRUE => copy s */
-      addr->pfr = pfr;
+      addr->pfr = TRUE;
       addr->mode = mode;
       addr->ignore_error = commands->noerror;
       addr->next = *generated;
@@ -1871,13 +1951,7 @@ while (commands != NULL)
       close(log_fd);
       log_fd = -1;
       }
-    log_filename = expand_string((char *)(commands->args[0]));
-    if (log_filename == NULL)
-      {
-      *error_pointer = string_sprintf("failed to expand \"%s\": %s",
-        (char *)(commands->args[0]), expand_string_message);
-      return FALSE;
-      }
+    log_filename = expargs[0];
     if (filter_test != NULL)
       {
       indent();
@@ -1887,13 +1961,7 @@ while (commands != NULL)
 
     case logwrite_command:
     if (commands->seen) *delivered = TRUE;
-    s = expand_string((char *)(commands->args[0]));
-    if (s == NULL)
-      {
-      *error_pointer = string_sprintf("failed to expand \"%s\": %s",
-        (char *)(commands->args[0]), expand_string_message);
-      return FALSE;
-      }
+    s = expargs[0];
 
     if (filter_test != NULL)
       {
@@ -1907,27 +1975,27 @@ while (commands != NULL)
     uid. This will be the case for normal user filter files unless the security
     option set to 2 and seteuid is not set for the forwardfile director.
 
-    The log_control values are:
+    The log_access values are:
       0     OK
       1     skip (verifying)
       2     locked out by configuration
       3     not running as user
     */
 
-    else if (log_control >= 2)   /* Locked out */
+    else if (log_access >= 2)   /* Locked out */
       {
       DEBUG(10)
-        debug_printf("filter log command aborted: euid=%d log_control=%d\n",
-        (int)geteuid(), log_control);
-      *error_pointer = (log_control == 2)? "logwrite command forbidden" :
+        debug_printf("filter log command aborted: euid=%ld log_access=%d\n",
+        (long int)geteuid(), log_access);
+      *error_pointer = (log_access == 2)? "logwrite command forbidden" :
         "logwrite command not permitted when filter not running as user";
       return FALSE;
       }
-    else if (log_control == 0)   /* Real delivery */
+    else if (log_access == 0)   /* Real delivery */
       {
       int len;
-      DEBUG(10) debug_printf("writing filter log as euid %d\n",
-        (int)geteuid());
+      DEBUG(10) debug_printf("writing filter log as euid %ld\n",
+        (long int)geteuid());
       if (log_fd < 0)
         {
         if (log_filename == NULL)
@@ -1952,6 +2020,10 @@ while (commands != NULL)
         return FALSE;
         }
       }
+    else
+      {
+      DEBUG(10) debug_printf("skipping logwrite (verifying or testing)\n");
+      }
     break;
 
     /* Headers is available only in the system filter. The command is
@@ -1960,15 +2032,7 @@ while (commands != NULL)
     case headers_command:
       {
       BOOL adding = (BOOL)(commands->args[1]);
-      char *s = expand_string((char *)(commands->args[0]));
-
-      if (s == NULL)
-        {
-        if (expand_string_forcedfail) break;
-        *error_pointer = string_sprintf("failed to expand \"%s\": %s",
-          (char *)(commands->args[0]), expand_string_message);
-        return FALSE;
-        }
+      s = expargs[0];
 
       if (filter_test != NULL)
         printf("Headers %s \"%s\"\n", adding? "add" : "remove",
@@ -2016,17 +2080,8 @@ while (commands != NULL)
 
     case fail_command:
     case freeze_command:
-    if ((char *)(commands->args[0]) != NULL)
-      {
-      filter_fmsg = expand_string((char *)(commands->args[0]));
-      if (filter_fmsg == NULL)
-        {
-        *error_pointer = string_sprintf("failed to expand \"%s\": %s",
-          (char *)(commands->args[0]), expand_string_message);
-        return FALSE;
-        }
-      }
-    else filter_fmsg = (filter_test == NULL)? NULL : "";
+    filter_fmsg = expargs[0];
+    if (filter_fmsg == NULL && filter_test != NULL) filter_fmsg = "";
 
     if (commands->command == fail_command)
       {
@@ -2063,14 +2118,22 @@ while (commands != NULL)
     return FALSE;
 
     case if_command:
-    condition_value =
-      test_condition((condition_block *)(commands->args[0]), delivered, TRUE);
-    if (*error_pointer != NULL) return FALSE;
-    output_indent += 2;
-    if (!interpret_commands((filter_cmd *)
-      (commands->args[condition_value? 1:2]),
-        generated, delivered, rewrite)) return FALSE;
-    output_indent -= 2;
+      {
+      char *save_address = filter_thisaddress;
+      BOOL ok = TRUE;
+      condition_value =
+        test_condition((condition_block *)(commands->args[0]), delivered, TRUE);
+      if (*error_pointer != NULL) ok = FALSE; else
+        {
+        output_indent += 2;
+        ok = interpret_commands((filter_cmd *)
+          (commands->args[condition_value? 1:2]),
+            generated, delivered, rewrite, log_access);
+        output_indent -= 2;
+        }
+      filter_thisaddress = save_address;
+      if (!ok) return FALSE;
+      }
     break;
 
 
@@ -2090,32 +2153,20 @@ while (commands != NULL)
       break;
       }
 
-    /* Expand all the string arguments of the command */
+    /* Check the contents of the strings. The type of string can be deduced
+    from the value of i. If it is > mailarg_index_text, we are dealing with a
+    file name, which cannot contain non-printing characters. If it is less
+    than mailarg_index_text we are dealing with something that will go in a
+    message header line, where newlines must be followed by white space. If
+    i is equal to mailarg_index_text its a text string for the body, where
+    anything goes. */
 
     for (i = 0; i < mailargs_string_count; i++)
       {
-      char *s = (char *)(commands->args[i]);
       char *p;
+      char *s = expargs[i];
 
       if (s == NULL) continue;
-      s = expand_string((char *)(commands->args[i]));
-
-      if (s == NULL)
-        {
-        *error_pointer = string_sprintf("failed to expand \"%s\" in "
-          "%s command: %s", (char *)(commands->args[i]),
-          command_list[commands->command],
-          expand_string_message);
-        return FALSE;
-        }
-
-      /* Check the contents of the strings. The type of string can be deduced
-      from the value of i. If it is > mailarg_index_text, we are dealing with a
-      file name, which cannot contain non-printing characters. If it is less
-      than mailarg_index_text we are dealing with something that will go in a
-      message header line, where newlines must be followed by white space. If
-      i is equal to mailarg_index_text its a text string for the body, where
-      anything goes. */
 
       for (p = s; *p != 0; p++)
         {
@@ -2211,7 +2262,21 @@ while (commands != NULL)
         commands->args[mailarg_index_expand] != NULL;
       addr->reply->return_message =
         commands->args[mailarg_index_return] != NULL;
-      for (i = 1; i < mailargs_string_count; i++)
+
+      if (commands->args[mailarg_index_once_repeat] != NULL)
+        {
+        addr->reply->once_repeat =
+          readconf_readtime(commands->args[mailarg_index_once_repeat], 0);
+        if (addr->reply->once_repeat < 0)
+          {
+          *error_pointer = string_sprintf("Bad time value for \"once_repeat\" "
+            "in mail or vacation command: %s",
+            commands->args[mailarg_index_once_repeat]);
+          return FALSE;
+          }
+        }
+
+      for (i = 1; i < mailargs_string_passed; i++)
         {
         char *ss = (char *)(commands->args[i]);
         *((char **)(((char *)(addr->reply)) + reply_offsets[i])) =
@@ -2224,19 +2289,11 @@ while (commands != NULL)
     case testprint_command:
     if (filter_test != NULL || debug_level >= 10)
       {
-      char *s = expand_string((char *)(commands->args[0]));
-      if (s == NULL)
-        {
-        *error_pointer =
-          string_sprintf("Failed to expand \"%s\" in testprint command: %s\n",
-            (char *)(commands->args[0]), expand_string_message);
-        return FALSE;
-        }
-
+      char *s = string_printing(expargs[0]);
       if (filter_test == NULL)
-        debug_printf("Filter: testprint: %s\n", string_printing(s));
+        debug_printf("Filter: testprint: %s\n", s);
       else
-        printf("Testprint: %s\n", string_printing(s));
+        printf("Testprint: %s\n", s);
       }
     }
 
@@ -2260,7 +2317,7 @@ Arguments:
   action      if non-null, allow special actions and return here
   error       where to pass back an error text
   log_access  0 => OK to obey the "log" command
-              1 => just skip it (verifying)
+              1 => just skip it (verifying or testing)
               2 => locked out by configuration
               3 => not running as user
   rewrite     if TRUE, newly generated addresses are rewritten
@@ -2295,7 +2352,6 @@ the log file variables. */
 
 if (action != NULL) *action = SPECIAL_NONE;
 special_action = action;
-log_control = (address_test_mode || verify_only)? 1 : log_access;
 error_pointer = error;
 *error_pointer = NULL;
 line_number = 1;
@@ -2311,12 +2367,20 @@ seen_force = FALSE;
 ptr = nextsigchar(ptr, TRUE);
 if (read_command_list(&ptr, &lastcmdptr, FALSE))
   {
-  interpret_commands(commands, generated, delivered, rewrite);
+  interpret_commands(commands, generated, delivered, rewrite, log_access);
   if (*error_pointer == NULL)
     {
     if (filter_test != NULL)
-      printf("At the end of filtering \"delivered\" is %s\n",
-        (*delivered)? "true" : "false");
+      {
+      printf("Filter processing ended:\n");
+      if (*delivered)
+        printf("  Filtering set up at least one significant delivery "
+               "or other action.\n"
+               "  No other deliveries will occur.\n");
+      else
+        printf("  Filtering did not set up a significant delivery.\n"
+               "  Normal delivery will occur.\n");
+      }
     else DEBUG(10)
       debug_printf("Filter: at end \"delivered\" is %s\n",
         (*delivered)? "true" : "false");
@@ -2380,8 +2444,7 @@ if (message_filter_uid_set)
   mac_seteuid(message_filter_uid);
   }
 
-DEBUG(9) debug_printf("running system filter as uid=%d gid=%d\n",
-  (int)geteuid(), (int)getegid());
+DEBUG(9) debug_print_ids("running system filter as");
 
 /* Now open the filter file and read it into memory. */
 
@@ -2431,8 +2494,7 @@ if (message_filter_uid_set)
   mac_seteuid(root_uid);
   mac_setegid(saved_egid);
   mac_seteuid(saved_euid);
-  DEBUG(9) debug_printf("after system filter restored uid=%d gid=%d\n",
-    (int)geteuid(), (int)getegid());
+  DEBUG(9) debug_print_ids("after system filter restored");
   }
 
 return yield;
@@ -2544,9 +2606,12 @@ for (; *tag != 0; s++, tag++)
 we might as well set up the message_body variable at the same time (when
 *really* filtering this is not read unless needed). The reading code is written
 out here rather than having options in read_message_data, in order to keep that
-function as efficient as possible. */
+function as efficient as possible. Handling message_body_end is somewhat more
+tedious. Pile it all into a circular buffer and sort out at the end. */
 
-message_body = body = store_get(message_body_visible + 1);
+message_body = body = store_malloc(message_body_visible + 1);
+message_body_end = store_malloc(message_body_visible + 1);
+s = message_body_end;
 body_len = 0;
 
 if (!dot_ended && !feof(stdin))
@@ -2556,6 +2621,9 @@ if (!dot_ended && !feof(stdin))
     while ((ch = getc(stdin)) != EOF)
       {
       if (body_len < message_body_visible) message_body[body_len++] = ch;
+      *s++ = ch;
+      if (s > message_body_end + message_body_visible)
+        s = message_body_end;
       message_size++;
       }
     }
@@ -2582,11 +2650,17 @@ if (!dot_ended && !feof(stdin))
         case 2:                         /* After "\n." */
         if (ch == '\n') goto READ_END;
         if (body_len < message_body_visible) message_body[body_len++] = '.';
+        *s++ = '.';
+        if (s > message_body_end + message_body_visible)
+          s = message_body_end;
         message_size++;
         ch_state = 0;
         break;
         }
       if (body_len < message_body_visible) message_body[body_len++] = ch;
+      *s++ = ch;
+      if (s > message_body_end + message_body_visible)
+        s = message_body_end;
       message_size++;
       }
     READ_END: ch = ch;  /* Some compilers don't like null statements */
@@ -2595,7 +2669,25 @@ if (!dot_ended && !feof(stdin))
 
 message_body[body_len] = 0;
 
-/* Convert newlines in the body to spaces */
+/* body_len stops at message_body_visible; it if got there, we may have
+wrapped round in message_body_end. */
+
+if (body_len >= message_body_visible)
+  {
+  int below = s - message_body_end;
+  int above = message_body_visible - below;
+  if (above > 0)
+    {
+    char *temp = store_get(below);
+    memcpy(temp, message_body_end, below);
+    memmove(message_body_end, s+1, above);
+    memcpy(message_body_end + above, temp, below);
+    message_body_end[message_body_visible] = 0;
+    }
+  }
+else *s = 0;
+
+/* Convert newlines in the body variables to spaces */
 
 while (*body != 0)
   {
@@ -2603,6 +2695,15 @@ while (*body != 0)
   body++;
   }
 
+body = message_body_end;
+while (*body != 0)
+  {
+  if (*body == '\n') *body = ' ';
+  body++;
+  }
+
+
+
 /* Now pass the filter file to the function that interprets it. Because
 filter_test is not NULL, the interpreter will output comments about what
 it is doing, but an error message will have to be output here. No need to
