@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one module, so that
@@ -20,15 +20,58 @@ Warning: Linking some objects which contain exception information sections
 
 As this may cause people to worry needlessly, include a dummy function here
 to stop the message from appearing. Make it reference itself to stop picky
-compilers complaining that it is unused. */
+compilers complaining that it is unused, and put in a dummy argument to stop
+even pickier compilers complaining about infinite loops. */
 
-static void dummy(void) { dummy(); }
+static void dummy(int x) { dummy(x-1); }
 
 
 /* For comments on these variables, see globals.h. I'm too idle to
 duplicate them here... */
 
+#ifdef EXIM_PERL
+char  *opt_perl_startup       = NULL;
+BOOL   opt_perl_at_start      = FALSE;
+BOOL   opt_perl_started       = FALSE;
+#endif
+
+#ifdef HAVE_AUTH
+char  *auth_hosts             = NULL;
+char  *host_auth_accept_relay = NULL;
+pcre  *regex_AUTH             = NULL;
+BOOL   smtp_authenticated     = FALSE;
+
+/* Generic options for auths, all of which live inside auth_instance
+data blocks and hence have the opt_public flag set. */
+
+optionlist optionlist_auths[] = {
+  { "driver",        opt_stringptr | opt_public,
+                 (void *)(offsetof(auth_instance, driver_name)) },
+  { "public_name",   opt_stringptr | opt_public,
+                 (void *)(offsetof(auth_instance, public_name)) },
+  { "server_set_id", opt_stringptr | opt_public,
+                 (void *)(offsetof(auth_instance, set_id)) }
+};
+
+int optionlist_auths_size = sizeof(optionlist_auths)/sizeof(optionlist);
+#endif
+
+#ifdef LOOKUP_LDAP
+char  *ldap_default_servers   = NULL;
+#endif
+
+#ifdef LOOKUP_MYSQL
+char  *mysql_servers          = NULL;
+#endif
+
+#ifdef SUPPORT_MOVE_FROZEN_MESSAGES
+BOOL   move_frozen_messages   = FALSE;
+#endif
+
+
 #ifdef SUPPORT_DSN
+/* This partial attempt at doing DSN was abandoned. This obsolete code is
+left here just in case. Nothing is documented. */
 BOOL   dsn                    = FALSE;
 char  *dsn_envid              = NULL;
 int    dsn_ret                = 0;
@@ -36,20 +79,48 @@ pcre  *regex_DSN              = NULL;
 BOOL   smtp_use_dsn           = FALSE;
 #endif
 
+
+/* Input-reading functions for messages, so we can use special ones for
+incoming TCP/IP. */
+
+int (*accept_getc)(FILE *)    = getc;
+int (*accept_ungetc)(int, FILE *) = ungetc;
+int (*accept_feof)(FILE *)    = feof;
+int (*accept_ferror)(FILE *)  = ferror;
+
+/* General global variables */
+
 BOOL   accept_8bitmime        = FALSE;
 BOOL   accept_call_bombout    = FALSE;
+int    accept_linecount       = 0;
+int    accept_messagecount    = 0;
 int    accept_timeout         = 0;
 address_item *addr_duplicate  = NULL;
-char  *address_directory_transport = "address_directory";
-char  *address_directory2_transport = "address_directory2";
-char  *address_file_transport = "address_file";
-char  *address_pipe_transport = "address_pipe";
-char  *address_reply_transport = "address_reply";
+char  *address_file           = NULL;
+char  *address_pipe           = NULL;
 BOOL   address_test_mode      = FALSE;
+gid_t *admin_groups           = NULL;
 BOOL   admin_user             = FALSE;
 BOOL   allow_unqualified_recipient = TRUE;    /* For local messages */
 BOOL   allow_unqualified_sender = TRUE;       /* Reset for SMTP */
 BOOL   always_bcc             = FALSE;
+char  *authenticated_id       = NULL;
+char  *authenticated_sender   = NULL;
+auth_instance *auths          = NULL;
+
+auth_instance auth_defaults   = {
+   NULL,                      /* chain pointer */
+   NULL,                      /* name */
+   NULL,                      /* info */
+   NULL,                      /* private options block pointer */
+   NULL,                      /* driver_name */
+   NULL,                      /* public_name */
+   NULL,                      /* set_id */
+   FALSE,                     /* client */
+   FALSE                      /* server */
+};
+
+char  *auth_defer_msg         = "reason not recorded";
 int    auto_thaw              = 0;
 
 int    batch_max              = -1;
@@ -57,16 +128,16 @@ char  *bi_command             = NULL;
 char  *big_buffer             = NULL;
 int    big_buffer_size        = BIG_BUFFER_SIZE;
 int    body_linecount         = 0;
+int    bsmtp_transaction_linecount = 0;
 
 BOOL   check_dns_names        = TRUE;
-BOOL   check_relay            = FALSE;
+char  *check_dns_names_pattern= "(?i)^(?>(?(1)\\.|())[^\\W_](?>[a-z0-9-]*[^\\W_])?)+$";
 int    check_log_inodes       = 0;
 int    check_log_space        = 0;
 int    check_spool_inodes     = 0;
 int    check_spool_space      = 0;
 int    clmacro_count          = 0;
 char  *clmacros[MAX_CLMACROS];
-BOOL   collapse_source_routes = FALSE;
 BOOL   config_changed         = FALSE;
 FILE  *config_file            = NULL;
 char  *config_filename        = CONFIGURE_FILE
@@ -78,21 +149,19 @@ int    continue_sequence      = 1;
 char  *continue_transport     = NULL;
 
 BOOL   daemon_listen          = FALSE;
-off_t  data_start_offset      = MESSAGE_ID_LENGTH + 3;
 FILE  *debug_file             = NULL;
 int    debug_level            = -1;
 pid_t  debug_pid              = 0;
-BOOL   debug_trace_memory     = FALSE;
-transport_info *debug_transport = NULL;
-char  *debug_transport_file   = NULL;
+int    debug_trace_memory     = 0;
 int    delay_warning[DELAY_WARNING_SIZE] = { DELAY_WARNING_SIZE, 1, 24*60*60 };
-char  *delay_warning_condition= NULL;
+char  *delay_warning_condition= "${if match{$h_precedence:}{(?i)bulk|list|junk}{no}{yes}}";
 BOOL   delivery_date_remove   = TRUE;
 char  *deliver_in_buffer      = NULL;
 char  *deliver_out_buffer     = NULL;
 int    deliver_datafile       = -1;
 char  *deliver_domain         = NULL;
 char  *deliver_domain_orig    = NULL;
+char  *deliver_domain_parent  = NULL;
 BOOL   deliver_firsttime      = FALSE;
 BOOL   deliver_force          = FALSE;
 BOOL   deliver_freeze         = FALSE;
@@ -103,12 +172,17 @@ char  *deliver_host_address   = NULL;
 int    deliver_load_max       = -1;
 char  *deliver_localpart      = NULL;
 char  *deliver_localpart_orig = NULL;
+char  *deliver_localpart_parent = NULL;
 char  *deliver_localpart_prefix = NULL;
 char  *deliver_localpart_suffix = NULL;
+BOOL   deliver_force_thaw     = FALSE;
 BOOL   deliver_manual_thaw    = FALSE;
 int    deliver_queue_load_max = -1;
 address_item *deliver_recipients = NULL;
 char  *deliver_selectstring   = NULL;
+BOOL   deliver_selectstring_regex = FALSE;
+char  *deliver_selectstring_sender = NULL;
+BOOL   deliver_selectstring_sender_regex = FALSE;
 director_instance *directors  = NULL;
 
 director_instance director_defaults = {
@@ -118,25 +192,17 @@ director_instance director_defaults = {
    NULL,                      /* private options block pointer */
    NULL,                      /* driver name */
    NULL,                      /* domains */
-   NULL,                      /* except_domains */
    NULL,                      /* local_parts */
-   NULL,                      /* except_local_parts */
    NULL,                      /* errors_to */
    NULL,                      /* extra_headers */
    NULL,                      /* remove_headers */
    NULL,                      /* debug_string */
    NULL,                      /* fallback_hosts */
    NULL,                      /* fallback_hostlist */
-   NULL,                      /* re_domains */
-   NULL,                      /* re_except_domains */
-   NULL,                      /* re_local_parts */
-   NULL,                      /* re_except_local_parts */
    TRUE,                      /* more */
    FALSE,                     /* verify_only */
    TRUE,                      /* verify_sender */
    TRUE,                      /* verify_recipient */
-   NULL,                      /* transport */
-   NULL,                      /* expanded_transport */
    NULL,                      /* expand_uid */
    NULL,                      /* expand_gid */
    (uid_t)(-1),               /* uid */
@@ -144,15 +210,16 @@ director_instance director_defaults = {
    FALSE,                     /* uid_set */
    FALSE,                     /* gid_set */
    FALSE,                     /* initgroups */
+   NULL,                      /* transport */
+   NULL,                      /* expanded_transport */
+   NULL,                      /* current_directory */
+   NULL,                      /* home_directory */
    NULL,                      /* condition */
    NULL,                      /* prefix */
    NULL,                      /* suffix */
    NULL,                      /* senders */
-   NULL,                      /* except_senders */
    NULL,                      /* require_files */
    NULL,                      /* new_director */
-   NULL,                      /* re_senders */
-   NULL,                      /* re_except_senders */
    NULL,                      /* new (director) */
    FALSE,                     /* unseen */
    FALSE,                     /* prefix_optional */
@@ -201,6 +268,9 @@ uid_t  exim_uid               = 0;
 BOOL   exim_uid_set           = FALSE;
 #endif
 
+BOOL   expand_forbid_exists   = FALSE;
+BOOL   expand_forbid_lookup   = FALSE;
+BOOL   expand_forbid_perl     = FALSE;
 int    expand_nlength[EXPAND_MAXN+1];
 int    expand_nmax            = -1;
 char  *expand_nstring[EXPAND_MAXN+1];
@@ -256,27 +326,27 @@ BOOL   headers_checks_fail    = TRUE;
 BOOL   headers_sender_verify  = FALSE;
 BOOL   headers_sender_verify_errmsg = FALSE;
 char  *helo_accept_junk_hosts = NULL;
-host_item *helo_accept_junk_hostlist = NULL;
-char  *helo_accept_junk_nets  = NULL;
-ip_net_item *helo_accept_junk_netlist = NULL;
 BOOL   helo_strict_syntax     = FALSE;
-BOOL   helo_verify            = FALSE;
-char  *helo_verify_except_hosts = NULL;
-host_item *helo_verify_except_hostlist = NULL;
-char  *helo_verify_except_nets = NULL;
-ip_net_item *helo_verify_except_netlist = NULL;
+char  *helo_verify            = NULL;
 char  *hold_domains           = NULL;
-char  *hold_domains_except    = NULL;
+char  *host_accept_relay      = NULL;
 BOOL   host_checking          = FALSE;
-char  *host_findbyaddr_msg    = "";
-char  *host_lookup_nets       = NULL;
-ip_net_item *host_lookup_netlist = NULL;
+BOOL   host_find_failed_syntax= FALSE;
+char  *host_lookup            = NULL;
+BOOL   host_lookup_failed     = FALSE;
+char  *host_lookup_msg        = "";
 int    host_number            = 0;
 char  *host_number_string     = NULL;
+char  *host_reject            = NULL;
+char  *host_reject_recipients = NULL;
 char  *hosts_treat_as_local   = NULL;
 
 BOOL   ignore_errmsg_errors   = FALSE;
 int    ignore_errmsg_errors_after = 0;
+BOOL   ignore_fromline_local  = FALSE;
+char  *ignore_fromline_hosts  = NULL;
+char  *interface_address      = NULL;
+
 int    journal_fd             = -1;
 
 int    keep_malformed         = 4*24*60*60;    /* 4 days */
@@ -298,18 +368,27 @@ char  *log_file_path          = LOG_FILE_PATH
 			  "\0<--------------Space to patch log_file_path->";
 BOOL   log_ip_options         = TRUE;
 int    log_level              = 5;
+int    log_queue_run_level    = 0;
 BOOL   log_received_sender    = FALSE;
 BOOL   log_received_recipients = FALSE;
 BOOL   log_refused_recipients = FALSE;
 BOOL   log_rewrites           = FALSE;
 BOOL   log_smtp_confirmation  = FALSE;
+BOOL   log_smtp_connections   = FALSE;
+BOOL   log_smtp_syntax_errors = FALSE;
 FILE  *log_stderr             = NULL;
 BOOL   log_subject            = FALSE;
 char  *lookup_key             = NULL;
+int    lookup_open_max        = 25;
+char  *lookup_value           = NULL;
 
 macro_item *macros            = NULL;
 char  **mailer_argv;
+int    max_username_length    = 0;
+int    message_age            = 0;
 char  *message_body           = NULL;
+char  *message_body_end       = NULL;
+int    message_body_size      = 0;
 int    message_body_visible   = 500;
 char  *message_filter         = NULL;
 
@@ -332,6 +411,7 @@ FILE  *message_log            = NULL;
 char  *message_precedence     = NULL;
 int    message_size           = 0;
 int    message_size_limit     = 0;
+BOOL   message_size_limit_count_recipients = FALSE;
 char   message_subdir[2]      = "\0";
 char  *message_reference      = NULL;
 
@@ -348,7 +428,6 @@ uid_t  originator_uid;
 
 BOOL   parse_allow_group      = FALSE;
 BOOL   parse_found_group      = FALSE;
-BOOL   passed_with_pending    = FALSE;
 char  *percent_hack_domains   = NULL;
 char  *pid_file_path          = PID_FILE_PATH
 			  "\0<--------------Space to patch pid_file_path->";
@@ -364,50 +443,35 @@ char  *prohibition_reason     = NULL;
 char  *qualify_domain_recipient = NULL;
 char  *qualify_domain_sender  = NULL;
 BOOL   queue_2stage           = FALSE;
+int    queue_interval         = -1;
 BOOL   queue_list_requires_admin = TRUE;
+BOOL   queue_only             = FALSE;
+char  *queue_only_file        = NULL;
+int    queue_only_load        = -1;
+BOOL   queue_remote           = FALSE;
+char  *queue_remote_domains   = NULL;
 BOOL   queue_run_force        = FALSE;
 BOOL   queue_run_in_order     = FALSE;
 BOOL   queue_run_local        = FALSE;
-BOOL   queue_running          = FALSE;
-int    queue_interval         = -1;
-BOOL   queue_only             = FALSE;
-int    queue_only_load        = -1;
-BOOL   queue_remote           = FALSE;
-char  *queue_remote_except    = NULL;
 int    queue_run_max          = 5;
+pid_t  queue_run_pid          = (pid_t)0;
+int    queue_run_pipe         = -1;
+BOOL   queue_running          = FALSE;
 BOOL   queue_smtp             = FALSE;
-char  *queue_smtp_except      = NULL;
+char  *queue_smtp_domains     = NULL;
 
 char  *rbl_domains            = NULL;
-char  *rbl_except_nets        = NULL;
-ip_net_item *rbl_except_netlist = NULL;
+char  *rbl_hosts              = "*";
 header_line *rbl_header       = NULL;
+BOOL   rbl_host               = FALSE;
+BOOL   rbl_log_rcpt_count     = FALSE;
+BOOL   rbl_log_headers        = FALSE;
+char  *rbl_msg_buffer         = NULL;
 BOOL   rbl_reject_recipients  = TRUE;
 BOOL   rbl_warn_header        = TRUE;
 char  *raw_sender             = NULL;
 char **raw_recipients         = NULL;
 
-re_block *re_dns_again_means_nonexist = NULL;
-re_block *re_hold_domains     = NULL;
-re_block *re_hold_domains_except = NULL;
-re_block *re_hosts_treat_as_local = NULL;
-re_block *re_local_domains    = NULL;
-re_block *re_percent_hack_domains = NULL;
-re_block *re_queue_remote_except = NULL;
-re_block *re_queue_smtp_except = NULL;
-re_block *re_receiver_verify_addresses = NULL;
-re_block *re_receiver_verify_senders = NULL;
-re_block *re_receiver_verify_senders_except = NULL;
-re_block *re_recipients_reject_except = NULL;
-re_block *re_relay_domains    = NULL;
-re_block *re_remote_sort      = NULL;
-re_block *re_sender_accept    = NULL;
-re_block *re_sender_accept_recipients = NULL;
-re_block *re_sender_address_relay = NULL;
-re_block *re_sender_reject    = NULL;
-re_block *re_sender_reject_except = NULL;
-re_block *re_sender_reject_recipients = NULL;
-
 gid_t  real_gid;
 uid_t  real_uid;
 char  *real_sender_address;
@@ -435,26 +499,21 @@ char  *received_protocol      = NULL;
 int    received_time          = 0;
 BOOL   receiver_try_verify    = FALSE;
 char  *receiver_unqualified_hosts = NULL;
-host_item *receiver_unqualified_hostlist = NULL;
-char  *receiver_unqualified_nets = NULL;
-ip_net_item *receiver_unqualified_netlist = NULL;
 BOOL   receiver_verify        = FALSE;
 char  *receiver_verify_addresses = NULL;
-char  *receiver_verify_except_hosts = NULL;
-host_item *receiver_verify_except_hostlist = NULL;
-char  *receiver_verify_except_nets = NULL;
-ip_net_item *receiver_verify_except_netlist = NULL;
+char  *receiver_verify_hosts  = "*";
 char  *receiver_verify_senders = NULL;
-char  *receiver_verify_senders_except = NULL;
 int    recipients_count       = 0;
 recipient_item *recipients_list = NULL;
 int    recipients_list_max    = 0;
 int    recipients_max         = 0;
 BOOL   recipients_max_reject  = FALSE;
 char  *recipients_reject_except = NULL;
+char  *recipients_reject_except_senders = NULL;
 BOOL   refuse_all_rcpts       = FALSE;
 BOOL   refuse_ip_options      = TRUE;
 char **regcomp_error_pointer  = NULL;
+pcre  *regex_check_dns_names  = NULL;
 pcre  *regex_From             = NULL;
 pcre  *regex_SIZE             = NULL;
 pcre  *regex_ismsgid          = NULL;
@@ -463,6 +522,7 @@ BOOL   relay_domains_include_local_mx = FALSE;
 BOOL   relay_need_either      = FALSE;
 int    remote_max_parallel    = 1;
 char  *remote_sort            = NULL;
+int    retry_data_expire      = 7*24*60*60;
 int    retry_interval_max     = 24*60*60;
 int    retry_maximum_timeout  = 0;        /* set from retry config */
 retry_config *retries         = NULL;
@@ -471,10 +531,7 @@ BOOL   return_path_remove     = TRUE;
 int    return_size_limit      = 100*1024;
 int    rewrite_existflags     = 0;
 rewrite_rule *rewrite_rules   = NULL;
-char  *rfc1413_except_hosts   = NULL;
-host_item *rfc1413_except_hostlist = NULL;
-char  *rfc1413_except_nets    = NULL;
-ip_net_item *rfc1413_except_netlist = NULL;
+char  *rfc1413_hosts          = "*";
 int    rfc1413_query_timeout  = 30;
 BOOL   rfc821_domains         = FALSE;
 uid_t  root_uid               = ROOT_UID;
@@ -488,25 +545,17 @@ router_instance router_defaults = {
    NULL,                      /* private options block pointer */
    NULL,                      /* driver name */
    NULL,                      /* domains */
-   NULL,                      /* except_domains */
    NULL,                      /* local_parts */
-   NULL,                      /* except_local_parts */
    NULL,                      /* errors_to */
    NULL,                      /* extra_headers */
    NULL,                      /* remove_headers */
    NULL,                      /* debug_string */
    NULL,                      /* fallback_hosts */
    NULL,                      /* fallback_hostlist */
-   NULL,                      /* re_domains */
-   NULL,                      /* re_except_domains */
-   NULL,                      /* re_local_parts */
-   NULL,                      /* re_except_local_parts */
    TRUE,                      /* more */
    FALSE,                     /* verify_only */
    TRUE,                      /* verify_sender */
    TRUE,                      /* verify_recipient */
-   NULL,                      /* transport instance */
-   NULL,                      /* expanded_transport */
    NULL,                      /* expand_uid */
    NULL,                      /* expand_gid */
    (uid_t)(-1),               /* uid */
@@ -514,90 +563,59 @@ router_instance router_defaults = {
    FALSE,                     /* uid_set */
    FALSE,                     /* gid_set */
    FALSE,                     /* initgroups */
+   NULL,                      /* transport instance */
+   NULL,                      /* expanded_transport */
    NULL,                      /* condition */
    NULL,                      /* senders */
-   NULL,                      /* except_senders */
    NULL,                      /* require_files */
-   NULL,                      /* re_senders */
-   NULL,                      /* re_except_senders */
    FALSE,                     /* unseen */
    FALSE,                     /* fail_verify_sender */
    FALSE,                     /* fail_verify_recipient */
    FALSE,                     /* pass_on_timeout */
    "freeze",                  /* self */
    self_freeze,               /* self_code */
-   FALSE                      /* self_rewrite */
+   FALSE,                     /* self_rewrite */
+   TRUE,                      /* sender_rewrite */
+   NULL                       /* translate_ip_address */
 };
 
+char  *running_status         = ">>>running<<<";
+
 BOOL   search_find_defer      = FALSE;
 int    security_level         = 0;
 char  *security_type          = NULL;
 char  *self_hostname          = NULL;
-char  *sender_accept          = NULL;
-char  *sender_accept_recipients = NULL;
 char  *sender_address         = NULL;
 BOOL   sender_address_forced  = FALSE;
 char  *sender_address_relay   = NULL;
 BOOL   sender_address_rewritten = FALSE;
 char  *sender_fullhost        = NULL;
 char  *sender_helo_name       = NULL;
-char  *sender_host_accept     = NULL;
-host_item *sender_host_accept_hosts = NULL;
-char  *sender_host_accept_relay = NULL;
-host_item *sender_host_accept_relay_hosts = NULL;
+char **sender_host_aliases    = NULL;
 char  *sender_host_address    = NULL;
+char  *sender_host_authenticated = NULL;
 char  *sender_host_name       = NULL;
 BOOL   sender_host_notsocket  = FALSE;
-char  *sender_host_reject     = NULL;
-char  *sender_host_reject_except = NULL;
-host_item *sender_host_reject_hosts = NULL;
-host_item *sender_host_reject_except_hosts = NULL;
-char  *sender_host_reject_recipients = NULL;
-host_item *sender_host_reject_recipients_hosts = NULL;
-char  *sender_host_reject_relay = NULL;
-host_item *sender_host_reject_relay_hosts = NULL;
-char  *sender_host_reject_relay_except = NULL;
-host_item *sender_host_reject_relay_except_hosts = NULL;
 BOOL   sender_host_unknown    = FALSE;
 char  *sender_ident           = NULL;
 BOOL   sender_local           = FALSE;
-char  *sender_net_accept      = NULL;
-ip_net_item *sender_net_accept_nets = NULL;
-char  *sender_net_accept_relay = NULL;
-ip_net_item *sender_net_accept_relay_nets = NULL;
-char  *sender_net_reject      = NULL;
-char  *sender_net_reject_except = NULL;
-ip_net_item *sender_net_reject_nets = NULL;
-ip_net_item *sender_net_reject_except_nets = NULL;
-char  *sender_net_reject_recipients = NULL;
-ip_net_item *sender_net_reject_recipients_nets = NULL;
-char  *sender_net_reject_relay = NULL;
-ip_net_item *sender_net_reject_relay_nets = NULL;
-char  *sender_net_reject_relay_except = NULL;
-ip_net_item *sender_net_reject_relay_except_nets = NULL;
 char  *sender_rcvhost         = NULL;
 char  *sender_reject          = NULL;
-char  *sender_reject_except   = NULL;
 char  *sender_reject_recipients = NULL;
 BOOL   sender_try_verify      = FALSE;
 char  *sender_unqualified_hosts = NULL;
-host_item *sender_unqualified_hostlist = NULL;
-char  *sender_unqualified_nets = NULL;
-ip_net_item *sender_unqualified_netlist = NULL;
 BOOL   sender_verify          = FALSE;
-BOOL   sender_verify_batch    = TRUE;
-char  *sender_verify_except_hosts = NULL;
-host_item *sender_verify_except_hostlist = NULL;
-char  *sender_verify_except_nets = NULL;
-ip_net_item *sender_verify_except_netlist = NULL;
+BOOL   sender_verify_batch    = FALSE;
 BOOL   sender_verify_fixup    = FALSE;
-BOOL   sender_verify_log_details = FALSE;
+char  *sender_verify_hosts    = "*";
 BOOL   sender_verify_reject   = TRUE;
 int    sender_verify_max_retry_rate = 12;
 volatile BOOL sigalrm_seen    = FALSE;
 char **sighup_argv            = NULL;
 int    smtp_accept_count      = 0;
+BOOL   smtp_accept_keepalive  = TRUE;
 int    smtp_accept_max        = 20;
+int    smtp_accept_max_per_host = 0;
 int    smtp_accept_queue      = 0;
 int    smtp_accept_queue_per_connection = 10;
 int    smtp_accept_reserve    = 0;
@@ -606,29 +624,21 @@ char  *smtp_banner            = "${primary_hostname} ESMTP "
                             "${tod_full}"
 			    "\0<---------------Space to patch smtp_banner->";
 BOOL   smtp_batched_input     = FALSE;
+BOOL   smtp_check_spool_space = TRUE;
 int    smtp_connect_backlog   = 5;
 FILE  *smtp_in                = NULL;
 BOOL   smtp_input             = FALSE;
 int    smtp_load_reserve      = -1;
-BOOL   smtp_log_connections   = FALSE;
 FILE  *smtp_out               = NULL;
 int    smtp_port              = -1;
 BOOL   smtp_reading_data      = FALSE;
 int    smtp_receive_timeout   = 5*60;
 char  *smtp_etrn_command      = NULL;
-host_item *smtp_etrn_hostlist = NULL;
 char  *smtp_etrn_hosts        = NULL;
-ip_net_item *smtp_etrn_netlist = NULL;
-char  *smtp_etrn_nets         = NULL;
 BOOL   smtp_etrn_serialize    = TRUE;
-host_item *smtp_expn_hostlist = NULL;
 char  *smtp_expn_hosts        = NULL;
-ip_net_item *smtp_expn_netlist = NULL;
-char  *smtp_expn_nets         = NULL;
-host_item *smtp_reserve_hostlist = NULL;
+char  *smtp_read_error        = "";
 char  *smtp_reserve_hosts     = NULL;
-ip_net_item *smtp_reserve_netlist = NULL;
-char  *smtp_reserve_nets      = NULL;
 BOOL   smtp_use_size          = FALSE;
 BOOL   smtp_verify            = FALSE;
 BOOL   split_spool_directory  = FALSE;
@@ -637,16 +647,10 @@ char  *spool_directory        = SPOOL_DIRECTORY
 char  *stderr_filename        = STDERR_FILE
                           "\0<--------------Space to patch stderr_filename->";
 
-int    store_max_malloc       = 0;
 int    store_pool             = POOL_MAIN;
 BOOL   strip_excess_angle_brackets = FALSE;
 BOOL   strip_trailing_dot     = FALSE;
 
-transport_instance *transport_address_directory = NULL;
-transport_instance *transport_address_directory2 = NULL;
-transport_instance *transport_address_file = NULL;
-transport_instance *transport_address_pipe = NULL;
-transport_instance *transport_address_reply = NULL;
 transport_instance *transports = NULL;
 
 transport_instance *transport_message_filter_directory = NULL;
@@ -674,18 +678,23 @@ transport_instance transport_defaults = {
    NULL,                     /* expand_gid */
    NULL,                     /* home_dir */
    NULL,                     /* current_dir */
+   NULL,                     /* warn_message */
    FALSE,                    /* deliver_as_creator */
    FALSE,                    /* initgroups */
    FALSE,                    /* bsmtp_helo */
    TRUE,                     /* multi-domain */
+   FALSE,                    /* overrides_hosts */
    NULL,                     /* shadow */
    NULL,                     /* shadow_condition */
    NULL,                     /* filter_command */
    NULL,                     /* add_headers */
    NULL,                     /* remove_headers */
+   NULL,                     /* return_path */
    NULL,                     /* debug_string */
+   FALSE,                    /* body_only */
    FALSE,                    /* delivery_date_add */
    FALSE,                    /* envelope_to_add */
+   FALSE,                    /* headers_only */
    FALSE,                    /* return_path_add */
    FALSE,                    /* return_output */
    FALSE,                    /* return_fail_output */
@@ -693,7 +702,8 @@ transport_instance transport_defaults = {
    FALSE,                    /* log_fail_output */
    FALSE,                    /* log_defer_output */
    2,                        /* BOOL, but set neither 1 nor 0 so can detect */
-   100                       /* max_addresses */
+   100,                      /* max_addresses */
+   0                         /* message_size_limit */
 };
 
 int    transport_count;
@@ -743,9 +753,10 @@ char  *warnmsg_delay          = NULL;
 char  *warnmsg_file           = NULL;
 char  *warnmsg_recipients     = NULL;
 
+char  *verify_address_parse_error = NULL;
 char  *verify_forced_errmsg   = NULL;
 BOOL   verify_only            = FALSE;
-char  *version_copyright      = "Copyright (c) University of Cambridge 1998";
+char  *version_copyright      = "Copyright (c) University of Cambridge 1999";
 char  *version_date           = "?";
 char  *version_cnumber        = "????";
 char  *version_string         = "?";
