@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one header, so that
@@ -13,7 +13,37 @@ operating systems. */
 extern int h_errno;
 #endif
 
+/* Now things that are present only when configured. */
+
+#ifdef EXIM_PERL
+extern char  *opt_perl_startup;       /* Startup code for Perl interpreter */
+extern BOOL   opt_perl_at_start;      /* Start Perl interpreter at start */
+extern BOOL   opt_perl_started;       /* Set once interpreter started */
+#endif
+
+#ifdef HAVE_AUTH
+extern char  *auth_hosts;             /* These must authenticate */
+extern char  *host_auth_accept_relay; /* Relay permitted if authenticated */
+extern pcre  *regex_AUTH;             /* For recognizing AUTH settings */
+extern BOOL   smtp_authenticated;     /* Sending client has authenticated */
+#endif
+
+#ifdef LOOKUP_LDAP
+extern char  *ldap_default_servers;   /* List of default servers */
+#endif
+
+#ifdef LOOKUP_MYSQL
+extern char  *mysql_servers;          /* List of servers and connect info */
+#endif
+
+#ifdef SUPPORT_MOVE_FROZEN_MESSAGES
+extern BOOL   move_frozen_messages;   /* Get them out of the normal directory */
+#endif
+
+
 #ifdef SUPPORT_DSN
+/* This partial attempt at doing DSN was abandoned. This obsolete code is
+left here just in case. Nothing is documented. */
 extern BOOL   dsn;                    /* FALSE if DSN not to be used */
 extern char  *dsn_envid;              /* DSN envid string */
 extern int    dsn_ret;                /* DSN ret type*/
@@ -21,22 +51,37 @@ extern pcre  *regex_DSN;              /* For recognizing DSN settings */
 extern BOOL   smtp_use_dsn;           /* Global for passed connections */
 #endif
 
+
+/* Input-reading functions for messages, so we can use special ones for
+incoming TCP/IP. */
+
+extern int (*accept_getc)(FILE *);
+extern int (*accept_ungetc)(int, FILE *);
+extern int (*accept_feof)(FILE *);
+extern int (*accept_ferror)(FILE *);
+
+/* General global variables */
+
 extern BOOL   accept_8bitmime;        /* Allow *BITMIME incoming */
 extern BOOL   accept_call_bombout;    /* Flag for crashing log */
+extern int    accept_linecount;       /* Mainly for BSMTP errors */
+extern int    accept_messagecount;    /* Mainly for BSMTP errors */
 extern int    accept_timeout;         /* For non-SMTP acceptance */
 extern address_item *addr_duplicate;  /* Duplicate address list */
-extern char  *address_directory_transport; /* Name of first alternative
-                                         transport for directory addresses */
-extern char  *address_directory2_transport; /* Name of second alternative
-                                         transport for directory addresses */
-extern char  *address_file_transport; /* Name of transport for file addresses */
-extern char  *address_pipe_transport; /* Name of transport for pipe addresses */
-extern char  *address_reply_transport; /* Name of transport for reply addresses */
+extern char  *address_file;           /* Name of file when delivering to one */
+extern char  *address_pipe;           /* Pipe command when delivering to one */
 extern BOOL   address_test_mode;      /* True for -bt */
+extern gid_t *admin_groups;           /* List of admin groups */
 extern BOOL   admin_user;             /* True if caller can do admin */
 extern BOOL   allow_unqualified_recipient; /* As it says */
 extern BOOL   allow_unqualified_sender; /* Ditto */
 extern BOOL   always_bcc;             /* Use bcc in all adding cases */
+extern char  *authenticated_id;       /* ID that was authenticated */
+extern char  *authenticated_sender;   /* From AUTH on MAIL */
+extern auth_info auths_available[];   /* Vector of available auth mechanisms */
+extern auth_instance *auths;          /* Chain of instantiated auths */
+extern auth_instance auth_defaults;   /* Default values */
+extern char  *auth_defer_msg;         /* Error message */
 extern int    auto_thaw;              /* Auto-thaw interval */
 
 extern int    batch_max;              /* Max SMTP batching from -oB */
@@ -44,16 +89,16 @@ extern char  *bi_command;             /* Command for -bi option */
 extern char  *big_buffer;             /* Used for various temp things */
 extern int    big_buffer_size;        /* Current size (can expand) */
 extern int    body_linecount;         /* Line count in body */
+extern int    bsmtp_transaction_linecount; /* Start of last transaction */
 
 extern BOOL   check_dns_names;        /* True for preliminary syntax check */
-extern BOOL   check_relay;            /* True if relay checking required */
+extern char  *check_dns_names_pattern;/* Regex for checking */
 extern int    check_log_inodes;       /* Minimum for message acceptance */
 extern int    check_log_space;        /* Minimum for message acceptance */
 extern int    check_spool_inodes;     /* Minimum for message acceptance */
 extern int    check_spool_space;      /* Minimum for message acceptance */
 extern int    clmacro_count;          /* Number of command line macros */
 extern char  *clmacros[];             /* Copy of them, for re-exec */
-extern BOOL   collapse_source_routes; /* True if collapsing required */
 extern BOOL   config_changed;         /* True if -C used */
 extern FILE  *config_file;            /* Configuration file */
 extern char  *config_filename;        /* Configuration file name */
@@ -64,13 +109,10 @@ extern int    continue_sequence;      /* Sequence num for continued delivery */
 extern char  *continue_transport;     /* Transport for continued delivery */
 
 extern BOOL   daemon_listen;          /* True if listening required */
-extern off_t  data_start_offset;      /* Offset after message id in data file */
 extern FILE  *debug_file;             /* Where to write debugging info */
 extern int    debug_level;            /* Just what its name implies */
 extern pid_t  debug_pid;              /* Pid to add to debug output */
-extern BOOL   debug_trace_memory;     /* For bad problems */
-extern transport_info *debug_transport; /* For catching all output */
-extern char  *debug_transport_file;   /* Non-NULL if debug transport wanted */
+extern int    debug_trace_memory;     /* For bad problems */
 extern int    delay_warning[];        /* Times between warnings */
 extern char  *delay_warning_condition; /* Condition string for warnings */
 extern BOOL   delivery_date_remove;   /* Remove delivery-date headers */
@@ -79,6 +121,7 @@ extern char  *deliver_out_buffer;
 extern int    deliver_datafile;       /* FD for data part of message */
 extern char  *deliver_domain;         /* The local domain for delivery */
 extern char  *deliver_domain_orig;    /* The original local domain for delivery */
+extern char  *deliver_domain_parent;  /* The parent domain for delivery */
 extern BOOL   deliver_firsttime;      /* True for first delivery attempt */
 extern BOOL   deliver_force;          /* TRUE if delivery was forced */
 extern BOOL   deliver_freeze;         /* TRUE if delivery is frozen */
@@ -90,12 +133,17 @@ extern char  *deliver_host_address;   /* Address for remote delivery filter */
 extern int    deliver_load_max;       /* No deliveries if load > this */
 extern char  *deliver_localpart;      /* The local part for delivery */
 extern char  *deliver_localpart_orig; /* The original local part for delivery */
+extern char  *deliver_localpart_parent; /* The parent local part for delivery */
 extern char  *deliver_localpart_prefix; /* The stripped prefix, if any */
 extern char  *deliver_localpart_suffix; /* The stripped suffix, if any */
+extern BOOL   deliver_force_thaw;     /* TRUE to force thaw in queue run */
 extern BOOL   deliver_manual_thaw;    /* TRUE if manually thawed */
 extern int    deliver_queue_load_max; /* Different value for queue running */
 extern address_item *deliver_recipients; /* Current set of addresses */
-extern char  *deliver_selectstring;   /* For selecting by string */
+extern char  *deliver_selectstring;   /* For selecting by recipient */
+extern BOOL   deliver_selectstring_regex; /* String is regex */
+extern char  *deliver_selectstring_sender; /* For selecting by sender */
+extern BOOL   deliver_selectstring_sender_regex; /* String is regex */
 extern director_info directors_available[]; /* Vector of available directors */
 extern director_instance *directors;  /* Chain of instantiated directors */
 extern director_instance director_defaults; /* Default values */
@@ -123,6 +171,9 @@ extern BOOL   exim_gid_set;           /* TRUE if exim_gid set */
 extern char  *exim_path;              /* Path to exec exim */
 extern uid_t  exim_uid;               /* Non-root uid for exim */
 extern BOOL   exim_uid_set;           /* TRUE if exim_uid set */
+extern BOOL   expand_forbid_exists;   /* Lock out exists checking */
+extern BOOL   expand_forbid_lookup;   /* Lock out lookups */
+extern BOOL   expand_forbid_perl;     /* Lock out Perl calls */
 extern int    expand_nlength[];       /* Lengths of numbered strings */
 extern int    expand_nmax;            /* Max numerical value */
 extern char  *expand_nstring[];       /* Numbered strings */
@@ -155,27 +206,27 @@ extern BOOL   headers_checks_fail;    /* If FALSE, just warn */
 extern BOOL   headers_sender_verify;  /* TRUE to verify sender from headers */
 extern BOOL   headers_sender_verify_errmsg; /* ditto, only if errmsg */
 extern char  *helo_accept_junk_hosts; /* Allowed to use junk args */
-extern host_item *helo_accept_junk_hostlist;
-extern char  *helo_accept_junk_nets;
-extern ip_net_item *helo_accept_junk_netlist;
 extern BOOL   helo_strict_syntax;     /* Allows _ in name if false */
-extern BOOL   helo_verify;            /* Check HELO argument */
-extern char  *helo_verify_except_hosts; /* Except for ... */
-extern host_item *helo_verify_except_hostlist; /* Munged data */
-extern char  *helo_verify_except_nets;
-extern ip_net_item *helo_verify_except_netlist;
+extern char  *helo_verify;            /* Check HELO argument */
 extern char  *hold_domains;           /* Hold up deliveries to these */
-extern char  *hold_domains_except;    /* Except for these */
+extern char  *host_accept_relay;      /* Accept relay only from these hosts */
 extern BOOL   host_checking;          /* Set when checking a host */
-extern char  *host_findbyaddr_msg;    /* Text for why it failed */
-extern char  *host_lookup_nets;       /* Nets for which IP addresses are always looked up */
-extern ip_net_item *host_lookup_netlist;
+extern BOOL   host_find_failed_syntax;/* DNS syntax check failure */
+extern char  *host_lookup;            /* For which IP addresses are always looked up */
+extern BOOL   host_lookup_failed;     /* TRUE if lookup failed */
+extern char  *host_lookup_msg;        /* Text for why it failed */
 extern int    host_number;            /* For sharing spools */
 extern char  *host_number_string;     /* For expanding */
+extern char  *host_reject;            /* Reject from these hosts */
+extern char  *host_reject_recipients; /* Reject RCPT from these hosts */
 extern char  *hosts_treat_as_local;   /* For routing */
 
 extern BOOL   ignore_errmsg_errors;   /* As it says */
 extern int    ignore_errmsg_errors_after; /* Or keep them for this time. */
+extern BOOL   ignore_fromline_local;  /* Local SMTP ignore fromline */
+extern char  *ignore_fromline_hosts;  /* Hosts permitted to send "From " */
+extern char  *interface_address;      /* Interface on incoming call */
+
 extern int    journal_fd;             /* Fd for journal file */
 
 extern int    keep_malformed;         /* Time to keep malformed messages */
@@ -196,19 +247,28 @@ extern char  *log_buffer;             /* For constructing log entries */
 extern char  *log_file_path;          /* If unset, use default */
 extern BOOL   log_ip_options;         /* For logging IP source routing */
 extern int    log_level;              /* Level of logging info */
+extern int    log_queue_run_level;    /* Log level for start/end */
 extern BOOL   log_received_sender;    /* Before rewriting */
 extern BOOL   log_received_recipients; /* As it says (before rewriting) */
 extern BOOL   log_refused_recipients; /* Sic */
 extern BOOL   log_rewrites;           /* Sic */
 extern BOOL   log_smtp_confirmation;  /* Log response after "." */
+extern BOOL   log_smtp_connections;   /* Verbose connection logging */
+extern BOOL   log_smtp_syntax_errors; /* As it says (for incoming smtp) */
 extern FILE  *log_stderr;             /* Copy of stderr for log use, or NULL */
 extern BOOL   log_subject;            /* As it says */
 extern char  *lookup_key;             /* For query expansion */
 extern lookup_info lookup_list[];     /* Vector of available lookups */
+extern int    lookup_open_max;        /* Max lookup files to cache */
+extern char  *lookup_value;           /* Value looked up from file */
 
 extern macro_item *macros;            /* Configuration macros */
+extern int    max_username_length;    /* For systems with broken getpwnam() */
 extern char  **mailer_argv;           /* Arg list for sending message */
+extern int    message_age;            /* In seconds */
 extern char  *message_body;           /* Start of message body for filter */
+extern char  *message_body_end;       /* End of message body for filter */
+extern int    message_body_size;      /* Sic */
 extern int    message_body_visible;   /* Amount visible in message_body */
 extern char  *message_filter;         /* Name of system filter file */
 
@@ -234,6 +294,7 @@ extern FILE  *message_log;            /* Open message log while delivering */
 extern char  *message_precedence;     /* From Precedence: header */
 extern int    message_size;           /* Size of message */
 extern int    message_size_limit;     /* As it says */
+extern BOOL   message_size_limit_count_recipients; /* As it says */
 extern char   message_subdir[];       /* Subdirectory for messages */
 extern char  *message_reference;      /* Reference for error messages */
 
@@ -243,6 +304,8 @@ extern uid_t  nobody_uid;             /* Safe, unprivileged, uid */
 extern BOOL   nobody_gid_set;         /* TRUE if nobody_gid set */
 extern BOOL   nobody_uid_set;         /* TRUE if nobody_uid set */
 
+extern optionlist optionlist_auths[];
+extern int    optionlist_auths_size;
 extern optionlist optionlist_directors[];  /* These option lists are made */
 extern int    optionlist_directors_size;   /* global so that readconf can */
 extern optionlist optionlist_routers[];    /* see them for printing out   */
@@ -257,7 +320,6 @@ extern uid_t  originator_uid;         /* Uid of ditto */
 
 extern BOOL   parse_allow_group;      /* Allow group syntax */
 extern BOOL   parse_found_group;      /* In the middle of a group */
-extern BOOL   passed_with_pending;    /* Passed an IP connection with more set */
 extern char  *percent_hack_domains;   /* Local domains for which '% operates */
 extern char  *pid_file_path;          /* For writing daemon pids */
 extern BOOL   preserve_message_logs;  /* Save msglog files */
@@ -275,47 +337,30 @@ extern BOOL   queue_2stage;           /* Run queue in 2-stage manner */
 extern BOOL   queue_list_requires_admin; /* TRUE if -bp requires admin */
 extern BOOL   queue_run_force;        /* TRUE to force during queue run */
 extern BOOL   queue_run_local;        /* Local deliveries only in queue run */
-extern BOOL   queue_running;          /* TRUE when running the queue */
+extern BOOL   queue_running;          /* TRUE for queue running process and */
+                                      /*   immediate children */
+extern pid_t  queue_run_pid;          /* PID of the queue running process or 0 */
+extern int    queue_run_pipe;         /* Pipe for synchronizing */
 extern int    queue_interval;         /* Queue running interval */
 extern BOOL   queue_only;             /* TRUE to disable immediate delivery */
 extern int    queue_only_load;        /* max load before auto-queue */
-extern BOOL   queue_remote;           /* Queue all remote deliveries... */
-extern char  *queue_remote_except;    /* ... except to these domains */
+extern char  *queue_only_file;        /* queue if file exists/not-exists */
+extern BOOL   queue_remote;           /* Queue all remote deliveries (-odqr) */
+extern char  *queue_remote_domains;   /* Queue these remote domains domains */
 extern BOOL   queue_run_in_order;     /* As opposed to random */
 extern int    queue_run_max;          /* Max queue runners */
-extern BOOL   queue_smtp;             /* TRUE to disable immediate STMP delivery */
-extern char  *queue_smtp_except;      /* Exception list */
-
-/* For keeping compiled regexs */
-
-extern re_block *re_dns_again_means_nonexist;
-extern re_block *re_hold_domains;
-extern re_block *re_hold_domains_except;
-extern re_block *re_hosts_treat_as_local;
-extern re_block *re_local_domains;
-extern re_block *re_percent_hack_domains;
-extern re_block *re_queue_remote_except;
-extern re_block *re_queue_smtp_except;
-extern re_block *re_receiver_verify_addresses;
-extern re_block *re_receiver_verify_senders;
-extern re_block *re_receiver_verify_senders_except;
-extern re_block *re_recipients_reject_except;
-extern re_block *re_relay_domains;
-extern re_block *re_remote_sort;
-extern re_block *re_sender_accept;
-extern re_block *re_sender_accept_recipients;
-extern re_block *re_sender_address_relay;
-extern re_block *re_sender_reject;
-extern re_block *re_sender_reject_except;
-extern re_block *re_sender_reject_recipients;
-extern re_block *re_SIZE;
+extern BOOL   queue_smtp;             /* Disable immediate STMP (-odqs)*/
+extern char  *queue_smtp_domains;     /* Ditto, for these domains */
 
 extern char  *raw_sender;             /* Before rewriting */
 extern char **raw_recipients;         /* Before rewriting */
 extern char  *rbl_domains;            /* Realtime Block List domains */
-extern char  *rbl_except_nets;        /* Nets for RBL isn't used */
-extern ip_net_item *rbl_except_netlist;
+extern char  *rbl_hosts;              /* Hosts for which RBL is used */
 extern header_line *rbl_header;       /* RBL header to add */
+extern BOOL   rbl_host;               /* True if RBL host but not rejected */
+extern BOOL   rbl_log_rcpt_count;
+extern BOOL   rbl_log_headers;
+extern char  *rbl_msg_buffer;         /* Pointer to TXT data */
 extern BOOL   rbl_reject_recipients;  /* TRUE to reject recipients if found */
 extern BOOL   rbl_warn_header;        /* Add header if warning */
 extern gid_t  real_gid;               /* Real gid */
@@ -330,25 +375,20 @@ extern char  *received_protocol;      /* Name of incoming protocol */
 extern int    received_time;          /* Time the message was received */
 extern BOOL   receiver_try_verify;    /* Accept if soft error while verifying */
 extern char  *receiver_unqualified_hosts; /* Permitted unqualified receivers */
-extern host_item *receiver_unqualified_hostlist;
-extern char  *receiver_unqualified_nets;
-extern ip_net_item *receiver_unqualified_netlist;
 extern BOOL   receiver_verify;        /* Verify receivers when received */
 extern char  *receiver_verify_addresses; /* ... but only these addresses */
-extern char  *receiver_verify_except_hosts; /* Don't verify for these hosts */
-extern host_item *receiver_verify_except_hostlist;
-extern char  *receiver_verify_except_nets;
-extern ip_net_item *receiver_verify_except_netlist;
+extern char  *receiver_verify_hosts;  /* Verify for these hosts */
 extern char  *receiver_verify_senders;/* Verify only from these senders */
-extern char  *receiver_verify_senders_except; /* Don't verify from these senders */
 extern int    recipients_count;       /* Number of recipients */
 extern recipient_item *recipients_list; /* List of recipient addresses */
 extern int    recipients_list_max;    /* Maximum number fitting in list */
 extern int    recipients_max;         /* Max permitted */
 extern int    recipients_max_reject;  /* If TRUE, reject whole message */
 extern char  *recipients_reject_except; /* Exceptions for policy rejections */
+extern char  *recipients_reject_except_senders; /* Exceptions for policy rejections */
 extern BOOL   refuse_all_rcpts;       /* Flag for SMTP verifying */
 extern BOOL   refuse_ip_options;      /* Refuse source routing */
+extern pcre  *regex_check_dns_names;  /* For DNS name checking */
 extern pcre  *regex_From;             /* For recognizing "From_" lines */
 extern pcre  *regex_SIZE;             /* For recognizing SIZE settings */
 extern pcre  *regex_ismsgid;          /* Compiled r.e. for message it */
@@ -358,6 +398,7 @@ extern BOOL   relay_need_either;      /* True for OR host, sender */
 extern int    remote_max_parallel;    /* Maximum parallel delivery */
 extern char  *remote_sort;            /* Remote domain sorting order */
 extern retry_config *retries;         /* Chain of retry config information */
+extern int    retry_data_expire;      /* When to expire retry data */
 extern int    retry_interval_max;     /* Absolute maximum */
 extern int    retry_maximum_timeout;  /* The maximum timeout */
 extern char  *return_path;            /* Return path for a message */
@@ -365,135 +406,80 @@ extern BOOL   return_path_remove;     /* Remove return-path headers */
 extern int    return_size_limit;      /* Limit messages returned to sender */
 extern int    rewrite_existflags;     /* Indicate which headers have rewrites */
 extern rewrite_rule *rewrite_rules;   /* Chain of rewriting rules */
-extern char  *rfc1413_except_hosts;   /* RFC 1413 exceptions */
-extern host_item *rfc1413_except_hostlist; /* Munged data */
-extern char  *rfc1413_except_nets;    /* RFC 1413 exceptions */
-extern ip_net_item *rfc1413_except_netlist; /* Munged data */
+extern char  *rfc1413_hosts;          /* RFC hosts */
 extern int    rfc1413_query_timeout;  /* Timeout on RFC 1413 calls */
 extern BOOL   rfc821_domains;         /* If set, syntax is 821, not 822 */
 extern uid_t  root_uid;               /* The uid for root */
 extern char  *route_option;           /* Option string for some routers */
-
 extern router_info routers_available[]; /* Vector of available routers */
 extern router_instance *routers;      /* Chain of instantiated routers */
 extern router_instance router_defaults; /* Default values */
+extern char  *running_status;         /* Flag string for testing */
 
 extern BOOL   search_find_defer;      /* Set TRUE if lookup defers */
 extern int    security_level;         /* For the set(e)uid stuff */
 extern char  *security_type;          /* Text name for above, set by config */
 extern char  *self_hostname;          /* Self host after routing->directors */
-extern char  *sender_accept;          /* Allow certain senders only */
-extern char  *sender_accept_recipients; /* Ditto, reject RCPT TO */
 extern char  *sender_address;         /* Envelope sender */
 extern BOOL   sender_address_forced;  /* Set by -f */
 extern char  *sender_address_relay;   /* Only non-nearby relay from these addresses */
 extern BOOL   sender_address_rewritten; /* True if rewritten by verify */
 extern char  *sender_fullhost;        /* Sender host name + address */
 extern char  *sender_helo_name;       /* Host name from HELO/EHLO */
-extern char  *sender_host_accept;     /* Accept only from these hosts */
-extern host_item *sender_host_accept_hosts; /* Munged data */
-extern char  *sender_host_accept_relay; /* Accept non-nearby relay only from these hosts */
-extern host_item *sender_host_accept_relay_hosts; /* Munged data */
 extern char  *sender_host_address;    /* IP address of sender, as chars */
+extern char **sender_host_aliases;    /* Points to list of alias names */
+extern char  *sender_host_authenticated; /* Name of authentication mechanism */
 extern char  *sender_host_name;       /* Host name from lookup, or "?" */
 extern BOOL   sender_host_notsocket;  /* Set for -bs and -bS */
-extern char  *sender_host_reject;     /* Reject from these hosts */
-extern char  *sender_host_reject_except; /* Except these hosts */
-extern host_item *sender_host_reject_hosts; /* Munged data */
-extern host_item *sender_host_reject_except_hosts; /* Munged data */
-extern char  *sender_host_reject_recipients; /* Reject RCPT from these hosts */
-extern host_item *sender_host_reject_recipients_hosts; /* Munged data */
-extern char  *sender_host_reject_relay; /* Reject non-nearby relay from these hosts */
-extern host_item *sender_host_reject_relay_hosts; /* Munged data */
-extern char  *sender_host_reject_relay_except; /* Exceptions */
-extern host_item *sender_host_reject_relay_except_hosts; /* Munged data */
 extern BOOL   sender_host_unknown;    /* TRUE for -bs and -bS except inetd */
 extern char  *sender_ident;           /* Sender identity via RFC 1413 */
 extern BOOL   sender_local;           /* TRUE for local senders */
-extern char  *sender_net_accept;      /* Accept only from these nets */
-extern ip_net_item *sender_net_accept_nets; /* Munged data */
-extern char  *sender_net_accept_relay; /* Accept non-nearby relay from these nets */
-extern ip_net_item *sender_net_accept_relay_nets; /* Munged data */
-extern char  *sender_net_reject;      /* Reject from these nets */
-extern char  *sender_net_reject_except; /* Except these nets */
-extern ip_net_item *sender_net_reject_nets; /* Munged data */
-extern ip_net_item *sender_net_reject_except_nets; /* Munged data */
-extern char  *sender_net_reject_recipients; /* Reject RCPT from these nets */
-extern ip_net_item *sender_net_reject_recipients_nets; /* Munged data */
-extern char  *sender_net_reject_relay; /* Reject non-nearby relay from these nets */
-extern ip_net_item *sender_net_reject_relay_nets; /* Munged data */
-extern char  *sender_net_reject_relay_except; /* Exceptions */
-extern ip_net_item *sender_net_reject_relay_except_nets; /* Munged data */
 extern char  *sender_rcvhost;         /* Host data for Received: */
 extern char  *sender_reject;          /* Spam filter list */
-extern char  *sender_reject_except;   /* Spam filter exception list */
 extern char  *sender_reject_recipients; /* Ditto, reject RCPT TO */
 extern BOOL   sender_try_verify;      /* Accept if soft error while verifying */
 extern char  *sender_unqualified_hosts; /* Permitted unqualified senders */
-extern host_item *sender_unqualified_hostlist;
-extern char  *sender_unqualified_nets;
-extern ip_net_item *sender_unqualified_netlist;
 extern BOOL   sender_verify;          /* TRUE when verifying senders */
 extern BOOL   sender_verify_batch;    /* TRUE if batch SMTP included */
-extern char  *sender_verify_except_hosts; /* Verification exceptions */
-extern host_item *sender_verify_except_hostlist; /* Munged data */
-extern char  *sender_verify_except_nets; /* Verification exceptions */
-extern ip_net_item *sender_verify_except_netlist; /* Munged data */
 extern BOOL   sender_verify_fixup;    /* Fix broken senders from headers */
-extern BOOL   sender_verify_log_details; /* For debugging */
+extern char  *sender_verify_hosts;    /* Verification hosts */
 extern BOOL   sender_verify_reject;   /* Reject if verify fails */
 extern int    sender_verify_max_retry_rate; /* Max retry rate */
 extern volatile BOOL sigalrm_seen;    /* Flag for sigalrm_handler */
 extern char **sighup_argv;            /* Args for re-execing after SIGHUP */
 extern int    smtp_accept_count;      /* Count of connections */
+extern BOOL   smtp_accept_keepalive;  /* Set keepalive on incoming */
 extern int    smtp_accept_max;        /* Max SMTP connections */
+extern int    smtp_accept_max_per_host; /* Max SMTP cons from one host/addr */
 extern int    smtp_accept_queue;      /* Queue after so many connections */
 extern int    smtp_accept_queue_per_connection; /* Queue after so many msgs */
 extern int    smtp_accept_reserve;    /* Reserve these SMTP connections */
 extern char  *smtp_banner;            /* Banner string (to be expanded) */
 extern BOOL   smtp_batched_input;     /* TRUE if SMTP batch (no interaction) */
+extern BOOL   smtp_check_spool_space; /* TRUE to check SMTP SIZE value */
 extern int    smtp_connect_backlog;   /* Max backlog permitted */
 extern char  *smtp_etrn_command;      /* Command to run */
-extern host_item *smtp_etrn_hostlist; /* Munged data */
 extern char  *smtp_etrn_hosts;        /* Accept ETRN only from these hosts */
-extern ip_net_item *smtp_etrn_netlist; /* Munged data */
-extern char  *smtp_etrn_nets;         /* Networks for ETRN command */
 extern BOOL   smtp_etrn_serialize;    /* Only one at once */
-extern host_item *smtp_expn_hostlist; /* Munged data */
 extern char  *smtp_expn_hosts;        /* Accept EXPN only from these hosts */
-extern ip_net_item *smtp_expn_netlist; /* Munged data */
-extern char  *smtp_expn_nets;         /* Networks for EXPN command */
 extern FILE  *smtp_in;                /* Incoming SMTP input file */
 extern BOOL   smtp_input;             /* TRUE if input is via SMTP */
 extern int    smtp_load_reserve;      /* Only from reserved if load > this */
-extern BOOL   smtp_log_connections;   /* Verbose connection logging */
 extern FILE  *smtp_out;               /* Incoming SMTP output file */
 extern int    smtp_port;              /* Alternate SMTP port number */
+extern char  *smtp_read_error;        /* Message for SMTP input error */
 extern BOOL   smtp_reading_data;      /* True if "." needed before response */
 extern int    smtp_receive_timeout;   /* Applies to each received line */
-extern host_item *smtp_reserve_hostlist; /* Munged data */
 extern char  *smtp_reserve_hosts;     /* Hosts for reserved slots */
-extern ip_net_item *smtp_reserve_netlist; /* Munged data */
-extern char  *smtp_reserve_nets;      /* Networks for reserved slots */
 extern BOOL   smtp_use_size;          /* Global for passed connections */
 extern BOOL   smtp_verify;            /* TRUE if VRFY permitted */
 extern BOOL   split_spool_directory;  /* TRUE to use multiple subdirs */
 extern char  *spool_directory;        /* Name of spool directory */
 extern char  *stderr_filename;        /* File for use with -df */
-extern int    store_max_malloc;       /* For keeping total */
 extern int    store_pool;             /* Current pool number */
 extern BOOL   strip_excess_angle_brackets; /* Surrounding route-addrs */
 extern BOOL   strip_trailing_dot;     /* Remove dots at ends of domains */
 
-/* Alternative transports for directory addresses; transports for file,
-pipe, and autoreply addresses. */
-
-extern transport_instance *transport_address_directory;
-extern transport_instance *transport_address_directory2;
-extern transport_instance *transport_address_file;
-extern transport_instance *transport_address_pipe;
-extern transport_instance *transport_address_reply;
-
 /* Ditto for system filter */
 
 extern transport_instance *transport_message_filter_directory;
@@ -530,6 +516,7 @@ extern char  *warnmsg_delay;          /* String form of delay time */
 extern char  *warnmsg_file;           /* Template for warning messages */
 extern char  *warnmsg_recipients;     /* Recipients of warning message */
 
+extern char  *verify_address_parse_error; /* Sic */
 extern char  *verify_forced_errmsg;   /* Message from :defer: or :fail: */
 extern BOOL   verify_only;            /* TRUE for -bv */
 extern char  *version_copyright;      /* Copyright notice */
