@@ -1,9 +1,8 @@
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for finding hosts, either by gethostbyname, gethostbyaddr, or via
@@ -85,9 +84,7 @@ char buffer[1024];
 
 if (list == NULL) return;
 
-for (name = string_nextinlist(&list, ':', buffer, sizeof(buffer));
-     name != NULL;
-     name = string_nextinlist(&list, ':', buffer, sizeof(buffer)))
+while ((name = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
   {
   host_item *h = store_get(sizeof(host_item));
   h->next = NULL;
@@ -213,52 +210,43 @@ DEBUG(5) debug_printf("sender_rcvhost = %s\n", sender_rcvhost);
 *************************************************/
 
 /* This function is called from a number of routers on receiving
-HOST_FOUND_LOCAL when looking up a supposedly remote host. There are
-two cases:
+HOST_FOUND_LOCAL when looking up a supposedly remote host. The action is
+controlled by a generic configuration option called "self" on each router,
+which can be one of:
 
-(1)  If the current address is source-routed, then we can remove the
-     first source-routing domain and try again with the remainder by
-     returning ISLOCAL. This will cause the new address to be routed
-     ab initio.
+   . freeze:                       Log the incident, freeze, and return ERROR
 
-(2)  For a non-source-routed address, the action is controlled by a
-     generic configuration option called "self" on each router, which
-     can be one of:
+   . defer:                        Log the incident and return DEFER
 
-     . freeze:                       Log the incident, freeze, and return ERROR
+   . send:                         Carry on with the delivery regardless -
+                                   this makes sense only if the SMTP
+                                   listener on this machine is a differently
+                                   configured MTA
 
-     . defer:                        Log the incident and return DEFER
+   . fail_soft                     The router just fails, and the address
+                                   gets passed to the next router, overriding
+                                   the setting of no_more
 
-     . send:                         Carry on with the delivery regardless -
-                                     this makes sense only if the SMTP
-                                     listener on this machine is a differently
-                                     configured MTA
+   . reroute:<new-domain>          Change the domain to the given domain
+                                   and return ISLOCAL so it gets passed back
+                                   to the directors or routers.
 
-     . fail_soft                     The router just fails, and the address
-                                     gets passed to the next router.
+   . reroute:rewrite:<new-domain>  The same, but headers containing the
+                                   old domain get rewritten.
 
-     . reroute:<new-domain>          Change the domain to the given domain
-                                     and return ISLOCAL so it gets passed back
-                                     to the directors or routers.
-
-     . reroute:rewrite:<new-domain>  The same, but headers containing the
-                                     old domain get rewritten.
-
-     . local                         Set addr->forced_local and return ISLOCAL,
-                                     so the address gets passed to the
-                                     directors.
+   . local                         Set addr->forced_local and return ISLOCAL,
+                                   so the address gets passed to the
+                                   directors.
 
 Arguments:
   addr     the address being routed
   host     the host that is local, with MX set (or -1 if MX not used)
-  code     the action to be taken (one of the self_xxx enums) for non-
-             source-routed addresses
+  code     the action to be taken (one of the self_xxx enums)
   rewrite  TRUE if rewriting headers required for ISLOCAL
   new      new domain to be used for ISLOCAL
 
-Returns:   (1) ISLOCAL for a source-routed address, or
-           (2) ERROR, DEFER, ISLOCAL, FAIL, FORCEFAIL, or OK, according
-               to the value of code.
+Returns:   ERROR, DEFER, ISLOCAL, FAILMORE, FORCEFAIL, or OK, according
+             to the value of code.
 */
 
 int
@@ -267,35 +255,6 @@ host_self_action(address_item *addr, host_item *host, int code, BOOL rewrite,
 {
 char *format, *msg;
 
-/* Handle a source-routed address */
-
-if (addr->local_part[0] == ',' || addr->local_part[0] == ':')
-  {
-  char *p = addr->local_part + 1;
-
-  if (addr->local_part[0] == ':')
-    {
-    while (*p != '@') p++;
-    *p = 0;
-    addr->domain = addr->route_domain = p+1;
-    addr->local_part++;
-    }
-  else
-    {
-    while (*p != ',' && *p != ':') p++;
-    addr->domain = addr->route_domain =
-      string_copyn(addr->local_part + 2, p - addr->local_part - 2);
-    addr->local_part = p;
-    }
-
-  DEBUG(2) debug_printf("source-routed address is local: rewrote\n  "
-    "domain = %s\n  local_part = %s\n", addr->route_domain,
-      addr->local_part);
-  return ISLOCAL;
-  }
-
-/* Handle a non-source-routed address */
-
 if (host->mx >= 0)
   {
   format = "lowest MX record for %s points to local host";
@@ -348,14 +307,15 @@ switch (code)
     }
   return OK;
 
-  case self_fail:
+  case self_fail:    /* This is soft failure; pass to next router */
   DEBUG(2)
     {
     debug_printf(format, addr->route_domain);
     debug_printf(": router failed soft\n");
     }
   addr->message = msg;
-  return FAIL;
+  addr->self_hostname = string_copy(host->name);
+  return FAILMORE;
 
   case self_forcefail:
   DEBUG(2)
@@ -382,16 +342,20 @@ returned.
 
 Problems:
 
-  (1) Solaris 2 has the SIOGIFNUM call to get the number of
-  interfaces, but other OS (including Solaris 1) appear not to. So just screw
-  in a largeish fixed number. Unfortunately, the www addressing scheme means
-  that some hosts have a very large number of virtual interfaces. Such hosts
-  are recommended to set local_interfaces to avoid problems with this (see
-  host_find_interfaces below).
+  (1) Solaris 2 has the SIOGIFNUM call to get the number of interfaces, but
+  other OS (including Solaris 1) appear not to. So just screw in a largeish
+  fixed number. Unfortunately, the www addressing scheme means that some hosts
+  have a very large number of virtual interfaces. Such hosts are recommended
+  to set local_interfaces to avoid problems with this (see host_find_interfaces
+  below).
 
   (2) If the standard code is run on IRIX, it does not return any alias
   interfaces. There is special purpose code for that operating system, which
-  uses the sysctl() function.
+  uses the sysctl() function. This is used if SYSCTL_IP_INTERFACES is set.
+
+  (3) Some experimental/developing OS (e.g. GNU/Hurd) do not have any means
+  of finding the interfaces. If NO_FIND_INTERFACES is set, a fudge-up is used
+  instead.
 
 Arguments:    none
 Returns:      a chain of ip_address_items, each pointing to a textual
@@ -399,12 +363,12 @@ Returns:      a chain of ip_address_items, each pointing to a textual
 */
 
 
+#ifndef NO_FIND_INTERFACES
 #ifndef SYSCTL_IP_INTERFACES
 
 /* This is the form of the function that uses ioctl() in a way that works on
-most versions of Unix. */
-
-#define MAXINTERFACES 250
+most versions of Unix. MAXINTERFACES is set in config.h and can be changed in
+Local/Makefile. */
 
 static ip_address_item *
 host_find_running_interfaces(void)
@@ -500,19 +464,18 @@ return yield;
 }
 
 
+#else  /* SYSCTL_IP_INTERFACES */
+
 /* This is the special form of the function using sysctl() which is the only
 form that returns all the aliases on IRIX systems. This code has its origins
 in a sample program that came from within SGI. */
 
-#else
-
 #include <sys/sysctl.h>
 #include <net/if_dl.h>
 #include <net/if_types.h>
 #include <net/soioctl.h>
 #include <net/route.h>
 
-
 #define ROUNDUP(a) ((a) > 0 ? (1 + (((a) - 1) | (sizeof(__uint64_t) -1))) \
                     : sizeof(__uint64_t))
 #ifdef _HAVE_SA_LEN
@@ -559,10 +522,7 @@ if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0)
 lim  = buf + needed;
 for (nextaddr = buf; nextaddr < lim; nextaddr += ifm->ifm_msglen)
   {
-  register struct sockaddr_dl *sdl;
-
   ifm = (struct if_msghdr *)nextaddr;
-  sdl = (struct sockaddr_dl *)(ifm +1);
 
   if (ifm->ifm_type != RTM_IFINFO)
     {
@@ -605,8 +565,37 @@ for (nextaddr = buf; nextaddr < lim; nextaddr += ifm->ifm_msglen)
 return yield;
 }
 
-#endif
+#endif  /* SYSCTL_IP_INTERFACES */
 
+#else  /* NO_FIND_INTERFACES */
+
+/* Some experimental or developing OS (e.g. GNU/Hurd) do not have the ioctls,
+and there is no other way to get a list of the (IP addresses of) local
+interfaces. Instead, we fake a local_interfaces="127.0.0.1" without setting it.
+If local_interfaces is unset, the default behaviour is to listen to all
+interfaces on a single socket. The only difference is that the interfaces will
+not be recognized as local. Hence, the user should add further local IPs to
+host_treat_as_local. However, setting local_interfaces is probably the best
+approach. */
+
+static ip_address_item *
+host_find_running_interfaces(void)
+{
+ip_address_item *yield = NULL;
+char *my_local_interfaces = "127.0.0.1"; /* local_interfaces */
+
+/* We know that local_interfaces is NULL when we come here from
+host_find_interfaces(). Also we know that if we set it to something else it
+will behave differently. In particular, it won't call us again, so this is not
+a loop. */
+
+local_interfaces = my_local_interfaces;
+yield = host_find_interfaces();
+local_interfaces = NULL;
+return yield;
+}
+
+#endif  /* NO_FIND_INTERFACES */
 
 
 
@@ -637,11 +626,9 @@ ip_address_item *next;
 
 if (local_interfaces == NULL) return host_find_running_interfaces();
 
-for (s = string_nextinlist(&listptr, ':', buffer, sizeof(buffer));
-     s != NULL;
-     s = string_nextinlist(&listptr, ':', buffer, sizeof(buffer)))
+while ((s = string_nextinlist(&listptr, ':', buffer, sizeof(buffer))) != NULL)
   {
-  if (!string_is_ip_address(s))
+  if (!string_is_ip_address(s, NULL))
     log_write(0, LOG_MAIN|LOG_PANIC_DIE, "Malformed IP address \"%s\" in "
       "local_interfaces", s);
 
@@ -669,19 +656,22 @@ return yield;
 
 
 /*************************************************
-*        Convert address to text                 *
+*        Convert network IP address to text      *
 *************************************************/
 
 /* Given an IPv4 or IPv6 address in binary, convert it to a text
 string and return the result in a piece of new store. The address can
-either be given directly, or passed over in a sockaddr structure.
+either be given directly, or passed over in a sockaddr structure. Note
+that this isn't the converse of host_aton() because of byte ordering
+differences. See host_nmtoa() below.
 
 Arguments:
   type       if < 0 then arg points to a sockaddr, else
              either AF_INET or AF_INET6
   arg        points to a sockaddr if type is < 0, or
              points to an IPv4 address (32 bits), or
-             points to an IPv6 address (128 bits)
+             points to an IPv6 address (128 bits),
+             in both cases, in network byte order
   buffer     if NULL, the result is returned in gotten store;
              else points to a buffer to hold the answer
 
@@ -745,6 +735,10 @@ return buffer;
 
 /* Given the textual form of an IP address, convert it to binary in an
 array of ints. IPv4 addresses occupy one int; IPv6 addresses occupy 4 ints.
+The result has the first byte in the most significant byte of the first int. In
+other words, the result is not in network byte order, but in host byte order.
+As a result, this is not the converse of host_ntoa(), which expects network
+byte order. See host_nmtoa() below.
 
 Arguments:
   address    points to the textual address, checked for syntax
@@ -784,7 +778,8 @@ if (strchr(address, ':') != NULL)
     int len = strcspn(p, ":");
     if (len == 0) nulloffset = ci;
     component[ci++] = p;
-    p += len + 1;
+    p += len;
+    if (*p == ':') p++;
     }
 
   /* If the final component contains a dot, it is a trailing v4 address.
@@ -831,81 +826,106 @@ return v4offset+1;
 }
 
 
-
-
 /*************************************************
-*       Convert address + mask to binary         *
+*           Apply mask to an IP address          *
 *************************************************/
 
-/* This function is given the textual form of an address and a mask, separated
-by a slash. It converts both to binary and returns them in vectors of ints.
-The number of ints used is returned. Both IPv4 and IPv6 addressing modes are
-supported.
+/* Mask an address held in 1 or 4 ints, with the ms bit in the ms bit of the
+first int, etc.
 
 Arguments:
-  s         the input string
-  address   points to vector of int for address
-  mask      points to vector of int for mask
-  error     where to put a message on error
+  count        the number of ints
+  binary       points to the ints to be masked
+  mask         the count of ms bits to leave, or -1 if no masking
 
-Returns:    number of vector words used, or 0 if there was a problem
+Returns:       nothing
 */
 
-int
-host_amton(char *s, int *address, int *mask, char **error)
+void
+host_mask(int count, int *binary, int mask)
 {
-int size;
-char *slash = strchr(s, '/');
-
-if (slash == NULL)
+int i;
+if (mask < 0) mask = 99999;
+for (i = 0; i < count; i++)
   {
-  *error = "\"/\" missing";
-  return 0;
-  }
-
-/* Temporarily terminate the string at the slash, and turn the initial
-address into binary. */
-
-*slash = 0;
-size = host_aton(s, address);
-*slash = '/';
-
-/* For IPv4 addresses, the mask may be another IPv4 address, containing dots,
-or an integer. The syntax has been checked. */
-
-if (size == 1 && strchr(slash+1, '.') != NULL) host_aton(slash+1, mask);
-
-/* For IPv6 addresses, and optionally for IPv4 addresses, the mask is just
-an integer, and we have to make up the bit pattern. */
-
-else
-  {
-  int i;
-  int mlen = atoi(slash+1);
-  for (i = 0; i < 4; i++)
+  int wordmask;
+  if (mask == 0) wordmask = 0;
+  else if (mask < 32)
     {
-    if (mlen >= 32)
-      {
-      mask[i] = -1;
-      mlen -= 32;
-      }
-    else if (mlen == 0) mask[i] = 0;
-    else
-      {
-      mask[i] = (-1) << (32 - mlen);
-      mlen = 0;
-      }
+    wordmask = (-1) << (32 - mask);
+    mask = 0;
     }
+  else
+    {
+    wordmask = -1;
+    mask -= 32;
+    }
+  binary[i] &= wordmask;
   }
-
-/* Return the number of ints used */
-
-return size;
 }
 
 
 
 
+/*************************************************
+*     Convert masked IP address in ints to text  *
+*************************************************/
+
+/* We can't use host_ntoa() because it assumes the binary values are in network
+byte order, and these are the result of host_aton(), which puts them in ints in
+host byte order. Also, we really want IPv6 addresses to be in a canonical
+format, so we output them with no abbreviation. However, we can't use the
+normal colon separator in them because it terminates keys in lsearch files, so
+use dot instead.
+
+Arguments:
+  count       1 or 4 (number of ints)
+  binary      points to the ints
+  mask        mask value; if < 0 don't add to result
+  buffer      big enough to hold the result
+
+Returns:      the number of characters placed in buffer, not counting
+              the final nul.
+*/
+
+int
+host_nmtoa(int count, int *binary, int mask, char *buffer)
+{
+int i, j;
+char *tt = buffer;
+
+if (count == 1)
+  {
+  j = binary[0];
+  for (i = 24; i >= 0; i -= 8)
+    {
+    sprintf(tt, "%d.", (j >> i) & 255);
+    while (*tt) tt++;
+    }
+  }
+else
+  {
+  for (i = 0; i < 4; i++)
+    {
+    j = binary[i];
+    sprintf(tt, "%04x.%04x.", (j >> 16) & 0xffff, j & 0xffff);
+    while (*tt) tt++;
+    }
+  }
+
+tt--;   /* lose final . */
+
+if (mask < 0)
+  *tt = 0;
+else
+  {
+  sprintf(tt, "/%d", mask);
+  while (*tt) tt++;
+  }
+
+return tt - buffer;
+}
+
 
 
 /*************************************************
@@ -960,9 +980,8 @@ for (h = host; h != last->next; h = h->next)
   {
   #ifndef STAND_ALONE
   if (hosts_treat_as_local != NULL &&
-      match_isinlist(h->name, hosts_treat_as_local, &re_hosts_treat_as_local,
-      TRUE))
-    goto FOUND_LOCAL;
+    match_isinlist(string_copylc(h->name), &hosts_treat_as_local, TRUE, TRUE,
+      NULL)) goto FOUND_LOCAL;
   #endif
 
   if (h->address != NULL)
@@ -1011,67 +1030,69 @@ return yield;
 
 
 /*************************************************
-*        Find host name given an IP address      *
+*        Find host name for incoming call        *
 *************************************************/
 
-/* For the moment, just use gethostbyaddr, and pass back a single string,
-either in permanent or pooled store.
+/* For the moment, just use gethostbyaddr, and pass back a single string, in
+permanent store. The incoming address is in sender_host_address, either in
+dotted-quad form for IPv4 or in colon-separated form for IPv6.
 
-Arguments:
-  address     an IPv4 address in dotted-quad form OR
-              an IPv6 address in colon-separated form
-  use_malloc  TRUE if permanent store is required
+Arguments:    none
+Returns:      TRUE on success, the answer being placed in the global variable
+              sender_host_name, with any aliases in a chain hung off
+              sender_host_aliases
 
-Returns:      the name of the host or NULL if not found
+The variable host_lookup_msg is set to an empty string on sucess, or to a
+reason for the failure otherwise, in a form suitable for tagging onto an error
+message, and also host_lookup_failed is set TRUE if the lookup failed. Any
+dynamically constructed string for host_lookup_msg must be in permanent store,
+because it might be used for several incoming messages on the same SMTP
+connection. */
 
-In addition, the variable host_findbyaddr_msg is set to an empty string on
-sucess, or to a reason for the failure otherwise, in a form suitable for
-tagging onto an error message. */
-
-char *
-host_find_byaddr(char *address, BOOL use_malloc)
+BOOL
+host_name_lookup(void)
 {
 char *s, *t, *yield;
+char **yield_aliases = NULL;
 host_item h, *hh;
 int len;
 struct hostent *hosts;
 struct in_addr addr;
 
-host_findbyaddr_msg = "";
-
-HDEBUG(2) debug_printf("looking up host name for %s\n", address);
+HDEBUG(2) debug_printf("looking up host name for %s\n", sender_host_address);
 
 /* Lookup on IPv6 system */
 
 #if HAVE_IPV6
-if (strchr(address, ':') != NULL)
+if (strchr(sender_host_address, ':') != NULL)
   {
   struct in6_addr addr6;
-  if (inet_pton(AF_INET6, address, &addr6) != 1)
-    log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", address);
+  if (inet_pton(AF_INET6, sender_host_address, &addr6) != 1)
+    log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", sender_host_address);
   hosts = gethostbyaddr((char *)(&addr6), sizeof(addr6), AF_INET6);
   }
 else
   {
-  if (inet_pton(AF_INET, address, &addr) != 1)
-    log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", address);
+  if (inet_pton(AF_INET, sender_host_address, &addr) != 1)
+    log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", sender_host_address);
   hosts = gethostbyaddr((char *)(&addr), sizeof(addr), AF_INET);
   }
 
 /* Do lookup on IPv4 system */
 
 #else
-addr.s_addr = (S_ADDR_TYPE)inet_addr(address);
+addr.s_addr = (S_ADDR_TYPE)inet_addr(sender_host_address);
 hosts = gethostbyaddr((char *)(&addr), sizeof(addr), AF_INET);
 #endif
 
-/* Failed to look up the host */
+/* Failed to look up the host. NB host_lookup_msg must be in permant store. */
 
 if (hosts == NULL)
   {
   HDEBUG(2) debug_printf("IP address lookup failed\n");
-  host_findbyaddr_msg = " (failed to find host name from IP address)";
-  return NULL;
+  host_lookup_msg = " (failed to find host name from IP address)";
+  host_lookup_failed = TRUE;
+  return FALSE;
   }
 
 /* Copy and lowercase the name. We do this before checking the forward lookup
@@ -1080,12 +1101,32 @@ systems, so we need to take a copy of the name first. */
 
 s = (char *)hosts->h_name;
 len = (int)strlen(s) + 1;
-t = yield = use_malloc? store_malloc(len) : store_get(len);
+t = yield = store_malloc(len);
 while (*s != 0) *t++ = tolower(*s++);
 *t = 0;
 
 HDEBUG(2) debug_printf("IP address lookup yielded %s\n", yield);
 
+/* If the host has aliases, build a copy of the alias list */
+
+if (hosts->h_aliases != NULL)
+  {
+  int count = 1;
+  char **aliases, **ptr;
+  for (aliases = hosts->h_aliases; *aliases != NULL; aliases++) count++;
+  ptr = yield_aliases = store_malloc(count * sizeof(char *));
+  for (aliases = hosts->h_aliases; *aliases != NULL; aliases++)
+    {
+    char *s = *aliases;
+    int len = (int)strlen(s) + 1;
+    char *t = *ptr++ = store_malloc(len);
+    while (*s != 0) *t++ = tolower(*s++);
+    *t = 0;
+    HDEBUG(2) debug_printf("Alias %s\n", ptr[-1]);
+    }
+  *ptr = NULL;
+  }
+
 /* We need to verify that a forward lookup on the name we found does indeed
 correspond to the address. This is for security: in principle a malefactor who
 happened to own a reverse zone could set it to point to any names at all. */
@@ -1097,21 +1138,43 @@ h.next = NULL;
 if (host_find_byname(&h, NULL, FALSE) == HOST_FIND_FAILED)
   {
   HDEBUG(2) debug_printf("no IP address found for %s\n", yield);
-  host_findbyaddr_msg = string_sprintf(
+  /* This must be in permanent store */
+  host_lookup_msg = string_copy_malloc(string_sprintf(
     " (no IP address found for %s, the host name obtained from %s)",
-    yield, address);
-  return NULL;
+    yield, sender_host_address));
+  host_lookup_failed = TRUE;
+  return FALSE;
   }
 
-else for (hh = &h; hh != NULL; hh = hh->next)
-  if (strcmp(hh->address, address) == 0) return yield;
+/* Found address(es) for the host name. If the incoming address is an IPv4
+address expressed in IPv6 format, we must compare the IPv4 part to any IPv4
+addresses. */
+
+else
+  {
+  char *address_ipv4 = (strncmp(sender_host_address, "::ffff:", 7) == 0)?
+    sender_host_address + 7 : sender_host_address;
+  for (hh = &h; hh != NULL; hh = hh->next)
+    {
+    if (strcmp(hh->address, (strchr(hh->address, ':') == NULL)?
+          address_ipv4 : sender_host_address) == 0)
+      {
+      sender_host_name = yield;
+      sender_host_aliases = yield_aliases;
+      return TRUE;
+      }
+    }
+  }
 
 HDEBUG(2) debug_printf("%s does not match any IP address for %s\n",
-  address, yield);
-host_findbyaddr_msg =
-  string_sprintf(" (%s does not match any IP address for %s)", address, yield);
+  sender_host_address, yield);
+/* This must be in permanent store */
+host_lookup_msg = string_copy_malloc(
+  string_sprintf(" (%s does not match any IP address for %s)",
+    sender_host_address, yield));
+host_lookup_failed = TRUE;
 
-return NULL;
+return FALSE;
 }
 
 
@@ -1148,7 +1211,6 @@ Returns:                 HOST_FIND_FAILED  Failed to find the host or domain
 int
 host_find_byname(host_item *host, char **fully_qualified_name, BOOL use_malloc)
 {
-int af = AF_INET6;
 int i, yield;
 char **addrlist;
 host_item *last = NULL;
@@ -1158,19 +1220,26 @@ round the loop twice. Note that we have ensured that AF_INET6 is defined
 even in an IPv4 world, which makes for slightly tidier code. */
 
 #if HAVE_IPV6
+int af = AF_INET6;
 int times = 2;
 #else
 int times = 1;
 #endif
 
+/* Initialize the flag that gets set for DNS syntax check errors, so that the
+interface to this function can be the same as host_find_bydns. */
+
+host_find_failed_syntax = FALSE;
+
 /* Loop to look up both kinds of address in an IPv6 world */
 
-for (i = 1; i <= times; af = AF_INET, i++)
+for (i = 1; i <= times; i++)
   {
   BOOL ipv4_addr;
 
   #if HAVE_IPV6
   struct hostent *hostdata = gethostbyname2(host->name, af);
+  af = AF_INET;
   #else
   struct hostent *hostdata = gethostbyname(host->name);
   #endif
@@ -1228,7 +1297,9 @@ NULL. */
 
 if (host->address == NULL) return HOST_FIND_FAILED;
 
-/* Now check to see if this is the local host. */
+/* Now check to see if this is the local host. This also checks for the
+presence of a host with an IP address, and returns HOST_FOUND, HOST_FOUND_LOCAL
+or HOST_FOUND_FAILED. */
 
 yield = host_scan_for_local_hosts(host, last, NULL);
 
@@ -1400,7 +1471,9 @@ Arguments:
   fully_qualified_name  if not NULL, return fully-qualified name
   removed               set TRUE if local host was removed from the list
 
-Returns:                HOST_FIND_FAILED  Failed to find the host or domain
+Returns:                HOST_FIND_FAILED  Failed to find the host or domain;
+                                          if there was a syntax error,
+                                          host_find_failed_syntax is set.
                         HOST_FIND_AGAIN   Could not resolve at this time
                         HOST_FOUND        Host found
                         HOST_FOUND_LOCAL  The lowest MX record points to this
@@ -1417,7 +1490,7 @@ host_find_bydns(host_item *host, BOOL mx_only, BOOL a_only,
 host_item *h, *last;
 dns_record *rr;
 int rc = DNS_FAIL;
-int mx_count, mx_addressed, yield;
+int yield;
 
 /* Set the default fully qualified name to the incoming name, initialize the
 resolver if necessary, and set up the relevant options. */
@@ -1425,7 +1498,10 @@ resolver if necessary, and set up the relevant options. */
 if (fully_qualified_name != NULL) *fully_qualified_name = host->name;
 dns_init(qualify_single, search_parents);
 
-/* Search the DNS for MX records, possibly via a CNAME. */
+/* Search the DNS for MX records, possibly via a CNAME. First, initialize the
+flag that gets set for DNS syntax check errors. */
+
+host_find_failed_syntax = FALSE;
 
 if (!a_only)
   {
@@ -1439,11 +1515,14 @@ if (!a_only)
   }
 
 /* If there were no MX records and mx_only is FALSE, or if a_only is TRUE,
-try for an A record. If we find it (or them) check to see that it isn't
-the local host. */
+try for an A record, unless there was a syntax error in the name that was used
+for the MX lookup. If we find it (or them) check to see that it isn't the local
+host. */
 
 if (rc != DNS_SUCCEED)
   {
+  if (host_find_failed_syntax) return HOST_FIND_FAILED;
+
   if (!mx_only || a_only)
     {
     host_item *next = host->next;
@@ -1486,7 +1565,7 @@ if (rc != DNS_SUCCEED)
 /* We have found one or more MX records. Sort them according to precedence, and
 create host_item blocks for them. For equal precedences one is supposed to
 randomize the order. To make this happen, the sorting is actually done on the
-MX valud * 100 + a random number. This is put into a host field called
+MX value * 100 + a random number. This is put into a host field called
 sort_key. Remove any duplicates that point to the same host, retaining only the
 one with the lowest precedence. We cannot yet check for precedence greater than
 that of the local host, because that test cannot be properly done until the
@@ -1494,11 +1573,10 @@ addresses have been found - an MX record may point to a name for this host
 which is not the primary hostname. */
 
 last = NULL;
-mx_count = mx_addressed = 0;
 
 for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL; rr = dns_next_rr(RESET_NEXT))
   {
-  int precedence, datalen;
+  int precedence;
   unsigned char *s;             /* MUST be unsigned for GETSHORT */
   char data[256];
 
@@ -1508,25 +1586,39 @@ for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL; rr = dns_next_rr(RESET_NEXT))
 
   /* Get the name of the host pointed to. */
 
-  datalen = dns_expand(s, data, 256);
+  (void)dns_expand(s, data, 256);
 
   /* Check that we haven't already got this host on the chain; if we have,
   keep only the lower precedence. This situation shouldn't occur, but you
-  never know what junk might get into the DNS. */
+  never know what junk might get into the DNS (and this case has been seen on
+  more than one occasion). */
 
-  if (last != NULL) for (h = host; h != last->next; h = h->next)
+  if (last != NULL)
     {
-    if (strcmpic(h->name, data) == 0)
+    host_item *prev = NULL;
+
+    for (h = host; h != last->next; prev = h, h = h->next)
       {
-      if (precedence < h->mx) h->mx = precedence;
-      goto NEXT_MX_RR;
+      if (strcmpic(h->name, data) == 0)
+        {
+        if (precedence >= h->mx) goto NEXT_MX_RR; /* Skip greater precedence */
+        if (h == host)                            /* Override first item */
+          {
+          h->mx = precedence;
+          host->sort_key = precedence * 100 + random_number(100);
+          goto NEXT_MX_RR;
+          }
+
+        /* Unwanted host item is not the first in the chain, so we can get
+        get rid of it by cutting it out. */
+
+        prev->next = h->next;
+        if (h == last) last = prev;
+        break;
+        }
       }
     }
 
-  /* Count the number of MX hosts we are setting up. */
-
-  mx_count++;
-
   /* If this is the first MX record, put the data into the existing host
   block. Otherwise, add a new block in the correct place; it it has to be
   before the first block, copy the first block's data to a new second block. */
@@ -1646,7 +1738,7 @@ for (rr = dns_next_rr(RESET_ADDITIONAL); rr != NULL;
     do
       {
       if (hh->address != NULL && strcmp(address, hh->address) == 0)
-        goto dns_next_rr;         /* Need goto to escape from inner loop */
+        goto DNS_NEXT_RR;         /* Need goto to escape from inner loop */
       thishostlast = hh;
       hh = hh->next;
       }
@@ -1656,7 +1748,10 @@ for (rr = dns_next_rr(RESET_ADDITIONAL); rr != NULL;
     an existing name. Create a copy of the current item, and give it
     the new address. RRs can be in arbitrary order, but one is supposed
     to randomize the addresses of multi-homed hosts, so compute a new
-    sorting key and do that. */
+    sorting key and do that. [Latest SMTP RFC says not to randomize multi-
+    homed hosts, but to rely on the resolver. I'm not happy about that -
+    caching in the resolver will not rotate as often as the nameserver
+    does.] */
 
     new_sort_key = h->mx * 100 + random_number(100);
     hh = store_get(sizeof(host_item));
@@ -1692,20 +1787,16 @@ for (rr = dns_next_rr(RESET_ADDITIONAL); rr != NULL;
     if (h == last) last = hh;       /* Inserted after last */
     }
 
-  /* The existing item doesn't have its address set yet. Set it, and
-  count the number of existing items we have filled in. */
+  /* The existing item doesn't have its address set yet, so just set it. */
 
-  else
-    {
-    h->address = address;
-    mx_addressed++;
-    }
+  else h->address = address;
 
-  /* Carry on to the next RR. It would be nice to be able to stop when
-  mx_addressed >= mx_count, but we can't be sure there won't be an additional
-  address for a multi-homed host further down the list. */
+  /* Carry on to the next RR. It would be nice to be able to be able to stop
+  when every host on the list has an address, but we can't be sure there won't
+  be an additional address for a multi-homed host further down the list, so
+  we have to continue to the end. */
 
-  dns_next_rr: continue;
+  DNS_NEXT_RR: continue;
   }
 
 /* Set the default yield to failure */
@@ -1717,27 +1808,19 @@ need to search for A or AAAA records explicitly. The names shouldn't point to
 CNAMES, but we use the general lookup function that handles them, just
 in case. If any lookup gives a soft error, change the default yield. */
 
-if (mx_addressed < mx_count)
+for (h = host; h != last->next; h = h->next)
   {
-  for (h = host; h != last->next; h = h->next)
+  if (h->address != NULL || h->status == hstatus_unusable) continue;
+  rc = set_address_from_dns(h, NULL);
+  if (rc != HOST_FOUND)
     {
-    if (h->address != NULL) continue;
-    rc = set_address_from_dns(h, NULL);
-
-    /* If the DNS lookup failed, mark this host item unusable and carry
-    on. If no addresses are found, we'll fail below. */
-
-    if (rc != HOST_FOUND)
+    h->status = hstatus_unusable;
+    if (rc == HOST_FIND_AGAIN)
       {
-      h->status = hstatus_unusable;
-      if (rc == HOST_FIND_AGAIN)
-        {
-        yield = rc;
-        h->why = hwhy_deferred;
-        }
-      else h->why = hwhy_failed;
+      yield = rc;
+      h->why = hwhy_deferred;
       }
-    else mx_addressed++;
+    else h->why = hwhy_failed;
     }
   }
 
@@ -1797,7 +1880,7 @@ host_check_rbl(char *address, char *domain, char **bufptr)
 {
 int bin[4];
 int rc;
-int type;
+int type = T_A;
 char name[256];
 char *t = name;
 
@@ -1805,6 +1888,11 @@ char *t = name;
 
 dns_init(FALSE, FALSE);
 
+/* If this is an IPv4 address mapped into IPv6 format, adjust the pointer
+to the IPv4 part only. */
+
+if (strncmp(address, "::ffff:", 7) == 0) address += 7;
+
 /* Handle IPv4 address: if HAVE_IPV6 is false, the result of
 host_aton() is always 1. */
 
@@ -1818,7 +1906,6 @@ if (host_aton(address, bin) == 1)
     while (*t) t++;
     x >>= 8;
     }
-  type = T_A;
   }
 
 /* Handle IPv6 address */
@@ -1843,10 +1930,9 @@ else
 
 
 strcpy(t, domain);
-HDEBUG(8) debug_printf("RBL lookup: %s\n", name);
 rc = dns_basic_lookup(name, type);
 
-/* If a buffer pointer exists, set it to point to the null string, or to a TXT
+/* Set the buffer pointer to point to the null string, or to a TXT
 record's data if present after success. */
 
 if (rc == DNS_SUCCEED)
@@ -1862,14 +1948,25 @@ if (rc == DNS_SUCCEED)
     if (rr != NULL)
       {
       int len = (rr->data)[0];
-      if (len > 127) len = 127;
+      if (len > 511) len = 127;
       *bufptr = string_sprintf("%.*s", len, (const char *)(rr->data + 1));
       }
 
-    DEBUG(8) debug_printf("RBL TXT: %s\n", *bufptr);
+    HDEBUG(8)
+      {
+      debug_printf("RBL lookup for %s succeeded\n", name);
+      debug_printf("=> that means it is black listed at %s\n", domain);
+      if (**bufptr != 0) debug_printf("%s\n", *bufptr);
+      }
     }
   }
 
+else HDEBUG(8)
+  {
+  debug_printf("RBL lookup for %s failed\n", name);
+  debug_printf("=> that means it's not black listed at %s\n", domain);
+  }
+
 return rc;
 }
 
@@ -1884,6 +1981,9 @@ return rc;
 
 #ifdef STAND_ALONE
 
+/* Note: an address for testing RBL is 192.203.178.39 */
+/* Note: an address for testing DUL is 192.203.178.4  */
+
 int main(int argc, char **argv)
 {
 host_item h;
@@ -1994,18 +2094,21 @@ while (fgets(buffer, 256, stdin) != NULL)
 
 printf("\n");
 
-printf("Testing host_find_byaddr\n");
+printf("Testing host_name_lookup\n");
 printf("> ");
 while (fgets(buffer, 256, stdin) != NULL)
   {
-  char *yield;
   int len = (int)strlen(buffer);
   while (len > 0 && isspace(buffer[len-1])) len--;
   buffer[len] = 0;
   if (strcmp(buffer, "q") == 0) break;
-  yield = host_find_byaddr(buffer, FALSE);      /* Debug causes printing */
-  if (yield == NULL)
-    printf("Lookup failed:%s\n", host_findbyaddr_msg);
+  sender_host_address = buffer;
+  sender_host_name = NULL;
+  sender_host_aliases = NULL;
+  host_lookup_msg = "";
+  host_lookup_failed = FALSE;
+  if (!host_name_lookup())  /* Debug causes printing */
+    printf("Lookup failed:%s\n", host_lookup_msg);
   printf("\n> ");
   }
 
