@@ -3,6 +3,7 @@
 **
 ** Author: Pr Emanuelsson <pell@lysator.liu.se>
 ** Hacked by: Peter Eriksson <pen@lysator.liu.se>
+** Also by: Philip Hazel <ph10@cus.cam.ac.uk>
 */
 #include <stdio.h>
 #include <ctype.h>
@@ -53,7 +54,7 @@ char *id_strtok __P3(char *, cp,
     {
 	while (*bp)
 	    bp++;
-	return cs;
+	return cp;  /* Fix by PH - was "return cs" */
     }
 
     /*
