@@ -60,16 +60,6 @@
 #endif /* HAVE_MMAP */
 
 
-/*
- * Compile these functions only if LOOKUP_CDB is defined. However,
- * some compilers don't like compiling empty modules, so keep them
- * happy with a dummy when skipping the rest.
- */
-
-#ifndef LOOKUP_CDB
-static void dummy(void) { dummy(); }
-#else
-
 #define CDB_HASH_SPLIT 256     /* num pieces the hash table is split into */
 #define CDB_HASH_MASK  255     /* mask to and off split value */
 #define CDB_HASH_ENTRY 8       /* how big each offset it */
@@ -289,13 +279,12 @@ int
 cdb_find(void *handle,
 	char *filename,
 	char *keystring,
-	int length,
+	int  key_len,
 	char **result,
 	char **errmsg)
 {
   struct cdb_state * cdbp = handle;
-  uint32 key_len,
-    item_key_len,
+  uint32 item_key_len,
     item_dat_len,
     key_hash,
     item_hash,
@@ -308,7 +297,6 @@ cdb_find(void *handle,
     hash_slotnm;
   int loop;
 
-  key_len = strlen(keystring);
   key_hash = cdb_hash((uschar *)keystring, key_len);
 
   hash_offset_entry = CDB_HASH_ENTRY * (key_hash & CDB_HASH_MASK);
@@ -446,6 +434,4 @@ struct cdb_state * cdbp = handle;
  store_free(cdbp);
 }
 
-#endif  /* LOOKUP_CDB */
-
 /* End of lookups/cdb.c */
