@@ -2,20 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
 #include "dbmdb.h"
 
-/* Compile these functions only if LOOKUP_DBM is defined. However, some
-compilers don't like compiling empty modules, so keep them happy with a
-dummy when skipping the rest. */
-
-#ifndef LOOKUP_DBM
-static void dummy(void) { dummy(); }
-#else
-
 
 /*************************************************
 *              Open entry point                  *
@@ -54,6 +46,7 @@ dbmdb_check(void *handle, char *filename, int modemask, int *owners,
   int *owngroups, char **errmsg)
 {
 int rc;
+handle = handle;    /* Keep picky compilers happy */
 
 #ifdef USE_DB
 rc = search_check_file(-1, filename, modemask, owners, owngroups,
@@ -88,7 +81,8 @@ return rc == 0;
 *              Find entry point                  *
 *************************************************/
 
-/* See local README for interface description */
+/* See local README for interface description. This function adds 1 to
+the keylength in order to include the terminating zero. */
 
 int
 dbmdb_find(void *handle, char *filename, char *keystring, int length,
@@ -97,6 +91,9 @@ dbmdb_find(void *handle, char *filename, char *keystring, int length,
 EXIM_DB *d = (EXIM_DB *)handle;
 EXIM_DATUM key, data;
 
+filename = filename;    /* Keep picky compilers happy */
+errmsg = errmsg;
+
 #ifdef EXIM_CLEAR_DATUMSTRUCT
 memset(&key, 0, sizeof(key));
 memset(&data, 0, sizeof(data));
@@ -106,7 +103,7 @@ EXIM_DATUM_DATA(key) = keystring;
 EXIM_DATUM_SIZE(key) = length + 1;
 if (EXIM_DBGET(d, key, data))
   {
-  *result = string_copy(EXIM_DATUM_DATA(data));
+  *result = string_copyn(EXIM_DATUM_DATA(data), EXIM_DATUM_SIZE(data));
   return OK;
   }
 return FAIL;
@@ -114,6 +111,21 @@ return FAIL;
 
 
 
+/*************************************************
+*      Find entry point - no zero on key         *
+*************************************************/
+
+/* See local README for interface description */
+
+int
+dbmnz_find(void *handle, char *filename, char *keystring, int length,
+  char **result, char **errmsg)
+{
+return dbmdb_find(handle, filename, keystring, length-1, result, errmsg);
+}
+
+
+
 /*************************************************
 *              Close entry point                 *
 *************************************************/
@@ -126,6 +138,4 @@ dbmdb_close(void *handle)
 EXIM_DBCLOSE((EXIM_DB *)handle);
 }
 
-#endif  /* LOOKUP_DBM */
-
 /* End of lookups/dbmdb.c */
