@@ -2,19 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
 #include "dnsdb.h"
 
-/* Compile these functions only if LOOKUP_DNSDB is defined. However, some
-compilers don't like compiling empty modules, so keep them happy with a
-dummy when skipping the rest. */
-
-#ifndef LOOKUP_DNSDB
-static void dummy(void) { dummy(); }
-#else
 
 
 /* Ancient systems (e.g. SunOS4) don't appear to have T_TXT defined in their
@@ -35,6 +28,8 @@ header files. */
 void *
 dnsdb_open(char *filename, char **errmsg)
 {
+filename = filename;   /* Keep picky compilers happy */
+errmsg = errmsg;       /* Ditto */
 return (void *)(-1);   /* Any non-0 value */
 }
 
@@ -52,7 +47,11 @@ dnsdb_find(void *handle, char *filename, char *keystring, int length,
 {
 int rc;
 dns_record *rr;
-char buffer[256];
+
+handle = handle;           /* Keep picky compilers happy */
+filename = filename;
+length = length;
+errmsg = errmsg;
 
 dns_init(FALSE, FALSE);    /* In case first time resolver is used. */
 
@@ -75,6 +74,4 @@ for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL;
 return OK;
 }
 
-#endif  /* LOOKUP_DNSDB */
-
 /* End of lookups/dnsdb.c */
