@@ -2,25 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
 #include "lsearch.h"
 
 
-/* Compile these functions only if LOOKUP_LSEARCH is defined. However, some
-compilers don't like compiling empty modules, so keep them happy with a
-dummy when skipping the rest. Also, some linkers are unhappy if an archive
-has no functions in it at all. This could happen if no lookup types at all are
-selected. That is most unlikely, but we might as well cater for it by making
-the dummy function here a global function. There is no need to do it for any
-other lookup type; one is enough. */
-
-#ifndef LOOKUP_LSEARCH
-void lsearch_dummy(void) { lsearch_dummy(); }
-#else
-
 
 /*************************************************
 *              Open entry point                  *
@@ -71,6 +59,9 @@ lsearch_find(void *handle, char *filename, char *keystring, int length,
 FILE *f = (FILE *)handle;
 char buffer[4096];
 
+filename = filename;  /* Keep picky compilers happy */
+errmsg = errmsg;
+
 rewind(f);
 while (fgets(buffer, sizeof(buffer), f) != NULL)
   {
@@ -134,6 +125,4 @@ lsearch_close(void *handle)
 fclose((FILE *)handle);
 }
 
-#endif  /* LOOKUP_LSEARCH */
-
 /* End of lookups/lsearch.c */
