@@ -2,18 +2,23 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
 #include "nisplus.h"
 
-/* Compile these functions only if LOOKUP_NISPLUS is defined. However, some
-compilers don't like compiling empty modules, so keep them happy with a
-dummy when skipping the rest. */
+/* We can't just compile this code and allow the library mechanism to omit the
+functions if they are not wanted, because we need to have the NIS+ header
+available for compiling. Therefore, compile these functions only if
+LOOKUP_NISPLUS is defined. However, some compilers don't like compiling empty
+modules, so keep them happy with a dummy when skipping the rest. Make it
+reference itself to stop picky compilers complaining that it is unused, and put
+in a dummy argument to stop even pickier compilers complaining about infinite
+loops. */
 
 #ifndef LOOKUP_NISPLUS
-static void dummy(void) { dummy(); }
+static void dummy(int x) { dummy(x-1); }
 #else
 
 
@@ -222,6 +227,40 @@ if (yield != NULL)
 return error_error;      /* FAIL or DEFER */
 }
 
+
+
+/*************************************************
+*               Quote entry point                *
+*************************************************/
+
+/* The only quoting that is necessary for NIS+ is to double any doublequote
+characters.
+
+Argument: the string to be quoted
+Returns:  the processed string
+*/
+
+char *
+nisplus_quote(char *s)
+{
+int count = 0;
+char *quoted;
+char *t = s;
+while (*t != 0) if (*t++ == '\"') count++;
+if (count == 0) return s;
+
+t = quoted = store_get((int)strlen(s) + count + 1);
+
+while (*s != 0)
+  {
+  *t++ = *s;
+  if (*s++ == '\"') *t++ = '\"';
+  }
+
+*t = 0;
+return quoted;
+}
+
 #endif  /* LOOKUP_NISPLUS */
 
 /* End of lookups/nisplus.c */
