@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -22,6 +22,12 @@ a string as a text string. This is sometimes useful for debugging output. */
 #define mac_expanded_string(s) mac_string(s)
 
 
+/* We need a special return code for "no recipients and failed to send an error
+message". ANSI C defines only EXIT_FAILURE and EXIT_SUCCESS. On the assumption
+that these are always 1 and 0 on Unix systems ... */
+
+#define EXIT_NORECIPIENTS 2
+
 /* Character-handling macros. It seems that the set of standard functions in
 ctype.h aren't actually all that useful. One reason for this is that email is
 international, so the concept of using a locale to vary what they do is not
@@ -125,10 +131,17 @@ enough to hold all the headers from a normal kind of message. */
 
 #define LOG_BUFFER_SIZE 8192
 
-/* The size of a big buffer for use in various places - usually for
-sprinting into before getting the correct store. */
+/* The initial size of a big buffer for use in various places. It gets put
+into big_buffer_size and in some circumstances increased. It should be at least
+as long as the maximum path length. */
 
+#if defined PATH_MAX && PATH_MAX > 1024
+#define BIG_BUFFER_SIZE PATH_MAX
+#elif defined MAXPATHLEN && MAXPATHLEN > 1024
+#define BIG_BUFFER_SIZE MAXPATHLEN
+#else
 #define BIG_BUFFER_SIZE 1024
+#endif
 
 /* A limit to the length of a sender address */
 
@@ -140,6 +153,11 @@ added to ensure it starts with a letter. */
 
 #define MESSAGE_ID_LENGTH 16
 
+/* The offset to the start of the data in the data file - this allows for
+the name of the data file to be present in the first line. */
+
+#define DATA_START_OFFSET (MESSAGE_ID_LENGTH+3)
+
 /* The length of the base names of spool files, which consist of an internal
 message id with a trailing "-H" or "-D" added. */
 
@@ -158,7 +176,7 @@ record. */
 /* Macros for trivial functions */
 
 #define mac_ismsgid(s) \
-  (pcre_exec(regex_ismsgid, NULL, s, (int)strlen(s), PCRE_EOPT, NULL, 0) >= 0)
+  (pcre_exec(regex_ismsgid,NULL,s,(int)strlen(s),0,PCRE_EOPT,NULL,0) >= 0)
 
 /* Macros for calling the memory allocation routines with
 tracing information for debugging. */
@@ -166,11 +184,12 @@ tracing information for debugging. */
 #define store_extend(addr,old,new) \
   store_extend_3(addr, old, new, __FILE__, __LINE__)
 
-#define store_free(addr)   store_free_3(addr, __FILE__, __LINE__)
-#define store_get(size)    store_get_3(size, __FILE__, __LINE__)
-#define store_malloc(size) store_malloc_3(size, __FILE__, __LINE__)
-#define store_mark(flag)   store_mark_3(flag, __FILE__, __LINE__)
-#define store_reset(addr)  store_reset_3(addr, __FILE__, __LINE__)
+#define store_free(addr)    store_free_3(addr, __FILE__, __LINE__)
+#define store_get(size)     store_get_3(size, __FILE__, __LINE__)
+#define store_malloc(size)  store_malloc_3(size, __FILE__, __LINE__)
+#define store_mark(flag)    store_mark_3(flag, __FILE__, __LINE__)
+#define store_release(addr) store_release_3(addr, __FILE__, __LINE__)
+#define store_reset(addr)   store_reset_3(addr, __FILE__, __LINE__)
 
 /* Options for dns_next_rr */
 
@@ -178,7 +197,7 @@ enum { RESET_NEXT, RESET_ANSWERS, RESET_ADDITIONAL };
 
 /* Argument values for the time-of-day function */
 
-enum { tod_log, tod_full, tod_bsdin };
+enum { tod_log, tod_full, tod_bsdin, tod_mbx };
 
 /* For identifying which kind of driver deferred */
 
@@ -195,10 +214,10 @@ enum {
   ERRMESS_BADARGADDRESS,    /* Bad address via argument list */
   ERRMESS_BADADDRESS,       /* Bad address read via -t */
   ERRMESS_NOADDRESS,        /* Message has no addresses */
+  ERRMESS_IGADDRESS,        /* All -t addresses ignored */
   ERRMESS_BADNOADDRESS,     /* Bad address via -t, leaving none */
-  ERRMESS_SPOOLWRITE,       /* Write to spool failed */
-  ERRMESS_MESSAGEREAD,      /* Read from input file failed */
-  ERRMESS_VLONGHEADER,      /* Excessively long header */
+  ERRMESS_IOERR,            /* I/O error while reading a message */
+  ERRMESS_VLONGHEADER,      /* Excessively long message header */
   ERRMESS_TOOBIG,           /* Message too big */
   ERRMESS_TOOMANYRECIP      /* Too many recipients */
 };
@@ -211,8 +230,9 @@ enum {
   ERRORS_STDERR             /* Write on stderr */
 };
 
-/* Returns from the directing, routing, and transport functions (not all
-apply to all of them) */
+/* Returns from the directing, routing, transport and authentication functions
+(not all apply to all of them). Some other functions also use these convenient
+values. */
 
 enum {
   OK,                       /* Success */
@@ -221,11 +241,15 @@ enum {
   DEFER,                    /* Failed soft */
   FAIL,                     /* Failed hard */
   FAIL_CONDITION,           /* Failed a condition test in a director */
+  FAILMORE,                 /* Fail, but override no_more */
   FORCEFAIL,                /* Failed, and don't pass on */
   ISLOCAL,                  /* Remote address turned out to be local */
   PASS,                     /* Pass to next - routers only */
   ERROR,                    /* Soft failed with internal or config error */
-  PANIC                     /* Hard failed with internal error */
+  PANIC,                    /* Hard failed with internal error */
+  BAD64,                    /* Bad base64 data (auth) */
+  UNEXPECTED,               /* Unexpected initial auth data */
+  FAIL_SEND                 /* send() failed in authenticator */
 };
 
 /* Returns from DNS lookup functions. */
@@ -260,27 +284,33 @@ to conflict with system errno values. */
 #define ERRNO_EXIMQUOTA      (-23)   /* Exim-imposed quota */
 #define ERRNO_HELD           (-24)
 #define ERRNO_FILTER_FAIL    (-25)   /* Delivery filter process failure */
-#define ERRNO_ADDHEADER_FAIL (-26)   /* Delivery add header failure */
+#define ERRNO_CHHEADER_FAIL  (-26)   /* Delivery add/remove header failure */
 #define ERRNO_WRITEINCOMPLETE (-27)  /* Delivery write incomplete error */
 #define ERRNO_EXPANDFAIL     (-28)   /* Some expansion failed */
 #define ERRNO_GIDFAIL        (-29)   /* Failed to get gid */
 #define ERRNO_UIDFAIL        (-30)   /* Failed to get uid */
+#define ERRNO_BADTRANSPORT   (-31)   /* Unset or non-existent transport */
+#define ERRNO_MBXLENGTH      (-32)   /* MBX length mismatch */
+#define ERRNO_UNKNOWNHOST    (-33)   /* Lookup failed in smtp transport */
+#define ERRNO_FORMATUNKNOWN  (-34)   /* Can't match format in appendfile */
+#define ERRNO_BADCREATE      (-35)   /* Creation outside home in appendfile */
 
 /* These must be last, so all retry deferments can easily be identified */
 
-#define ERRNO_RETRY_BASE     (-31)   /* Base to test against */
-#define ERRNO_DRETRY         (-31)   /* Not time for directing */
-#define ERRNO_RRETRY         (-32)   /* Not time for routing */
-#define ERRNO_LRETRY         (-33)   /* Not time for local delivery */
-#define ERRNO_HRETRY         (-34)   /* Not time for any remote host */
-#define ERRNO_LOCAL_ONLY     (-35)   /* Local-only delivery */
+#define ERRNO_RETRY_BASE     (-41)   /* Base to test against */
+#define ERRNO_DRETRY         (-41)   /* Not time for directing */
+#define ERRNO_RRETRY         (-42)   /* Not time for routing */
+#define ERRNO_LRETRY         (-43)   /* Not time for local delivery */
+#define ERRNO_HRETRY         (-44)   /* Not time for any remote host */
+#define ERRNO_LOCAL_ONLY     (-45)   /* Local-only delivery */
 
 /* Special actions to take after failure or deferment. */
 
 enum {
   SPECIAL_NONE,             /* No special action */
   SPECIAL_FREEZE,           /* Freeze message */
-  SPECIAL_FAIL              /* Fail the delivery */
+  SPECIAL_FAIL,             /* Fail the delivery */
+  SPECIAL_WARN              /* Send a warning message */
 };
 
 
@@ -305,10 +335,10 @@ enum { hn_bcc, hn_cc, hn_date, hn_from, hn_msgid, hn_sender, hn_to,
 
 /* Types of item in options lists */
 
-enum { opt_stringptr, opt_domainlist, opt_transportptr, opt_int,
+enum { opt_stringptr, opt_transportptr, opt_int,
   opt_octint, opt_mkint, opt_fixed, opt_time, opt_timelist, opt_bool,
-  opt_bool_verify, opt_uid, opt_gid, opt_uidlist, opt_gidlist, opt_expand_uid,
-  opt_expand_gid, opt_searchtype, opt_local_batch };
+  opt_bool_verify, opt_bool_set, opt_uid, opt_gid, opt_uidlist, opt_gidlist,
+  opt_expand_uid, opt_expand_gid, opt_searchtype, opt_local_batch };
 
 /* There's a high-ish bit which is used to flag duplicate options, kept
 for compatibility, which shouldn't be output. Also used for hidden options
@@ -331,9 +361,7 @@ enum { v_none, v_sender, v_recipient, v_expn };
 
 #define vopt_is_recipient   0x01
 #define vopt_local          0x02
-#define vopt_log_details    0x04
-#define vopt_address_test   0x08
-#define vopt_expn           0x10
+#define vopt_expn           0x04
 
 /* Options for lookup functions */
 
@@ -388,8 +416,9 @@ enum { self_freeze, self_defer, self_send, self_reroute, self_local,
 #define EXTRACTED_INCLUDEFAIL  (+1)    /* :include: failed */
 #define EXTRACTED_OK            0
 #define EXTRACTED_DEFER        (-1)    /* :defer: */
-#define EXTRACTED_FAIL         (-2)    /* :fail: */
-#define EXTRACTED_ERROR        (-3)    /* some problem */
+#define EXTRACTED_UNKNOWN      (-2)    /* :unknown: */
+#define EXTRACTED_FAIL         (-3)    /* :fail: */
+#define EXTRACTED_ERROR        (-4)    /* some problem */
 
 /* Flags for log_write() */
 
@@ -432,10 +461,10 @@ enum {
 order as for SMTP local batching above. */
 
 enum {
-  local_batch_off,       /* not doing batched delivery */
-  local_batch_one,       /* each address separate */
-  local_batch_domain,    /* batch identical domains */
-  local_batch_all        /* batch all addresses */
+  local_batch_off,      /* not doing batched delivery */
+  local_batch_one,      /* each address separate */
+  local_batch_domain,   /* batch identical domains */
+  local_batch_all       /* batch all addresses */
 };
 
 /* Returns from the spool_read_header() function */
@@ -449,13 +478,14 @@ enum {
 
 /* Options for transport_write_message */
 
-#define topt_add_return_path    0x01
-#define topt_add_delivery_date  0x02
-#define topt_add_envelope_to    0x04
-#define topt_from_hack          0x08
-#define topt_use_crlf           0x10
-#define topt_smtp_dots          0x20  /* Double start-line dots */
-#define topt_end_dot            0x40  /* Send terminting dot */
+#define topt_add_return_path    0x001
+#define topt_add_delivery_date  0x002
+#define topt_add_envelope_to    0x004
+#define topt_use_crlf           0x008  /* Terminate lines with CRLF */
+#define topt_end_dot            0x010  /* Send terminting dot line */
+#define topt_no_headers         0x020  /* Omit headers */
+#define topt_no_body            0x040  /* Omit body */
+#define topt_escape_headers     0x080  /* Apply escape check to headers */
 
 /* Flags for recipient_block */
 
