@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for parsing addresses */
@@ -51,7 +51,8 @@ return recipient;
 
 /* Scan over a string looking for the termination of an address at a comma,
 or end of the string. It's the source-routed addresses which cause much pain
-here.
+here. Although Exim ignores source routes, it must recognize such addresses, so
+we cannot get rid of this logic.
 
 Argument:
   s        pointer to the start of an address
@@ -80,7 +81,7 @@ while (*s != 0 && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
 
   else if (*s == '\"')
     {
-    while (*(++s) != 0)
+    while (*(++s) != 0 && (*s != '\n' || !nl_ends))
       {
       if (*s == '\\' && s[1] != 0) s++;
         else if (*s == '\"') { s++; break; }
@@ -93,7 +94,7 @@ while (*s != 0 && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
   else if (*s == '(')
     {
     int level = 1;
-    while (*(++s) != 0)
+    while (*(++s) != 0 && (*s != '\n' || !nl_ends))
       {
       if (*s == '\\' && s[1] != 0) s++;
         else if (*s == '(') level++;
@@ -379,10 +380,11 @@ for (;;)
   s = skip_comment(s);
 
   /* If we have read a null component at this point, give an error unless it is
-  the first component of the local part and an empty local part is permitted,
-  in which case just return normally. */
+  terminated by a dot - an extension to RFC 822 - or if it is the first
+  component of the local part and an empty local part is permitted, in which
+  case just return normally. */
 
-  if (t == tsave)
+  if (t == tsave && *s != '.')
     {
     if (t == tt && !allow_null)
       *error = "missing or malformed local part";
@@ -404,9 +406,9 @@ return s;
 *            Read route part of route-addr       *
 *************************************************/
 
-/* The pointer is at the initial "@" on entry. Return it following
-the terminating colon. If collapse_source_routes is set, return
-an empty string.
+/* The pointer is at the initial "@" on entry. Return it following the
+terminating colon. Exim no longer supports the use of source routes, but it is
+required to accept the syntax.
 
 Arguments:
   s          current character pointer
@@ -419,7 +421,7 @@ Returns:     new character pointer
 static uschar *
 read_route(uschar *s, uschar *t, char **errorptr)
 {
-uschar *t_init = t;
+BOOL commas = FALSE;
 *errorptr = NULL;
 
 while (*s == '@')
@@ -430,13 +432,22 @@ while (*s == '@')
   t += (int)strlen((const char *)t);
   if (*s != ',') break;
   *t++ = *s++;
+  commas = TRUE;
   s = skip_comment(s);
   }
 
 if (*s == ':') *t++ = *s++;
-  else *errorptr = "colon expected after route";
 
-if (collapse_source_routes) t = t_init;
+/* If there is no colon, and there were no commas, the most likely error
+is in fact a missing local part in the address rather than a missing colon
+after the route. */
+
+else *errorptr = commas?
+  "colon expected after route list" :
+  "no local part";
+
+/* Terminate the route and return */
+
 *t = 0;
 return skip_comment(s);
 }
@@ -498,9 +509,14 @@ point the error pointer to the reason. The arguments "start" and "end" are used
 to return the offsets of the first and one past the last characters in the
 original mailbox of the address that has been extracted, to aid in re-writing.
 The argument "domain" is set to point to the first character after "@" in the
-final part of the returned address, or zero if there is no @. Note: if the
-address is of the form @domain,...:route-addr "domain" refers to the final
-route-addr.
+final part of the returned address, or zero if there is no @.
+
+Exim no longer supports the use of source routed addresses (those of the form
+@domain,...:route_addr). It recognizes the syntax, but collapses such addresses
+down to their final components. Formerly, collapse_source_routes had to be set
+to achieve this effect. RFC 1123 allows collapsing with MAY, while the revision
+of RFC 821 had increased this to SHOULD, so I've gone for it, because it makes
+a lot of code elsewhere in Exim much simpler.
 
 There are some special fudges here for handling RFC 822 group address notation
 which may appear in certain headers. If the flag parse_allow_group is set
@@ -615,14 +631,15 @@ if (*s == '<')
   s = skip_comment(s);
 
   /* Read an optional series of routes, each of which is a domain. They
-  are separated by commas and terminated by a colon. If collapse_source_routes
-  is set, the syntax will be checked, but nothing will be returned in t. */
+  are separated by commas and terminated by a colon. However, we totally ignore
+  such routes (RFC 1123 says we MAY, and the revision of RFC 821 says we
+  SHOULD). */
 
   if (*s == '@')
     {
     s = read_route(s, t, errorptr);
     if (*errorptr != NULL) goto PARSE_FAILED;
-    t += (int)strlen((const char *)t);
+    *t = 0;                  /* Ensure route is ignored - probably overkill */
     source_routed = TRUE;
     }
 
@@ -662,14 +679,14 @@ if (*s == '<')
 addr-spec, on a strict reading of the RFC, and the rest of the string
 should be the domain. However, for flexibility we allow for a route-address
 not enclosed in <> as well, which is indicated by an empty first local
-part preceding '@'. */
+part preceding '@'. The source routing is, however, ignored. */
 
 else if (*t == 0)
   {
   uschar *domainptr = yield;
   s = read_route(s, t, errorptr);
   if (*errorptr != NULL) goto PARSE_FAILED;
-  t += (int)strlen((const char *)t);
+  *t = 0;         /* Ensure route is ignored - probably overkill */
   s = read_addr_spec(s, t, 0, errorptr, &domainptr);
   if (*errorptr != NULL) goto PARSE_FAILED;
   *domain = domainptr - yield;
@@ -962,8 +979,8 @@ Yield 0 if the string was successfully parsed; non-zero otherwise, with
 a message pointed to by the error variable. A positive value is given if
 the failure is simply a failure to open or subsequently stat an :include: file
 - some callers treat this as a soft error - and a negative value is given
-otherwise. The error numbers are now numbered, with special values for :fail:
-and :defer:.
+otherwise. The error numbers are now numbered, with special values for :fail:,
+:unknown: and :defer:.
 
 If optional is TRUE, then failure to generate any addresses, without a syntax
 error, is not an error.
@@ -976,13 +993,14 @@ Arguments:
                      .forward file or a list of entries in an alias file)
   incoming domain  Domain of the incoming address; used to qualify unqualified
                      local parts preceded by \
-  anchor           Where to hang the chain of newly-created addresses
+  anchor           Where to hang the chain of newly-created addresses. This
+                     should be initialized to NULL.
   error            Where to return an error text
   optional         If TRUE and no addresses generated, treat as success;
                      otherwise no addresses generated is an error
   allow_specials   If TRUE, allow :blackhole: to generate no addresses without
                      an error, even if optional is FALSE, and also recognize
-                     :defer: and :fail:
+                     :defer:, :unknown:, and :fail:
   rewrite          TRUE if generated addresses are to be passed through the
                      rewriting functions
   directory        If NULL, no checks are done on :include: files
@@ -1023,7 +1041,10 @@ for (;;)
   if (*s == 0)
     {
     if (count > 0 || optional) return EXTRACTED_OK;
-    *error = "no addresses generated";
+    *error = (syntax_errors != NULL && *syntax_errors != NULL)?
+      string_sprintf("no addresses generated: syntax error in %s: %s",
+        (*syntax_errors)->text2, (*syntax_errors)->text1) :
+      "no addresses generated";
     return EXTRACTED_ERROR;
     }
 
@@ -1071,8 +1092,8 @@ for (;;)
 
   /* Handle special addresses if permitted. If the address is :blackhole:
   then ignore it, and set the "optional" flag so that generating no addresses
-  at all in this expansion is no longer an error. For :defer: or :fail: we have
-  to set up the error message and give up right away. */
+  at all in this expansion is no longer an error. For :defer:, :unknown:, or
+  :fail: we have to set up the error message and give up right away. */
 
   if (allow_specials)
     {
@@ -1085,6 +1106,7 @@ for (;;)
       }
 
     if      (strncmp(s, ":defer:", 7) == 0) special = EXTRACTED_DEFER;
+    else if (strncmp(s, ":unknown:", 9) == 0) special = EXTRACTED_UNKNOWN;
     else if (strncmp(s, ":fail:", 6) == 0) special = EXTRACTED_FAIL;
 
     if (special != 0)
@@ -1368,7 +1390,7 @@ for (;;)
       /* Address was successfully parsed. Rewrite, and then make an address
       block in the MAIN storage pool. */
 
-      recipient = rewrite? rewrite_address(recipient, TRUE, TRUE) :
+      recipient = rewrite? rewrite_address(recipient, TRUE, FALSE) :
         rewrite_address_qualify(recipient, TRUE);
 
       store_pool = POOL_MAIN;
