@@ -7,11 +7,19 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
+#include <locale.h>
 
 /* Use the internal info for displaying the results of pcre_study(). */
 
 #include "internal.h"
+
+/* It is possible to compile this test program without including support for
+testing the POSIX interface, though this is not available via the standard
+Makefile. */
+
+#if !defined NOPOSIX
 #include "pcreposix.h"
+#endif
 
 #ifndef CLOCKS_PER_SEC
 #ifdef CLK_TCK
@@ -21,6 +29,8 @@
 #endif
 #endif
 
+#define LOOPREPEAT 20000
+
 
 static FILE *outfile;
 static int log_store = 0;
@@ -32,14 +42,15 @@ code as contained in pcre.c under the DEBUG macro. */
 
 static const char *OP_names[] = {
   "End", "\\A", "\\B", "\\b", "\\D", "\\d",
-  "\\S", "\\s", "\\W", "\\w", "Cut", "\\Z", "^", "$", "Any", "chars",
-  "not",
+  "\\S", "\\s", "\\W", "\\w", "\\Z", "\\z",
+  "Opt", "^", "$", "Any", "chars", "not",
   "*", "*?", "+", "+?", "?", "??", "{", "{", "{",
   "*", "*?", "+", "+?", "?", "??", "{", "{", "{",
   "*", "*?", "+", "+?", "?", "??", "{", "{", "{",
   "*", "*?", "+", "+?", "?", "??", "{", "{",
-  "class", "negclass", "Ref",
-  "Alt", "Ket", "KetRmax", "KetRmin", "Assert", "Assert not", "Once",
+  "class", "Ref",
+  "Alt", "Ket", "KetRmax", "KetRmin", "Assert", "Assert not",
+  "AssertB", "AssertB not", "Reverse", "Once", "Cond", "Cref",
   "Brazero", "Braminzero", "Bra"
 };
 
@@ -48,33 +59,49 @@ static void print_internals(pcre *re)
 {
 unsigned char *code = ((real_pcre *)re)->code;
 
-printf("------------------------------------------------------------------\n");
+fprintf(outfile, "------------------------------------------------------------------\n");
 
 for(;;)
   {
   int c;
   int charlength;
 
-  printf("%3d ", code - ((real_pcre *)re)->code);
+  fprintf(outfile, "%3d ", (int)(code - ((real_pcre *)re)->code));
 
   if (*code >= OP_BRA)
     {
-    printf("%3d Bra %d", (code[1] << 8) + code[2], *code - OP_BRA);
+    fprintf(outfile, "%3d Bra %d", (code[1] << 8) + code[2], *code - OP_BRA);
     code += 2;
     }
 
   else switch(*code)
     {
     case OP_END:
-    printf("    %s\n", OP_names[*code]);
-    printf("------------------------------------------------------------------\n");
+    fprintf(outfile, "    %s\n", OP_names[*code]);
+    fprintf(outfile, "------------------------------------------------------------------\n");
     return;
 
+    case OP_OPT:
+    fprintf(outfile, " %.2x %s", code[1], OP_names[*code]);
+    code++;
+    break;
+
+    case OP_COND:
+    fprintf(outfile, "%3d Cond", (code[1] << 8) + code[2]);
+    code += 2;
+    break;
+
+    case OP_CREF:
+    fprintf(outfile, " %.2d %s", code[1], OP_names[*code]);
+    code++;
+    break;
+
     case OP_CHARS:
     charlength = *(++code);
-    printf("%3d ", charlength);
+    fprintf(outfile, "%3d ", charlength);
     while (charlength-- > 0)
-      if (isprint(c = *(++code))) printf("%c", c); else printf("\\x%02x", c);
+      if (isprint(c = *(++code))) fprintf(outfile, "%c", c);
+        else fprintf(outfile, "\\x%02x", c);
     break;
 
     case OP_KETRMAX:
@@ -83,8 +110,15 @@ for(;;)
     case OP_KET:
     case OP_ASSERT:
     case OP_ASSERT_NOT:
+    case OP_ASSERTBACK:
+    case OP_ASSERTBACK_NOT:
     case OP_ONCE:
-    printf("%3d %s", (code[1] << 8) + code[2], OP_names[*code]);
+    fprintf(outfile, "%3d %s", (code[1] << 8) + code[2], OP_names[*code]);
+    code += 2;
+    break;
+
+    case OP_REVERSE:
+    fprintf(outfile, "%3d %s", (code[1] << 8) + code[2], OP_names[*code]);
     code += 2;
     break;
 
@@ -101,36 +135,36 @@ for(;;)
     case OP_TYPEQUERY:
     case OP_TYPEMINQUERY:
     if (*code >= OP_TYPESTAR)
-      printf("    %s", OP_names[code[1]]);
-    else if (isprint(c = code[1])) printf("    %c", c);
-      else printf("    \\x%02x", c);
-    printf("%s", OP_names[*code++]);
+      fprintf(outfile, "    %s", OP_names[code[1]]);
+    else if (isprint(c = code[1])) fprintf(outfile, "    %c", c);
+      else fprintf(outfile, "    \\x%02x", c);
+    fprintf(outfile, "%s", OP_names[*code++]);
     break;
 
     case OP_EXACT:
     case OP_UPTO:
     case OP_MINUPTO:
-    if (isprint(c = code[3])) printf("    %c{", c);
-      else printf("    \\x%02x{", c);
-    if (*code != OP_EXACT) printf(",");
-    printf("%d}", (code[1] << 8) + code[2]);
-    if (*code == OP_MINUPTO) printf("?");
+    if (isprint(c = code[3])) fprintf(outfile, "    %c{", c);
+      else fprintf(outfile, "    \\x%02x{", c);
+    if (*code != OP_EXACT) fprintf(outfile, ",");
+    fprintf(outfile, "%d}", (code[1] << 8) + code[2]);
+    if (*code == OP_MINUPTO) fprintf(outfile, "?");
     code += 3;
     break;
 
     case OP_TYPEEXACT:
     case OP_TYPEUPTO:
     case OP_TYPEMINUPTO:
-    printf("    %s{", OP_names[code[3]]);
-    if (*code != OP_TYPEEXACT) printf("0,");
-    printf("%d}", (code[1] << 8) + code[2]);
-    if (*code == OP_TYPEMINUPTO) printf("?");
+    fprintf(outfile, "    %s{", OP_names[code[3]]);
+    if (*code != OP_TYPEEXACT) fprintf(outfile, "0,");
+    fprintf(outfile, "%d}", (code[1] << 8) + code[2]);
+    if (*code == OP_TYPEMINUPTO) fprintf(outfile, "?");
     code += 3;
     break;
 
     case OP_NOT:
-    if (isprint(c = *(++code))) printf("    [^%c]", c);
-      else printf("    [^\\x%02x]", c);
+    if (isprint(c = *(++code))) fprintf(outfile, "    [^%c]", c);
+      else fprintf(outfile, "    [^\\x%02x]", c);
     break;
 
     case OP_NOTSTAR:
@@ -139,33 +173,32 @@ for(;;)
     case OP_NOTMINPLUS:
     case OP_NOTQUERY:
     case OP_NOTMINQUERY:
-    if (isprint(c = code[1])) printf("    [^%c]", c);
-      else printf("    [^\\x%02x]", c);
-    printf("%s", OP_names[*code++]);
+    if (isprint(c = code[1])) fprintf(outfile, "    [^%c]", c);
+      else fprintf(outfile, "    [^\\x%02x]", c);
+    fprintf(outfile, "%s", OP_names[*code++]);
     break;
 
     case OP_NOTEXACT:
     case OP_NOTUPTO:
     case OP_NOTMINUPTO:
-    if (isprint(c = code[3])) printf("    [^%c]{", c);
-      else printf("    [^\\x%02x]{", c);
-    if (*code != OP_NOTEXACT) printf(",");
-    printf("%d}", (code[1] << 8) + code[2]);
-    if (*code == OP_NOTMINUPTO) printf("?");
+    if (isprint(c = code[3])) fprintf(outfile, "    [^%c]{", c);
+      else fprintf(outfile, "    [^\\x%02x]{", c);
+    if (*code != OP_NOTEXACT) fprintf(outfile, ",");
+    fprintf(outfile, "%d}", (code[1] << 8) + code[2]);
+    if (*code == OP_NOTMINUPTO) fprintf(outfile, "?");
     code += 3;
     break;
 
     case OP_REF:
-    printf("    \\%d", *(++code));
+    fprintf(outfile, "    \\%d", *(++code));
     code++;
     goto CLASS_REF_REPEAT;
 
     case OP_CLASS:
-    case OP_NEGCLASS:
       {
       int i, min, max;
-      if (*code++ == OP_CLASS) printf("    [");
-        else printf("   ^[");
+      code++;
+      fprintf(outfile, "    [");
 
       for (i = 0; i < 256; i++)
         {
@@ -174,18 +207,18 @@ for(;;)
           int j;
           for (j = i+1; j < 256; j++)
             if ((code[j/8] & (1 << (j&7))) == 0) break;
-          if (i == '-' || i == ']') printf("\\");
-          if (isprint(i)) printf("%c", i); else printf("\\x%02x", i);
+          if (i == '-' || i == ']') fprintf(outfile, "\\");
+          if (isprint(i)) fprintf(outfile, "%c", i); else fprintf(outfile, "\\x%02x", i);
           if (--j > i)
             {
-            printf("-");
-            if (j == '-' || j == ']') printf("\\");
-            if (isprint(j)) printf("%c", j); else printf("\\x%02x", j);
+            fprintf(outfile, "-");
+            if (j == '-' || j == ']') fprintf(outfile, "\\");
+            if (isprint(j)) fprintf(outfile, "%c", j); else fprintf(outfile, "\\x%02x", j);
             }
           i = j;
           }
         }
-      printf("]");
+      fprintf(outfile, "]");
       code += 32;
 
       CLASS_REF_REPEAT:
@@ -198,16 +231,16 @@ for(;;)
         case OP_CRMINPLUS:
         case OP_CRQUERY:
         case OP_CRMINQUERY:
-        printf("%s", OP_names[*code]);
+        fprintf(outfile, "%s", OP_names[*code]);
         break;
 
         case OP_CRRANGE:
         case OP_CRMINRANGE:
         min = (code[1] << 8) + code[2];
         max = (code[3] << 8) + code[4];
-        if (max == 0) printf("{%d,}", min);
-        else printf("{%d,%d}", min, max);
-        if (*code == OP_CRMINRANGE) printf("?");
+        if (max == 0) fprintf(outfile, "{%d,}", min);
+        else fprintf(outfile, "{%d,%d}", min, max);
+        if (*code == OP_CRMINRANGE) fprintf(outfile, "?");
         code += 4;
         break;
 
@@ -220,12 +253,12 @@ for(;;)
     /* Anything else is just a one-node item */
 
     default:
-    printf("    %s", OP_names[*code]);
+    fprintf(outfile, "    %s", OP_names[*code]);
     break;
     }
 
   code++;
-  printf("\n");
+  fprintf(outfile, "\n");
   }
 }
 
@@ -248,7 +281,9 @@ compiled re. */
 
 static void *new_malloc(size_t size)
 {
-if (log_store) fprintf(outfile, "Store size request: %d\n", (int)size);
+if (log_store)
+  fprintf(outfile, "Memory allocation (code space): %d\n",
+    (int)((int)size - offsetof(real_pcre, code[0])));
 return malloc(size);
 }
 
@@ -266,6 +301,7 @@ int study_options = 0;
 int op = 1;
 int timeit = 0;
 int showinfo = 0;
+int showstore = 0;
 int posix = 0;
 int debug = 0;
 int done = 0;
@@ -280,7 +316,8 @@ outfile = stdout;
 
 while (argc > 1 && argv[op][0] == '-')
   {
-  if (strcmp(argv[op], "-s") == 0) log_store = 1;
+  if (strcmp(argv[op], "-s") == 0 || strcmp(argv[op], "-m") == 0)
+    showstore = 1;
   else if (strcmp(argv[op], "-t") == 0) timeit = 1;
   else if (strcmp(argv[op], "-i") == 0) showinfo = 1;
   else if (strcmp(argv[op], "-d") == 0) showinfo = debug = 1;
@@ -288,6 +325,12 @@ while (argc > 1 && argv[op][0] == '-')
   else
     {
     printf("*** Unknown option %s\n", argv[op]);
+    printf("Usage: pcretest [-d] [-i] [-p] [-s] [-t] [<input> [<output>]]\n");
+    printf("  -d   debug: show compiled code; implies -i\n"
+           "  -i   show information about compiled pattern\n"
+           "  -p   use POSIX interface\n"
+           "  -s   output store information\n"
+           "  -t   time compilation and execution\n");
     return 1;
     }
   op++;
@@ -320,9 +363,8 @@ if (argc > 2)
 
 pcre_malloc = new_malloc;
 
-/* Heading line, then prompt for first re if stdin */
+/* Heading line, then prompt for first regex if stdin */
 
-fprintf(outfile, "Testing Perl-Compatible Regular Expressions\n");
 fprintf(outfile, "PCRE version %s\n\n", pcre_version());
 
 /* Main loop */
@@ -331,17 +373,26 @@ while (!done)
   {
   pcre *re = NULL;
   pcre_extra *extra = NULL;
+
+#if !defined NOPOSIX  /* There are still compilers that require no indent */
   regex_t preg;
+#endif
+
   const char *error;
-  unsigned char *p, *pp;
+  unsigned char *p, *pp, *ppp;
+  unsigned const char *tables = NULL;
   int do_study = 0;
-  int do_debug = 0;
+  int do_debug = debug;
+  int do_G = 0;
+  int do_g = 0;
+  int do_showinfo = showinfo;
+  int do_showrest = 0;
   int do_posix = 0;
   int erroroffset, len, delimiter;
 
   if (infile == stdin) printf("  re> ");
   if (fgets((char *)buffer, sizeof(buffer), infile) == NULL) break;
-  if (infile != stdin) fprintf(outfile, (char *)buffer);
+  if (infile != stdin) fprintf(outfile, "%s", (char *)buffer);
 
   p = buffer;
   while (isspace(*p)) p++;
@@ -352,9 +403,9 @@ while (!done)
 
   delimiter = *p++;
 
-  if (isalnum(delimiter))
+  if (isalnum(delimiter) || delimiter == '\\')
     {
-    fprintf(outfile, "** Delimiter must not be alphameric\n");
+    fprintf(outfile, "** Delimiter must not be alphameric or \\\n");
     goto SKIP_DATA;
     }
 
@@ -362,7 +413,12 @@ while (!done)
 
   for(;;)
     {
-    while (*pp != 0 && *pp != delimiter) pp++;
+    while (*pp != 0)
+      {
+      if (*pp == '\\' && pp[1] != 0) pp++;
+        else if (*pp == delimiter) break;
+      pp++;
+      }
     if (*pp != 0) break;
 
     len = sizeof(buffer) - (pp - buffer);
@@ -379,9 +435,15 @@ while (!done)
       done = 1;
       goto CONTINUE;
       }
-    if (infile != stdin) fprintf(outfile, (char *)pp);
+    if (infile != stdin) fprintf(outfile, "%s", (char *)pp);
     }
 
+  /* If the first character after the delimiter is backslash, make
+  the pattern end with backslash. This is purely to provide a way
+  of testing for the error message when a pattern ends with backslash. */
+
+  if (pp[1] == '\\') *pp++ = '\\';
+
   /* Terminate the pattern at the delimiter */
 
   *pp++ = 0;
@@ -390,22 +452,47 @@ while (!done)
 
   options = 0;
   study_options = 0;
+  log_store = showstore;  /* default from command line */
+
   while (*pp != 0)
     {
     switch (*pp++)
       {
+      case 'g': do_g = 1; break;
       case 'i': options |= PCRE_CASELESS; break;
       case 'm': options |= PCRE_MULTILINE; break;
       case 's': options |= PCRE_DOTALL; break;
       case 'x': options |= PCRE_EXTENDED; break;
+
+      case '+': do_showrest = 1; break;
       case 'A': options |= PCRE_ANCHORED; break;
-      case 'D': do_debug = 1; break;
+      case 'D': do_debug = do_showinfo = 1; break;
       case 'E': options |= PCRE_DOLLAR_ENDONLY; break;
+      case 'G': do_G = 1; break;
+      case 'I': do_showinfo = 1; break;
+      case 'M': log_store = 1; break;
+
+#if !defined NOPOSIX
       case 'P': do_posix = 1; break;
+#endif
+
       case 'S': do_study = 1; break;
-      case 'I': study_options |= PCRE_CASELESS; break;
       case 'U': options |= PCRE_UNGREEDY; break;
       case 'X': options |= PCRE_EXTRA; break;
+
+      case 'L':
+      ppp = pp;
+      while (*ppp != '\n' && *ppp != ' ') ppp++;
+      *ppp = 0;
+      if (setlocale(LC_CTYPE, (const char *)pp) == NULL)
+        {
+        fprintf(outfile, "** Failed to set locale \"%s\"\n", pp);
+        goto SKIP_DATA;
+        }
+      tables = pcre_maketables();
+      pp = ppp;
+      break;
+
       case '\n': case ' ': break;
       default:
       fprintf(outfile, "** Unknown option '%c'\n", pp[-1]);
@@ -414,8 +501,10 @@ while (!done)
     }
 
   /* Handle compiling via the POSIX interface, which doesn't support the
-  timing, showing, or debugging options. */
+  timing, showing, or debugging options, nor the ability to pass over
+  local character tables. */
 
+#if !defined NOPOSIX
   if (posix || do_posix)
     {
     int rc;
@@ -438,23 +527,26 @@ while (!done)
   /* Handle compiling via the native interface */
 
   else
+#endif  /* !defined NOPOSIX */
+
     {
     if (timeit)
       {
       register int i;
       clock_t time_taken;
       clock_t start_time = clock();
-      for (i = 0; i < 4000; i++)
+      for (i = 0; i < LOOPREPEAT; i++)
         {
-        re = pcre_compile((char *)p, options, &error, &erroroffset);
+        re = pcre_compile((char *)p, options, &error, &erroroffset, tables);
         if (re != NULL) free(re);
         }
       time_taken = clock() - start_time;
-      fprintf(outfile, "Compile time %.2f milliseconds\n",
-        ((double)time_taken)/(4 * CLOCKS_PER_SEC));
+      fprintf(outfile, "Compile time %.3f milliseconds\n",
+        ((double)time_taken * 1000.0) /
+        ((double)LOOPREPEAT * (double)CLOCKS_PER_SEC));
       }
 
-    re = pcre_compile((char *)p, options, &error, &erroroffset);
+    re = pcre_compile((char *)p, options, &error, &erroroffset, tables);
 
     /* Compilation failed; go back for another re, skipping to blank line
     if non-interactive. */
@@ -478,16 +570,16 @@ while (!done)
           }
         fprintf(outfile, "\n");
         }
-      continue;
+      goto CONTINUE;
       }
 
     /* Compilation succeeded; print data if required */
 
-    if (showinfo || do_debug)
+    if (do_showinfo)
       {
       int first_char, count;
 
-      if (debug || do_debug) print_internals(re);
+      if (do_debug) print_internals(re);
 
       count = pcre_info(re, &options, &first_char);
       if (count < 0) fprintf(outfile,
@@ -505,6 +597,10 @@ while (!done)
             ((options & PCRE_DOLLAR_ENDONLY) != 0)? " dollar_endonly" : "",
             ((options & PCRE_EXTRA) != 0)? " extra" : "",
             ((options & PCRE_UNGREEDY) != 0)? " ungreedy" : "");
+
+        if (((((real_pcre *)re)->options) & PCRE_ICHANGED) != 0)
+          fprintf(outfile, "Case state changes\n");
+
         if (first_char == -1)
           {
           fprintf(outfile, "First char at start or follows \\n\n");
@@ -520,6 +616,16 @@ while (!done)
           else
             fprintf(outfile, "First char = %d\n", first_char);
           }
+
+        if (((((real_pcre *)re)->options) & PCRE_REQCHSET) != 0)
+          {
+          int req_char = ((real_pcre *)re)->req_char;
+          if (isprint(req_char))
+            fprintf(outfile, "Req char = \'%c\'\n", req_char);
+          else
+            fprintf(outfile, "Req char = %d\n", req_char);
+          }
+        else fprintf(outfile, "No req char\n");
         }
       }
 
@@ -533,12 +639,13 @@ while (!done)
         register int i;
         clock_t time_taken;
         clock_t start_time = clock();
-        for (i = 0; i < 4000; i++)
+        for (i = 0; i < LOOPREPEAT; i++)
           extra = pcre_study(re, study_options, &error);
         time_taken = clock() - start_time;
         if (extra != NULL) free(extra);
-        fprintf(outfile, "  Study time %.2f milliseconds\n",
-          ((double)time_taken)/(4 * CLOCKS_PER_SEC));
+        fprintf(outfile, "  Study time %.3f milliseconds\n",
+          ((double)time_taken * 1000.0)/
+          ((double)LOOPREPEAT * (double)CLOCKS_PER_SEC));
         }
 
       extra = pcre_study(re, study_options, &error);
@@ -550,7 +657,7 @@ while (!done)
       /* This looks at internal information. A bit kludgy to do it this
       way, but it is useful for testing. */
 
-      else if (showinfo || do_debug)
+      else if (do_showinfo)
         {
         real_pcre_extra *xx = (real_pcre_extra *)extra;
         if ((xx->options & PCRE_STUDY_MAPPED) == 0)
@@ -592,19 +699,24 @@ while (!done)
   for (;;)
     {
     unsigned char *q;
+    unsigned char *bptr = dbuffer;
     int count, c;
-    int offsets[30];
+    int copystrings = 0;
+    int getstrings = 0;
+    int getlist = 0;
+    int start_offset = 0;
+    int offsets[45];
     int size_offsets = sizeof(offsets)/sizeof(int);
 
     options = 0;
 
-    if (infile == stdin) printf("  data> ");
+    if (infile == stdin) printf("data> ");
     if (fgets((char *)buffer, sizeof(buffer), infile) == NULL)
       {
       done = 1;
       goto CONTINUE;
       }
-    if (infile != stdin) fprintf(outfile, (char *)buffer);
+    if (infile != stdin) fprintf(outfile, "%s", (char *)buffer);
 
     len = (int)strlen((char *)buffer);
     while (len > 0 && isspace(buffer[len-1])) len--;
@@ -658,20 +770,22 @@ while (!done)
         options |= PCRE_NOTBOL;
         continue;
 
-        case 'E':
-        options |= PCRE_DOLLAR_ENDONLY;
+        case 'C':
+        while(isdigit(*p)) n = n * 10 + *p++ - '0';
+        copystrings |= 1 << n;
         continue;
 
-        case 'I':
-        options |= PCRE_CASELESS;
+        case 'G':
+        while(isdigit(*p)) n = n * 10 + *p++ - '0';
+        getstrings |= 1 << n;
         continue;
 
-        case 'M':
-        options |= PCRE_MULTILINE;
+        case 'L':
+        getlist = 1;
         continue;
 
-        case 'S':
-        options |= PCRE_DOTALL;
+        case 'N':
+        options |= PCRE_NOTEMPTY;
         continue;
 
         case 'O':
@@ -691,6 +805,7 @@ while (!done)
     /* Handle matching via the POSIX interface, which does not
     support timing. */
 
+#if !defined NOPOSIX
     if (posix || do_posix)
       {
       int rc;
@@ -699,8 +814,8 @@ while (!done)
       if ((options & PCRE_NOTBOL) != 0) eflags |= REG_NOTBOL;
       if ((options & PCRE_NOTEOL) != 0) eflags |= REG_NOTEOL;
 
-      rc = regexec(&preg, (char *)dbuffer, sizeof(pmatch)/sizeof(regmatch_t),
-        pmatch, eflags);
+      rc = regexec(&preg, (const char *)bptr,
+        sizeof(pmatch)/sizeof(regmatch_t), pmatch, eflags);
 
       if (rc != 0)
         {
@@ -714,69 +829,157 @@ while (!done)
           {
           if (pmatch[i].rm_so >= 0)
             {
-            fprintf(outfile, "%2d: ", i);
+            fprintf(outfile, "%2d: ", (int)i);
             pchars(dbuffer + pmatch[i].rm_so,
               pmatch[i].rm_eo - pmatch[i].rm_so);
             fprintf(outfile, "\n");
+            if (i == 0 && do_showrest)
+              {
+              fprintf(outfile, " 0+ ");
+              pchars(dbuffer + pmatch[i].rm_eo, len - pmatch[i].rm_eo);
+              fprintf(outfile, "\n");
+              }
             }
           }
         }
       }
 
-    /* Handle matching via the native interface */
+    /* Handle matching via the native interface - repeats for /g and /G */
 
     else
+#endif  /* !defined NOPOSIX */
+
+    for (;;)
       {
       if (timeit)
         {
         register int i;
         clock_t time_taken;
         clock_t start_time = clock();
-        for (i = 0; i < 4000; i++)
-          count = pcre_exec(re, extra, (char *)dbuffer, len, options, offsets,
-            size_offsets);
+        for (i = 0; i < LOOPREPEAT; i++)
+          count = pcre_exec(re, extra, (char *)bptr, len,
+            (do_g? start_offset : 0), options, offsets, size_offsets);
         time_taken = clock() - start_time;
-        fprintf(outfile, "Execute time %.2f milliseconds\n",
-          ((double)time_taken)/(4 * CLOCKS_PER_SEC));
+        fprintf(outfile, "Execute time %.3f milliseconds\n",
+          ((double)time_taken * 1000.0)/
+          ((double)LOOPREPEAT * (double)CLOCKS_PER_SEC));
         }
 
-      count = pcre_exec(re, extra, (char *)dbuffer, len, options, offsets,
-        size_offsets);
+      count = pcre_exec(re, extra, (char *)bptr, len,
+        (do_g? start_offset : 0), options, offsets, size_offsets);
 
       if (count == 0)
         {
         fprintf(outfile, "Matched, but too many substrings\n");
-        count = size_offsets/2;
+        count = size_offsets/3;
         }
 
       if (count >= 0)
         {
         int i;
-        count *= 2;
-        for (i = 0; i < count; i += 2)
+        for (i = 0; i < count * 2; i += 2)
           {
           if (offsets[i] < 0)
             fprintf(outfile, "%2d: <unset>\n", i/2);
           else
             {
             fprintf(outfile, "%2d: ", i/2);
-            pchars(dbuffer + offsets[i], offsets[i+1] - offsets[i]);
+            pchars(bptr + offsets[i], offsets[i+1] - offsets[i]);
             fprintf(outfile, "\n");
+            if (i == 0)
+              {
+              start_offset = offsets[1];
+              if (do_showrest)
+                {
+                fprintf(outfile, " 0+ ");
+                pchars(bptr + offsets[i+1], len - offsets[i+1]);
+                fprintf(outfile, "\n");
+                }
+              }
             }
           }
+
+        for (i = 0; i < 32; i++)
+          {
+          if ((copystrings & (1 << i)) != 0)
+            {
+            char copybuffer[16];
+            int rc = pcre_copy_substring((char *)bptr, offsets, count,
+              i, copybuffer, sizeof(copybuffer));
+            if (rc < 0)
+              fprintf(outfile, "copy substring %d failed %d\n", i, rc);
+            else
+              fprintf(outfile, "%2dC %s (%d)\n", i, copybuffer, rc);
+            }
+          }
+
+        for (i = 0; i < 32; i++)
+          {
+          if ((getstrings & (1 << i)) != 0)
+            {
+            const char *substring;
+            int rc = pcre_get_substring((char *)bptr, offsets, count,
+              i, &substring);
+            if (rc < 0)
+              fprintf(outfile, "get substring %d failed %d\n", i, rc);
+            else
+              {
+              fprintf(outfile, "%2dG %s (%d)\n", i, substring, rc);
+              free((void *)substring);
+              }
+            }
+          }
+
+        if (getlist)
+          {
+          const char **stringlist;
+          int rc = pcre_get_substring_list((char *)bptr, offsets, count,
+            &stringlist);
+          if (rc < 0)
+            fprintf(outfile, "get substring list failed %d\n", rc);
+          else
+            {
+            for (i = 0; i < count; i++)
+              fprintf(outfile, "%2dL %s\n", i, stringlist[i]);
+            if (stringlist[i] != NULL)
+              fprintf(outfile, "string list not terminated by NULL\n");
+            free((void *)stringlist);
+            }
+          }
+
         }
       else
         {
-        if (count == -1) fprintf(outfile, "No match\n");
-          else fprintf(outfile, "Error %d\n", count);
+        if (start_offset == 0)
+          {
+          if (count == -1) fprintf(outfile, "No match\n");
+            else fprintf(outfile, "Error %d\n", count);
+          }
+        start_offset = -1;
+        }
+
+      if ((!do_g && !do_G) || start_offset <= 0) break;
+      if (do_G)
+        {
+        bptr += start_offset;
+        len -= start_offset;
         }
       }
     }
 
   CONTINUE:
+
+#if !defined NOPOSIX
   if (posix || do_posix) regfree(&preg);
+#endif
+
   if (re != NULL) free(re);
   if (extra != NULL) free(extra);
+  if (tables != NULL)
+    {
+    free((void *)tables);
+    setlocale(LC_CTYPE, "C");
+    }
   }
 
 fprintf(outfile, "\n");
