@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions that operate on the input queue. */
@@ -26,10 +26,11 @@ been delivered in the meantime by some other means.
 The global variables queue_force and queue_local may be set to cause forced
 deliveries or local-only deliveries, respectively.
 
-If deliver_selectstring is not NULL, skip messages whose recipients do not
-contain the string. As this option is typically used when a machine comes back
-online, we want to ensure that at least one delivery attempt takes place, so
-force the first one.
+If deliver_selectstring[_sender] is not NULL, skip messages whose recipients do
+not contain the string. As this option is typically used when a machine comes
+back online, we want to ensure that at least one delivery attempt takes place,
+so force the first one. The selecting string can optionally be a regex, or
+refer to the sender instead of recipients.
 
 If queue_2stage is set, the queue is scanned twice. The first time, queue_smtp
 is set so that routing is done for all messages. Thus in the second run those
@@ -46,28 +47,86 @@ Returns:     nothing
 void
 queue_run(char *start_id, char *stop_id, BOOL recurse)
 {
-BOOL force_delivery = queue_run_force || deliver_selectstring != NULL;
+BOOL force_delivery = queue_run_force || deliver_selectstring != NULL ||
+  deliver_selectstring_sender != NULL;
 queue_filename *f;
+pcre *selectstring_regex = NULL;
+pcre *selectstring_regex_sender = NULL;
+char *log_detail = NULL;
+
+/* Turn off the command line flag that causes remote processing not to happen,
+unless queue_run_local is set, in which case force it. Cancel any specific
+remote queue domains. Turn off the flag that causes SMTP deliveries not to
+happen, unless doing a 2-stage queue run, when the SMTP flag gets set. Save the
+queue_runner's pid and the flag that indicates any deliveries run directly from
+this process. Deliveries that are run by handing on TCP/IP channels have
+queue_run_pid set, but not queue_running. */
+
+queue_remote = queue_run_local;
+queue_remote_domains = NULL;
+
+queue_smtp = queue_2stage;
+queue_smtp_domains = NULL;
+
+queue_run_pid = getpid();
+queue_running = TRUE;
+
+/* Log the true start of a queue run, and fancy options */
 
 if (!recurse)
-  log_write(0, LOG_MAIN, "Start queue run: pid=%d%s%s", getpid(),
-    (deliver_selectstring == NULL)? "" : " -R ",
-    (deliver_selectstring == NULL)? "" : deliver_selectstring);
+  {
+  char extras[8];
+  char *p = extras;
 
-/* Turn off the flags that cause SMTP deliveries and remote processing not to
-happen, unless doing a 2-stage queue run, when the SMTP flag gets set. Set
-the flag that indicates a queue run is in progress. */
+  if (queue_2stage) *p++ = 'q';
+  if (queue_run_force) *p++ = 'f';
+  if (deliver_force_thaw) *p++ = 'f';
+  if (queue_run_local) *p++ = 'l';
+  *p = 0;
 
-queue_remote = FALSE;
-queue_smtp = queue_2stage;
-queue_running = TRUE;
+  p = big_buffer;
+  sprintf(p, "pid=%d", (int)queue_run_pid);
+  while (*p != 0) p++;
+
+  if (extras[0] != 0)
+    {
+    sprintf(p, " -q%s", extras);
+    while (*p != 0) p++;
+    }
+
+  if (deliver_selectstring != NULL)
+    {
+    sprintf(p, " -R%s %s", deliver_selectstring_regex? "r" : "",
+      deliver_selectstring);
+    while (*p != 0) p++;
+    }
+
+  if (deliver_selectstring_sender != NULL)
+    {
+    sprintf(p, " -S%s %s", deliver_selectstring_sender_regex? "r" : "",
+      deliver_selectstring_sender);
+    while (*p != 0) p++;
+    }
+
+  log_detail = string_copy(big_buffer);
+  log_write(log_queue_run_level, LOG_MAIN, "Start queue run: %s", log_detail);
+  }
+
+/* If deliver_selectstring is a regex, compile it. */
+
+if (deliver_selectstring != NULL && deliver_selectstring_regex)
+  selectstring_regex = regex_must_compile(deliver_selectstring, TRUE, FALSE);
+
+if (deliver_selectstring_sender != NULL && deliver_selectstring_sender_regex)
+  selectstring_regex_sender =
+    regex_must_compile(deliver_selectstring_sender, TRUE, FALSE);
 
 /* Run the queue. If a start or finish id is given, we must take the queue in
 its natural order. Otherwise "randomize" it so we don't always do things in the
 same order. If deliver_selectstring is set, check for an undelivered address
-that matches the string. This does mean we have to read the header file, and
-then again when actually delivering, but it's cheaper than forking a delivery
-process for each file. */
+or a sender address that matches the string. This does mean we have to read the
+header file, and then again when actually delivering, but it's cheaper than
+forking a delivery process for each file. */
 
 for (f = queue_get_spool_list(!queue_run_in_order &&
        start_id == NULL && stop_id == NULL);
@@ -78,6 +137,7 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
     {
     pid_t pid;
     int status;
+    int pfd[2];
     struct stat statbuf;
     char buffer[256];
 
@@ -85,58 +145,115 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
     sprintf(buffer, "%s/input/%s/%s", spool_directory, message_subdir, f->text);
     if (stat(buffer, &statbuf) < 0) continue;
 
-    /* Check for a matching address if deliver_selectstring is set. If so,
-    we do a fully delivery - don't want to omit other addresses since
-    their routing might trigger re-writing etc. */
+    /* Check for a matching address if deliver_selectstring[_sender} is set.
+    If so, we do a fully delivery - don't want to omit other addresses since
+    their routing might trigger re-writing etc. Ensure the store used in
+    reading the header file is scavenged. */
 
-    if (deliver_selectstring != NULL)
+    if (deliver_selectstring != NULL || deliver_selectstring_sender != NULL)
       {
-      BOOL found = FALSE;
       void *reset_point = store_get(0);
       if (spool_read_header(f->text, FALSE, TRUE) != spool_read_OK) continue;
 
-      if (recipients_list != NULL)
+      if (deliver_freeze && !deliver_force_thaw)
+        {
+        log_write(5, LOG_MAIN, "Message is frozen");
+        store_reset(reset_point);
+        continue;  /* With next message */
+        }
+
+      /* Sender matching */
+
+      if (deliver_selectstring_sender != NULL)
+        {
+        if (!(deliver_selectstring_sender_regex?
+              (pcre_exec(selectstring_regex_sender, NULL, sender_address,
+                (int)strlen(sender_address), 0, PCRE_EOPT, NULL, 0) >= 0)
+              :
+              (strstric(sender_address, deliver_selectstring_sender, FALSE) !=
+                NULL)))
+          {
+          DEBUG(9) debug_printf("%s: sender address did not match %s\n",
+            f->text, deliver_selectstring_sender);
+          store_reset(reset_point);
+          continue;     /* With next message */
+          }
+        }
+
+      /* Recipient matching */
+
+      if (deliver_selectstring != NULL)
         {
         int i;
         for (i = 0; i < recipients_count; i++)
           {
-          if (
-            strstric(recipients_list[i].address, deliver_selectstring, FALSE)
-              != NULL &&
-                tree_search_addr(tree_nonrecipients, recipients_list[i].address,
-                  FALSE) == NULL)
-            {
-            found = TRUE;
+          char *address = recipients_list[i].address;
+          if ((deliver_selectstring_regex?
+               (pcre_exec(selectstring_regex, NULL, address,
+                 (int)strlen(address), 0, PCRE_EOPT, NULL, 0) >= 0)
+               :
+               (strstric(address, deliver_selectstring, FALSE) != NULL))
+              &&
+              tree_search_addr(tree_nonrecipients, address, FALSE) == NULL)
             break;
-            }
+          }
+
+        if (i >= recipients_count)
+          {
+          DEBUG(9) debug_printf("%s: no recipient address matched %s\n",
+            f->text, deliver_selectstring);
+          store_reset(reset_point);
+          continue;      /* With next message */
           }
         }
 
+      /* Message matches -R and -S (whichever are set) */
+
       store_reset(reset_point);
-
-      /* If no addresses matched the string, skip this message. */
-
-      if (!found)
-        {
-        DEBUG(9) debug_printf("%s: no addresses matched %s\n",
-          f->text, deliver_selectstring);
-        continue;
-        }
       }
 
-    /* OK, got a message we want to deliver; cut the -H off the name first */
+    /* OK, got a message we want to deliver. Create a pipe which will
+    serve as a means of detecting when all the processes created by the
+    delivery process are finished. This is relevant when the delivery
+    process passes one or more SMTP channels on to its own children. The
+    pipe gets passed down; by reading on it here we detect when the last
+    descendent dies by the unblocking of the read. It's a pity that for
+    most of the time the pipe isn't used, but creating a pipe should be
+    pretty cheap. */
+
+    if (pipe(pfd) < 0)
+      {
+      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "failed to create pipe in queue "
+        "runner process %d: %s", queue_run_pid, strerror(errno));
+      }
+    queue_run_pipe = pfd[pipe_write];  /* To ensure it gets passed on. */
+
+    /* Make sure it isn't stdin. This seems unlikely, but just to be on the
+    safe side... */
+
+    if (queue_run_pipe == 0)
+      {
+      queue_run_pipe = dup(queue_run_pipe);
+      close(0);
+      }
+
+    /* Now deliver the message; get the id by cutting the -H off the file
+    name. */
 
     set_process_info("running queue: %s", f->text);
     f->text[SPOOL_NAME_LENGTH-2] = 0;
     if ((pid = fork()) == 0)
-      _exit(deliver_message(f->text, force_delivery, queue_run_local, FALSE));
-
+      _exit(deliver_message(f->text, force_delivery, FALSE));
     if (pid < 0)
-      log_write(0, LOG_PANIC_DIE, "fork of delivery process failed\n");
+      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "fork of delivery process from "
+        "queue runner %d failed\n", queue_run_pid);
 
+    /* Close the writing end of the synchronizing pipe in this process,
+    then wait for the first level process to terminate. */
+
+    close(pfd[pipe_write]);
     set_process_info("running queue: waiting for %s (%d)", f->text, pid);
     while (wait(&status) != pid);
-    set_process_info("running queue");
 
     /* A successful return means a delivery was attempted; turn off the
     force flag for any subsequent calls unless queue_force is set. */
@@ -147,14 +264,29 @@ for (f = queue_get_spool_list(!queue_run_in_order &&
     was too high to attempt a delivery. Abandon the queue run. A message
     has already been written to the log by the delivery process. */
 
-    if ((status & 0x00ff) == 0 && (status & 0xff00) == (DEFER << 8)) break;
+    else if ((status & 0x00ff) == 0)
+      {
+      if (((status & 0xff00) >> 8) == DEFER) break;
+      }
 
     /* If the process crashed, tell somebody */
 
-    if ((status & 0x00ff) != 0)
+    else
+      {
       log_write(0, LOG_MAIN|LOG_PANIC,
         "queue run: process %d crashed with signal %d while delivering %s",
         (int)pid, status & 0x00ff, f->text);
+      }
+
+    /* Before continuing, wait till the pipe gets closed at the far end. This
+    tells us that any children created by the delivery to re-use any SMTP
+    channels have all finished. Since no process actually writes to the pipe,
+    the mere fact that read() unblocks is enough. */
+
+    set_process_info("running queue: waiting for children of %d", pid);
+    (void)read(pfd[pipe_read], buffer, sizeof(buffer));
+    (void)close(pfd[pipe_read]);
+    set_process_info("running queue");
     }
   }
 
@@ -174,9 +306,7 @@ if (!recurse)
   {
   if (smtp_etrn_serialize && deliver_selectstring != NULL)
     transport_end_serialized("etrn-runs", deliver_selectstring);
-  log_write(0, LOG_MAIN, "End queue run: pid=%d%s%s", getpid(),
-    (deliver_selectstring == NULL)? "" : " -R ",
-    (deliver_selectstring == NULL)? "" : deliver_selectstring);
+  log_write(log_queue_run_level, LOG_MAIN, "End queue run: %s", log_detail);
   }
 }
 
@@ -193,8 +323,10 @@ the point is simply to ensure that the same order doesn't occur every time, in
 case a particular message is causing a remote MTA to barf - we would like to
 try other messages to that MTA first. If the argument is FALSE, sort the list
 according to the file name. This should give the order in which the messages
-arrived. It is used only for presentation to humans, so the insertion sort
-that it does is not part of the normal operational code.
+arrived. It is normally used only for presentation to humans, in which case the
+insertion sort that it does is not part of the normal operational code.
+However, if the queue_run_in order is set, or if a -q option specifies starting
+and stopping messages, the sort is required.
 
 If any single-character sub-directories are encountered in the initial
 scan, they are subsequently scanned for message files too. The returned
@@ -392,6 +524,8 @@ Arguments:
    option     0 => list top-level recipients, with "D" for those delivered
               1 => list only undelivered top-level recipients
               2 => as 0, plus any generated delivered recipients
+              If 8 is added to any of these values, the queue is listed in
+                random order.
    list       => first of any message ids to list
    count      count of message ids; 0 => all
 
@@ -404,13 +538,13 @@ queue_list(int option, char **list, int count)
 int i;
 int now = (int)time(NULL);
 void *reset_point;
-queue_filename *f;
+queue_filename *f = NULL;
 
 /* If given a list of messages, build a chain containing their ids. */
 
 if (count > 0)
   {
-  queue_filename *last;
+  queue_filename *last = NULL;
   for (i = 0; i < count; i++)
     {
     queue_filename *next =
@@ -423,9 +557,11 @@ if (count > 0)
     }
   }
 
-/* Otherwise get a list of the entire queue. */
+/* Otherwise get a list of the entire queue, in order if necessary. */
 
-else f = queue_get_spool_list(FALSE);
+else f = queue_get_spool_list(option >= 8);
+
+if (option >= 8) option -= 8;
 
 /* Now scan the chain and print information, resetting store used
 each time. If queue_list_requires_admin is TRUE and the caller is not
@@ -453,10 +589,15 @@ for (; f != NULL; f = f->next)
 
   if (env_read)
     {
+    int ptr;
+    FILE *jread;
     struct stat statbuf;
+
     sprintf(big_buffer, "%s/input/%s/%s", spool_directory, message_subdir,
       f->text);
-    big_buffer[(int)strlen(big_buffer)-1] = 'D';
+    ptr = (int)strlen(big_buffer)-1;
+    big_buffer[ptr] = 'D';
+
     if (stat(big_buffer, &statbuf) == 0) size = message_size + statbuf.st_size;
     i = (now - received_time)/60;  /* minutes on queue */
     if (i > 90)
@@ -465,6 +606,21 @@ for (; f != NULL; f = f->next)
       if (i > 72) printf("%2dd ", (i + 12)/24); else printf("%2dh ", i);
       }
     else printf("%2dm ", i);
+
+    /* Collect delivered addresses from any J file */
+
+    big_buffer[ptr] = 'J';
+    jread = fopen(big_buffer, "r");
+    if (jread != NULL)
+      {
+      while (fgets(big_buffer, big_buffer_size, jread) != NULL)
+        {
+        int n = (int)strlen(big_buffer);
+        big_buffer[n-1] = 0;
+        tree_add_nonrecipient(big_buffer+1, big_buffer[0] == 'Y');
+        }
+      fclose(jread);
+      }
     }
 
   fprintf(stdout, "%s ", string_format_size(size, big_buffer));
@@ -674,7 +830,8 @@ if (!admin_user && (action != MSG_REMOVE || real_uid != originator_uid))
 /* Set up the user name for logging. */
 
 pw = getpwuid(real_uid);
-username = (pw != NULL)? pw->pw_name : string_sprintf("uid %d", (int)real_uid);
+username = (pw != NULL)?
+  pw->pw_name : string_sprintf("uid %ld", (long int)real_uid);
 
 /* Take the necessary action. */
 
@@ -693,7 +850,7 @@ switch(action)
     deliver_freeze = TRUE;
     deliver_manual_thaw = FALSE;
     deliver_frozen_at = time(NULL);
-    if (spool_write_header(id) >= 0)
+    if (spool_write_header(id, FALSE) >= 0)
       {
       printf("is now frozen\n");
       log_write(0, LOG_MAIN, "frozen by %s", username);
@@ -701,9 +858,8 @@ switch(action)
     else
       {
       yield = FALSE;
-      printf("could not be frozen: failed to rewrite "
-        "spool header file\n");
-      log_write(0, LOG_MAIN, "failed to rewrite header file");
+      printf("could not be frozen: failed to rewrite spool header file: %s\n",
+        strerror(errno));
       }
     }
   break;
@@ -719,7 +875,7 @@ switch(action)
     {
     deliver_freeze = FALSE;
     deliver_manual_thaw = TRUE;
-    if (spool_write_header(id) >= 0)
+    if (spool_write_header(id, FALSE) >= 0)
       {
       printf("is no longer frozen\n");
       log_write(0, LOG_MAIN, "unfrozen by %s", username);
@@ -727,9 +883,8 @@ switch(action)
     else
       {
       yield = FALSE;
-      printf("could not be unfrozen: failed to rewrite "
-        "spool header file\n");
-      log_write(0, LOG_MAIN, "failed to rewrite header file");
+      printf("could not be unfrozen: failed to rewrite spool header file: %s\n",
+        strerror(errno));
       }
     }
   break;
@@ -786,7 +941,7 @@ switch(action)
     {
     tree_add_nonrecipient(recipients_list[i].address, FALSE);
     }
-  if (spool_write_header(id) >= 0)
+  if (spool_write_header(id, FALSE) >= 0)
     {
     printf("has been modified\n");
     for (i = 0; i < recipients_count; i++)
@@ -797,9 +952,7 @@ switch(action)
     {
     yield = FALSE;
     printf("- failed to rewrite spool header file "
-      "while marking all recipients delivered\n");
-    log_write(0, LOG_MAIN, "failed to rewrite header file while "
-      "marking all recipients delivered");
+      "while marking all recipients delivered_ %s\n", strerror(errno));
     }
   break;
 
@@ -888,15 +1041,13 @@ switch(action)
 
     if (yield)
       {
-      if (spool_write_header(id) >= 0)
+      if (spool_write_header(id, FALSE) >= 0)
         printf("has been modified\n");
       else
         {
         yield = FALSE;
-        printf("- failed to rewrite spool header file "
-          "while %s\n", doing);
-        log_write(0, LOG_MAIN, "failed to rewrite header file while %s",
-          doing);
+        printf("- failed to rewrite spool header file while %s: %s\n", doing,
+          strerror(errno));
         }
       }
     }
@@ -1013,4 +1164,56 @@ deliver_datafile = -1;
 return yield;
 }
 
+
+
+/*************************************************
+*       Check the queue_only_file condition      *
+*************************************************/
+
+/* The queue_only_file option forces certain kinds of queueing if a given file
+exists.
+
+Arguments:  none
+Returns:    nothing
+*/
+
+void
+queue_check_only(void)
+{
+BOOL *set;
+struct stat statbuf;
+char *s, *ss, *name;
+char buffer[1024];
+
+if (queue_only_file == NULL) return;
+
+s = queue_only_file;
+while ((ss = string_nextinlist(&s, ':', buffer, sizeof(buffer))) != NULL)
+  {
+  if (strncmp(ss, "remote", 6) == 0)
+    {
+    name = "queue_remote";
+    set = &queue_remote;
+    ss += 6;
+    }
+  else if (strncmp(ss, "smtp", 4) == 0)
+    {
+    name = "queue_smtp";
+    set = &queue_smtp;
+    ss += 4;
+    }
+  else
+    {
+    name = "queue_only";
+    set = &queue_only;
+    }
+
+  if (stat(ss, &statbuf) == 0)
+    {
+    *set = TRUE;
+    DEBUG(2) debug_printf("%s set because %s exists\n", name, ss);
+    }
+  }
+}
+
 /* End of queue.c */
