@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with retrying unsuccessful deliveries. */
@@ -17,7 +17,7 @@
 *************************************************/
 
 /* This function is passed a host_item which contains a host name and an
-address string. Its job is to set the status of the address if it is not
+IP address string. Its job is to set the status of the address if it is not
 already set (indicated by hstatus_unknown). The possible values are:
 
    hstatus_usable    the address is not listed in the unusable tree, and does
@@ -31,87 +31,135 @@ already set (indicated by hstatus_unknown). The possible values are:
    hstatus_unusable_expired  as above, but also the retry time has expired
                      for this address.
 
-If a retry record is retrieved from the hints database, the time of last
-trying is filled into the last_try field of the host block.
+If a host retry record is retrieved from the hints database, the time of last
+trying is filled into the last_try field of the host block. If a host is
+generally usable, a check is made to see if there is a retry delay on this
+specific message at this host.
 
 Arguments:
   host                pointer to a host item
   include_ip_address  TRUE to include the address in the key - this is
                         usual, but sometimes is not wanted
-  is_expired          set TRUE if the host has expired but is usable because
-                        its retry time has come; undefined except when the
-                        return is hstatus_usable
+  retry_host_key      where to put a pointer to the key for the host-specific
+                        retry record, if one is read and the host is usable
+  retry_message_key   where to put a pointer to the key for the message+host
+                        retry record, if one is read and the host is usable
 
-Returns:    NULL if no retry record was read; otherwise
-            pointer to the key name of the retry record
+Returns:    TRUE if the host has expired but is usable because
+             its retry time has come
 */
 
-char *
-retry_check_address(host_item *host, BOOL include_ip_address, BOOL *is_expired)
+BOOL
+retry_check_address(host_item *host, BOOL include_ip_address,
+  char **retry_host_key, char **retry_message_key)
 {
-char *key;
+BOOL yield = FALSE;
+time_t now = time(NULL);
+char *host_key, *message_key;
 open_db dbblock;
 open_db *dbm_file;
 tree_node *node;
-dbdata_retry *retry_record;
+dbdata_retry *host_retry_record, *message_retry_record;
+
+*retry_host_key = *retry_message_key = NULL;
 
 DEBUG(9) debug_printf("checking status of %s\n", host->name);
 
 /* Do nothing if status already set; otherwise initialize status as usable. */
 
-if (host->status != hstatus_unknown) return NULL;
+if (host->status != hstatus_unknown) return FALSE;
 host->status = hstatus_usable;
 
-/* Generate the key for the unusable tree and the retry database. */
+/* Generate the host key for the unusable tree and the retry database. Ensure
+host names are lower cased (that's what %S does). */
 
-key = include_ip_address?
-      string_sprintf("T:%s:%s", host->name, host->address) :
-      string_sprintf("T:%s", host->name);
+host_key = include_ip_address?
+  string_sprintf("T:%S:%s", host->name, host->address) :
+  string_sprintf("T:%S", host->name);
 
-/* Search the tree of unusable addresses. This is filled in when deliveries
+/* Generate the message-specific key */
+
+message_key = string_sprintf("%s:%s", host_key, message_id);
+
+/* Search the tree of unusable IP addresses. This is filled in when deliveries
 fail, because the retry database itself is not updated until the end of all
 deliveries (so as to do it all in one go). The tree records addresses that have
 become unusable during this delivery process (i.e. those that will get put into
 the retry database when it is updated). */
 
-node = tree_search(tree_unusable, key);
+node = tree_search(tree_unusable, host_key);
 if (node != NULL)
   {
   DEBUG(9) debug_printf("found in tree of unusables\n");
   host->status = (node->data.val > 255)?
     hstatus_unusable_expired : hstatus_unusable;
   host->why = node->data.val & 255;
-  return NULL;
+  return FALSE;
   }
 
 /* Open the retry database, giving up if there isn't one. Otherwise, search for
-the retry record, and then close the database again. */
+the retry records, and then close the database again. */
 
-if ((dbm_file = dbfn_open("retry", O_RDONLY, &dbblock)) == NULL) return NULL;
-retry_record = dbfn_read(dbm_file, key);
+if ((dbm_file = dbfn_open("retry", O_RDONLY, &dbblock)) == NULL) return FALSE;
+host_retry_record = dbfn_read(dbm_file, host_key);
+message_retry_record = dbfn_read(dbm_file, message_key);
 dbfn_close(dbm_file);
 
-/* If there's no retry record, return NULL. */
+/* Ignore the data if it is too old - too long since it was written */
 
-if (retry_record == NULL)
+if (host_retry_record == NULL)
   {
-  DEBUG(9) debug_printf("no retry record\n");
-  return NULL;
+  DEBUG(9) debug_printf("no host retry record\n");
+  }
+else if (now - host_retry_record->time_stamp > retry_data_expire)
+  {
+  host_retry_record = NULL;
+  DEBUG(9) debug_printf("host retry record too old\n");
   }
 
-/* Set the status according to the retry data, and return the key. If
-it is usable, set is_expired. */
-
-if (time(NULL) < retry_record->next_try && !deliver_force)
+if (message_retry_record == NULL)
   {
-  host->status = (retry_record->expired)?
-    hstatus_unusable_expired : hstatus_unusable;
-  host->why = hwhy_retry;
-  host->last_try = retry_record->last_try;
+  DEBUG(9) debug_printf("no message retry record\n");
+  }
+else if (now - message_retry_record->time_stamp > retry_data_expire)
+  {
+  message_retry_record = NULL;
+  DEBUG(9) debug_printf("message retry record too old\n");
   }
-else *is_expired = retry_record->expired;
 
-return key;
+/* If there's a host-specific retry record, check for reaching the retry
+time (or forcing). If not, set the appropriate unusable flag and return FALSE.
+Otherwise arrange to return TRUE if this is an expired host. */
+
+if (host_retry_record != NULL)
+  {
+  *retry_host_key = host_key;
+  if (time(NULL) < host_retry_record->next_try && !deliver_force)
+    {
+    host->status = (host_retry_record->expired)?
+      hstatus_unusable_expired : hstatus_unusable;
+    host->why = hwhy_retry;
+    host->last_try = host_retry_record->last_try;
+    return FALSE;
+    }
+  yield = host_retry_record->expired;
+  }
+
+/* It's OK to try the host. If there's a message-specific retry record, check
+for reaching its retry time (or forcing). If not, mark the host unusable. */
+
+if (message_retry_record != NULL)
+  {
+  *retry_message_key = message_key;
+  if (time(NULL) < message_retry_record->next_try && !deliver_force)
+    {
+    host->status = hstatus_unusable;
+    host->why = hwhy_retry;
+    return FALSE;
+    }
+  }
+
+return yield;
 }
 
 
@@ -123,27 +171,30 @@ return key;
 
 /* Retry items are chained onto an address when it is deferred either by
 a director, router, or transport, or if it succeeds or fails and there was a
-previous retry item that now needs to be deleted.
+previous retry item that now needs to be deleted. Sometimes there can be both
+kinds of item: for example, if routing was deferred but then succeeded, and
+delivery then deferred. In that case there is a delete item for the routing
+retry, and an updating item for the delivery.
 
-They are used at the end to update the retry database. The key name is built
-from a type letter and the domain, optionally with "<local_part>@" on the front
-- this is used for local destinations - or passed in externally for remote
-destinations, which might be more complex (SMTP destinations consist of the
-domain + the IP address).
+These items are used at the end of a delivery attempt to update the retry
+database. The key name is built from a type letter and the domain, optionally
+with "<local_part>@" on the front - this is used for local destinations - or
+passed in externally for remote destinations, which might be more complex (SMTP
+destinations consist of the domain + the IP address).
 
 Arguments:
   addr    the address block
   type    the type letter: one of 'D', 'R', or 'T'
   use_at  if TRUE, "addr->local_part@" is included after "type:"
   key     if NULL, addr->domain is used, otherwise key
-  delete  the delete flag, copied into the block
+  flags   delete, host, and message flags, copied into the block
 
 Returns:  nothing
 */
 
 void
 retry_add_item(address_item *addr, char *type, BOOL use_at, char *key,
-  BOOL delete)
+  int flags)
 {
 retry_item *rti = store_get(sizeof(retry_item));
 rti->next = addr->retries;
@@ -155,10 +206,10 @@ rti->destination = string_sprintf("%s:%s%s%s", type,
 rti->basic_errno = addr->basic_errno;
 rti->more_errno = addr->more_errno;
 rti->message = addr->message;
-rti->delete = delete;
+rti->flags = flags;
 
-DEBUG(7) debug_printf("added retry item for %s: errno=%d %d\n",
-  rti->destination, rti->basic_errno, rti->more_errno);
+DEBUG(7) debug_printf("added retry item for %s: errno=%d %d flags=%d\n",
+  rti->destination, rti->basic_errno, rti->more_errno, flags);
 }
 
 
@@ -217,13 +268,15 @@ for (yield = retries; yield != NULL; yield = yield->next)
   handling that specific error, and if so, check any additional
   error information if required. Special code is required for quota
   errors, as these can either be system quota errors, or Exim's own
-  quota imposition, which has a different error number. */
+  quota imposition, which has a different error number. Full partitions
+  are also treated in the same way as quota errors. */
 
   if (yield->basic_errno != 0)
     {
     if (yield->basic_errno == ERRNO_EXIMQUOTA)
       {
-      if ((basic_errno != ERRNO_EXIMQUOTA && basic_errno != errno_quota) ||
+      if ((basic_errno != ERRNO_EXIMQUOTA && basic_errno != errno_quota &&
+           basic_errno != ENOSPC) ||
           (yield->more_errno != 0 && yield->more_errno > more_errno))
         continue;
       }
@@ -239,14 +292,13 @@ for (yield = retries; yield != NULL; yield = yield->next)
   otherwise it's a remote host name. Either way, we can persuade the
   match_address function to handle it. */
 
-  if (match_address_list(destination, domain, dlist, &(yield->re), -1, 0))
-    break;
+  if (match_address_list(destination, domain, &dlist, -1, 0, FALSE)) break;
 
   /* If the match failed, have a go at matching on the alternative destination,
   which is normally the domain as opposed to the host for a remote delivery. */
 
   if (alternate != NULL &&
-      match_address_list(alternate, 0, dlist, &(yield->re), -1, 0)) break;
+      match_address_list(alternate, 0, &dlist, -1, 0, FALSE)) break;
   }
 
 *colon = replace;
@@ -260,9 +312,9 @@ return yield;
 *              Update retry database             *
 *************************************************/
 
-/* Update the retry data for any destinations that were deferred, or delete it
-for destinations that succeeded or failed hard. This is done all in one go to
-minimize opening/closing/locking of the database file.
+/* Update the retry data for any directing/routing/transporting that was
+deferred, or delete it for those that succeeded after a previous defer. This is
+done all in one go to minimize opening/closing/locking of the database file.
 
 Note that, because SMTP delivery involves a list of destinations to try, there
 may be defer-type retry information for some of them even when the message was
@@ -292,7 +344,7 @@ DEBUG(2) debug_printf("Processing retry items\n");
 
 /* Three-times loop to handle succeeded, failed, and deferred addresses.
 Deferred addresses must be handled after failed ones, because some may be moved
-to the failed chain if all their destinations have timed out. */
+to the failed chain if they have timed out. */
 
 for (i = 0; i < 3; i++)
   {
@@ -317,11 +369,14 @@ for (i = 0; i < 3; i++)
 
   while ((endaddr = *paddr) != NULL)
     {
-    BOOL timed_out = TRUE;
+    BOOL timed_out = FALSE;
     retry_item *rti;
 
     for (addr = endaddr; addr != NULL; addr = addr->parent)
       {
+      int update_count = 0;
+      int timedout_count = 0;
+
       DEBUG(5) debug_printf("%s%s\n", addr->orig, (addr->retries == NULL)?
         ": no retry items" : "");
 
@@ -349,38 +404,55 @@ for (i = 0; i < 3; i++)
           return;
           }
 
-        /* Handle the case of a request to delete the retry info for this
-        destination. Keep the timed_out flag for the end address only. */
+        /* If there are no deferred addresses, that is, if this message is
+        completing, and the retry item is for a message-specific SMTP error,
+        force it to be deleted, because there's no point in keeping data for
+        no-longer-existing messages. This situation can occur when a domain has
+        two hosts and a message-specific error occurs for the first of them,
+        but the address gets delivered to the second one. This optimization
+        doesn't succeed in cleaning out all the dead entries, but it helps. */
 
-        if (rti->delete)
+        if (*addr_defer == NULL && (rti->flags & rf_message) != 0)
+          rti->flags |= rf_delete;
+
+        /* Handle the case of a request to delete the retry info for this
+        destination. */
+
+        if ((rti->flags & rf_delete) != 0)
           {
           (void)dbfn_delete(dbm_file, rti->destination);
           DEBUG(5) debug_printf("deleted retry information for %s\n",
             rti->destination);
-          if (addr == endaddr) timed_out = FALSE;
           continue;
           }
 
+        /* Count the number of non-delete retry items. This is so that we
+        can compare it to the count of timed_out ones, to check whether
+        all are timed out. */
+
+        update_count++;
+
         /* Get the retry information for this destination and error code, if
         any. If this item is for a remote host with ip address, then pass
         the domain name as an alternative to search for. If no retry
         information is found, we can't generate a retry time, so there is
-        no point updating the database. */
+        no point updating the database. This retry item is timed out. */
 
         if ((retry = retry_find_config(rti->destination+2,
-             (strchr(rti->destination+2, ':') != NULL)? addr->domain : NULL,
+             ((rti->flags & rf_host) != 0)? addr->domain : NULL,
              rti->basic_errno, rti->more_errno)) == NULL)
           {
           DEBUG(5) debug_printf("No configured retry item for %s%s%s\n",
             rti->destination,
-            (strchr(rti->destination+2, ':') != NULL)? " or " : "",
-            (strchr(rti->destination+2, ':') != NULL)? addr->domain : "");
+            ((rti->flags & rf_host) != 0)? " or " : "",
+            ((rti->flags & rf_host) != 0)? addr->domain : "");
+          if (addr == endaddr) timedout_count++;
           continue;
           }
 
         DEBUG(5)
           {
-          if (strchr(rti->destination+2, ':') != NULL)
+          if ((rti->flags & rf_host) != 0)
             debug_printf("retry for %s (%s) = %s\n", rti->destination,
               addr->domain, retry->destination);
           else
@@ -391,12 +463,18 @@ for (i = 0; i < 3; i++)
         /* Set up the message for the database retry record. */
 
         message = (rti->basic_errno > 0)? strerror(rti->basic_errno) :
-          (rti->message == NULL)? "unknown error" : rti->message;
+          (rti->message == NULL)?
+          "unknown error" : string_printing(rti->message);
         message_length = strlen(message);
 
-        /* Read a retry record from the database or construct a new one */
+        /* Read a retry record from the database or construct a new one.
+        Ignore an old one if it is too old since it was last updated. */
 
         retry_record = dbfn_read(dbm_file, rti->destination);
+        if (retry_record != NULL &&
+            now - retry_record->time_stamp > retry_data_expire)
+          retry_record = NULL;
+
         if (retry_record == NULL)
           {
           retry_record = store_get(sizeof(dbdata_retry) + message_length);
@@ -435,18 +513,19 @@ for (i = 0; i < 3; i++)
 
         if (rule != NULL)
           {
-          if (addr == endaddr) timed_out = FALSE;
           retry_record->expired = FALSE;
           }
 
         /* Otherwise, set the retry timeout expired, and set the final rule
         as the one from which to compute the next retry time. Subsequent
-        messages will fail immediately until the retry time is reached. */
+        messages will fail immediately until the retry time is reached (unless
+        there are other, still active, retries). */
 
         else
           {
           rule = final_rule;
           retry_record->expired = TRUE;
+          if (addr == endaddr) timedout_count++;
           }
 
         /* There is a special case to consider when some messages get through
@@ -457,14 +536,18 @@ for (i = 0; i < 3; i++)
         stay on the queue longer than the final retry time. In order to
         avoid this, we check, using the time of arrival of the message, to
         see if it has been on the queue for more than the final cutoff time,
-        and if so, cause the message to get bounced, and the retry time to
+        and if so, cause this retry item to time out, and the retry time to
         be set to "now" so that any subsequent messages in the same condition
         also get tried. We search for the last rule onwards from the one that
-        is in use. If there are no retry rules, it will be null and timed_out
-        will already be set. */
+        is in use. If there are no retry rules for the item, rule will be null
+        and timedout_count will already have been updated.
+
+        This implements "timeout this rule if EITHER the host (or routing or
+        directing) has been failing for more than the maximum time, OR if the
+        message has been on the queue for more than the maximum time." */
 
         if (received_time < retry_record->first_failed &&
-            addr == endaddr && !timed_out && rule != NULL)
+            addr == endaddr && !retry_record->expired && rule != NULL)
           {
           retry_rule *last_rule;
           for (last_rule = rule;
@@ -472,7 +555,8 @@ for (i = 0; i < 3; i++)
                last_rule = last_rule->next);
           if (now - received_time > last_rule->timeout)
             {
-            timed_out = TRUE;
+            DEBUG(5) debug_printf("on queue longer than maximum retry\n");
+            timedout_count++;
             rule = NULL;
             }
           }
@@ -535,6 +619,15 @@ for (i = 0; i < 3; i++)
         dbfn_write(dbm_file, rti->destination, retry_record,
           sizeof(dbdata_retry) + message_length);
         }                            /* Loop for each retry item */
+
+      /* If all the non-delete retry items are timed out, the address is
+      timed out. */
+
+      if (update_count > 0 && update_count == timedout_count)
+        {
+        DEBUG(5) debug_printf("timed out: all retries expired\n");
+        timed_out = TRUE;
+        }
       }                              /* Loop for address & parents */
 
     /* If this is a deferred address, and retry processing was requested by
