@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with rewriting headers */
@@ -52,12 +52,11 @@ return (parse_find_at(s) != NULL)? s :
 *               Rewrite a single address         *
 *************************************************/
 
-/* The yield is the input address if there is no rewriting to be done. For
-source-routed addresses, it is only the final route-addr that gets considered.
-Assume the input is a valid address, except in the case of SMTP-time rewriting,
-which is handled specially. When this function is called while processing
-filter and forward files, the uid may be that of the user. Ensure it is reset
-while expanding a replacement, in case that involves file lookups.
+/* The yield is the input address if there is no rewriting to be done. Assume
+the input is a valid address, except in the case of SMTP-time rewriting, which
+is handled specially. When this function is called while processing filter and
+forward files, the uid may be that of the user. Ensure it is reset while
+expanding a replacement, in case that involves file lookups.
 
 Arguments:
   s           address to rewrite
@@ -66,7 +65,8 @@ Arguments:
   whole       if not NULL, set TRUE if any rewriting rule contained the
                 "whole" bit and it is a header that is being rewritten
   add_header  if TRUE and rewriting occurs, add an "X-rewrote-xxx" header
-                if headers are in existence
+                if headers are in existence; this should be TRUE only when
+                a message is being received, not during delivery
   name        name of header, for use when adding X-rewrote-xxxx
 
 Returns:      new address if rewritten; the input address if no change;
@@ -81,15 +81,13 @@ rewrite_rule *rule;
 char *yield = s;
 char *subject = s;
 char *domain = NULL;
-char *user = NULL;
 BOOL uid_checked = FALSE;
 BOOL uid_set = FALSE;
 BOOL done = FALSE;
 uid_t saved_euid = 0;
 gid_t saved_egid = 0;
-int pre_length = 0;
 int rule_number = 1;
-int yield_start, yield_end;
+int yield_start = 0, yield_end = 0;
 
 if (whole != NULL) *whole = FALSE;
 
@@ -119,45 +117,32 @@ for (rule = rewrite_rules;
 
   if ((flag & rewrite_smtp) != 0)
     {
-    if (!match_check_string(subject, rule->key, &(rule->re), 0, TRUE, NULL))
-      continue;
+    if (!match_check_string(subject, rule->key, 0, TRUE, FALSE, NULL)) continue;
     new = expand_string(rule->replacement);
     }
 
   /* All other rewrites expect the input to be a valid address, so local part
-  and domain variables can be set for expansion. */
+  and domain variables can be set for expansion. For the first rule, to be
+  applied to this address, domain will be NULL and needs to be set. */
 
   else
     {
-    /* If this is the first rule we are trying on this address, set domain
-    to point past the last '@' in the address and user to the start of the
-    address or following the last ':' if source-routed. Since Exim 0.56, the
-    parsing has insisted that there be a domain after the ':' in a source-
-    routed address. */
-
-    if (domain == NULL)
-      {
-      domain = strrchr(subject, '@') + 1;
-      if (subject[0] != '@')
-        {
-        user = subject;
-        pre_length = 0;
-        }
-      else
-        {
-        user = strchr(subject, ':') + 1;
-        pre_length = user - subject;
-        }
-      }
+    if (domain == NULL) domain = strrchr(subject, '@') + 1;
 
     /* Use the general function for matching an address against a list (here
     just one item, so set the separator to 0). */
 
-    if (!match_address_list(user, domain - user, rule->key, &(rule->re), 0, 0))
+    if (!match_address_list(subject, domain - subject, &(rule->key), 0, 0, FALSE))
       continue;
 
     /* The source address matches, and numerical variables have been
-    set up. Expand the replacement string, changing the uid/gid first if
+    set up. If the replacement string consists of precisely "*" then no
+    rewriting is required for this address - the behaviour is as for "fail"
+    in the replacement expansion, but assuming the quit flag. */
+
+    if (strcmp(rule->replacement, "*") == 0) break;
+
+    /* Otherwise, expand the replacement string, changing the uid/gid first if
     necessary. While doing the expansion, set $local_part and $domain to the
     appropriate values, restoring whatever value they previously had afterwards.
     */
@@ -174,20 +159,19 @@ for (rule = rewrite_rules;
         if (exim_gid_set) mac_setegid(exim_gid);
         mac_seteuid(uid);
         uid_set = TRUE;
-        DEBUG(2) debug_printf("rewrite_one set euid=%d egid=%d\n",
-          (int)geteuid(), (int)getegid());
+        DEBUG(2) debug_print_ids("rewrite_one");
         }
       }
 
     save_localpart = deliver_localpart;
     save_domain = deliver_domain;
 
-    /* We have user pointing to "localpart@domain" and domain pointing to
+    /* We have subject pointing to "localpart@domain" and domain pointing to
     the domain. Temporarily terminate the local part so that it can be
     set up as an expansion variable */
 
     domain[-1] = 0;
-    deliver_localpart = user;
+    deliver_localpart = subject;
     deliver_domain = domain;
 
     new = expand_string(rule->replacement);
@@ -211,16 +195,6 @@ for (rule = rewrite_rules;
     break;
     }
 
-  /* If this was a source-routed address, put back the initial part on the
-  front of what was generated. */
-
-  if (pre_length != 0)
-    {
-    char *fullnew = store_get(pre_length + (int)strlen(new) + 1);
-    sprintf(fullnew, "%.*s%s", pre_length, subject, new);
-    new = fullnew;
-    }
-
   /* Validate what has been generated, qualifying if permitted to do so. */
 
   if ((rule->flags & rewrite_qualify) != 0)
@@ -248,7 +222,11 @@ for (rule = rewrite_rules;
         yield, new, rule_number);
 
     /* A header will only actually be added if header_last is non-NULL,
-    i.e. during message reception or delivery. */
+    i.e. during message reception or delivery, but add_header should not
+    be set TRUE during delivery, as otherwise multiple instances of the header
+    can fill up the -H file and make it embarrassingly large. We don't need
+    to set header_changed because the -H file always gets written at the end of
+    message reception. */
 
     if (add_header)
       header_add(htype_old, "X-rewrote-%s: %s\n", name, subject);
@@ -329,8 +307,7 @@ if (uid_set)
   mac_seteuid(root_uid);
   mac_setegid(saved_egid);
   mac_seteuid(saved_euid);
-  DEBUG(2) debug_printf("restored euid=%d egid=%d\n", (int)geteuid(),
-    (int)getegid());
+  DEBUG(2) debug_print_ids("restored");
   }
 expand_nmax = -1;
 return yield;
@@ -352,7 +329,8 @@ and debugging purposes.
 Arguments:
   s             the address to be considered
   is_recipient  TRUE for recipient addresses; FALSE otherwise
-  add_header    add "X-rewrote-xxx" header when rewriting
+  add_header    add "X-rewrote-xxx" header when rewriting; this is
+                  set TRUE only for calls from the reception functions
 
 Returns:        possibly rewritten address
 */
@@ -365,7 +343,7 @@ s = rewrite_address_qualify(s, is_recipient);
 if ((rewrite_existflags & flag) != 0)
   {
   char *new = rewrite_one(s, flag, NULL, add_header, is_recipient?
-    "recipient" : "sender");
+    "original-recipient" : "sender");
   if (new != s) s = new;
   }
 return s;
@@ -418,13 +396,17 @@ DEBUG(5) debug_printf("rewrite_one_header: type=%c:\n  %s", h->type, h->text);
 
 parse_allow_group = TRUE;     /* Allow group syntax */
 
-/* Loop for multiple addresses in the header */
+/* Loop for multiple addresses in the header. We have to go through them all
+in case any need qualifying, even if there's no rewriting. Pathological headers
+may have thousands of addresses in them, so cause the store to be reset for
+any that don't actually get rewritten. */
 
 while (*s != 0)
   {
   char *sprev;
   char *ss = parse_find_address_end(s, FALSE);
   char *recipient, *new, *errmess;
+  void *reset_point = store_get(0);
   BOOL changed = FALSE;
   int terminator = *ss;
   int start, end, domain;
@@ -447,67 +429,32 @@ while (*s != 0)
 
   /* If routed_old is not NULL, this is a rewrite caused by a router,
   consisting of changing routed_old into routed_new, and applying to all
-  headers. The new value should always be fully qualified, but it may be
-  something that has an explicit re-write rule set, so we need to check the
-  configured rules subsequently as well. (Example: there's an explicit rewrite
-  turning *.foo.com into foo.com, and an address is supplied as abc@xyz, which
-  the DNS lookup turns into abc@xyz.foo.com). However, if no change is made
-  here, don't bother carrying on. */
+  headers. If the header address has no domain, it is excluded, since a router
+  rewrite affects domains only. The new value should always be fully qualified,
+  but it may be something that has an explicit re-write rule set, so we need to
+  check the configured rules subsequently as well. (Example: there's an
+  explicit rewrite turning *.foo.com into foo.com, and an address is supplied
+  as abc@xyz, which the DNS lookup turns into abc@xyz.foo.com). However, if no
+  change is made here, don't bother carrying on. */
 
   if (routed_old != NULL)
     {
-    new = NULL;
-
-    /* For source-routed addresses, the router will have been looking
-    at the first one. */
-
-    if (recipient[0] == '@')
+    if (domain <= 0 || strcmpic(recipient+domain, routed_old) != 0) continue;
+    recipient[domain-1] = 0;
+    new = string_sprintf("%s@%s", recipient, routed_new);
+    DEBUG(2)
       {
-      int firstterm;
-      char *t = strpbrk(recipient, ":,");
-      if (t == NULL) return NULL;  /* paranoid precaution */
-      firstterm = *t;
-      *t = 0;
-
-      if (strcmpic(recipient+1, routed_old) == 0)
-        {
-        *t = firstterm;
-        new = string_sprintf("@%s%s", routed_new, t);
-        DEBUG(2)
-          debug_printf("%s rewritten by router as %s\n", recipient, new);
-        }
+      recipient[domain-1] = '@';
+      debug_printf("%s rewritten by router as %s\n", recipient, new);
       }
-
-    /* For non-source-routed addresses, "domain" will be pointing to
-    the domain, and should be non-zero, since a router can't route without
-    a domain! */
-
-    else if (domain > 0 && strcmpic(recipient+domain, routed_old) == 0)
-      {
-      recipient[domain-1] = 0;
-      new = string_sprintf("%s@%s", recipient, routed_new);
-      DEBUG(2)
-        {
-        recipient[domain-1] = '@';
-        debug_printf("%s rewritten by router as %s\n", recipient, new);
-        }
-      }
-
-    /* If there isn't a new address, do no more on this iteration. */
-
-    if (new != NULL)
-      {
-      recipient = new;
-      changed = TRUE;
-      }
-    else continue;
+    recipient = new;
+    changed = TRUE;
     }
 
-
   /* This is not a router-inspired rewrite. Ensure the address is fully
   qualified if that is permitted. If an unqualified address was received
   from a host that isn't listed, do not continue rewriting this address.
-  Sender, From or Reply-to headers are treated as senders, the rest as
+  Sender, From or Reply-To headers are treated as senders, the rest as
   recipients. This matters only when there are different qualify strings. */
 
   else
@@ -619,6 +566,10 @@ while (*s != 0)
     newh = newnewh;
     }
 
+  /* If the address was not changed, reset so as to reuse the store it
+  was extracted into. */
+
+  else store_reset(reset_point);
   }
 
 parse_allow_group = FALSE;  /* Reset group flags */
@@ -709,13 +660,32 @@ void rewrite_test(char *s)
 {
 char *recipient, *error;
 int i, start, end, domain;
+BOOL done_smtp = FALSE;
+
+if (rewrite_existflags == 0)
+  {
+  printf("No rewrite rules are defined\n");
+  return;
+  }
+
+/* Do SMTP rewrite only if a rule with the S flag exists. */
 
 if ((rewrite_existflags & rewrite_smtp) != 0)
   {
   char *new = rewrite_one(s, rewrite_smtp, NULL, FALSE, "");
-  if (new != s) printf("    SMTP: %s\n", new);
+  if (new != s)
+    {
+    printf("    SMTP: %s\n", new);
+    done_smtp = TRUE;
+    }
   }
 
+/* Do the other rewrites only if a rule without the S flag exists */
+
+if ((rewrite_existflags & ~(rewrite_smtp|rewrite_X)) == 0) return;
+
+/* Qualify if necessary before extracting the address */
+
 if (parse_find_at(s) == NULL)
   s = string_sprintf("%s@%s", s, qualify_domain_recipient);
 
@@ -723,7 +693,8 @@ recipient = parse_extract_address(s, &error, &start, &end, &domain, FALSE);
 
 if (recipient == NULL)
   {
-  printf("Syntax error in address: %s\n", error);
+  if (!done_smtp)
+    printf("Syntax error in %s\n%c%s\n", s, toupper(error[0]), error+1);
   return;
   }
 
