@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with routing, and the list of generic router options. */
@@ -36,12 +36,6 @@ optionlist optionlist_routers[] = {
                  (void *)offsetof(router_instance, driver_name) },
   { "errors_to",          opt_stringptr|opt_public,
                  (void *)(offsetof(router_instance, errors_to)) },
-  { "except_domains",     opt_stringptr|opt_public,
-                 (void *)offsetof(router_instance, except_domains) },
-  { "except_local_parts", opt_stringptr|opt_public,
-                 (void *)offsetof(router_instance, except_local_parts) },
-  { "except_senders",     opt_stringptr|opt_public,
-                 (void *)offsetof(router_instance, except_senders) },
   { "fail_verify",        opt_bool_verify|opt_hidden|opt_public,
                  (void *)offsetof(router_instance, fail_verify_sender) },
   { "fail_verify_recipient", opt_bool|opt_public,
@@ -70,6 +64,10 @@ optionlist optionlist_routers[] = {
                  (void *)(offsetof(router_instance, self)) },
   { "senders",            opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, senders) },
+  #ifdef SUPPORT_TRANSLATE_IP_ADDRESS
+  { "translate_ip_address", opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, translate_ip_address) },
+  #endif
   { "transport",          opt_transportptr|opt_public,
                  (void *)offsetof(router_instance, transport) },
   { "unseen",             opt_bool|opt_public,
@@ -230,38 +228,56 @@ return TRUE;
 
 
 
+
 /*************************************************
-*             Set transport                      *
+*       Get transport for a driver               *
 *************************************************/
 
 /* If expand_transport is set on a driver, it must be expanded each time and
 used as a transport name. This action is identical for both routers and
-directors, and so the relevant data is in the same place in the options blocks
-for both. This function is therefore also called from direct_address().
+directors, and this function is called from both of them.
 
-Argument:
-  dr            points to the router (director) instance block
-  driver_type   "router" or "transport"
-  errmsg        where to put error message
+Arguments:
+  tp            the fixed transport from the driver (may be NULL, but see below)
+  expand_tp     the expansion string (may be NULL)
+  addr          the address being processed
+  tpptr         where to put the answer
+  driver_name   for error messages
+  require_name  if not NULL, fixed transport may not be NULL; this text
+                used in the error message
 
-Returns:        TRUE if successful; false if something went wrong
+Returns:        TRUE if a transport has been placed in tpptr; this may be NULL
+                if tp == expand_tp == require_name == NULL.
+                FALSE if there's a problem, in which case
+                addr->message contains a message, and addr->basic_errno has
+                ERRNO_BADTRANSPORT set in it.
 */
 
 BOOL
-route_set_transport(router_instance *dr, char *driver_type, char **errmsg)
+route_get_transport(transport_instance *tp, char *expand_tp, address_item *addr,
+  transport_instance **tpptr, char *driver_name, char *require_name)
 {
 char *ss;
-char *s = dr->expand_transport;
-transport_instance *tp;
 
-if (s == NULL) return TRUE;    /* Fixed transport may be set */
+/* If there's nothing to expand, use the fixed transport. For some calls, it is
+required to be set. */
 
-ss = expand_string(s);
+if (expand_tp == NULL)
+  {
+  *tpptr = tp;
+  if (tp != NULL || require_name == NULL) return TRUE;
+  addr->basic_errno = ERRNO_BADTRANSPORT;
+  addr->message = string_sprintf("%s unset in %s driver", require_name,
+    driver_name);
+  return FALSE;
+  }
+
+ss = expand_string(expand_tp);
 if (ss == NULL)
   {
-  *errmsg = string_sprintf("failed to expand dynamic transport "
-    "name \"%s\" for %s %s: %s", s, dr->name, driver_type,
-      expand_string_message);
+  addr->basic_errno = ERRNO_BADTRANSPORT;
+  addr->message = string_sprintf("failed to expand dynamic transport "
+    "\"%s\" in %s driver: %s", expand_tp, driver_name, expand_string_message);
   return FALSE;
   }
 
@@ -270,13 +286,14 @@ for (tp = transports; tp != NULL; tp = tp->next)
   if (strcmp(tp->name, ss) == 0)
     {
     DEBUG(9) debug_printf("dynamic transport \"%s\" set\n", ss);
-    dr->transport = tp;
+    *tpptr = tp;
     return TRUE;
     }
   }
 
-*errmsg = string_sprintf("dynamic transport \"%s\" not found "
-  "for %s %s", ss, dr->name, driver_type);
+addr->basic_errno = ERRNO_BADTRANSPORT;
+addr->message = string_sprintf("dynamic transport \"%s\" not found "
+  "in %s driver", ss, driver_name);
 return FALSE;
 }
 
@@ -308,7 +325,6 @@ BOOL
 route_skip_driver(router_instance *dr, char *local_part, char *domain,
   int verify, BOOL *pmore, char *driver_type)
 {
-
 /* Variables to hold data from lookup of a domain or local part. */
 
 domain_data = NULL;
@@ -325,41 +341,31 @@ if ((verify == v_none || verify == v_expn) && dr->verify_only)
 
 /* Skip this driver if domain mismatch */
 
-if (dr->domains != NULL &&
-     !match_isinlist_get(domain, dr->domains, &(dr->re_domains),
-       TRUE, &domain_data))
+if (dr->domains != NULL)
   {
-  DEBUG(9) debug_printf("%s %s skipped: domain mismatch\n",
-    dr->name, driver_type);
-  return TRUE;
-  }
-
-if (dr->except_domains != NULL &&
-     match_isinlist(domain, dr->except_domains, &(dr->re_except_domains), TRUE))
-  {
-  DEBUG(9) debug_printf("%s %s skipped: except domain mismatch\n",
-    dr->name, driver_type);
-  return TRUE;
+  char *domains = expand_string_panic(dr->domains, dr->name, driver_type);
+  if (!match_isinlist(domain, &domains, TRUE, TRUE, &domain_data))
+    {
+    DEBUG(9) debug_printf("%s %s skipped: domain mismatch\n",
+      dr->name, driver_type);
+    return TRUE;
+    }
   }
 
 /* Skip this driver if local part mismatch */
 
-if (dr->local_parts != NULL &&
-     !match_isinlist_get(local_part, dr->local_parts,
-       &(dr->re_local_parts), FALSE, &local_part_data))
+if (dr->local_parts != NULL)
   {
-  DEBUG(9) debug_printf("%s %s skipped: local part mismatch\n",
-    dr->name, driver_type);
-  return TRUE;
-  }
-
-if (dr->except_local_parts != NULL &&
-     match_isinlist(local_part, dr->except_local_parts,
-       &(dr->re_except_local_parts), FALSE))
-  {
-  DEBUG(9) debug_printf("%s %s skipped: except local part mismatch\n",
-    dr->name, driver_type);
-  return TRUE;
+  char *local_parts = expand_string_panic(dr->local_parts, dr->name,
+    driver_type);
+  if (!match_isinlist(local_part, &local_parts, FALSE,
+        (driver_type[0] == 'd' && locally_caseless),
+        &local_part_data))
+    {
+    DEBUG(9) debug_printf("%s %s skipped: local part mismatch\n",
+      dr->name, driver_type);
+    return TRUE;
+    }
   }
 
 /* If this driver's "more" flag is FALSE, arrange that no subsequent
@@ -391,6 +397,81 @@ return FALSE;
 
 
 
+/*************************************************
+*            Handle an unseen routing            *
+*************************************************/
+
+/* This function is called when an address is routed by a router with "unseen"
+set, or directed by a director with "unseen" set. It must make a clone of the
+address, for handling by subsequent drivers. The clone will fall through to the
+next router or director, by the anti-looping rules. The original address must
+be replaced by an invented "parent" which has the routed/directed address plus
+the clone as its children. This is necessary in case the address is at the top
+level - we don't want to mark it complete until both deliveries have been done.
+
+A new unique field must be made, so that the record of the delivery isn't a
+record of the original address, and checking for already delivered has
+therefore to be done here. If the delivery has happened, then take the base
+address off whichever delivery queue it is on - it will always be the top item.
+
+Arguments:
+  name         router or director name
+  addr         address that was routed/directed
+  addr_local   chain of local-delivery addresses
+  addr_remote  chain of remote-delivery addresses
+  addr_new     chain for newly created addresses
+
+Returns:       nothing
+*/
+
+void
+route_unseen(char *name, address_item *addr, address_item **addr_local,
+  address_item **addr_remote, address_item **addr_new)
+{
+address_item *parent = deliver_make_addr(addr->orig, TRUE);
+address_item *new = deliver_make_addr(addr->orig, TRUE);
+
+/* The invented parent is a copy that replaces the original; note that
+this copies its parent pointer. It has two children. */
+
+*parent = *addr;
+parent->child_count = 2;
+
+/* The routed/directed address gets a new parent. */
+
+addr->parent = parent;
+
+/* Set up the cloned address and put it on the new address queue */
+
+new->parent = parent;
+new->ignore_error |= addr->ignore_error;
+new->errors_address = addr->errors_address;
+new->next = *addr_new;
+*addr_new = new;
+
+DEBUG(2) debug_printf("\"unseen\" set: replicated %s\n", addr->orig);
+
+/* Make a new unique field, to distinguish from the normal one. */
+
+addr->unique = string_sprintf("%s/%s", addr->unique, name);
+
+/* If the address has been directed or routed to a transport, see if it
+was previously delivered. If so, we take it off the relevant queue so that
+it isn't delivered again. Otherwise, it was an alias or something, and the
+addresses it generated are handled in the normal way. */
+
+if (addr->transport != NULL &&
+    tree_search_addr(tree_nonrecipients, addr->unique, FALSE) != NULL)
+  {
+  DEBUG(2)
+    debug_printf("\"unseen\" delivery previously done - discarded\n");
+  parent->child_count--;
+  if (*addr_remote == addr) *addr_remote = addr->next;
+  if (*addr_local == addr) *addr_local = addr->next;
+  }
+}
+
+
 
 /*************************************************
 *                 Route one address              *
@@ -409,7 +490,8 @@ the local domains. The action can be one of:
   . return PASS - meaning the router did something to the address, and wants
       it passed to the next router.
 
-FAIL and PASS have the same effect, except for logging.
+Although FAIL and PASS are conceptually different, in fact they have the same
+effect, except for debugging output.
 
 The verify flag is set if this is being called for verification rather than
 delivery. If the router doesn't have its "verify" flag set, it is skipped.
@@ -453,8 +535,10 @@ for (r = routers; more && r != NULL; r = r->next)
   int yield = FALSE;
   int rc;
 
-  /* Check the conditions that are common to both routers and directors. */
+  /* Check the conditions that are common to both routers and directors. We
+  need the expansions set in case anything therein is expanded. */
 
+  deliver_set_expansions(addr);
   if (route_skip_driver(r, addr->local_part, addr->route_domain,
     verify, &more, "router")) continue;
 
@@ -480,12 +564,9 @@ for (r = routers; more && r != NULL; r = r->next)
     }
   if (loop_detected) continue;
 
-  /* Before calling the router, set up values that may be used in string
-  expansions and output any debugging string. Afterwards unset them, just in
-  case. Also, unset the rewrite flag; routers must explicitly request
-  rewriting. */
+  /* Before calling the router, output any debugging string. Also, unset the
+  rewrite flag; routers must explicitly request rewriting. */
 
-  deliver_set_expansions(addr);
   debug_print_string(r->debug_string);
   addr->rewrite_headers = FALSE;
 
@@ -507,11 +588,9 @@ for (r = routers; more && r != NULL; r = r->next)
     break;
 
     case OK:
-    if ((r->except_senders != NULL &&
-         (rc = match_sender(r->except_senders, &(r->re_except_senders)))
-            == OK) ||
-        (r->senders != NULL &&
-         (rc = match_sender(r->senders, &(r->re_senders))) != OK))
+    if (r->senders != NULL &&
+         (rc = match_sender(
+           expand_string_panic(r->senders, r->name, "router"))) != OK)
       {
       if (rc == DEFER)
         {
@@ -534,12 +613,7 @@ for (r = routers; more && r != NULL; r = r->next)
     else
       {
       addr->local_host_removed = FALSE;
-      if (!route_set_transport(r, "router", &(addr->message)))
-        {
-        yield = DEFER;
-        DEBUG(9) debug_printf("%s", addr->message);
-        }
-      else yield = (r->info->code)(r, addr, addr_local, addr_remote,
+      yield = (r->info->code)(r, addr, addr_local, addr_remote,
         verify != v_none);
       }
     break;
@@ -548,7 +622,7 @@ for (r = routers; more && r != NULL; r = r->next)
   deliver_set_expansions(NULL);
 
   /* If succeeded while verifying but fail_verify is set, convert into
-  a failure. */
+  a failure, and take it off the local or remote delivery list. */
 
   if (((verify == v_sender && r->fail_verify_sender) ||
        (verify == v_recipient && r->fail_verify_recipient)) &&
@@ -556,23 +630,11 @@ for (r = routers; more && r != NULL; r = r->next)
     {
     addr->message = string_sprintf("%s router forced verify failure",
       r->name);
+    if (*addr_remote == addr) *addr_remote = addr->next;
+    if (*addr_local == addr) *addr_local = addr->next;
     break;
     }
 
-  /* If router succeeded and "unseen" is set, make a new address with this
-  as parent, to be handled by other routers. */
-
-  if (yield == OK && r->unseen)
-    {
-    address_item *new = deliver_make_addr(addr->orig, TRUE);
-    new->parent = addr;
-    new->ignore_error |= addr->ignore_error;
-    new->errors_address = addr->errors_address;
-    addr->child_count++;
-    new->next = *addr_new;
-    *addr_new = new;
-    }
-
   /* Router modified the address; loop for next router. */
 
   if (yield == PASS)
@@ -582,8 +644,8 @@ for (r = routers; more && r != NULL; r = r->next)
     continue;
     }
 
-
-  /* Failed to handle this address; loop for next router. */
+  /* Failed to handle this address; loop for next router, respecting the
+  setting of "more". */
 
   if (yield == FAIL)
     {
@@ -591,6 +653,15 @@ for (r = routers; more && r != NULL; r = r->next)
     continue;
     }
 
+  /* Failed to handle this address; loop for next router, ignoring the
+  setting of "more". */
+
+  if (yield == FAILMORE)
+    {
+    DEBUG(2) debug_printf("%s router failed; \"more\" overridden\n", r->name);
+    more = TRUE;
+    continue;
+    }
 
   /* Router wants this address to be failed; do not loop for next router. */
 
@@ -600,7 +671,6 @@ for (r = routers; more && r != NULL; r = r->next)
     break;
     }
 
-
   /* Deferral means we are finished with this address, as does an internal
   or configuration failure. */
 
@@ -685,20 +755,86 @@ for (r = routers; more && r != NULL; r = r->next)
       r->name, old_route_domain);
     if (!addr->rewrite_headers && strcmp(addr->domain, old_route_domain) == 0)
       addr->domain = addr->route_domain;
+    addr->router = r;
     return ISLOCAL;
     }
 
 
   /* The only remaining possibility is that the router succeeded. */
 
+
+  /* If the translate_ip_address options is set and host addresses were
+  associated with the address, run them through the translation. This feature
+  is for weird and wonderful situations (the amateur packet radio people need
+  it) or very broken networking, so it is included in the binary only if
+  requested. */
+
+  #ifdef SUPPORT_TRANSLATE_IP_ADDRESS
+
+  if (r->translate_ip_address != NULL)
+    {
+    host_item *h;
+    for (h = addr->host_list; h != NULL; h = h->next)
+      {
+      if (h->address != NULL)
+        {
+        char *newaddress;
+        deliver_host_address = h->address;
+        newaddress = expand_string(r->translate_ip_address);
+        deliver_host_address = NULL;
+        if (newaddress == NULL)
+          {
+          if (!expand_string_forcedfail)
+            {
+            addr->basic_errno = ERRNO_EXPANDFAIL;
+            addr->message = string_sprintf("translate_ip_address expansion "
+              "failed: %s", expand_string_message);
+            return DEFER;
+            }
+          }
+        else
+          {
+          DEBUG(2) debug_printf("%s [%s] translated to %s\n",
+            h->name, h->address, newaddress);
+          if (string_is_ip_address(newaddress, NULL))
+            {
+            h->address = newaddress;
+            }
+          else
+            {
+            char *oldname = h->name;
+            char *oldaddress = h->address;
+
+            h->name = newaddress;
+            h->address = NULL;
+
+            switch (host_find_byname(h, NULL, TRUE))
+              {
+              case HOST_FOUND_LOCAL:
+              DEBUG(2) debug_printf("translated address is local host\n");
+              addr->router = r;
+              return ISLOCAL;
+
+              case HOST_FIND_FAILED:
+              addr->basic_errno = ERRNO_UNKNOWNHOST;
+              addr->message = string_sprintf("host %s not found when "
+                "translating %s [%s]", h->name, oldname, oldaddress);
+              return DEFER;
+              }
+            }
+          }
+        }
+      }
+    }
+  #endif  /* SUPPORT_TRANSLATE_IP_ADDRESS */
+
   DEBUG(2)
     {
-    debug_printf("routed by %s router:\n  deliver to ", r->name);
+    host_item *h;
 
-    if (addr->local_part[0] == ',' || addr->local_part[0] == ':')
-      debug_printf("@%s%s\n", addr->domain, addr->local_part);
-    else
-      debug_printf("%s@%s\n", addr->local_part, addr->domain);
+    debug_printf("routed by %s router%s:\n  deliver to %s@%s\n", r->name,
+      (r->unseen)? " (unseen)" : "",
+      addr->local_part, addr->domain);
 
     debug_printf("  transport: %s\n", (addr->transport == NULL)?
       "<none>" : addr->transport->name);
@@ -706,35 +842,43 @@ for (r = routers; more && r != NULL; r = r->next)
     if (addr->errors_address != NULL)
       debug_printf("  errors to %s\n", addr->errors_address);
 
-    if (addr->host_list != NULL)
+    for (h = addr->host_list; h != NULL; h = h->next)
       {
-      host_item *h;
-      for (h = addr->host_list; h != NULL; h = h->next)
-        {
-        debug_printf("  host %s ", h->name);
-        if (h->address != NULL)
-          debug_printf("[%s] ", h->address);
-        if (h->mx >= 0) debug_printf("MX=%d", h->mx);
-        debug_printf("\n");
-        }
+      debug_printf("  host %s ", h->name);
+      if (h->address != NULL)
+        debug_printf("[%s] ", h->address);
+      if (h->mx >= 0) debug_printf("MX=%d", h->mx);
+      debug_printf("\n");
       }
     }
 
   /* Record which router did it, and return success, having cleared any
-  temporary error message set by a router that failed. */
+  temporary error message set by a router that failed, and handled the
+  "unseen" option. */
 
   addr->router = r;
   addr->message = NULL;
+  if (r->unseen)
+    route_unseen(r->name, addr, addr_local, addr_remote, addr_new);
   return OK;
   }
 
 /* No routers accepted this address; fail it, with a default message if nothing
 is set. */
 
+DEBUG(2)
+  {
+  if (!more && r != NULL)
+    debug_printf("no_more set: routing abandoned\n");
+  else
+    debug_printf("end of routers reached\n");
+  }
+
 if (addr->message == NULL)
   addr->message = string_sprintf("unrouteable mail domain \"%s\"",
     addr->route_domain);
 
+deliver_set_expansions(NULL);    /* In case final one was skipped */
 return FAIL;
 }
 
