@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -34,6 +34,8 @@ optionlist domainlist_router_options[] = {
       (void *)(offsetof(domainlist_router_options_block, owners)) },
   { "owngroups",      opt_gidlist,
       (void *)(offsetof(domainlist_router_options_block, owngroups)) },
+  { "qualify_single",     opt_bool,
+      (void *)(offsetof(domainlist_router_options_block, qualify_single)) },
   { "route_file",     opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_file)) },
   { "route_list",     opt_stringptr,
@@ -42,6 +44,8 @@ optionlist domainlist_router_options[] = {
       (void *)(offsetof(domainlist_router_options_block, route_queries)) },
   { "route_query",    opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_query)) },
+  { "search_parents",     opt_bool,
+      (void *)(offsetof(domainlist_router_options_block, search_parents)) },
   { "search_type",    opt_searchtype,
       (void *)(offsetof(domainlist_router_options_block, search_type)) }
 };
@@ -61,12 +65,13 @@ domainlist_router_options_block domainlist_router_option_defaults = {
   hff_freeze,   /* host_find_failed code */
   NULL,         /* owners */
   NULL,         /* owngroups */
+  TRUE,         /* qualify_single */
+  FALSE,        /* search_parents */
   "freeze",     /* host_find_failed */
   NULL,         /* route_file */
   NULL,         /* route_queries */
   NULL,         /* route_query */
-  NULL,         /* route_list */
-  NULL          /* re_list */
+  NULL          /* route_list */
 };
 
 
@@ -103,9 +108,11 @@ else
 
 /* One of route_list or route_file or route_query must be specified */
 
-if (ob->route_file == NULL && ob->route_list == NULL && ob->route_query == NULL)
+if (ob->route_file == NULL && ob->route_list == NULL && ob->route_query == NULL
+  && ob->route_queries == NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
-    "either route_list or route_file or route_query required", rblock->name);
+    "either route_list, route_file, route_query, or route_queries required",
+    rblock->name);
 
 /* A search type is mandatory if a file name or query is given. */
 
@@ -252,10 +259,10 @@ BOOL individual_transport_set = FALSE;
 int lookup_type, rc;
 host_item *h, *next_h, *prev;
 char *routelist_item = NULL;
-char *domain, *hostlist, *newhostlist, *options, *name, *listptr;
+char *domain, *hostlist, *newhostlist, *options, *listptr;
 domainlist_router_options_block *ob =
   (domainlist_router_options_block *)(rblock->options_block);
-transport_instance *transport = rblock->transport;
+transport_instance *transport;
 char route_buffer[1024];
 
 DEBUG(2) debug_printf("%s router called for %s\n  route_domain = %s\n",
@@ -266,14 +273,9 @@ both are specified. The list takes precedence over the file. */
 
 if (ob->route_list != NULL)
   {
-  re_block **rechain = &(ob->re_list);
-
   listptr = ob->route_list;
-  for (routelist_item = string_nextinlist(&listptr, ';', route_buffer,
-         sizeof(route_buffer));
-       routelist_item != NULL;
-       routelist_item = string_nextinlist(&listptr, ';', route_buffer,
-         sizeof(route_buffer)))
+  while ((routelist_item = string_nextinlist(&listptr, ';', route_buffer,
+         sizeof(route_buffer))) != NULL)
     {
     DEBUG(9) debug_printf("routelist_item = %s\n", routelist_item);
     if (!parse_routelist_item(routelist_item, &domain, &hostlist, &options))
@@ -284,7 +286,8 @@ if (ob->route_list != NULL)
     thing and $1, $2, etc. as appropriate. If no match, advance the regular
     expression compiled pointer if we have just passed a regular expression. */
 
-    if (match_check_string(addr->route_domain, domain, rechain, 0, TRUE, NULL))
+    if (match_check_string(addr->route_domain, domain, 0, TRUE, TRUE,
+        &lookup_value))
       {
       matched = TRUE;
       break;
@@ -296,8 +299,6 @@ if (ob->route_list != NULL)
         addr->message = "lookup defer in route_list";
         return DEFER;
         }
-      if (domain[0] == '^' && *rechain != NULL)
-        rechain = &((*rechain)->next);
       }
     }
   }
@@ -380,6 +381,10 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL ||
   if (handle == NULL)
     {
     addr->message = string_sprintf("%s router: %s", rblock->name, error);
+
+    if (errno == ERRNO_BADMODE)
+      error = string_sprintf("%s (check the modemask option)", error);
+
     log_write(0, LOG_MAIN, "%s router: %s", rblock->name, error);
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
@@ -398,17 +403,17 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL ||
   if (ob->route_queries != NULL)
     {
     char *onequery;
-    for (onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE);
-         onequery != NULL;
-         onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE))
+    while ((onequery = string_nextinlist(&query, ':', big_buffer,
+           BIG_BUFFER_SIZE)) != NULL)
       {
-      routelist_item = search_find(handle, filename, onequery, ob->search_type,
+      routelist_item = search_find(handle, filename, onequery,
         ob->partial_match, &expand_setup, &error);
       if (routelist_item != NULL || search_find_defer) break;
       }
     }
-  else routelist_item = search_find(handle, filename, query, ob->search_type,
-    ob->partial_match, &expand_setup, &error);
+  else
+    routelist_item = search_find(handle, filename, query, ob->partial_match,
+      &expand_setup, &error);
 
   /* If we found the domain, parse the resulting data; in this case,
   parsing never fails as an empty entry is permitted. Turn on the
@@ -448,6 +453,7 @@ it must not be present. */
 DEBUG(9) debug_printf("original hostlist=%s options=%s\n", hostlist, options);
 
 newhostlist = expand_string_copy(hostlist);
+lookup_value = NULL;                        /* Finished with */
 expand_nmax = -1;
 
 /* If the expansion was forced to fail, just fail the router. Otherwise
@@ -526,13 +532,19 @@ rc = direct_get_munge_headers(addr, (director_instance *)rblock, "router",
   &(addr->extra_headers), &(addr->remove_headers));
 if (rc != OK) return rc;
 
+/* If an individual transport is not set, get the transport for this router, if
+any. It might be expanded, or it might be unset. */
+
+if (!individual_transport_set &&
+    !route_get_transport(rblock->transport, rblock->expand_transport,
+      addr, &transport, rblock->name, NULL))
+  return ERROR;
+
 /* Deal with the case of a local transport. A single host is allowed (can
 be used, for example, as an expansion in a file name). */
 
 if (transport != NULL && transport->info->local)
   {
-  ugid_block ugid;
-
   if (hostlist[0] != 0)
     {
     host_item *h;
@@ -568,10 +580,12 @@ if (transport != NULL && transport->info->local)
 
 
 /* There is either no transport or a remote transport. A host list is mandatory
-in either case. */
+in either case, except when verifying, in which case the address is just
+accepted. */
 
 if (hostlist[0] == 0)
   {
+  if (verify) goto ROUTED;
   log_write(0, LOG_MAIN, "Error in %s router: no host(s) specified for %s",
     rblock->name, domain);
   addr->message =
@@ -628,8 +642,8 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
     rc = host_find_byname(h, &canonical_name, FALSE);
   else
     rc = host_find_bydns(h, (lookup_type == lk_bydns_mx),
-      (lookup_type == lk_bydns_a), FALSE, FALSE, &canonical_name,
-      &(addr->local_host_removed));
+      (lookup_type == lk_bydns_a), ob->qualify_single, ob->search_parents,
+      &canonical_name, &(addr->local_host_removed));
 
   /* Temporary failure defers, unless pass_on_timeout is set */
 
@@ -648,7 +662,7 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
 
   if (rc == HOST_FIND_FAILED)
     {
-    if (ob->hff_code == hff_fail_soft) return FAIL;
+    if (ob->hff_code == hff_fail_soft) return FAILMORE;
 
     addr->message = string_sprintf("lookup of host \"%s\" failed in %s router",
       h->name, rblock->name);
@@ -684,9 +698,9 @@ for (h = addr->host_list; h != NULL; prev = h, h = next_h)
 
 /* Finally, since we have done all the routing here, there must be a transport
 defined for these hosts. It will be a remote one, as a local transport is
-dealt with above. */
+dealt with above. However, we don't need one if verifying only. */
 
-if (transport == NULL)
+if (transport == NULL && !verify)
     {
     log_write(0, LOG_MAIN, "Error in %s router: no transport defined",
       rblock->name);
@@ -697,6 +711,8 @@ if (transport == NULL)
 
 /* Fill in the transport, queue for remote delivery. */
 
+ROUTED:
+
 addr->transport = transport;
 addr->next = *addr_remote;
 addr->fallback_hosts = rblock->fallback_hostlist;
