@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the domainlist router */
@@ -16,12 +16,13 @@ typedef struct {
   int   hff_code;
   int  *owners;
   int  *owngroups;
+  BOOL qualify_single;
+  BOOL search_parents;
   char *host_find_failed;
   char *route_file;
   char *route_queries;
   char *route_query;
   char *route_list;
-  re_block *re_list;
 } domainlist_router_options_block;
 
 /* Data for reading the private options. */
