@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -73,7 +73,6 @@ ipliteral_router_entry(
   address_item **addr_remote,     /* add it to this if successful & remote */
   BOOL verify)                    /* TRUE if verifying */
 {
-transport_instance *transport = rblock->transport;
 /*
 ipliteral_router_options_block *ob =
   (ipliteral_router_options_block *)(rblock->options_block);
@@ -93,7 +92,7 @@ host name to "(unnamed)". */
 if (domain[0] != '[' || domain[len-1] != ']') return FAIL;
 domain[len-1] = 0;  /* temporarily */
 
-if (!string_is_ip_address(domain+1))
+if (!string_is_ip_address(domain+1, NULL))
   {
   domain[len-1] = ']';
   return FAIL;
@@ -142,7 +141,10 @@ just verifying, there need not be a transport, in which case it doesn't matter
 which queue we put the address on. This is all now handled by the route_queue()
 function. */
 
-addr->transport = transport;
+if (!route_get_transport(rblock->transport, rblock->expand_transport,
+      addr, &(addr->transport), rblock->name, NULL))
+  return ERROR;
+
 return route_queue(addr, addr_local, addr_remote, rblock, "router")?
   OK : ERROR;
 }
