@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -23,15 +23,17 @@ optionlist iplookup_router_options[] = {
       (void *)(offsetof(iplookup_router_options_block, hosts)) },
   { "optional", opt_bool,
       (void *)(offsetof(iplookup_router_options_block, optional)) },
+  { "port",     opt_int,
+      (void *)(offsetof(iplookup_router_options_block, service)) },
   { "protocol", opt_stringptr,
       (void *)(offsetof(iplookup_router_options_block, protocol_name)) },
-  { "query",   opt_stringptr,
+  { "query",    opt_stringptr,
       (void *)(offsetof(iplookup_router_options_block, query)) },
-  { "reroute", opt_stringptr,
+  { "reroute",  opt_stringptr,
       (void *)(offsetof(iplookup_router_options_block, reroute)) },
   { "response_pattern", opt_stringptr,
       (void *)(offsetof(iplookup_router_options_block, response_pattern)) },
-  { "service",  opt_int,
+  { "service",  opt_int | opt_hidden,
       (void *)(offsetof(iplookup_router_options_block, service)) },
   { "timeout",  opt_time,
       (void *)(offsetof(iplookup_router_options_block, timeout)) }
@@ -74,6 +76,10 @@ iplookup_router_init(router_instance *rblock)
 iplookup_router_options_block *ob =
   (iplookup_router_options_block *)(rblock->options_block);
 
+/* Don't rewrite sender domains as a result of this router */
+
+rblock->sender_rewrite = FALSE;
+
 /* A service and a host list must be given */
 
 if (ob->service < 0)
@@ -104,7 +110,8 @@ if (ob->protocol_name != NULL)
 /* If a response pattern is given, compile it now to get the error early. */
 
 if (ob->response_pattern != NULL)
-  ob->re_response_pattern = regex_must_compile(ob->response_pattern);
+  ob->re_response_pattern =
+    regex_must_compile(ob->response_pattern, FALSE, TRUE);
 }
 
 
@@ -136,6 +143,9 @@ int count, query_len, rc;
 int error_yield = (ob->optional)? PASS : DEFER;
 int yield = PASS;
 
+addr_local = addr_local;    /* Keep picky compilers happy */
+addr_remote = addr_remote;
+
 DEBUG(2) debug_printf("%s router called for %s: route_domain = %s\n",
   rblock->name, addr->orig, addr->route_domain);
 
@@ -168,11 +178,8 @@ being a host list. */
 net_port = htons(ob->service);
 listptr = ob->hosts;
 
-for (hostname = string_nextinlist(&listptr, ':', host_buffer,
-       sizeof(host_buffer));
-     hostname != NULL;
-     hostname = string_nextinlist(&listptr, ':', host_buffer,
-        sizeof(host_buffer)))
+while ((hostname = string_nextinlist(&listptr, ':', host_buffer,
+       sizeof(host_buffer))) != NULL)
   {
   host_item *h;
 
@@ -182,7 +189,7 @@ for (hostname = string_nextinlist(&listptr, ':', host_buffer,
   host->address = NULL;
   host->next = NULL;
 
-  if (string_is_ip_address(host->name)) host->address = host->name;
+  if (string_is_ip_address(host->name, NULL)) host->address = host->name;
     else if (host_find_byname(host, NULL, FALSE) == HOST_FIND_FAILED) continue;
 
   /* Loop for possible multiple IP addresses for the given name. */
@@ -362,13 +369,6 @@ if (hostname == NULL)
   }
 
 
-/* We have received a response from the querying service. This router might do
-routing according to the local part. Hence we mustn't subsequently copy its
-routing for another address. */
-
-addr->routed_by_domain = FALSE;
-
-
 /* If a response pattern was supplied, match the returned string against it. A
 failure to match causes the router to fail. After a successful match, the
 numerical variables for expanding the rerouted address are set up. */
@@ -429,7 +429,7 @@ else reroute = reply;
 
 /* We should now have a new address in the form user@domain. This must be
 used to update the local part, the domain that is being routed, and the
-actual address domain. */
+actual address domain, except when verifying. */
 
 domain = strchr(reroute, '@');
 if (domain == NULL)
@@ -467,7 +467,7 @@ if (rc != OK) return rc;
 /* Check whether the new domain is actually a local domain. If so, we have to
 pass back this address to the directors. */
 
-if (match_isinlist(domain, local_domains, &re_local_domains, TRUE))
+if (match_isinlist(domain, &local_domains, TRUE, TRUE, NULL))
   yield = ISLOCAL;
 
 /* Cancel numerical expansion variables, and return */
