@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -21,10 +21,6 @@ optionlist lookuphost_router_options[] = {
       (void *)(offsetof(lookuphost_router_options_block, gethostbyname)) },
   { "mx_domains",         opt_stringptr,
       (void *)(offsetof(lookuphost_router_options_block, mx_domains)) },
-  { "mx_domains_except",  opt_stringptr,
-      (void *)(offsetof(lookuphost_router_options_block, mx_domains_except)) },
-  { "non_mx_domains",     opt_stringptr|opt_hidden,
-      (void *)(offsetof(lookuphost_router_options_block, mx_domains_except)) },
   { "qualify_single",     opt_bool,
       (void *)(offsetof(lookuphost_router_options_block, qualify_single)) },
   { "rewrite_headers",    opt_bool,
@@ -50,10 +46,7 @@ lookuphost_router_options_block lookuphost_router_option_defaults = {
   FALSE,           /* search_parents */
   TRUE,            /* rewrite_headers */
   NULL,            /* widen_domains */
-  NULL,            /* mx_domains */
-  NULL,            /* mx_domains_except */
-  NULL,            /* re_mx_domains */
-  NULL             /* re_mx_domains_except */
+  NULL             /* mx_domains */
 };
 
 
@@ -69,8 +62,10 @@ to be set up. */
 void
 lookuphost_router_init(router_instance *rblock)
 {
+/*
 lookuphost_router_options_block *ob =
   (lookuphost_router_options_block *)(rblock->options_block);
+*/
 
 /* There must be a transport, unless just verifying. */
 
@@ -102,46 +97,90 @@ int rc;
 lookuphost_router_options_block *ob =
   (lookuphost_router_options_block *)(rblock->options_block);
 char *widen = NULL;
+char *pre_widen = addr->route_domain;
+char *post_widen = NULL;
 char *fully_qualified_name;
 char *listptr;
 char widen_buffer[256];
 
-
 DEBUG(2)
   debug_printf("%s router called for %s\n  %s lookup: route_domain = %s\n",
   rblock->name, addr->orig, (ob->gethostbyname)? "gethostbyname" : "dns",
   addr->route_domain);
 
-
-/* Set up an initial host item, and then call the appropriate function to fill
-in its address and chain on additional host_items if necessary. */
-
-h.next = NULL;
-h.name = addr->route_domain;
-h.address = NULL;
-h.mx = -1;
-h.status = hstatus_unknown;
-h.why = hwhy_unknown;
-h.last_try = 0;
-
-
-/* Loop to cope with explicit widening of domains as configured. */
+/* Set up the first of any widening domains. The code further down copes with
+either pre- or post-widening, but at present there is no way to turn on
+pre-widening, as actually doing so seems like a rather bad idea, and nobody has
+requested it. Pre-widening would cause local abbreviated names to take
+precedence over global names. For example, if the domain is "xxx.ch" it might
+be something in the "ch" toplevel domain, but it also might be xxx.ch.xyz.com.
+The choice of pre- or post-widening affects which takes precedence. If ever
+somebody comes up with some kind of requirement for pre-widening, presumably
+with some conditions under which it is done, it can be selected here. */
 
 if (ob->widen_domains != NULL)
   {
   listptr = ob->widen_domains;
   widen = string_nextinlist(&listptr, ':', widen_buffer, sizeof(widen_buffer));
+
+/****
+  if (some condition requiring pre-widening)
+    {
+    post_widen = pre_widen;
+    pre_widen = NULL;
+    }
+****/
+
   }
 
+/* Loop to cope with explicit widening of domains as configured. This code
+copes with widening that may happen before or after the original name. The
+decision as to which is taken above. */
+
 for (;;)
   {
+  if (pre_widen != NULL)
+    {
+    h.name = pre_widen;
+    pre_widen = NULL;
+    }
+  else if (widen != NULL)
+    {
+    h.name = string_sprintf("%s.%s", addr->route_domain, widen);
+    widen = string_nextinlist(&listptr,':',widen_buffer,sizeof(widen_buffer));
+    DEBUG(2) debug_printf("%s router widened %s to %s\n", rblock->name,
+      addr->route_domain, h.name);
+    }
+  else if (post_widen != NULL)
+    {
+    h.name = post_widen;
+    post_widen = NULL;
+    DEBUG(2) debug_printf("%s router trying %s after widening failed\n",
+      rblock->name, h.name);
+    }
+  else return FAIL;
+
+  /* Set up the rest of the initial host item. Others may get chained on if
+  there is more than one IP address. We set it up here instead of outside the
+  loop so as to re-initialize if a previous try succeeded but was rejected
+  because of not having an MX record. */
+
+  h.next = NULL;
+  h.address = NULL;
+  h.mx = -1;
+  h.status = hstatus_unknown;
+  h.why = hwhy_unknown;
+  h.last_try = 0;
+
+  /* Call the appropriate function to fill in the address and chain on
+  additional host_items if necessary. */
+
   if (ob->gethostbyname)
     rc = host_find_byname(&h, &fully_qualified_name, FALSE);
 
   /* Unfortunately, we cannot set the mx_only option in advance, because the
   DNS lookup may extend an unqualified name. Therefore, we must do the test
-  subsequently. The MX-only test cannot be applied to source-routed addresses,
-  which can be identified by a local-part starting with ',' or ':'. */
+  subsequently. */
 
   else
     {
@@ -150,20 +189,15 @@ for (;;)
 
     if ((rc == HOST_FOUND || rc == HOST_FOUND_LOCAL) &&
         h.mx < 0 &&                            /* Found A records only */
-        addr->local_part[0] != ',' &&          /* Not source routed */
-        addr->local_part[0] != ':' &&
-        (ob->mx_domains_except == NULL ||      /* Not in mx_domains_except list */
-        !match_isinlist(fully_qualified_name, ob->mx_domains_except,
-        &(ob->re_mx_domains_except), TRUE)) &&
         (ob->mx_domains != NULL &&             /* In mx-domains list */
-        match_isinlist(fully_qualified_name, ob->mx_domains,
-        &(ob->re_mx_domains), TRUE)) &&
+        match_isinlist(fully_qualified_name, &(ob->mx_domains), TRUE, TRUE,
+          NULL)) &&
         !match_isinlist(fully_qualified_name,  /* Not turned into a local */
-          local_domains, &re_local_domains, TRUE))              /* domain */
+          &local_domains, TRUE, TRUE, NULL))   /* domain */
       {
       DEBUG(2) debug_printf("%s router rejected %s: no MX record(s)\n",
         rblock->name, fully_qualified_name);
-      return FAIL;
+      continue;
       }
     }
 
@@ -197,21 +231,16 @@ for (;;)
   if (h.mx >= 0 && h.address == NULL)
     {
     addr->message = "all relevant MX records point to non-existent hosts";
-    if (string_is_ip_address(h.name))
+    if (string_is_ip_address(h.name, NULL))
       addr->message = string_sprintf("%s: it appears that the DNS operator for "
          "this domain has installed an invalid MX record with an IP address "
          "instead of a domain name on the right hand side", addr->message);
     return FAIL;
     }
 
-  /* If there are any configured widening domains, widen the name we
-  have got and try again. Otherwise, fail. */
+  /* If there's a syntax error, do not continue with any widening. */
 
-  if (widen == NULL) return FAIL;
-  h.name = string_sprintf("%s.%s", addr->route_domain, widen);
-  widen = string_nextinlist(&listptr, ':', widen_buffer, sizeof(widen_buffer));
-  DEBUG(2) debug_printf("%s router widened %s to %s\n", rblock->name,
-    addr->route_domain, h.name);
+  if (host_find_failed_syntax) return FAIL;
   }
 
 
@@ -227,8 +256,7 @@ if (strcmp(addr->route_domain, fully_qualified_name) != 0)
   addr->route_domain = fully_qualified_name;
   addr->rewrite_headers = ob->rewrite_headers;
   if (rc == HOST_FOUND_LOCAL ||
-      match_isinlist(fully_qualified_name, local_domains, &re_local_domains,
-        TRUE))
+      match_isinlist(fully_qualified_name, &local_domains, TRUE, TRUE, NULL))
     {
     addr->rewrite_headers = TRUE;
     return ISLOCAL;
@@ -290,9 +318,13 @@ addr->host_list[0] = h;
 /* Fill in the transport, queue the address for local or remote delivery, and
 yield success. If just verifying, there may not be a transport, in which case
 it doesn't matter which queue is used. This is all now handled by the
-route_queue() function. */
+route_queue() function. Initialization insists on the presence of a transport
+of some kind. */
+
+if (!route_get_transport(rblock->transport, rblock->expand_transport,
+      addr, &(addr->transport), rblock->name, NULL))
+  return ERROR;
 
-addr->transport = rblock->transport;
 return route_queue(addr, addr_local, addr_remote, rblock, "router")?
   OK : ERROR;
 }
