@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -15,9 +15,6 @@ typedef struct {
   BOOL rewrite_headers;
   char *widen_domains;
   char *mx_domains;
-  char *mx_domains_except;
-  re_block *re_mx_domains;
-  re_block *re_mx_domains_except;
 } lookuphost_router_options_block;
 
 /* Data for reading the private options. */
