@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,20 +14,20 @@
 /* Options specific to the queryprogram router. */
 
 optionlist queryprogram_router_options[] = {
-  { "*set_group",   opt_bool | opt_hidden,
+  { "*set_command_group",   opt_bool | opt_hidden,
       (void *)(offsetof(queryprogram_router_options_block, gid_set)) },
-  { "*set_user",    opt_bool | opt_hidden,
+  { "*set_command_user",    opt_bool | opt_hidden,
       (void *)(offsetof(queryprogram_router_options_block, uid_set)) },
   { "command",      opt_stringptr,
       (void *)(offsetof(queryprogram_router_options_block, command)) },
+  { "command_group",opt_gid,
+      (void *)(offsetof(queryprogram_router_options_block, gid)) },
+  { "command_user", opt_uid,
+      (void *)(offsetof(queryprogram_router_options_block, uid)) },
   { "current_directory", opt_stringptr,
       (void *)(offsetof(queryprogram_router_options_block, current_directory)) },
-  { "group",        opt_gid,
-      (void *)(offsetof(queryprogram_router_options_block, gid)) },
   { "timeout",      opt_time,
-      (void *)(offsetof(queryprogram_router_options_block, timeout)) },
-  { "user",         opt_uid,
-      (void *)(offsetof(queryprogram_router_options_block, uid)) }
+      (void *)(offsetof(queryprogram_router_options_block, timeout)) }
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -99,7 +99,7 @@ char *s, *expanded;
 char *argv[20];
 char *returned[5];
 char buffer[256];
-transport_instance *transport = rblock->transport;
+transport_instance *transport = NULL;
 queryprogram_router_options_block *ob =
   (queryprogram_router_options_block *)(rblock->options_block);
 char *current_directory = (ob->current_directory == NULL)? "/" :
@@ -108,11 +108,6 @@ char *current_directory = (ob->current_directory == NULL)? "/" :
 DEBUG(2) debug_printf("%s router called for %s: route_domain = %s\n",
   rblock->name, addr->orig, addr->route_domain);
 
-/* This router might do routing according to the local part. Hence we mustn't
-subsequently copy its routing for another address. */
-
-addr->routed_by_domain = FALSE;
-
 /* If one of uid/gid is not set, we need to have "nobody"s values available. */
 
 if (!ob->uid_set || !ob->gid_set)
@@ -145,8 +140,8 @@ if (!ob->uid_set || !ob->gid_set)
 uid = (ob->uid_set)? ob->uid : nobody_uid;
 gid = (ob->gid_set)? ob->gid : nobody_gid;
 
-DEBUG(2) debug_printf("uid=%d gid=%d current_directory=%s\n", uid, gid,
-  current_directory);
+DEBUG(2) debug_printf("uid=%ld gid=%ld current_directory=%s\n",
+  (long int)uid, (long int)gid, current_directory);
 
 /* Create a pipe to read the standard output of the subprocess. */
 
@@ -188,17 +183,16 @@ for (i = 0; *s != 0 && i < 19; i++)
 argv[i] = NULL;
 
 
-/* Create the child process */
+/* Create the child process and make it a group leader */
 
 pid = child_open(argv, NULL, 0077, &uid, &gid, &fd_in, pfd[pipe_write],
-  current_directory);
+  current_directory, TRUE);
 
 if (pid < 0)
   {
-  log_write(0, LOG_MAIN, "%s router: couldn't create child process",
-    rblock->name);
-  addr->message = string_sprintf("%s router couldn't create child process",
-    rblock->name);
+  addr->message = string_sprintf("%s router couldn't create child process: %s",
+    rblock->name, strerror(errno));
+  log_write(0, LOG_MAIN, "%s", addr->message);
   return DEFER;
   }
 
@@ -211,12 +205,26 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
   if (rc > 0)
     addr->message = string_sprintf("%s router: command returned non-zero code %d",
       rblock->name, rc);
+
   else if (rc == -256)
+    {
+    uid_t euid = geteuid();
+
     addr->message = string_sprintf("%s router: command timed out",
       rblock->name);
+
+    /* Kill the whole process group; we have to get back the privilege in
+    order to do this. */
+
+    if (euid != root_uid) mac_seteuid(root_uid);
+    killpg(pid, SIGKILL);
+    if (euid != root_uid) mac_seteuid(euid);
+    }
+
   else if (rc == -257)
     addr->message = string_sprintf("%s router: wait() failed: %s",
       rblock->name, strerror(errno));
+
   else
     addr->message = string_sprintf("%s router: command killed by signal %d",
       rblock->name, -rc);
@@ -306,11 +314,16 @@ if (returned[1][0] != 0 && strcmp(returned[1], "+") != 0)
       "command", rblock->name, returned[1]);
     addr->message = string_sprintf("%s router: unknown transport name %s "
       "yielded by command", rblock->name, returned[1]);
-    addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
   }
 
+/* Second field was empty or "+"; get the transport for this router, if
+any. It might be expanded, or it might be unset (which is not an error). */
+
+if (!route_get_transport(rblock->transport, rblock->expand_transport,
+      addr, &transport, rblock->name, NULL))
+  return ERROR;
 
 /* If the third field is non-null and not "+", it is a new route_address to
 replace the current one. If the fourth field is non-null and not "+", it gives
