@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* A set of functions to search databases in various formats. An open
@@ -12,7 +12,11 @@ lookups subdirectory and the functions here form a generic interface.
 
 Caching is used to improve performance. Open files are cached until a tidyup
 function is called, and for each file the result of the last lookup is cached.
-All this data is held in malloc'd store so as to be independent of the stacking
+However, if too many files are opened, some of those that are not in use have
+to be closed. Those open items that use real files are kept on a LRU chain to
+help with this.
+
+All the data is held in malloc'd store so as to be independent of the stacking
 store pools. */
 
 #include "exim.h"
@@ -20,7 +24,18 @@ store pools. */
 
 /* Tree in which to cache open files until tidyup called. */
 
-tree_node *search_tree = NULL;
+static tree_node *search_tree = NULL;
+
+/* Two-way chain of open databases that use real files. This is maintained in
+recently-used order for the purposes of closing the least recently used when
+too many files are open. */
+
+static tree_node *open_top = NULL;
+static tree_node *open_bot = NULL;
+
+/* Count of open databases that use real files */
+
+static int open_filecount = 0;
 
 
 
@@ -105,7 +120,9 @@ if ((fd >= 0 && fstat(fd, &statbuf) != 0) ||
 if ((statbuf.st_mode & S_IFMT) != S_IFREG ||
     (statbuf.st_mode & modemask) != 0)
   {
-  *errmsg = string_sprintf("%s (%s lookup): wrong mode", filename, type);
+  *errmsg = string_sprintf("%s (%s lookup): file mode %.4o should not contain "
+    "%.4o", filename, type,  statbuf.st_mode & 07777,
+    statbuf.st_mode & modemask);
   errno = ERRNO_BADMODE;
   return +1;
   }
@@ -117,7 +134,8 @@ if (owners != NULL)
     if (owners[i] == statbuf.st_uid) { uid_ok = TRUE; break; }
   if (!uid_ok)
     {
-    *errmsg = string_sprintf("%s (%s lookup): wrong owner", filename, type);
+    *errmsg = string_sprintf("%s (%s lookup): file has wrong owner", filename,
+      type);
     errno = ERRNO_BADUGID;
     return +1;
     }
@@ -130,7 +148,8 @@ if (owngroups != NULL)
     if (owngroups[i] == statbuf.st_gid) { gid_ok = TRUE; break; }
   if (!gid_ok)
     {
-    *errmsg = string_sprintf("%s (%s lookup): wrong group", filename, type);
+    *errmsg = string_sprintf("%s (%s lookup): file has wrong group", filename,
+      type);
     errno = ERRNO_BADUGID;
     return +1;
     }
@@ -148,9 +167,11 @@ return 0;
 /* When search_open is called it caches the "file" that it opens in
 search_tree. The name of the tree node is a concatenation of the search type
 with the file name. For query-style lookups, the file name is empty. Real
-files are closed only when this tidyup routine is called, typically at the end
-of sections of code where a number of lookups might occur. There is also a
-general tidyup function which is called for the lookup driver, if it exists.
+files are normally closed only when this tidyup routine is called, typically at
+the end of sections of code where a number of lookups might occur. However, if
+too many files are open simultaneously, some get closed beforehand. They can't
+be removed from the tree. There is also a general tidyup function which is
+called for the lookup driver, if it exists.
 
 First, there is an internal, recursive subroutine.
 
@@ -167,7 +188,7 @@ search_cache *c = (search_cache *)(t->data.ptr);
 if (t->left != NULL) tidyup_subtree(t->left);
 if (t->right != NULL) tidyup_subtree(t->right);
 
-if (lookup_list[search_type].close != NULL)
+if (c->handle != NULL && lookup_list[search_type].close != NULL)
   lookup_list[search_type].close(c->handle);
 
 if (c->key != NULL) store_free(c->key);
@@ -197,13 +218,13 @@ if (search_tree != NULL)
   tidyup_subtree(search_tree);
   search_tree = NULL;
   }
+open_top = open_bot = NULL;
+open_filecount = 0;
 
-/* Call the tidyup entry for any drivers that have one. */
+/* Call the general tidyup entry for any drivers that have one. */
 
 for (li = lookup_list; li->name[0] != 0; li++)
-  {
   if (li->tidy != NULL) (li->tidy)();
-  }
 }
 
 
@@ -213,7 +234,7 @@ for (li = lookup_list; li->name[0] != 0; li++)
 *             Open search database               *
 *************************************************/
 
-/* A mode, and lists of owners and groups are passed over for checking in
+/* A mode, and lists of owners and groups, are passed over for checking in
 the cases where the database is one or more files. Return NULL, with a message
 pointed to by message, in cases of error.
 
@@ -234,7 +255,11 @@ types such as NIS+.
 Before opening, scan the tree of cached files to see if this file is already
 open for the correct search type. If so, return the saved handle. If not, put
 the handle in the tree for possible subsequent use. See search_tidyup above for
-closing the cached files.
+closing all the cached files.
+
+A count of open databases which use real files is maintained, and if this
+gets too large, we have to close a cached file. Its entry remains in the tree,
+but is marked closed.
 
 Arguments:
   filename       the name of the file for single-key+file style lookups,
@@ -249,9 +274,9 @@ Arguments:
   message        points to a char * which can be set to point to an error
                  message when something goes wrong
 
-Returns:         an identifying handle for the open database; for single-key
-                 search types this is the pointer to the tree block in the
-                 cache of open files
+Returns:         an identifying handle for the open database;
+                 this is the pointer to the tree block in the
+                 cache of open files; return NULL on open failure
 */
 
 void *
@@ -261,49 +286,94 @@ search_open(char *filename, int search_type, int modemask, int *owners,
 void *handle;
 tree_node *t;
 search_cache *c;
+lookup_info *lk = lookup_list + search_type;
 char keybuffer[256];
 
-DEBUG(9) debug_printf("search_open: %s \"%s\"\n",
-  lookup_list[search_type].name,
+DEBUG(9) debug_printf("search_open: %s \"%s\"\n", lk->name,
   (filename == NULL)? "NULL" : filename);
 
 /* See if we already have this open for this type of search, and if so,
 pass back the tree block as the handle. The key for the tree node is the search
-type plus '0' concatenated with the file name. */
+type plus '0' concatenated with the file name. There may be entries in the tree
+with closed files if a lot of files have been opened. */
 
 sprintf(keybuffer, "%c%.254s", search_type + '0',
   (filename == NULL)? "" : filename);
 
 if ((t = tree_search(search_tree, keybuffer)) != NULL)
   {
-  DEBUG(9) debug_printf("  cached\n");
-  return t;
+  c = (search_cache *)(t->data.ptr);
+  if (c->handle != NULL)
+    {
+    DEBUG(9) debug_printf("  cached open\n");
+    return t;
+    }
+  DEBUG(9) debug_printf("  cached closed\n");
   }
 
-/* Otherwise, open the file or database - each search type has its own code,
-which is now split off into separately compiled modules. If opening is
-successful, call the file-checking function if there is one, and if all is
-still well, enter the open database into the tree. */
+/* Otherwise, we need to open the file or database - each search type has its
+own code, which is now split off into separately compiled modules. Before doing
+this, if the search type is one that uses real files, check on the number that
+we are holding open in the cache. If the limit is reached, close the least
+recently used one. */
 
-handle = lookup_list[search_type].open(filename, message);
+if (lk->type == lookup_absfile && open_filecount >= lookup_open_max)
+  {
+  if (open_bot == NULL)
+    log_write(0, LOG_MAIN|LOG_PANIC, "too many lookups open, but can't find "
+      "one to close");
+  else
+    {
+    search_cache *c = (search_cache *)(open_bot->data.ptr);
+    DEBUG(9) debug_printf("Too many lookup files open\n  closing %s\n",
+      open_bot->name);
+    open_bot = c->up;
+    if (open_bot != NULL)
+      ((search_cache *)(open_bot->data.ptr))->down = NULL;
+    else
+      open_top = NULL;
+    ((lookup_list + c->search_type)->close)(c->handle);
+    c->handle = NULL;
+    open_filecount--;
+    }
+  }
+
+/* If opening is successful, call the file-checking function if there is one,
+and if all is still well, enter the open database into the tree. */
+
+handle = lk->open(filename, message);
 if (handle == NULL) return handle;
 
-if (lookup_list[search_type].check != NULL &&
-   !lookup_list[search_type].check(handle, filename, modemask, owners,
-     owngroups, message))
+if (lk->check != NULL &&
+   !lk->check(handle, filename, modemask, owners, owngroups, message))
   {
-  lookup_list[search_type].close(handle);
+  lk->close(handle);
   return NULL;
   }
 
-t = store_malloc(sizeof(tree_node) + (int)strlen(keybuffer));
-strcpy(t->name, keybuffer);
+/* If this is a search type that uses real files, keep count. */
+
+if (lk->type == lookup_absfile) open_filecount++;
+
+/* If we found a previously opened entry in the tree, re-use it; otherwise
+insert a new entry. On re-use, leave any cached lookup data and the lookup
+count alone. */
+
+if (t == NULL)
+  {
+  t = store_malloc(sizeof(tree_node) + (int)strlen(keybuffer));
+  t->data.ptr = c = store_malloc(sizeof(search_cache));
+  c->key = NULL;
+  c->data = NULL;
+  strcpy(t->name, keybuffer);
+  tree_insertnode(&search_tree, t);
+  }
+else c = t->data.ptr;
 
-t->data.ptr = c = store_malloc(sizeof(search_cache));
 c->handle = handle;
-c->key = NULL;
-c->data = NULL;
-tree_insertnode(&search_tree, t);
+c->search_type = search_type;
+c->up = c->down = NULL;
+
 return t;
 }
 
@@ -315,21 +385,18 @@ return t;
 *  Internal function: Find one item in database  *
 *************************************************/
 
-/* The answer is always put into ???. If the key contains a colon, then it is
-treated as a double key: the first part is the key for the record in the file,
-and the remainder is a subkey that is used to extract a subfield from the main
-data. Subfields are specified as subkey=value in the records.
-
-The last lookup is cached by file name and key - using the handle is no good
-as it isn't unique enough.
+/* The answer is always put into dynamic store. If the key contains a colon,
+then it is treated as a double key: the first part is the key for the record in
+the file, and the remainder is a subkey that is used to extract a subfield from
+the main data. Subfields are specified as subkey=value in the records. The last
+lookup for each handle is cached.
 
 Arguments:
-  handle       the handle from search_open
+  handle       the handle from search_open; points to tree node
   filename     the filename that was handed to search_open, or
                NULL for query-style searches
   keystring    the keystring for single-key+file lookups, or
                the querystring for query-style lookups
-  type         the type of lookup
   errmsg       somewhere to point to an error message
 
 Returns:       a pointer to a dynamic string containing the answer,
@@ -338,11 +405,13 @@ Returns:       a pointer to a dynamic string containing the answer,
 */
 
 static char *
-internal_search_find(void *handle, char *filename, char *keystring, int type,
+internal_search_find(void *handle, char *filename, char *keystring,
   char **errmsg)
 {
-int length;
-search_cache *c = (search_cache *)(((tree_node *)handle)->data.ptr);
+tree_node *t = (tree_node *)handle;
+search_cache *c = (search_cache *)(t->data.ptr);
+int search_type = t->name[0] - '0';
+
 char *subkey = NULL;
 char *colon = NULL;
 
@@ -364,7 +433,7 @@ if (filename != NULL)
 
 DEBUG(9) debug_printf("internal_search_find: file=\"%s\"\n  "
   "type=%s key=\"%s\"\n", (filename == NULL)? "NULL" : filename,
-  lookup_list[type].name, keystring);
+  lookup_list[search_type].name, keystring);
 
 /* Insurance. If the keystring is empty, just fail. */
 
@@ -376,6 +445,7 @@ file. */
 if (c->key == NULL || strcmp(c->key, keystring) != 0)
   {
   char *data = NULL;
+  int keylength = (int)strlen(keystring);
 
   DEBUG(9)
     {
@@ -399,8 +469,7 @@ if (c->key == NULL || strcmp(c->key, keystring) != 0)
   like FAIL, except that search_find_defer is set so the caller can
   distinguish if necessary. */
 
-  length = (int)strlen(keystring);
-  if (lookup_list[type].find(c->handle, filename, keystring, length,
+  if (lookup_list[search_type].find(c->handle, filename, keystring, keylength,
       &data, errmsg) == DEFER)
     {
     search_find_defer = TRUE;
@@ -411,12 +480,12 @@ if (c->key == NULL || strcmp(c->key, keystring) != 0)
 
   else
     {
-    int len = (int)strlen(keystring) + 1;
+    int len = keylength + 1;
     c->key = store_malloc(len);
     memcpy(c->key, keystring, len);
     if (data != NULL)
       {
-      int len = (int)strlen(data) + 1;
+      len = (int)strlen(data) + 1;
       c->data = store_malloc(len);
       memcpy(c->data, data, len);
       }
@@ -470,7 +539,6 @@ Arguments:
                NULL for query-style searches
   keystring    the keystring for single-key+file lookups, or
                the querystring for query-style lookups
-  type         the type of lookup
   partial      if the value is greater than 2000, subtract 2048 and
                set the "try * to the left of @" flag; then
                if the value is greater than 1000, subtract 1024 to
@@ -488,18 +556,62 @@ Returns:       a pointer to a dynamic string containing the answer,
 */
 
 char *
-search_find(void *handle, char *filename, char *keystring, int type,
-  int partial, int *expand_setup, char **errmsg)
+search_find(void *handle, char *filename, char *keystring, int partial,
+  int *expand_setup, char **errmsg)
 {
+tree_node *t = (tree_node *)handle;
 BOOL set_null_wild = FALSE;
 BOOL do_star = FALSE;
 BOOL do_at = FALSE;
 char *yield;
 
-DEBUG(9) debug_printf("search_find: file=\"%s\"\n  type=%s key=\"%s\" "
-  "partial=%d\n",
-  (filename == NULL)? "NULL" : filename,
-  lookup_list[type].name, keystring, partial);
+DEBUG(9) debug_printf("search_find: file=\"%s\"\n  key=\"%s\" partial=%d\n",
+  (filename == NULL)? "NULL" : filename, keystring, partial);
+
+/* Arrange to put this database at the top of the LRU chain if it is a type
+that opens real files. */
+
+if (open_top != (tree_node *)handle &&
+    lookup_list[t->name[0]-'0'].type == lookup_absfile)
+  {
+  search_cache *c = (search_cache *)(t->data.ptr);
+  tree_node *up = c->up;
+  tree_node *down = c->down;
+
+  /* Cut it out of the list. A newly opened file will a NULL up pointer.
+  Otherwise there will be a non-NULL up pointer, since we checked above that
+  this block isn't already at the top of the list. */
+
+  if (up != NULL)
+    {
+    ((search_cache *)(up->data.ptr))->down = down;
+    if (down != NULL)
+      ((search_cache *)(down->data.ptr))->up = up;
+    else open_bot = up;
+    }
+
+  /* Now put it at the head of the list. */
+
+  c->up = NULL;
+  c->down = open_top;
+  if (open_top == NULL) open_bot = t; else
+    ((search_cache *)(open_top->data.ptr))->up = t;
+  open_top = t;
+  }
+
+DEBUG(9)
+  {
+  tree_node *t = open_top;
+  debug_printf("LRU list:\n");
+  while (t != NULL)
+    {
+    search_cache *c = (search_cache *)(t->data.ptr);
+    debug_printf("  %s\n", t->name);
+    if (t == open_bot) debug_printf("  End\n");
+    t = c->down;
+    }
+  }
+
 
 /* Determine whether independent "*" or "*@" matching is wanted and adjust the
 partial count. If there is no filename, partial matching can never happen. */
@@ -522,19 +634,26 @@ else partial = -1;
 /* First of all, try to match the key string verbatim. If matched a complete
 entry but could have been partial, flag to set up variables. */
 
-yield = internal_search_find(handle, filename, keystring, type, errmsg);
+yield = internal_search_find(handle, filename, keystring, errmsg);
 if (search_find_defer) return NULL;
 if (yield != NULL) { if (partial >= 0) set_null_wild = TRUE; }
 
-/* Not matched a complete entry; handle partial lookups, but only if
-the full search didn't defer. */
+/* Not matched a complete entry; handle partial lookups, but only if the full
+search didn't defer. Don't use string_sprintf() to construct the initial key,
+just in case the original key is too long for the string_sprintf() buffer (it
+*has* happened!). */
 
 else if (partial >= 0)
   {
-  char *keystring2 = string_sprintf("*.%s", keystring);
+  int len = (int)strlen(keystring);
+  char *keystring2 = store_get(len + 3);
+
+  strcpy(keystring2 + 2, keystring);
+  keystring2[0] = '*';
+  keystring2[1] = '.';
 
   DEBUG(9) debug_printf("trying partial match %s\n", keystring2);
-  yield = internal_search_find(handle, filename, keystring2, type, errmsg);
+  yield = internal_search_find(handle, filename, keystring2, errmsg);
   if (search_find_defer) return NULL;
 
   /* The key in its entirety did not match a wild entry; try chopping off
@@ -552,7 +671,7 @@ else if (partial >= 0)
       while (keystring3[1] != '.' && keystring3[1] != 0) keystring3++;
       *keystring3 = '*';
       DEBUG(9) debug_printf("trying partial match %s\n", keystring3);
-      yield = internal_search_find(handle, filename, keystring3, type, errmsg);
+      yield = internal_search_find(handle, filename, keystring3, errmsg);
       if (search_find_defer) return NULL;
       if (yield != NULL)
         {
@@ -592,7 +711,7 @@ if (yield == NULL && do_at)
     *atat = '*';
 
     DEBUG(9) debug_printf("trying default match %s\n", atat);
-    yield = internal_search_find(handle, filename, atat, type, errmsg);
+    yield = internal_search_find(handle, filename, atat, errmsg);
     *atat = savechar;
     if (search_find_defer) return NULL;
 
@@ -616,7 +735,7 @@ is the whole key, and the second is empty. */
 if (yield == NULL && do_star && partial != 0)
   {
   DEBUG(9) debug_printf("trying to match *\n");
-  yield = internal_search_find(handle, filename, "*", type, errmsg);
+  yield = internal_search_find(handle, filename, "*", errmsg);
   if (yield != NULL && expand_setup != NULL && *expand_setup >= 0)
     {
     *expand_setup += 1;
