@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for reading spool files. */
@@ -84,7 +84,10 @@ if (fcntl(deliver_datafile, F_SETLK, &lock_data) < 0)
 in the count, but add one for the newline before the data. */
 
 if (fstat(deliver_datafile, &statbuf) == 0)
-  message_size = statbuf.st_size - MESSAGE_ID_LENGTH - 3 + 1;
+  {
+  message_body_size = statbuf.st_size - MESSAGE_ID_LENGTH - 3;
+  message_size = message_body_size + 1;
+  }
 
 return TRUE;
 }
@@ -207,17 +210,22 @@ int
 spool_read_header(char *name, BOOL read_headers, BOOL subdir_set)
 {
 FILE *f;
-int n, uid, gid;
+int n;
+long int uid, gid;
 BOOL inheader = FALSE;
 char originator[64];
 
 /* Ensure all pointers to store that will be obtained are NULL, just
 in case. */
 
+authenticated_id = NULL;
+authenticated_sender = NULL;
 header_list = header_last = NULL;
 sender_address = NULL;
 sender_host_address = NULL;
 sender_host_name = NULL;
+sender_host_authenticated = NULL;
+interface_address = NULL;
 sender_helo_name = NULL;
 sender_fullhost = NULL;
 sender_ident = NULL;
@@ -243,34 +251,22 @@ errno = 0;
 DEBUG(5) debug_printf("Opened spool file %s\n", name);
 
 /* The first line of a spool file contains the message id followed by -H (i.e.
-the file name), in order to make the file self-identifying. However, versions
-of Exim before 0.57 did not do this, so make this line optional for
-compatibility. Change made in October 1996. After a year or so, the optionality
-can be removed. Set data_start_offset to the length of the message id + 3
-because the identification will also be present in the data file if it is in
-the header file. */
+the file name), in order to make the file self-identifying. */
 
-data_start_offset = 0;
+if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
+if ((int)strlen(big_buffer) != MESSAGE_ID_LENGTH + 3 ||
+    strncmp(big_buffer, name, MESSAGE_ID_LENGTH + 2) != 0)
+  goto SPOOL_FORMAT_ERROR;
+
+/* The next three lines in the header file are in a fixed format. The first
+contains the login, uid, and gid of the user who caused the file to be written.
+The second contains the mail address of the message's sender, enclosed in <>.
+The third contains the time the message was received, and the number of warning
+messages for delivery delays that have been sent. */
 
 if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 
-if ((int)strlen(big_buffer) == MESSAGE_ID_LENGTH + 3)
-  {
-  if (strncmp(big_buffer, name, MESSAGE_ID_LENGTH + 2) == 0)
-    {
-    data_start_offset = MESSAGE_ID_LENGTH + 3;
-    if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
-    }
-  }
-
-/* We now have the first proper data line in the big buffer. The first three
-data lines of spool files are in a fixed format. The first contains the login,
-uid, and gid of the user who caused the file to be written. The second contains
-the mail address of the message's sender, enclosed in <>. The third contains
-the time the message was received, and the number of warning messages for
-delivery delays that have been sent. */
-
-if (sscanf(big_buffer, "%s %d %d", originator, &uid, &gid) != 3)
+if (sscanf(big_buffer, "%s %ld %ld", originator, &uid, &gid) != 3)
   goto SPOOL_FORMAT_ERROR;
 originator_login = string_copy(originator);
 originator_uid = (uid_t)uid;
@@ -289,22 +285,28 @@ if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (sscanf(big_buffer, "%d %d", &received_time, &warning_count) != 2)
   goto SPOOL_FORMAT_ERROR;
 
-DEBUG(5) debug_printf("user=%s uid=%d gid=%d sender=%s\n", originator_login,
-  (int)originator_uid, (int)originator_gid, sender_address);
+message_age = time(NULL) - received_time;
+
+DEBUG(5) debug_printf("user=%s uid=%ld gid=%ld sender=%s\n",
+  originator_login, (long int)originator_uid, (long int)originator_gid,
+  sender_address);
 
 /* Now there may be a number of optional lines, each starting with "-".
 First set default values for the options. */
 
 sender_local = FALSE;
 header_names = header_names_normal;
-body_linecount = -1;
+body_linecount = 0;
 message_linecount = 0;
 deliver_freeze = FALSE;
 deliver_frozen_at = 0;
+host_lookup_failed = FALSE;
 local_error_message = FALSE;
 user_null_sender = FALSE;
 
 #ifdef SUPPORT_DSN
+/* This partial attempt at doing DSN was abandoned. This obsolete code is
+left here just in case. Nothing is documented. */
 dsn_ret = 0;
 dsn_envid = NULL;
 #endif
@@ -318,6 +320,8 @@ for (;;)
   if (strcmp(big_buffer, "-local") == 0) sender_local = TRUE;
     else if (strcmp(big_buffer, "-localerror") == 0)
       local_error_message = TRUE;
+    else if (strcmp(big_buffer, "-host_lookup_failed") == 0)
+      host_lookup_failed = TRUE;
     else if (strcmp(big_buffer, "-resent") == 0)
       header_names = header_names_resent;
     else if (strncmp(big_buffer, "-body_linecount", 15) == 0)
@@ -333,8 +337,14 @@ for (;;)
       deliver_manual_thaw = TRUE;
     else if (strcmp(big_buffer, "-user_null_sender") == 0)
       user_null_sender = TRUE;
+    else if (strncmp(big_buffer, "-auth_id", 8) == 0)
+      authenticated_id = string_copy(big_buffer + 9);
+    else if (strncmp(big_buffer, "-auth_sender", 12) == 0)
+      authenticated_sender = string_copy(big_buffer + 13);
 
     #ifdef SUPPORT_DSN
+    /* This partial attempt at doing DSN was abandoned. This obsolete code is
+    left here just in case. Nothing is documented. */
     else if (strncmp(big_buffer, "-dsn_ret", 8) == 0)
       dsn_ret= atoi(big_buffer + 8);
 
@@ -344,8 +354,12 @@ for (;;)
 
     else if (strncmp(big_buffer, "-host_address", 13) == 0)
       sender_host_address = string_copy(big_buffer + 14);
+    else if (strncmp(big_buffer, "-host_auth", 10) == 0)
+      sender_host_authenticated = string_copy(big_buffer + 11);
     else if (strncmp(big_buffer, "-host_name", 10) == 0)
       sender_host_name = string_copy(big_buffer + 11);
+    else if (strncmp(big_buffer, "-interface_address", 18) == 0)
+      interface_address = string_copy(big_buffer + 19);
     else if (strncmp(big_buffer, "-helo_name", 10) == 0)
       sender_helo_name = string_copy(big_buffer + 11);
     else if (strncmp(big_buffer, "-ident", 6) == 0)
@@ -408,7 +422,9 @@ for (n = 0; n < recipients_count; n++)
   if (nn < 2) goto SPOOL_FORMAT_ERROR;
 
   /* Look back from the end for digits/commas and a space, indicating the
-  presence of flags, pno data, and possibly the DSN orcpt field. */
+  presence of flags, pno data, and possibly the DSN orcpt field.
+  The partial attempt at doing DSN was abandoned. This obsolete code is
+  left here just in case. Nothing is documented. */
 
   p = big_buffer + nn - 2;
   while (isdigit(*p) || *p == ',') p--;
@@ -431,7 +447,11 @@ for (n = 0; n < recipients_count; n++)
   recipients_list[n].pno = pno;
 
   #ifdef SUPPORT_DSN
+  /* This partial attempt at doing DSN was abandoned. This obsolete code is
+  left here just in case. Nothing is documented. */
   recipients_list[n].orcpt = orcpt;
+  #else
+  orcpt = orcpt;   /* Keep picky compilers happy */
   #endif
   }
 
@@ -457,7 +477,6 @@ while ((n = fgetc(f)) != EOF)
   ungetc(n, f);
   fscanf(f, "%d%c ", &n, flag);
   if (!mac_isprint(flag[0])) goto SPOOL_FORMAT_ERROR;
-
   if (flag[0] != '*') message_size += n;  /* Omit non-transmitted headers */
 
   if (read_headers)
@@ -505,18 +524,15 @@ while ((n = fgetc(f)) != EOF)
     }
   }
 
-/* We have successfully read the data in the header file. If the value of
-body_linecount is set (it will be except for spool files written by older
-Exims - the test was written on 15Apr98 and can be removed in a couple of
-years time) set up message_linecount as the total count. Close the file and
-give a positive response. */
-
-message_linecount = (body_linecount >= 0)?
-  message_linecount + body_linecount : -1;
+/* We have successfully read the data in the header file. Update the message
+line count by adding the body linecount to the header linecount. Close the file
+and give a positive response. */
 
 DEBUG(5) debug_printf("body_linecount=%d message_linecount=%d\n",
   body_linecount, message_linecount);
 
+message_linecount += body_linecount;
+
 fclose(f);
 return spool_read_OK;
 
