@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Exim gets and frees all its store through these functions. In the original
@@ -76,6 +76,14 @@ static storeblock *current_block[2] = { NULL, NULL };
 static void *next_yield[2] = { NULL, NULL };
 static int yield_length[2] = { -1, -1 };
 
+/* pool_malloc holds the amount of memory used by the store pools; this goes up
+and down as store is reset or released. nonpool_malloc is the total got by
+malloc from other calls; this doesn't go down because it is just freed by
+pointer. */
+
+static int pool_malloc = 0;
+static int nonpool_malloc = 0;
+
 
 
 /*************************************************
@@ -99,15 +107,33 @@ store_get_3(int size, char *filename, int linenumber)
 {
 void *yield;
 
+/* Round up the size to a multiple of the alignment. Although this looks a
+messy statement, because "alignment" is a constant expression, the compiler can
+do a reasonable job of optimizing, especially if the value of "alignment" is a
+power of two. I checked this with -O2, and gcc did very well, compiling it to 4
+instructions on a Sparc (alignment = 8). */
+
+if (size % alignment != 0) size += alignment - (size % alignment);
+
 /* If there isn't room in the current block, get a new one. The minimum
 size is STORE_BLOCK_SIZE, and we would expect this to be the norm, since
 these functions are mostly called for small amounts of store. */
 
 if (size > yield_length[store_pool])
   {
-  int length = (size > STORE_BLOCK_SIZE)? size : STORE_BLOCK_SIZE;
-  storeblock *newblock = store_malloc(length + sizeof(storeblock));
+  int length = (size <= STORE_BLOCK_SIZE)? STORE_BLOCK_SIZE : size;
+  int mlength = length + sizeof(storeblock);
+  storeblock *newblock;
 
+  #ifdef DEBUG_STORE
+  if (debug_trace_memory > 0)
+    {
+    pool_malloc += mlength;           /* Used in pools */
+    nonpool_malloc -= mlength;        /* Exclude from overall total */
+    }
+  #endif
+
+  newblock = store_malloc(mlength);
   newblock->next = NULL;
   newblock->length = length;
 
@@ -126,19 +152,16 @@ pointer. */
 yield = next_yield[store_pool];
 
 #ifdef DEBUG_STORE
-if (debug_trace_memory)
-  debug_printf("---%d Get %6d %4d %-14s %4d\n", store_pool, yield, size,
-    filename, linenumber);
+if (debug_trace_memory > 0)
+  {
+  if (debug_trace_memory == 1)
+    debug_printf("---%d Get %5d\n", store_pool, size);
+  else
+    debug_printf("---%d Get %6d %5d %-14s %4d\n", store_pool, yield, size,
+      filename, linenumber);
+  }
 #endif
 
-/* Round up the size to a multiple of the alignment. Although this looks a
-messy statement, because "alignment" is a constant expression, the compiler can
-do a reasonable job of optimizing, especially if the value of "alignment" is a
-power of two. I checked this with -O2, and gcc did very well, compiling it to 4
-instructions on a Sparc (alignment = 8). */
-
-if (size % alignment != 0) size += alignment - (size % alignment);
-
 /* Update next pointer and number of bytes left in the current block. */
 
 next_yield[store_pool] = (void *)((char *)next_yield[store_pool] + size);
@@ -186,9 +209,14 @@ if ((char *)ptr + rounded_oldsize != (char *)(next_yield[store_pool]) ||
   return FALSE;
 
 #ifdef DEBUG_STORE
-if (debug_trace_memory)
-  debug_printf("---%d Ext %6d %4d %-14s %4d\n", store_pool, ptr, newsize,
-    filename, linenumber);
+if (debug_trace_memory > 0)
+  {
+  if (debug_trace_memory == 1)
+    debug_printf("---%d Ext %5d\n", store_pool, newsize);
+  else
+    debug_printf("---%d Ext %6d %5d %-14s %4d\n", store_pool, ptr, newsize,
+      filename, linenumber);
+  }
 #endif
 
 if (newsize % alignment != 0) newsize += alignment - (newsize % alignment);
@@ -224,7 +252,6 @@ store_reset_3(void *ptr, char *filename, int linenumber)
 storeblock *bb;
 storeblock *b = current_block[store_pool];
 char *bc = (char *)b + sizeof(storeblock);
-void **chainptr;
 int newlength;
 
 /* See if the place is in the current block - as it often will be. Otherwise,
@@ -249,12 +276,6 @@ yield_length[store_pool] = newlength - (newlength % alignment);
 next_yield[store_pool] = (char *)ptr + (newlength % alignment);
 current_block[store_pool] = b;
 
-#ifdef DEBUG_STORE
-if (debug_trace_memory)
-  debug_printf("---%d Rst %6d   ** %-14s %4d\n", store_pool, ptr,
-    filename, linenumber);
-#endif
-
 /* Free any subsequent blocks, if any */
 
 bb = b->next;
@@ -264,14 +285,84 @@ while (bb != NULL)
   {
   b = bb;
   bb = bb->next;
+
+  #ifdef DEBUG_STORE
+  if (debug_trace_memory > 0) pool_malloc -= b->length;
+  #endif
+
   store_free_3(b, filename, linenumber);
   }
+
+#ifdef DEBUG_STORE
+if (debug_trace_memory > 0)
+  {
+  if (debug_trace_memory == 1)
+    debug_printf("---%d Rst    ** %d\n", store_pool, pool_malloc);
+  else
+    debug_printf("---%d Rst %6d    ** %-14s %4d %d\n", store_pool, ptr,
+      filename, linenumber, pool_malloc);
+  }
+#endif
 }
 
 
 
 
 
+/************************************************
+*             Release store                     *
+************************************************/
+
+/* This function is specifically provided for use when reading very
+long strings, e.g. header lines. When the string gets longer than a
+complete block, it gets copied to a new block. It is helpful to free
+the old block iff the previous copy of the string is at its start,
+and therefore the only thing in it. Otherwise, for very long strings,
+dead store can pile up somewhat disastrously. This function checks that
+the pointer it is given is the first thing in a block, and if so,
+releases that block.
+
+Arguments:
+  block       block of store to consider
+  filename    source file from which called
+  linenumber  line number in source file
+
+Returns:      nothing
+*/
+
+void
+store_release_3(void *block, char *filename, int linenumber)
+{
+storeblock *b;
+
+/* It will never be the first block, so no need to check that. */
+
+for (b = chainbase[store_pool]; b != NULL; b = b->next)
+  {
+  storeblock *bb = b->next;
+  if (bb != NULL && (char *)block == (char *)bb + sizeof(storeblock))
+    {
+    b->next = bb->next;
+    #ifdef DEBUG_STORE
+    if (debug_trace_memory > 0)
+      {
+      pool_malloc -= bb->length;
+      if (debug_trace_memory == 1)
+        debug_printf("-Release       %d\n", pool_malloc);
+      else
+        debug_printf("-Release %6d %-20s %4d %d\n", bb, filename,
+          linenumber, pool_malloc);
+      }
+    #endif
+    free(bb);
+    return;
+    }
+  }
+}
+
+
+
+
 /*************************************************
 *                Malloc store                    *
 *************************************************/
@@ -301,11 +392,15 @@ if (yield == NULL)
     "called from line %d of %s", size, linenumber, filename);
 
 #ifdef DEBUG_STORE
-if (debug_trace_memory)
+if (debug_trace_memory > 0)
   {
-  store_max_malloc += size;
-  debug_printf("---- Malloc %6d %5d (%-14s %4d) %d\n", yield, size, filename,
-    linenumber, store_max_malloc);
+  nonpool_malloc += size;
+  if (debug_trace_memory == 1)
+    debug_printf("--Malloc %5d %d %d\n", size, pool_malloc,
+      nonpool_malloc);
+  else
+    debug_printf("--Malloc %6d %5d %-14s %4d %d %d\n", yield, size,
+      filename, linenumber, pool_malloc, nonpool_malloc);
   }
 #endif
 
@@ -317,7 +412,7 @@ return yield;
 *             Free store                        *
 ************************************************/
 
-/* This function is called by the macro store_free()
+/* This function is called by the macro store_free().
 
 Arguments:
   block       block of store to free
@@ -331,8 +426,13 @@ void
 store_free_3(void *block, char *filename, int linenumber)
 {
 #ifdef DEBUG_STORE
-if (debug_trace_memory)
-  debug_printf("---- Free %6d (%-14s %4d)\n", block, filename, linenumber);
+if (debug_trace_memory > 0)
+  {
+  if (debug_trace_memory == 1)
+    debug_printf("----Free\n");
+  else
+    debug_printf("----Free %6d %-20s %4d\n", block, filename, linenumber);
+  }
 #endif
 free(block);
 }
