@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Miscellaneous string-handling functions */
@@ -19,23 +19,48 @@ static char  *hex_digits = "0123456789abcdef";
 
 /* This used just to be a regular expression, but with IPv6 things are a bit
 more complicated. If the address contains a colon, it is assumed to be a v6
-address (assuming HAVE_IPV6 is set).
+address (assuming HAVE_IPV6 is set). If a mask is permitted and one is present,
+the '/' in the input gets changed to 0, and maskptr points to its offset.
+
+Arguments:
+  s         a string
+  maskptr   NULL if no mask is permitted to follow
+            otherwise, points to an int where the '/' offset is put, or
+            zero if there is no mask given
 
-Argument:   a string
 Returns:    0 if the string is not a textual representation of an IP address
             4 if it is an IPv4 address
             6 if it is an IPv6 address
 */
 
 int
-string_is_ip_address(char *s)
+string_is_ip_address(char *s, int *maskptr)
 {
 int i;
 int yield = 4;
 
+/* If an optional mask is permitted, check for it. If found, change the / to
+a zero, and pass back the offset where this was done. */
+
+if (maskptr != NULL)
+  {
+  char *ss = s + (int)strlen(s);
+  *maskptr = 0;
+  if (s != ss && isdigit(*(--ss)))
+    {
+    while (ss > s && isdigit(ss[-1])) ss--;
+    if (ss > s && *(--ss) == '/')
+      {
+      *ss = 0;
+      *maskptr = ss - s;
+      }
+    }
+  }
+
 /* A colon anywhere in the string => IPv6 address */
 
 #if HAVE_IPV6
+
 if (strchr(s, ':') != NULL)
   {
   BOOL had_double_colon = FALSE;
@@ -106,6 +131,7 @@ if (strchr(s, ':') != NULL)
 
   if (!v4end) return (*s == 0)? yield : 0;
   }
+
 #endif
 
 /* Test for IPv4 address, which may be the tail-end of an IPv6 address. */
@@ -122,92 +148,6 @@ return (*s == 0)? yield : 0;
 
 
 
-
-/*************************************************
-*       Test for IP network representation       *
-*************************************************/
-
-/* This used just to be a regular expression, but with IPv6 things are a bit
-more complicated. If the address contains a colon, it is assumed to be a v6
-address (assuming HAVE_IPV6 is set).
-
-Argument:
-  s         string to test
-  error     where to put error message
-
-Returns:    0 if the string is not a textual representation of an IP network
-            4 for an IPv4 network; this is x.x.x.x/m.m.m.m
-            6 for an IPv6 network; this is <v6 address>/int
-*/
-
-int
-string_is_ip_network(char *s, char **error)
-{
-BOOL yield;
-char *slash = strchr(s, '/');
-
-/* The string must contain a slash to separate the address and the mask. */
-
-if (slash == NULL)
-  {
-  *error = "\"/\" missing";
-  return 0;
-  }
-
-/* Temporarily terminate at the slash and check that the first part
-is valid. */
-
-*slash = 0;
-yield = string_is_ip_address(s);
-*slash = '/';
-if (yield == 0)
-  {
-  *error = "invalid IP address";
-  return 0;
-  }
-
-/* Now test the part after the slash. */
-
-s = slash + 1;
-
-/* The mask for an IPv6 address is just an integer, and an integer is
-also allowed for an IPv4 address. */
-
-if (yield == 6 || strchr(s, '.') == NULL)
-  {
-  while (isdigit(*s)) s++;
-  if (*s == 0) return 6;
-  *error = "non-digit in IP mask";
-  return 0;
-  }
-
-/* Mask for an IPv4 address that contains a dot must be an IPv4 address,
-and must partition the 32-bits into high-order 1s and low-order 0s. This
-test is done by some bit-flipping magic! */
-
-if (string_is_ip_address(s) == 4)
-  {
-  int i;
-  BOOL ones = TRUE;
-  for (i = 0; i < 4; s++, i++)
-    {
-    int c = strtol(s, &s, 10);
-    if ((ones && c != ((-(c & (-c))) & 255)) || (!ones && c != 0))
-      {
-      *error = "IP net mask contains mixed bits";
-      return 0;
-      }
-    if (c != 255) ones = FALSE;
-    }
-  return 4;
-  }
-
-*error = "invalid IP net mask";
-return 0;
-}
-
-
-
 /*************************************************
 *              Format message size               *
 *************************************************/
@@ -495,6 +435,78 @@ return ss;
 }
 
 
+/*************************************************
+*     Copy space-terminated or quoted string     *
+*************************************************/
+
+/* I should have invented this function at the start; because I didn't, there
+are various places in the code that could use it but don't. With luck they may
+get converted as time goes by.
+
+This function copies from a string until its end, or until whitespace is
+encountered, unless the string begins with a double quote, in which case the
+terminating quote is sought, and escaping within the string is done. The length
+of a de-quoted string can be no longer than the original, since escaping always
+turns n characters into 1 character.
+
+Argument:  pointer to the pointer to the first character, which gets updated
+Returns:   the new string
+*/
+
+char *
+string_dequote(char **sptr)
+{
+char *s = *sptr;
+char *t, *yield;
+
+/* First find the end of the string */
+
+if (*s != '\"')
+  {
+  while (*s != 0 && !isspace(*s)) s++;
+  }
+else
+  {
+  s++;
+  while (*s != 0 && *s != '\"')
+    {
+    if (*s == '\\') (void)string_interpret_escape(&s);
+    s++;
+    }
+  if (*s != 0) s++;
+  }
+
+/* Get enough store to copy into */
+
+t = yield = store_get(s - *sptr + 1);
+s = *sptr;
+
+/* Do the copy */
+
+if (*s != '\"')
+  {
+  while (*s != 0 && !isspace(*s)) *t++ = *s++;
+  }
+else
+  {
+  s++;
+  while (*s != 0 && *s != '\"')
+    {
+    if (*s == '\\') *t++ = string_interpret_escape(&s);
+      else *t++ = *s++;
+    }
+  if (*s != 0) s++;
+  }
+
+/* Update the pointer and return the terminated copy */
+
+*sptr = s;
+*t = 0;
+return yield;
+}
+
+
+
 
 /*************************************************
 *          Format a string and save it           *
@@ -514,7 +526,7 @@ char *
 string_sprintf(char *format, ...)
 {
 va_list ap;
-char buffer[8192];
+char buffer[STRING_SPRINTF_BUFFER_SIZE];
 va_start(ap, format);
 if (!string_vformat(buffer, sizeof(buffer), format, ap))
   log_write(0, LOG_PANIC_DIE, "string_sprintf expansion was longer than %d",
@@ -610,6 +622,7 @@ while (*s)
       }
     cl = tolower(*p);
     cu = toupper(*p);
+    s++;
     }
   else if (yield != NULL)
     {
@@ -618,7 +631,7 @@ while (*s)
     cl = tolower(*p);
     cu = toupper(*p);
     }
-  s++;
+  else s++;
   }
 return NULL;
 }
@@ -692,6 +705,12 @@ if (p + len >= *size)
   {
   int oldsize = *size;
   while (*size <= p + len) *size += 50;
+
+  /* We must *not* use store_release() here as is done in a
+  similar bit of code in accept.c because we cannot guarantee
+  that there are no other calls to the store_ functions between
+  calls to string_cat(). */
+
   if (!store_extend(string, oldsize, *size))
     {
     char *newstring = store_get(*size);
@@ -851,6 +870,7 @@ while (*format != 0)
     break;
 
     case 's':
+    case 'S':                   /* Forces *lower* case */
     s = va_arg(ap, char *);
     if (s == NULL) s = "NULL";
     slen = (int)strlen(s);
@@ -886,7 +906,10 @@ while (*format != 0)
       width = precision = last - p - 1;
       }
     sprintf(p, "%*.*s", width, precision, s);
-    while (*p) p++;
+    if (format[-1] == 'S')
+      while (*p) { *p = tolower(*p); p++; }
+    else
+      while (*p) p++;
     if (!yield) goto END_FORMAT;
     break;
 
@@ -945,8 +968,8 @@ doesn't seem much we can do about that. */
 (void)string_vformat(buffer+15, sizeof(buffer) - 15, format, ap);
 
 return (eno == EACCES)?
-  string_sprintf("%s: %s (euid=%d egid=%d)", buffer, strerror(eno),
-    geteuid(), getegid()) :
+  string_sprintf("%s: %s (euid=%ld egid=%ld)", buffer, strerror(eno),
+    (long int)geteuid(), (long int)getegid()) :
   string_sprintf("%s: %s", buffer, strerror(eno));
 }
 
@@ -986,32 +1009,26 @@ if (dlocal)
     (int)strlen(addr->local_part));
 
 /* A routed local delivery or any remote delivery starts with the full
-address, which might be source routed. If it is the same as the original
+address. If it is the same as the original
 address, and no parents are to be shown, unset the flag for adding the
 top address. */
 
 else
   {
-  char buffer[512];
-  if (addr->local_part[0] == ',' || addr->local_part[0] == ':')
-    {
-    yield = string_cat(yield, &size, &ptr, "@", 1);
-    yield = string_cat(yield, &size, &ptr, addr->domain,
-      (int)strlen(addr->domain) );
-    yield = string_cat(yield, &size, &ptr, addr->local_part,
-      (int)strlen(addr->local_part));
-    }
-  else
-    {
-    yield = string_cat(yield, &size, &ptr, addr->local_part,
-      (int)strlen(addr->local_part));
-    yield = string_cat(yield, &size, &ptr, "@", 1);
-    yield = string_cat(yield, &size, &ptr, addr->domain,
-      (int)strlen(addr->domain) );
-    }
+  yield = string_cat(yield, &size, &ptr, addr->local_part,
+    (int)strlen(addr->local_part));
+  yield = string_cat(yield, &size, &ptr, "@", 1);
+  yield = string_cat(yield, &size, &ptr, addr->domain,
+    (int)strlen(addr->domain) );
   yield[ptr] = 0;
 
-  if (strcmp(yield, topaddr->orig) == 0 && addr->onetime_parent == NULL &&
+  /* If the address we are going to print is the same as the top address,
+  don't add on the top address. First of all, do a caseless comparison; if
+  this succeeds, do a caseful comparison on the local parts. */
+
+  if (strcmpic(yield, topaddr->orig) == 0 &&
+      strncmp(yield, topaddr->orig, strchr(yield, '@') - yield) == 0 &&
+      addr->onetime_parent == NULL &&
       (!all_parents || addr->parent == NULL || addr->parent == topaddr))
     add_topaddr = FALSE;
   }
@@ -1076,19 +1093,10 @@ printf("Testing is_ip_address\n");
 
 while (fgets(buffer, 256, stdin) != NULL)
   {
+  int offset;
   buffer[(int)strlen(buffer) - 1] = 0;
-  printf("%d\n", string_is_ip_address(buffer));
-  }
-
-printf("Testing is_ip_network\n");
-
-while (fgets(buffer, 256, stdin) != NULL)
-  {
-  int rc;
-  char *error;
-  buffer[(int)strlen(buffer) - 1] = 0;
-  printf("%d %s\n", rc = string_is_ip_network(buffer, &error),
-    (rc == 0)? error : "");
+  printf("%d\n", string_is_ip_address(buffer, NULL));
+  printf("%d %d %s\n", string_is_ip_address(buffer, &offset), offset, buffer);
   }
 
 /* This is a horrible lash-up, but it serves its purpose. */
