@@ -2,15 +2,16 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 /* Definitions of various structures. We have to pre-declare some because of
-mutually recursive definitions in the directors, routers, and transports
+mutually recursive definitions in the auths, directors, routers, and transports
 blocks. */
 
 struct address_item;
+struct auth_info;
 struct driver_info;
 struct director_info;
 struct transport_info;
@@ -33,15 +34,6 @@ typedef struct {
   void *value;
 } optionlist;
 
-
-/* Chain of compiled regular expressions for handling lists */
-
-typedef struct re_block {
-  struct re_block *next;
-  pcre *re;
-} re_block;
-
-
 /* Block for holding a uid and gid, possibly unset, and an initgroups flag. */
 
 typedef struct ugid_block {
@@ -52,10 +44,9 @@ typedef struct ugid_block {
   BOOL  initgroups;
 } ugid_block;
 
-
 /* Structure for holding information about a host for use mainly by
-routers, but also used when checking lists of hosts. Looking up host
-addresses is done in both cases, using this structure. */
+routers, but also used when checking lists of hosts and when transporting.
+Looking up host addresses is done using this structure. */
 
 typedef struct host_item {
   struct host_item *next;
@@ -67,9 +58,9 @@ typedef struct host_item {
   int   status;                  /* Usable, unusable, or unknown */
   int   why;                     /* Why host is unusable */
   int   last_try;                /* Time of last try if known */
+  BOOL  update_waiting;          /* Turned off if wait db not to be updated */
   /* Used when checking */
   char *ident_string;            /* RFC 1413 ident string */
-  re_block *compiled_name;       /* When name is wildcarded */
 } host_item;
 
 /* Directors, routers, and transports have similar data blocks. Each driver
@@ -131,20 +122,25 @@ typedef struct transport_instance {
   char *expand_gid;              /* )                                  */
   char *home_dir;                /* )                                  */
   char *current_dir;             /* )                                  */
+  char *warn_message;            /* ) Text for the message             */
   BOOL  deliver_as_creator;      /* )                                  */
   BOOL  initgroups;              /* )                                  */
   BOOL  bsmtp_helo;              /* )                                  */
                                  /**************************************/
   BOOL  multi_domain;            /* Applies to remote transports only  */
+  BOOL  overrides_hosts;         /* Ditto                              */
                                  /**************************************/
   struct transport_instance *shadow;  /* Shadow, to use on success. */
   char *shadow_condition;        /* Condition for running it */
   char *filter_command;          /* For on-the-fly-filtering */
   char *add_headers;             /* Add these headers */
   char *remove_headers;          /* Remove these headers */
+  char *return_path;             /* Overriding return path */
   char *debug_string;            /* Debugging output */
+  BOOL  body_only;               /* Deliver only the body */
   BOOL  delivery_date_add;       /* Add Delivery-Date header */
   BOOL  envelope_to_add;         /* Add Envelope-To header */
+  BOOL  headers_only;            /* Deliver only the headers */
   BOOL  return_path_add;         /* Add Return-Path header */
   BOOL  return_output;           /* TRUE if output should always be returned */
   BOOL  return_fail_output;      /* ditto, but only on failure */
@@ -153,6 +149,7 @@ typedef struct transport_instance {
   BOOL  log_defer_output;
   BOOL  retry_use_local_part;    /* Defaults true for local, false for remote */
   int   max_addresses;           /* Max amalgamation for remote delivery */
+  int   message_size_limit;      /* Biggest message this transport handles */
 } transport_instance;
 
 
@@ -193,25 +190,17 @@ typedef struct director_instance {
   /* so they aren't in this block.                                    */
   /********************************************************************/
   char *domains;                 /* Specific domains */
-  char *except_domains;          /* Specific not domains */
   char *local_parts;             /* Specific local parts */
-  char *except_local_parts;      /* Specific not local parts */
   char *errors_to;               /* Errors address */
   char *extra_headers;           /* Additional headers */
   char *remove_headers;          /* Those to remove */
   char *debug_string;            /* Debugging output */
   char *fallback_hosts;          /* For remote transports */
   host_item *fallback_hostlist;
-  re_block *re_domains;          /* Used when domains are wildcarded */
-  re_block *re_except_domains;
-  re_block *re_local_parts;
-  re_block *re_except_local_parts;
   BOOL  more;                    /* If FALSE, do no more if this one fails */
   BOOL  verify_only;             /* Skip this director if not verifying */
   BOOL  verify_sender;           /* Use this director when verifying a sender */
   BOOL  verify_recipient;        /* Use this director when verifying a recipient*/
-  transport_instance *transport; /* Assigned transport */
-  char *expand_transport;        /* Dynamic transport string */
   char *expand_uid;              /* Expanded uid string */
   char *expand_gid;              /* Expanded gid string */
   uid_t uid;                     /* Fixed uid value */
@@ -221,15 +210,16 @@ typedef struct director_instance {
   BOOL  initgroups;              /* True if initgroups is required */
   /********************************************************************/
 
+  transport_instance *transport; /* Assigned transport */
+  char *expand_transport;        /* Dynamic transport string */
+  char *current_directory;       /* For use during delivery */
+  char *home_directory;          /* Ditto */
   char *condition;               /* General condition */
   char *prefix;                  /* Address prefix */
   char *suffix;                  /* Address suffix */
   char *senders;                 /* Specific senders */
-  char *except_senders;          /* Specific not senders */
   char *require_files;           /* Required (or not) for director to run */
   char *new_director;            /* Name of director for generated address */
-  re_block *re_senders;
-  re_block *re_except_senders;
   struct director_instance *new; /* Actual director for generated address */
   BOOL  unseen;                  /* If TRUE, carry on even after success */
   BOOL  prefix_optional;         /* Just what it says */
@@ -279,25 +269,17 @@ typedef struct router_instance {
   /* so they aren't in this block.                                    */
   /********************************************************************/
   char *domains;                 /* Specific domains */
-  char *except_domains;          /* Specific not domains */
   char *local_parts;             /* Specific local parts */
-  char *except_local_parts;      /* Specific not local parts */
   char *errors_to;               /* Errors address */
   char *extra_headers;           /* Additional headers */
   char *remove_headers;          /* Removed headers */
   char *debug_string;            /* Debugging output */
   char *fallback_hosts;          /* For remote transports */
   host_item *fallback_hostlist;
-  re_block *re_domains;          /* Used when domains are wildcarded */
-  re_block *re_except_domains;
-  re_block *re_local_parts;
-  re_block *re_except_local_parts;
   BOOL  more;                    /* If FALSE, do no more if this one fails */
   BOOL  verify_only;             /* Skip this router if not verifying */
   BOOL  verify_sender;           /* Use this router when verifying a sender */
   BOOL  verify_recipient;        /* Use this router when verifying a recipient*/
-  transport_instance *transport; /* Assigned transport */
-  char *expand_transport;        /* Dynamic transport string */
   char *expand_uid;              /* Expanded uid string */
   char *expand_gid;              /* Expanded gid string */
   uid_t uid;                     /* Fixed uid value */
@@ -307,12 +289,11 @@ typedef struct router_instance {
   BOOL  initgroups;              /* True if initgroups is required */
   /********************************************************************/
 
+  transport_instance *transport; /* Assigned transport */
+  char *expand_transport;        /* Dynamic transport string */
   char *condition;               /* General condition */
   char *senders;                 /* Specific senders */
-  char *except_senders;          /* Specific not senders */
   char *require_files;           /* Files required (or not) for router to run */
-  re_block *re_senders;
-  re_block *re_except_senders;
   BOOL  unseen;                  /* If TRUE carry on, even after success */
   BOOL  fail_verify_sender;      /* Fail verify if sender match this router */
   BOOL  fail_verify_recipient;   /* Fail verify if recipient match this router */
@@ -320,6 +301,8 @@ typedef struct router_instance {
   char *self;                    /* Text option for handling self reference */
   int   self_code;               /* Encoded version of same */
   BOOL  self_rewrite;            /* TRUE to rewrite headers if making local */
+  BOOL  sender_rewrite;          /* TRUE to rewrite verified sender */
+  char *translate_ip_address;    /* IP address translation fudgery */
 } router_instance;
 
 
@@ -369,9 +352,49 @@ typedef struct lookup_info {
   void (*close)(                 /* close function */
     void *);                     /* handle */
   void (*tidy)(void);            /* tidy function */
+  char *(*quote)(                /* quoting function */
+    char *);                     /* string to quote */
 } lookup_info;
 
 
+/* Structure for holding information about the configured authentication
+mechanisms */
+
+typedef struct auth_instance {
+  struct auth_instance *next;
+  char *name;                    /* Exim instance name */
+  struct auth_info *info;        /* Pointer to driver info block */
+  void *options_block;           /* Pointer to private options */
+  char *driver_name;             /* Must be first */
+  char *public_name;             /* Advertised name */
+  char *set_id;                  /* String to set as authenticated id */
+  BOOL  client;                  /* TRUE if client option(s) set */
+  BOOL  server;                  /* TRUE if server options(s) set */
+} auth_instance;
+
+
+/* Structure for holding information about an authentication mechanism */
+
+typedef struct auth_info {
+  char *driver_name;             /* e.g. "condition" */
+  int (*servercode)(             /* server function */
+    auth_instance *,             /* the instance data */
+    char *);                     /* rest of AUTH command */
+  void (*init)(                  /* initialization function */
+    struct auth_instance *);
+  int (*clientcode)(             /* client function */
+    struct auth_instance *,
+    int,                         /* socket */
+    int,                         /* command timeout */
+    char *,                      /* buffer for reading response */
+    int);                        /* sizeof buffer */
+  optionlist *options;           /* Table of private options names */
+  int  *options_count;           /* -> Number of entries in table */
+  void *options_block;           /* Points to default private block */
+  int   options_len;             /* Length of same in bytes */
+} auth_info;
+
+
 /* Structure for holding a single IP address; used for the chain of
 addresses and ports for the local host. Make the char string large enough
 to hold an IPv6 address. */
@@ -381,22 +404,6 @@ typedef struct ip_address_item {
   char address[46];
 } ip_address_item;
 
-/* Structure for holding information about an IP net; used for controlling
-incoming and outgoing SMTP calls. The size field gives the number of words
-used in the address and mask fields. It's a bit untidy screwing in 4 here, but
-it makes things less messy. When we get to IPv8 this may have to change...
-The ability to do file lookups for networks is also available (at a cost, of
-course). If filename != NULL, is is scanned instead of using the other
-fields. */
-
-typedef struct ip_net_item {
-  struct ip_net_item *next;
-  int size;
-  int address[4];
-  int mask[4];
-  char *filename;
-} ip_net_item;
-
 
 /* Information about a soft delivery failure, for use when calculating
 retry information. It's separate from the address block, because there
@@ -409,15 +416,21 @@ typedef struct retry_item {
   int   basic_errno;             /* error code for this destination */
   int   more_errno;              /* additional error information */
   char *message;                 /* local error message */
-  BOOL  delete;                  /* TRUE when retry info is to be deleted */
+  int   flags;                   /* see below */
 } retry_item;
 
+/* Retry data flags */
+
+#define rf_delete   0x0001       /* retry info is to be deleted */
+#define rf_host     0x0002       /* retry info is for a remote host */
+#define rf_message  0x0004       /* retry info is for a host+message */
 
 /* Information about a constructed message that is to be sent using the
 autoreply transport. This is pointed to from the address block. */
 
 typedef struct reply_item {
   char *from;                    /* ) */
+  char *reply_to;                /* ) */
   char *to;                      /* ) */
   char *cc;                      /* ) specific header fields */
   char *bcc;                     /* ) */
@@ -428,6 +441,7 @@ typedef struct reply_item {
   BOOL  file_expand;             /* expand the body */
   char *logfile;                 /* file to keep a log in */
   char *oncelog;                 /* file to keep records in for once only */
+  time_t once_repeat;            /* time to repeat "once only" */
   BOOL  return_message;          /* send back the original message */
 } reply_item;
 
@@ -478,7 +492,6 @@ typedef struct address_item {
   BOOL local_part_quoted;        /* local part was quoted */
   BOOL forced_local;             /* by a self option on a router */
   BOOL delivered;                /* already delivered */
-  BOOL routed_by_domain;         /* routing did not depend on local part */
   BOOL ignore_error;             /* ignore delivery error */
   BOOL local_host_removed;       /* local host was backup */
   header_line *extra_headers;    /* additional headers */
@@ -562,7 +575,6 @@ typedef struct retry_rule {
 typedef struct retry_config {
   struct retry_config *next;
   char *destination;
-  re_block *re;
   int  basic_errno;
   int  more_errno;
   retry_rule *rules;
@@ -574,7 +586,6 @@ typedef struct rewrite_rule {
   struct rewrite_rule *next;
   int flags;
   char *key;
-  re_block *re;
   char *replacement;
 } rewrite_rule;
 
@@ -593,10 +604,15 @@ typedef struct tree_node {
 } tree_node;
 
 /* Structure for holding the handle and the cached last lookup for searches.
-This block is pointed to by the tree entry for the open file. */
+This block is pointed to by the tree entry for the file. The file can get
+closed if too many are opened at once. There is a LRU chain for deciding which
+to close. */
 
 typedef struct search_cache {
-  void *handle;                   /* lookup handle */
+  void *handle;                   /* lookup handle, or NULL if closed */
+  int search_type;                /* search type */
+  tree_node *up;                  /* LRU up pointer */
+  tree_node *down;                /* LRU down pointer */
   char *key;                      /* the key of the last lookup */
   char *data;                     /* the data of the last lookup */
 } search_cache;
@@ -611,4 +627,12 @@ typedef struct {
   u_char *data;                   /* pointer to data */
 } dns_record;
 
+/* Structure used for holding intermediate data during MD5 computations. */
+
+typedef struct md5 {
+  unsigned int length;
+  unsigned int abcd[4];
+  }
+md5;
+
 /* End of structs.h */
