@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* A function for returning the time of day in various formats */
@@ -85,12 +85,25 @@ else
     strftime(timebuf + len, sizeof(timebuf) - len, " %Y", &local);
     }
 
+  /* tod_mbx: format used in MBX mailboxes - subtly different to tod_full */
+
+  #ifdef SUPPORT_MBX
+  else if (type == tod_mbx)
+    {
+    int len;
+    (void) sprintf(timebuf, "%02d-", local.tm_mday);
+    len = strlen(timebuf);
+    len += strftime(timebuf + len, sizeof(timebuf) - len, "%b-%Y %H:%M:%S", &local);
+    (void) sprintf(timebuf + len, " %+03d%02d", diff_hour, diff_min);
+    }
+  #endif
+
   /* tod_full: format used in Received: headers. */
 
   else
     {
     int len = strftime(timebuf, sizeof(timebuf), "%a, ", &local);
-    (void) sprintf(timebuf + len, "%d ", local.tm_mday);
+    (void) sprintf(timebuf + len, "%02d ", local.tm_mday);
     len += strlen(timebuf + len);
     len += strftime(timebuf + len, sizeof(timebuf) - len, "%b %Y %H:%M:%S", &local);
     (void) sprintf(timebuf + len, " %+03d%02d", diff_hour, diff_min);
