@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* General functions concerned with transportation, and generic options for all
@@ -12,10 +12,14 @@ transports. */
 #include "exim.h"
 
 
-/* Newline status flag and current pointer for write_chunk() */
+/* Static data for write_chunk() */
 
-static BOOL was_nl;
-static char *chunk_ptr;
+static char *chunk_ptr;           /* chunk pointer */
+static char *nl_check;            /* string to look for at line start */
+static int   nl_check_length;     /* length of same */
+static char *nl_escape;           /* string to insert */
+static int   nl_escape_length;    /* length of same */
+static int   nl_partial_match;    /* length matched at chunk end */
 
 
 /* Generic options for transports, all of which live inside transport_instance
@@ -26,6 +30,8 @@ certain transports. */
 optionlist optionlist_transports[] = {
   { "add_headers",      opt_stringptr|opt_public|opt_hidden,
                  (void *)offsetof(transport_instance, add_headers) },
+  { "body_only",        opt_bool|opt_public,
+                 (void *)offsetof(transport_instance, body_only) },
   { "debug_print",      opt_stringptr | opt_public,
                  (void *)offsetof(transport_instance, debug_string) },
   { "delivery_date_add", opt_bool|opt_public,
@@ -36,10 +42,16 @@ optionlist optionlist_transports[] = {
                  (void *)(offsetof(transport_instance, envelope_to_add)) },
   { "headers_add",      opt_stringptr|opt_public,
                  (void *)offsetof(transport_instance, add_headers) },
+  { "headers_only",     opt_bool|opt_public,
+                 (void *)offsetof(transport_instance, headers_only) },
   { "headers_remove",   opt_stringptr|opt_public,
                  (void *)offsetof(transport_instance, remove_headers) },
+  { "message_size_limit", opt_int|opt_public,
+                 (void *)offsetof(transport_instance, message_size_limit) },
   { "remove_headers",   opt_stringptr|opt_public|opt_hidden,
                  (void *)offsetof(transport_instance, remove_headers) },
+  { "return_path",      opt_stringptr|opt_public,
+                 (void *)(offsetof(transport_instance, return_path)) },
   { "return_path_add",   opt_bool|opt_public,
                  (void *)(offsetof(transport_instance, return_path_add)) },
   { "shadow_condition", opt_stringptr|opt_public,
@@ -96,6 +108,11 @@ for (t = transports; t != NULL; t = t->next)
         "shadow transport %s (set on %s transport) is not a local transport",
           tt->name, t->name);
     }
+
+  if (t->body_only && t->headers_only)
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+      "%s transport: body_only and headers_only are mutually exclusive",
+      t->name);
   }
 }
 
@@ -245,57 +262,78 @@ return transport_write_block(fd, big_buffer, strlen(big_buffer));
 
 /* Subroutine used by transport_write_message() to scan character chunks for
 newlines and act appropriately. The object is to minimise the number of writes.
-In practice, either from_hack or (use_crlf & smtp_dots) is likely to be set,
-but not both. However, the code works with any combination. The output byte
-stream is buffered up in deliver_out_buffer, which is written only when it gets
-full, thus minimizing write operations and TCP packets.
+The output byte stream is buffered up in deliver_out_buffer, which is written
+only when it gets full, thus minimizing write operations and TCP packets.
 
-If the from hack or smtp_dots options are requested, we must do the necessary
-at the start of the chunk if the previous last char was '\n'. This is handled
-by a static flag.
+Static data is used to handle the case when the last character of the previous
+chunk was NL, or matched part of the data that has to be escaped.
 
 Arguments:
   fd         file descript to write to
   chunk      pointer to data to write
   len        length of data to write
-  from_hack  TRUE if the "from hack" is wanted
   usr_crlf   TRUE if CR LF is wanted at the end of each line
-  smtp_dots  TRUE if the SMTP dot-escaping algorithm is wanted
+
+In addition, the static nl_xxx variables must be set as required.
 
 Returns:     TRUE on success, FALSE on failure (with errno preserved)
 */
 
 static BOOL
-write_chunk(int fd, char *chunk, int len, BOOL from_hack, BOOL use_crlf,
-  BOOL smtp_dots)
+write_chunk(int fd, char *chunk, int len, BOOL use_crlf)
 {
 char *start = chunk;
 char *end = chunk + len;
 register char *ptr;
+int mlen = DELIVER_OUT_BUFFER_SIZE - nl_escape_length - 2;
 
-/* Scan the buffer for newlines while copying into the output
-buffer. First handle the case when the preceding char was NL. */
+/* The assumption is made that the check string will never stretch over move
+than one chunk since the only time there are partial matches is when copying
+the body in large buffers. There is always enough room in the buffer for an
+escape string, since the loop below ensures this for each character it
+processes, and it won't have stuck in the escape string if it left a partial
+match. */
 
-if (was_nl)
+if (nl_partial_match >= 0)
   {
-  if (from_hack && len >= 5 && strncmp(start, "From ", 5) == 0)
-    *chunk_ptr++ = '>';
-  else if (smtp_dots && *start == '.')
-    *chunk_ptr++ = '.';
-  was_nl = FALSE;
+  if (nl_check_length > 0 && len >= nl_check_length &&
+      strncmp(start, nl_check + nl_partial_match,
+        nl_check_length - nl_partial_match) == 0)
+    {
+    strncpy(chunk_ptr, nl_escape, nl_escape_length);
+    chunk_ptr += nl_escape_length;
+    start += nl_check_length - nl_partial_match;
+    }
+
+  /* The partial match was a false one. Insert the characters carried over
+  from the previous chunk. */
+
+  else if (nl_partial_match > 0)
+    {
+    strncpy(chunk_ptr, nl_check, nl_partial_match);
+    chunk_ptr += nl_partial_match;
+    }
+
+  nl_partial_match = -1;
   }
 
-/* Now process the characters in the chunk. */
+/* Now process the characters in the chunk. Whenever we hit a newline we check
+for possible escaping. The code for the non-NL route should be as fast as
+possible. */
 
 for (ptr = start; ptr < end; ptr++)
   {
   register int ch;
 
-  /* If there isn't room for another 4 characters, flush the buffer. */
+  /* Flush the buffer if it has reached the threshold - we want to leave enough
+  room for the next char, plus a possible extra CR for an LF, plus the escape
+  string. */
 
-  if ((len = chunk_ptr - deliver_out_buffer) > DELIVER_OUT_BUFFER_SIZE - 5)
+  if (chunk_ptr - deliver_out_buffer > mlen)
     {
-    if (!transport_write_block(fd, deliver_out_buffer, len)) return FALSE;
+    if (!transport_write_block(fd, deliver_out_buffer,
+          chunk_ptr - deliver_out_buffer))
+      return FALSE;
     chunk_ptr = deliver_out_buffer;
     }
 
@@ -306,28 +344,39 @@ for (ptr = start; ptr < end; ptr++)
     /* Insert CR before NL if required */
 
     if (use_crlf) *chunk_ptr++ = '\r';
-
-    /* Now the NL */
-
     *chunk_ptr++ = '\n';
 
-    /* If it happens that \n is the last character in the chunk, set the
-    flag for next time (and for checking the final char). */
+    /* The check_string test (formerly "from hack") replaces the specific
+    string at the start of a line with an escape string (e.g. "From " becomes
+    ">From " or "." becomes "..". It is a case-sensitive test. The length
+    check above ensures there is always enough room to insert this string. */
 
-    if (left <= 0) was_nl = TRUE;
+    if (nl_check_length > 0)
+      {
+      if (left >= nl_check_length &&
+          strncmp(ptr+1, nl_check, nl_check_length) == 0)
+        {
+        strncpy(chunk_ptr, nl_escape, nl_escape_length);
+        chunk_ptr += nl_escape_length;
+        ptr += nl_check_length;
+        }
 
-    /* The "from hack" inserts ">" before any line starting with "From ". It
-    is a case-sensitive test. */
+      /* Handle the case when there isn't enough left to match the whole
+      check string, but there may be a partial match. We remember how many
+      characters matched, and finish processing this chunk. */
 
-    else if (from_hack && left >= 5 && strncmp(ptr+1, "From ", 5) == 0)
-      *chunk_ptr++ = '>';
+      else if (left <= 0) nl_partial_match = 0;
 
-    /* The smtp dot convention inserts a dot before any line starting with a
-    dot. (We don't need to test left >= 1 because we know it isn't <= 0.) */
-
-    else if (smtp_dots && ptr[1] == '.') *chunk_ptr++ = '.';
+      else if (strncmp(ptr+1, nl_check, left) == 0)
+        {
+        nl_partial_match = left;
+        ptr = end;
+        }
+      }
     }
 
+  /* Not a NL character */
+
   else *chunk_ptr++ = ch;
   }
 
@@ -355,14 +404,12 @@ file descriptor deliver_datafile. Make sure we start it at the beginning.
 . If add_delivery_date is TRUE, a "delivery-date:" header is added to the
   message. It gives the time and date that delivery took place.
 
-. If from_hack is true, the "from hack" is done to lines starting with the text
-  "From " (what a shoddy specification). The from hack is not applied to
-  headers.
+. If check_string is not null, the start of each line is checked for that
+  string. If it is found, it is replaced by escape_string. This used to be
+  the "from hack" for files, and "smtp_dots" for escaping SMTP dots.
 
 . If use_crlf is true, newlines are turned into CRLF (SMTP output).
 
-. If smtp_dots is true, lines starting with . get an extra . added.
-
 The yield is TRUE if all went well, and FALSE if not. Exit *immediately* after
 any writing or reading error, leaving the code in errno intact. Error exits
 can include timeouts for certain transports, which are requested by setting
@@ -376,11 +423,10 @@ Arguments:
     add_return_path       if TRUE, add a "return-path" header
     add_envelope_to       if TRUE, add a "envelope-to" header
     add_delivery_date     if TRUE, add a "delivery-date" header
-    from_hack             if TRUE, apply the "from hack"
     use_crlf              if TRUE, turn NL into CR LF
-    smtp_dots             if TRUE, implement the SMTP dot escaping algorithm
     dot_end               if TRUE, send a terminating "." line at the end
-  errors_to             if not NULL, used for the return-path header
+    no_headers            if TRUE, omit the headers
+    no_body               if TRUE, omit the body
   size_limit            if > 0, this is a limit to the size of message written;
                           it is used when returning messages to their senders,
                           and is approximate rather than exact, owing to chunk
@@ -389,7 +435,9 @@ Arguments:
                           expanded, and must be in correct RFC 822 format as
                           it is transmitted verbatim; NULL => no additions,
                           and so does empty string or forced expansion fail
-  remove_headers        a colon-separated list of headers to remove, or NULL.
+  remove_headers        a colon-separated list of headers to remove, or NULL
+  check_string          a string to check for at the start of lines, or NULL
+  escape_string         a string to insert in front of any check string
 
 Returns:                TRUE on success; FALSE (with errno) on failure.
                         In addition, the global variable transport_count
@@ -398,208 +446,243 @@ Returns:                TRUE on success; FALSE (with errno) on failure.
 
 static BOOL
 internal_transport_write_message(address_item *addr, int fd, int options,
-  char *errors_to, int size_limit, char *add_headers, char *remove_headers)
+  int size_limit, char *add_headers, char *remove_headers, char *check_string,
+  char *escape_string)
 {
 int written = 0;
 int len;
 header_line *h;
-BOOL from_hack = (options & topt_from_hack) != 0;
 BOOL use_crlf  = (options & topt_use_crlf)  != 0;
-BOOL smtp_dots = (options & topt_smtp_dots) != 0;
 
-/* Initialize newline flag, and pointer in output buffer. */
+/* Initialize pointer in output buffer. */
 
-was_nl = TRUE;
 chunk_ptr = deliver_out_buffer;
 
-/* Add return-path: if requested, and there is one. */
+/* Set up the data for start-of-line data checking and escaping */
 
-if ((options & topt_add_return_path) != 0)
+nl_partial_match = -1;
+if (check_string != NULL && escape_string != NULL)
   {
-  char buffer[SENDER_ADDRESS_MAXLENGTH + 20];
-  sprintf(buffer, "Return-path: <%.*s>\n", SENDER_ADDRESS_MAXLENGTH,
-    (errors_to != NULL)? errors_to : user_null_sender? "" : sender_address);
-  len = (int)strlen(buffer);
-  if (!write_chunk(fd, buffer, len, FALSE, use_crlf, FALSE)) return FALSE;
+  nl_check = check_string;
+  nl_check_length = (int)strlen(nl_check);
+  nl_escape = escape_string;
+  nl_escape_length = (int)strlen(nl_escape);
   }
+else nl_check_length = nl_escape_length = 0;
 
-/* Add envelope-to: if requested, picking up from all the addresses, and
-any relevant duplicates. */
+/* Whether the escaping mechanism is applied to headers or not is controlled by
+an option (set for SMTP, not otherwise). Negate the length if not wanted till
+after the headers. */
 
-if ((options & topt_add_envelope_to) != 0)
+if ((options & topt_escape_headers) == 0) nl_check_length = -nl_check_length;
+
+/* Write the headers if required, including any that have to be added */
+
+if ((options & topt_no_headers) == 0)
   {
-  address_item *p;
-  char buffer[256];
-  strcpy(buffer, "Envelope-to: ");
-  len = (int)strlen(buffer);
+  /* Add return-path: if requested. */
 
-  for (p = addr; p != NULL; p = p->next)
+  if ((options & topt_add_return_path) != 0)
     {
-    address_item *pp, *dup;
+    char buffer[SENDER_ADDRESS_MAXLENGTH + 20];
+    sprintf(buffer, "Return-path: <%.*s>\n", SENDER_ADDRESS_MAXLENGTH,
+      user_null_sender? "" : return_path);
+    len = (int)strlen(buffer);
+    if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
+    }
 
-    /* Scan for any duplicates that relate to this address. */
+  /* Add envelope-to: if requested, picking up from all the addresses, and
+  any relevant duplicates. */
 
-    for (dup = addr_duplicate; dup != NULL; dup = dup->next)
+  if ((options & topt_add_envelope_to) != 0)
+    {
+    address_item *p;
+    char buffer[256];
+    strcpy(buffer, "Envelope-to: ");
+    len = (int)strlen(buffer);
+
+    for (p = addr; p != NULL; p = p->next)
       {
-      if (strcmp(dup->unique, p->unique) == 0)
+      address_item *pp, *dup;
+
+      /* Scan for any duplicates that relate to this address. */
+
+      for (dup = addr_duplicate; dup != NULL; dup = dup->next)
         {
-        address_item *dup_orig;
-        for (dup_orig = dup; dup_orig->parent != NULL;
-          dup_orig = dup_orig->parent);
-        sprintf(buffer + len, "%.200s,\n ", dup_orig->orig);
-        len += (int)strlen(dup_orig->orig) + 3;
-        if (!write_chunk(fd, buffer, len, FALSE, use_crlf, FALSE)) return FALSE;
-        buffer[0] = 0;
-        len = 0;
+        if (strcmp(dup->unique, p->unique) == 0)
+          {
+          address_item *dup_orig;
+          for (dup_orig = dup; dup_orig->parent != NULL;
+            dup_orig = dup_orig->parent);
+          sprintf(buffer + len, "%.200s,\n ", dup_orig->orig);
+          len += (int)strlen(dup_orig->orig) + 3;
+          if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
+          buffer[0] = 0;
+          len = 0;
+          }
         }
+
+      /* Now list the parent of this address */
+
+      for (pp = p; pp->parent != NULL; pp = pp->parent);
+      strncpy(buffer + len, pp->orig, 200);
+      len += (int)strlen(pp->orig);
+
+      if (p->next == NULL)
+        {
+        strcpy(buffer + len, "\n");
+        len += 1;
+        }
+      else
+        {
+        strcpy(buffer + len, ",\n ");
+        len += 3;
+        }
+
+      if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
+      buffer[0] = 0;
+      len = 0;
+      }
+    }
+
+  /* Add delivery-date: if requested. */
+
+  if ((options & topt_add_delivery_date) != 0)
+    {
+    char buffer[100];
+    sprintf(buffer, "Delivery-date: %s\n", tod_stamp(tod_full));
+    len = strlen(buffer);
+    if (!write_chunk(fd, buffer, len, use_crlf)) return FALSE;
+    }
+
+  /* Then the message's headers. Don't write any that are flagged as "old"; that
+  means they were rewritten, or are a record of envelope rewriting, or were
+  removed (e.g. Bcc). If remove_headers is not null, skip any headers that
+  match any entries therein. Then check addr->remove_headers too, provided that
+  addr is not NULL. */
+
+  if (remove_headers != NULL)
+    {
+    char *s = expand_string(remove_headers);
+    if (s == NULL && !expand_string_forcedfail)
+      {
+      errno = ERRNO_CHHEADER_FAIL;
+      return FALSE;
+      }
+    remove_headers = s;
+    }
+
+  for (h = header_list; h != NULL; h = h->next)
+    {
+    int i;
+    char *list = NULL;
+    BOOL include_header;
+
+    if (h->type == htype_old) continue;
+
+    include_header = TRUE;
+    list = remove_headers;
+
+    for (i = 0; i < 2; i++)    /* For remove_headers && addr->remove_headers */
+      {
+      if (list != NULL)
+        {
+        char *s, *ss;
+        char buffer[128];
+        while ((s = string_nextinlist(&list, ':', buffer, sizeof(buffer)))
+                != NULL)
+          {
+          int len = (int)strlen(s);
+          if (strncmpic(h->text, s, len) != 0) continue;
+          ss = h->text + len;
+          while (*ss == ' ' || *ss == '\t') ss++;
+          if (*ss == ':') break;
+          }
+        if (s != NULL) { include_header = FALSE; break; }
+        }
+      if (addr != NULL) list = addr->remove_headers;
       }
 
-    /* Now list the parent of this address */
+    if (include_header && !write_chunk(fd, h->text, h->slen, use_crlf))
+      return FALSE;
+    }
 
-    for (pp = p; pp->parent != NULL; pp = pp->parent);
-    strncpy(buffer + len, pp->orig, 200);
-    len += (int)strlen(pp->orig);
+  /* Add on any address-specific headers. If there are multiple addresses,
+  they will all have the same headers in order to be batched. */
 
-    if (p->next == NULL)
+  if (addr != NULL)
+    for (h = addr->extra_headers; h!= NULL; h = h->next)
+      if (!write_chunk(fd, h->text, h->slen, use_crlf)) return FALSE;
+
+  /* If a string containing additional headers exists, expand it and write
+  out the result. This is done last so that if it (deliberately or accidentally)
+  isn't in header format, it won't mess up any other headers. An empty string or
+  a forced expansion failure are noops. */
+
+  if (add_headers != NULL)
+    {
+    char *s = expand_string(add_headers);
+    if (s == NULL)
       {
-      strcpy(buffer + len, "\n");
-      len += 1;
+      if (!expand_string_forcedfail)
+        {
+        errno = ERRNO_CHHEADER_FAIL;
+        return FALSE;
+        }
       }
     else
       {
-      strcpy(buffer + len, ",\n ");
-      len += 3;
-      }
-
-    if (!write_chunk(fd, buffer, len, FALSE, use_crlf, FALSE)) return FALSE;
-    buffer[0] = 0;
-    len = 0;
-    }
-  }
-
-/* Add delivery-date: if requested. */
-
-if ((options & topt_add_delivery_date) != 0)
-  {
-  char buffer[100];
-  sprintf(buffer, "Delivery-date: %s\n", tod_stamp(tod_full));
-  len = strlen(buffer);
-  if (!write_chunk(fd, buffer, len, FALSE, use_crlf, FALSE)) return FALSE;
-  }
-
-/* Then the message's headers. Don't write any that are flagged as "old"; that
-means they were rewritten, or are a record of envelope rewriting, or were
-removed (e.g. Bcc). If remove_headers is not null, skip any headers that
-match any entries therein. Then check addr->remove_headers too, provided that
-addr is not NULL. */
-
-for (h = header_list; h != NULL; h = h->next)
-  {
-  int i;
-  char *list;
-  BOOL include_header;
-
-  if (h->type == htype_old) continue;
-
-  include_header = TRUE;
-  list = remove_headers;
-
-  for (i = 0; i < 2; i++)    /* For remove_headers && addr->remove_headers */
-    {
-    if (list != NULL)
-      {
-      char *s, *ss;
-      char buffer[128];
-      while ((s = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
+      int len = (int)strlen(s);
+      if (len > 0)
         {
-        int len = (int)strlen(s);
-        if (strncmpic(h->text, s, len) != 0) continue;
-        ss = h->text + len;
-        while (*ss == ' ' || *ss == '\t') ss++;
-        if (*ss == ':') break;
+        if (!write_chunk(fd, s, len, use_crlf)) return FALSE;
+        if (s[len-1] != '\n' && !write_chunk(fd, "\n", 1, use_crlf))
+          return FALSE;
         }
-      if (s != NULL) { include_header = FALSE; break; }
       }
-    if (addr != NULL) list = addr->remove_headers;
     }
 
-  if (include_header &&
-      !write_chunk(fd, h->text, h->slen, FALSE, use_crlf, smtp_dots))
+  /* Separate headers from body with a blank line */
+
+  if (!write_chunk(fd, "\n", 1, use_crlf)) return FALSE;
+  }
+
+/* If the body is required, ensure that the data for check strings (formerly
+the "from hack") is enabled by negating the length if necessary. (It will be
+negative in cases where it isn't to apply to the headers). Then ensure the body
+is positioned at the start of its file (following the message id), then write
+it, applying the size limit if required. */
+
+if ((options & topt_no_body) == 0)
+  {
+  nl_check_length = abs(nl_check_length);
+  nl_partial_match = 0;
+  lseek(deliver_datafile, DATA_START_OFFSET, SEEK_SET);
+  while ((len = read(deliver_datafile, deliver_in_buffer,
+           DELIVER_IN_BUFFER_SIZE)) > 0)
+    {
+    if (!write_chunk(fd, deliver_in_buffer, len, use_crlf)) return FALSE;
+    if (size_limit > 0)
+      {
+      written += len;
+      if (written > size_limit) break;
+      }
+    }
+
+  /* Finished with the check string */
+
+  nl_check_length = nl_escape_length = 0;
+
+  /* A read error on the body will have left len == -1 and errno set. */
+
+  if (len != 0) return FALSE;
+
+  /* If requested, add a terminating "." line (SMTP output). */
+
+  if ((options & topt_end_dot) != 0 && !write_chunk(fd, ".\n", 2, use_crlf))
     return FALSE;
   }
 
-/* Add on any address-specific headers. If there are multiple addresses,
-they will all have the same headers in order to be batched. */
-
-if (addr != NULL)
-  for (h = addr->extra_headers; h!= NULL; h = h->next)
-    if (!write_chunk(fd, h->text, h->slen, FALSE, use_crlf, smtp_dots))
-      return FALSE;
-
-/* If a string containing additional headers exists, expand it and write
-out the result. This is done last so that if it (deliberately or accidentally)
-isn't in header format, it won't mess up any other headers. An empty string or
-a forced expansion failure are noops. */
-
-if (add_headers != NULL)
-  {
-  char *s = expand_string(add_headers);
-  if (s == NULL)
-    {
-    if (!expand_string_forcedfail)
-      {
-      errno = ERRNO_ADDHEADER_FAIL;
-      return FALSE;
-      }
-    }
-  else
-    {
-    int len = (int)strlen(s);
-    if (len > 0)
-      {
-      if (!write_chunk(fd, s, len, FALSE, use_crlf, smtp_dots))
-        return FALSE;
-      if (s[len-1] != '\n' && !write_chunk(fd, "\n", 1, FALSE, use_crlf,
-        smtp_dots)) return FALSE;
-      }
-    }
-  }
-
-/* Separate headers from body with a blank line */
-
-if (!write_chunk(fd, "\n", 1, FALSE, use_crlf, smtp_dots)) return FALSE;
-
-/* Ensure the body is positioned at the start of its file, following the
-message id if present, then write it, applying the size limit if required. */
-
-lseek(deliver_datafile, data_start_offset, SEEK_SET);
-while ((len = read(deliver_datafile, deliver_in_buffer,
-         DELIVER_IN_BUFFER_SIZE)) > 0)
-  {
-  if (!write_chunk(fd, deliver_in_buffer, len, from_hack, use_crlf, smtp_dots))
-    return FALSE;
-  if (size_limit > 0)
-    {
-    written += len;
-    if (written > size_limit) break;
-    }
-  }
-
-/* A read error on the body will have left len == -1 and errno set. */
-
-if (len != 0) return FALSE;
-
-/* If the last character of the body was not '\n', manufacture one. */
-
-if (!was_nl && !write_chunk(fd, "\n", 1, FALSE, use_crlf, smtp_dots))
-  return FALSE;
-
-/* If requested, add a terminating "." line (SMTP output). */
-
-if ((options & topt_end_dot) != 0 &&
-    !write_chunk(fd, ".\n", 2, FALSE, use_crlf, FALSE))
-  return FALSE;
-
 /* Write out any remaining data in the buffer before returning. */
 
 return (len = chunk_ptr - deliver_out_buffer) <= 0 ||
@@ -622,13 +705,15 @@ down the given fd. At the end, tidy up the pipes and the processes.
 Arguments:     as for internal_transport_write_message() above
 
 Returns:       TRUE on success; FALSE (with errno) for any failure
+                 transport_count is incremented by the count of bytes written
 */
 
 BOOL
 transport_write_message(address_item *addr, int fd, int options,
-  char *errors_to, int size_limit, char *add_headers, char *remove_headers)
+  int size_limit, char *add_headers, char *remove_headers, char *check_string,
+  char *escape_string)
 {
-BOOL from_hack, use_crlf, smtp_dots;
+BOOL use_crlf;
 int rc;
 int pfd[2];
 int len, fd_read, fd_write, yield;
@@ -639,8 +724,8 @@ pid_t filter_pid, write_pid;
 the actual work, passing it the incoming fd. */
 
 if (transport_filter_argv == NULL)
-  return internal_transport_write_message(addr, fd, options, errors_to,
-    size_limit, add_headers, remove_headers);
+  return internal_transport_write_message(addr, fd, options, size_limit,
+    add_headers, remove_headers, check_string, escape_string);
 
 /* Otherwise the message must be written through a filter. Create a pipe for
 stdout, saving the reading end. */
@@ -658,7 +743,7 @@ write_pid = (pid_t)(-1);
 fd_write = -1;
 
 filter_pid = child_open(transport_filter_argv, NULL, 077, NULL, NULL,
-  &fd_write, pfd[pipe_write], NULL);
+  &fd_write, pfd[pipe_write], NULL, FALSE);
 
 close(pfd[pipe_write]);
 if (filter_pid < 0)
@@ -670,7 +755,8 @@ if (filter_pid < 0)
 DEBUG(3) debug_printf("process %d running as transport filter\n", filter_pid);
 
 /* Fork subprocess to write the message to the filter, and return the result
-via a(nother) pipe. */
+via a(nother) pipe. While writing to the filter, we do not do the CRLF,
+smtp dots, or check string processing. */
 
 if (pipe(pfd) != 0)
   {
@@ -683,11 +769,10 @@ if ((write_pid = fork()) == 0)
   BOOL rc;
   close(fd_read);
   close(pfd[pipe_read]);
+  nl_check_length = nl_escape_length = 0;
   rc = internal_transport_write_message(addr, fd_write,
-    (options &
-      ~(topt_from_hack | topt_use_crlf |
-        topt_smtp_dots | topt_end_dot)),
-    errors_to, size_limit, add_headers, remove_headers);
+    (options & ~(topt_use_crlf | topt_end_dot)),
+    size_limit, add_headers, remove_headers, NULL, NULL);
   save_errno = errno;
   write(pfd[pipe_write], (void *)&rc, sizeof(BOOL));
   write(pfd[pipe_write], (void *)&save_errno, sizeof(int));
@@ -718,14 +803,25 @@ up for writing timeouts. If not, we must set it up here. */
 
 DEBUG(9) debug_printf("copying from the filter\n");
 
-from_hack = (options & topt_from_hack) != 0;
+/* Set up the special processing to be done during the copying */
+
 use_crlf  = (options & topt_use_crlf)  != 0;
-smtp_dots = (options & topt_smtp_dots) != 0;
+
+nl_partial_match = -1;
+if (check_string != NULL && escape_string != NULL)
+  {
+  nl_check = check_string;
+  nl_check_length = (int)strlen(nl_check);
+  nl_escape = escape_string;
+  nl_escape_length = (int)strlen(nl_escape);
+  }
+else nl_check_length = nl_escape_length = 0;
 
 transport_count = 0;
-was_nl = TRUE;
 chunk_ptr = deliver_out_buffer;
 
+/* Copy the output of the filter */
+
 for (;;)
   {
   if (transport_write_timeout <= 0)  /* Yes, that is "<=" see above */
@@ -749,7 +845,7 @@ for (;;)
 
   /* Otherwise, write the block down the original fd. */
 
-  if (!write_chunk(fd, deliver_in_buffer, len, from_hack, use_crlf, smtp_dots))
+  if (!write_chunk(fd, deliver_in_buffer, len, use_crlf))
     {
     save_errno = errno;
     goto TIDY_UP;
@@ -762,8 +858,7 @@ if (len != 0) { save_errno = errno; goto TIDY_UP; }
 
 /* If requested, add a terminating "." line (SMTP output). */
 
-if ((options & topt_end_dot) != 0 &&
-    !write_chunk(fd, ".\n", 2, FALSE, use_crlf, FALSE))
+if ((options & topt_end_dot) != 0 && !write_chunk(fd, ".\n", 2, use_crlf))
   {
   save_errno = errno;
   goto TIDY_UP;
@@ -870,7 +965,8 @@ better.
 Old records should eventually get swept up by the exim_tidydb utility.
 
 Arguments:
-  hostlist  list of hosts that this message could be sent to
+  hostlist  list of hosts that this message could be sent to;
+              the update_waiting flag is set if a host is to be noted
   tpname    name of the transport
 
 Returns:    nothing
@@ -897,7 +993,8 @@ if (dbm_file == NULL)
   }
 
 /* Scan the list of hosts for which this message is waiting, and ensure
-that the message id is in each host record. */
+that the message id is in each host record for those that have the
+update_waiting flag set. */
 
 for (host = hostlist; host!= NULL; host = host->next)
   {
@@ -906,7 +1003,11 @@ for (host = hostlist; host!= NULL; host = host->next)
   char *s;
   int i, host_length;
 
-  /*Skip if this is the same host as we just processed; otherwise remember
+  /* Skip if the update_waiting flag is not set. */
+
+  if (!host->update_waiting) continue;
+
+  /* Skip if this is the same host as we just processed; otherwise remember
   the name for next time. */
 
   if (strcmp(prevname, host->name) == 0) continue;
@@ -1057,7 +1158,8 @@ sprintf(buffer, "wait-%.200s", transport_name);
 dbm_file = dbfn_open(buffer, O_RDWR|O_CREAT, &dbblock);
 if (dbm_file == NULL)
   {
-  log_write(0, LOG_MAIN, "Failed to open %s database", buffer);
+  log_write(0, LOG_MAIN, "Failed to open %s database: %s", buffer,
+    strerror(errno));
   return FALSE;
   }
 
@@ -1220,11 +1322,14 @@ int status;
 if ((pid = vfork()) == 0)
   {
   int fd, i;
-  char *argv[13 + MAX_CLMACROS];
+  char *argv[17 + MAX_CLMACROS];
 
   /* Disconnect entirely from the parent process. */
 
-  if (vfork() != 0) _exit(EXIT_SUCCESS);
+  if ((pid = vfork()) != 0)
+    {
+    _exit(EXIT_SUCCESS);
+    }
 
   /* Set up the calling arguments */
 
@@ -1246,10 +1351,24 @@ if ((pid = vfork()) == 0)
     }
 
   #ifdef SUPPORT_DSN
+  /* This partial attempt at doing DSN was abandoned. This obsolete code is
+  left here just in case. Nothing is documented. */
   if (smtp_use_dsn) argv[i++] = "-MCD";
   #endif
 
+  #ifdef HAVE_AUTH
+  if (smtp_authenticated) argv[i++] = "-MCA";
+  #endif
+
   if (smtp_use_size) argv[i++] = "-MCS";
+
+  if (queue_run_pid != (pid_t)0)
+    {
+    argv[i++] = "-MCQ";
+    argv[i++] = string_sprintf("%d", queue_run_pid);
+    argv[i++] = string_sprintf("%d", queue_run_pipe);
+    }
+
   argv[i++] = "-MC";
   argv[i++] = transport_name;
   argv[i++] = hostname;
@@ -1257,12 +1376,14 @@ if ((pid = vfork()) == 0)
   argv[i++] = id;
   argv[i++] = NULL;
 
-  /* Arrange for the channel to be on stdin, with all other
-  file descriptors closed (except for debug_file while debugging). */
+  /* Arrange for the channel to be on stdin, with all other file descriptors
+  closed, except for the queue runner's synchronizing pipe if present, and the
+  debug_file while debugging). */
 
   for (fd = mac_maxfd; fd >= 0; fd--)
-    if (fd != socket_fd && (debug_file == NULL || fd != fileno(debug_file)))
-      close(fd);
+    if (fd != socket_fd && fd != queue_run_pipe &&
+      (debug_file == NULL || fd != fileno(debug_file)))
+        close(fd);
 
   if (socket_fd != 0)
     {
@@ -1270,12 +1391,14 @@ if ((pid = vfork()) == 0)
     close(socket_fd);
     }
 
-  DEBUG(4) debug_printf("fork %s %s %s %s %s %s %s %s %s %s\n", argv[0],
+  DEBUG(4) debug_printf("fork %s %s %s %s %s %s %s %s %s %s %s %s\n", argv[0],
     argv[1], argv[2], argv[3], argv[4], argv[5],
       (argv[6] == NULL)? "" : argv[6],
       (argv[7] == NULL)? "" : argv[7],
       (argv[8] == NULL)? "" : argv[8],
-      (argv[9] == NULL)? "" : argv[9]);
+      (argv[9] == NULL)? "" : argv[9],
+      (argv[10] == NULL)? "" : argv[10],
+      (argv[11] == NULL)? "" : argv[11]);
 
   execv(argv[0], argv);
 
@@ -1561,8 +1684,8 @@ if (expand_arguments)
     /* Handle special fudge for passing an address list */
 
     if (addr != NULL &&
-        strcmp(argv[i], "$pipe_addresses") == 0 ||
-        strcmp(argv[i], "${pipe_addresses}") == 0)
+        (strcmp(argv[i], "$pipe_addresses") == 0 ||
+         strcmp(argv[i], "${pipe_addresses}") == 0))
       {
       int additional;
 
