@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -14,10 +14,16 @@ typedef struct {
   char *suffix;
   char *create_file_string;
   char *quota;
+  char *quota_warn_threshold;
   char *maildir_tag;
   char *mailstore_prefix;
   char *mailstore_suffix;
+  char *check_string;
+  char *escape_string;
+  char *file_format;
   int   quota_value;
+  int   quota_warn_threshold_value;
+  int   quota_filecount;
   int   mode;
   int   dirmode;
   int   lockfile_mode;
@@ -26,19 +32,27 @@ typedef struct {
   int   lock_interval;
   int   maildir_retries;
   int   create_file;
+  int   options;
   BOOL  allow_symlink;
   BOOL  check_group;
   BOOL  create_directory;
   BOOL  notify_comsat;
   BOOL  require_lockfile;
   BOOL  use_lockfile;
+  BOOL  set_use_lockfile;
   BOOL  use_fcntl;
+  BOOL  set_use_fcntl;
+  BOOL  use_mbx_lock;
+  BOOL  set_use_mbx_lock;
   BOOL  use_crlf;
   BOOL  from_hack;
+  BOOL  set_from_hack;
   BOOL  file_must_exist;
   BOOL  mode_fail_narrower;
   BOOL  maildir_format;
   BOOL  mailstore_format;
+  BOOL  mbx_format;
+  BOOL  quota_warn_threshold_is_percent;
 } appendfile_transport_options_block;
 
 /* Restricted creation options */
