@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -50,6 +50,10 @@ optionlist autoreply_transport_options[] = {
       (void *)(offsetof(autoreply_transport_options_block, mode)) },
   { "once",              opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, oncelog)) },
+  { "once_repeat",       opt_time,
+      (void *)(offsetof(autoreply_transport_options_block, once_repeat)) },
+  { "reply_to",          opt_stringptr,
+      (void *)(offsetof(autoreply_transport_options_block, reply_to)) },
   { "return_message",    opt_bool,
       (void *)(offsetof(autoreply_transport_options_block, return_message)) },
   { "subject",           opt_stringptr,
@@ -72,6 +76,7 @@ int autoreply_transport_options_count =
 
 autoreply_transport_options_block autoreply_transport_option_defaults = {
   NULL,           /* from */
+  NULL,           /* reply_to */
   NULL,           /* to */
   NULL,           /* cc */
   NULL,           /* bcc */
@@ -82,6 +87,7 @@ autoreply_transport_options_block autoreply_transport_option_defaults = {
   NULL,           /* logfile */
   NULL,           /* oncelog */
   0600,           /* mode */
+  0,              /* once_repeat */
   FALSE,          /* file_expand */
   FALSE,          /* file_optional */
   FALSE           /* return message */
@@ -180,11 +186,12 @@ autoreply_transport_entry(
 int fd, pid, rc;
 int log_fd = -1;
 BOOL file_expand, return_message;
-char *from, *to, *cc, *bcc, *subject, *headers, *text, *file;
+char *from, *reply_to, *to, *cc, *bcc, *subject, *headers, *text, *file;
 char *logfile, *oncelog;
-char *now = NULL;
 header_line *h;
 EXIM_DB *dbm_file = NULL;
+time_t now = time(NULL);
+time_t once_repeat;
 FILE *f;
 FILE *ff = NULL;
 autoreply_transport_options_block *ob =
@@ -206,6 +213,7 @@ if (addr->reply != NULL)
   {
   DEBUG(9) debug_printf("taking data from address\n");
   from = addr->reply->from;
+  reply_to = addr->reply->reply_to;
   to = addr->reply->to;
   cc = addr->reply->cc;
   bcc = addr->reply->bcc;
@@ -215,6 +223,7 @@ if (addr->reply != NULL)
   file = addr->reply->file;
   logfile = addr->reply->logfile;
   oncelog = addr->reply->oncelog;
+  once_repeat = addr->reply->once_repeat;
   file_expand = addr->reply->file_expand;
   return_message = addr->reply->return_message;
   }
@@ -222,6 +231,7 @@ else
   {
   DEBUG(9) debug_printf("taking data from transport\n");
   from = ob->from;
+  reply_to = ob->reply_to;
   to = ob->to;
   cc = ob->cc;
   bcc = ob->bcc;
@@ -231,11 +241,14 @@ else
   file = ob->file;
   logfile = ob->logfile;
   oncelog = ob->oncelog;
+  once_repeat = ob->once_repeat;
   file_expand = ob->file_expand;
   return_message = ob->return_message;
 
   if ((from  != NULL &&
         (from = checkexpand(from, addr, tblock->name, FALSE)) == NULL) ||
+      (reply_to    != NULL &&
+        (reply_to = checkexpand(reply_to, addr, tblock->name, FALSE)) == NULL) ||
       (to    != NULL &&
         (to = checkexpand(to, addr, tblock->name, FALSE)) == NULL) ||
       (cc    != NULL &&
@@ -264,7 +277,6 @@ if (dont_deliver)
   {
   debug_printf("*** delivery by %s transport bypassed by -N option\n",
     tblock->name);
-  addr->transport_return = OK;
   return;
   }
 
@@ -298,18 +310,41 @@ if (oncelog != NULL && to != NULL)
 
   if (EXIM_DBGET(dbm_file, key_datum, result_datum))
     {
-    DEBUG(9) debug_printf("message already sent to %s\n", to);
-    log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
-    if (log_fd >= 0)
+    time_t then;
+    DEBUG(9) debug_printf("message previously sent to %s\n", to);
+
+    /* If the datum size is that of a binary time, we are in the new world
+    where messages are sent periodically. Otherwise the file is an old one,
+    where the datum was filled with a tod_log time, which is assumed to be
+    different in size. For that, only one message is ever sent. This change
+    introduced at Exim 3.00. In a couple of years' time the test on the size
+    can be abolished. */
+
+    if (EXIM_DATUM_SIZE(result_datum) == sizeof(time_t))
       {
-      char *ptr = log_buffer;
-      if (now == NULL) now = tod_stamp(tod_log);
-      sprintf(ptr, "%s\n  previously sent to %.200s\n", now, to);
-      while(*ptr) ptr++;
-      write(log_fd, log_buffer, ptr - log_buffer);
+      memcpy(&then, EXIM_DATUM_DATA(result_datum), sizeof(time_t));
+      }
+    else then = now;
+
+    if (once_repeat <= 0 || now - then < once_repeat)
+      {
+      DEBUG(9) debug_printf("repeat time not reached\n");
+      log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
+      if (log_fd >= 0)
+        {
+        char *ptr = log_buffer;
+        sprintf(ptr, "%s\n  previously sent to %.200s\n", tod_stamp(tod_log),
+          to);
+        while(*ptr) ptr++;
+        write(log_fd, log_buffer, ptr - log_buffer);
+        }
+      EXIM_DBCLOSE(dbm_file);
+      return;
+      }
+    else
+      {
+      DEBUG(9) debug_printf("repeat time reached\n");
       }
-    EXIM_DBCLOSE(dbm_file);
-    return;
     }
   }
 
@@ -331,16 +366,16 @@ if (file != NULL)
 /* Make a subprocess to send the message */
 
 pid = child_open(mailer_argv, NULL, 077, NULL, NULL, &fd,
-  (debug_file != NULL)? fileno(debug_file) : -1, NULL);
+  (debug_file != NULL)? fileno(debug_file) : -1, NULL, FALSE);
 
 /* Creation of child failed; defer this delivery. */
 
 if (pid < 0)
   {
-  DEBUG(2) debug_printf("Failed to create child process\n");
   addr->transport_return = DEFER;
   addr->message = string_sprintf("Failed to create child process to send "
-    "message from %s transport", tblock->name);
+    "message from %s transport: %s", tblock->name, strerror(errno));
+  DEBUG(2) debug_printf("%s\n", addr->message);
   return;
   }
 
@@ -351,6 +386,7 @@ are newlines in it which might, if placed earlier, screw up other headers. */
 f = fdopen(fd, "w");
 
 if (from != NULL) fprintf(f, "From: %s\n", from);
+if (reply_to != NULL) fprintf(f, "Reply-To: %s\n", reply_to);
 if (to != NULL) fprintf(f, "To: %s\n", to);
 if (cc != NULL) fprintf(f, "Cc: %s\n", cc);
 if (bcc != NULL) fprintf(f, "Bcc: %s\n", bcc);
@@ -419,11 +455,13 @@ if (return_message)
 
   fflush(f);
   transport_write_message(addr, fileno(f),
+    (tblock->body_only? topt_no_headers : 0) |
+    (tblock->headers_only? topt_no_body : 0) |
     (tblock->return_path_add? topt_add_return_path : 0) |
-      (tblock->delivery_date_add? topt_add_delivery_date : 0) |
-      (tblock->envelope_to_add? topt_add_envelope_to : 0),
-    NULL, return_size_limit,
-    tblock->add_headers, tblock->remove_headers);
+    (tblock->delivery_date_add? topt_add_delivery_date : 0) |
+    (tblock->envelope_to_add? topt_add_envelope_to : 0),
+    return_size_limit, tblock->add_headers, tblock->remove_headers,
+    NULL, NULL);
   }
 
 /* End the message and wait for the child process to end; no timeout. */
@@ -437,27 +475,42 @@ missing out a message rather than risking sending more than one. */
 if (dbm_file != NULL)
   {
   EXIM_DATUM key_datum, value_datum;
-  if (now == NULL) now = tod_stamp(tod_log);
   #ifdef EXIM_CLEAR_DATUMSTRUCT
   memset(&key_datum, 0, sizeof(key_datum));
   memset(&value_datum, 0, sizeof(value_datum));
   #endif
   EXIM_DATUM_DATA(key_datum) = to;
   EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
-  EXIM_DATUM_DATA(value_datum) = now;
-  EXIM_DATUM_SIZE(value_datum) = (int)strlen(now) + 1;
+
+  /* Many OS define the datum value, sensibly, as a void *. However, there
+  are some which still have char *. By casting this address to a char * we
+  can avoid warning messages from the char * systems. */
+
+  EXIM_DATUM_DATA(value_datum) = (char *)(&now);
+
+  EXIM_DATUM_SIZE(value_datum) = (int)sizeof(time_t);
   EXIM_DBPUT(dbm_file, key_datum, value_datum);
   EXIM_DBCLOSE(dbm_file);
   }
 
 /* If sending failed, defer to try again - but if once is set the next
-try will skip, of course. */
+try will skip, of course. However, if there were no recipients in the
+message, we do not fail. */
 
 if (rc != 0)
   {
-  addr->transport_return = DEFER;
-  addr->message = string_sprintf("Failed to send message from %s "
-    "transport (%d)", tblock->name, rc);
+  if (rc == EXIT_NORECIPIENTS)
+    {
+    DEBUG(1) debug_printf("%s transport: message contained no recipients\n",
+      tblock->name);
+    }
+  else
+    {
+    addr->transport_return = DEFER;
+    addr->message = string_sprintf("Failed to send message from %s "
+      "transport (%d)", tblock->name, rc);
+    return;
+    }
   }
 
 /* Log the sending of the message if successful and required. If the file
@@ -471,14 +524,13 @@ writing. */
 
 if (logfile != NULL)
   {
-  if (now == NULL) now = tod_stamp(tod_log);
   if (log_fd < 0)
     log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
   if (log_fd >= 0)
     {
     char *ptr = log_buffer;
     DEBUG(9) debug_printf("logging message details\n");
-    sprintf(ptr, "%s\n", now);
+    sprintf(ptr, "%s\n", tod_stamp(tod_log));
     while(*ptr) ptr++;
     if (from != NULL)
       {
