@@ -2,13 +2,14 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
   char *from;
+  char *reply_to;
   char *to;
   char *cc;
   char *bcc;
@@ -19,6 +20,7 @@ typedef struct {
   char *logfile;
   char *oncelog;
   int   mode;
+  time_t once_repeat;
   BOOL  file_expand;
   BOOL  file_optional;
   BOOL  return_message;
