@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -16,7 +16,7 @@ order (note that "_" comes before the lower case letters). Those starting
 with "*" are not settable by the user but are used by the option-reading
 software for alternative value types. Some options are stored in the transport
 instance block so as to be publicly visible; these are flagged with opt_public.
-The "directory" option is obsolete, being replaced by "home_directory". */
+*/
 
 optionlist pipe_transport_options[] = {
   { "*expand_group",     opt_stringptr | opt_hidden | opt_public,
@@ -37,12 +37,16 @@ optionlist pipe_transport_options[] = {
       (void *)(offsetof(transport_instance, local_smtp)) },
   { "bsmtp_helo",        opt_bool | opt_public,
       (void *)(offsetof(transport_instance, bsmtp_helo)) },
+  { "check_string",      opt_stringptr,
+      (void *)(offsetof(pipe_transport_options_block, check_string)) },
   { "command",           opt_stringptr,
       (void *)(offsetof(pipe_transport_options_block, cmd)) },
   { "current_directory", opt_stringptr | opt_public,
       (void *)(offsetof(transport_instance, current_dir)) },
-  { "directory",         opt_stringptr | opt_public | opt_hidden,
-      (void *)(offsetof(transport_instance, home_dir)) },
+  { "environment",       opt_stringptr,
+      (void *)(offsetof(pipe_transport_options_block, environment)) },
+  { "escape_string",     opt_stringptr,
+      (void *)(offsetof(pipe_transport_options_block, escape_string)) },
   { "freeze_exec_fail",  opt_bool,
       (void *)(offsetof(pipe_transport_options_block, freeze_exec_fail)) },
   { "from_hack",         opt_bool,
@@ -104,15 +108,19 @@ int pipe_transport_options_count =
 pipe_transport_options_block pipe_transport_option_defaults = {
   NULL,           /* cmd */
   NULL,           /* allow_commands */
+  NULL,           /* environment */
   "/usr/bin",     /* path */
   "From ${if def:return_path{$return_path}{MAILER-DAEMON}} ${tod_bsdinbox}\n",
                   /* prefix */
   "\n",           /* suffix */
   mac_expanded_string(EX_TEMPFAIL) ":"    /* temp_errors */
   mac_expanded_string(EX_CANTCREAT),
+  NULL,           /* check_string */
+  NULL,           /* escape_string */
   022,            /* umask */
   20480,          /* max_output */
   60*60,          /* timeout */
+  0,              /* options */
   FALSE,          /* freeze_exec_fail */
   FALSE,          /* from_hack */
   FALSE,          /* ignore_status */
@@ -154,11 +162,12 @@ if (tblock->uid_set && !tblock->gid_set && tblock->expand_gid == NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "user set without group for the %s transport", tblock->name);
 
-/* Temp_errors must consist only of digits and colons */
+/* Temp_errors must consist only of digits and colons, but there can be
+spaces round the colons, so allow them too. */
 
 if (ob->temp_errors != NULL)
   {
-  size_t p = strspn(ob->temp_errors, "0123456789:");
+  size_t p = strspn(ob->temp_errors, "0123456789: ");
   if (ob->temp_errors[p] != 0)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
       "temp_errors option must be a list of numbers in the %s transport",
@@ -178,10 +187,17 @@ if (tblock->log_output && tblock->log_fail_output)
     "both log_output and log_fail_output set for %s transport",
     tblock->name);
 
-/* If batch SMTP is set, ensure the generic local batch option matches */
+/* If batch SMTP is set, ensure the generic local batch option matches. They
+take the same set of values. Also force the check and escape strings, and
+arrange that headers are also escaped. */
 
-if (tblock->local_smtp != local_batch_off)
+if (tblock->local_smtp != local_smtp_off)
+  {
   tblock->local_batch = tblock->local_smtp;
+  ob->check_string = ".";
+  ob->escape_string = "..";
+  ob->options |= topt_escape_headers;
+  }
 
 /* The restrict_to_path  and use_shell options are incompatible */
 
@@ -196,6 +212,17 @@ if (ob->allow_commands && ob->use_shell)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "both allow_commands and use_shell set for %s transport",
     tblock->name);
+
+/* Set up the bitwise options for transport_write_message from the various
+driver options. Only one of body_only and headers_only can be set. */
+
+ob->options |=
+  (tblock->body_only? topt_no_headers : 0) |
+  (tblock->headers_only? topt_no_body : 0) |
+  (tblock->return_path_add? topt_add_return_path : 0) |
+  (tblock->delivery_date_add? topt_add_delivery_date : 0) |
+  (tblock->envelope_to_add? topt_add_envelope_to : 0) |
+  (ob->use_crlf? topt_use_crlf : 0);
 }
 
 
@@ -258,9 +285,7 @@ if (ob->allow_commands != NULL)
     return FALSE;
     }
 
-  for (p = string_nextinlist(&s, ':', buffer, sizeof(buffer));
-       p != NULL;
-       p = string_nextinlist(&s, ':', buffer, sizeof(buffer)))
+  while ((p = string_nextinlist(&s, ':', buffer, sizeof(buffer))) != NULL)
     {
     if (strcmp(p, argv[0]) == 0) { permitted = TRUE; break; }
     }
@@ -304,9 +329,7 @@ if (argv[0][0] != '/')
   char *listptr = ob->path;
   char buffer[1024];
 
-  for (p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer));
-       p != NULL;
-       p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer)))
+  while ((p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer))) != NULL)
     {
     struct stat statbuf;
     sprintf(big_buffer, "%.256s/%.256s", p, argv[0]);
@@ -342,7 +365,6 @@ Arguments:
   expand_arguments   true if expansion is to occur
   addr               chain of addresses
   tname              the transport name
-  ob                 the transport options block
 
 Returns:             TRUE if all went well; otherwise an error will be
                      set in the first address and FALSE returned
@@ -350,7 +372,7 @@ Returns:             TRUE if all went well; otherwise an error will be
 
 static BOOL
 set_up_shell_command(char ***argvptr, char *cmd, BOOL expand_arguments,
-  address_item *addr, char *tname, pipe_transport_options_block *ob)
+  address_item *addr, char *tname)
 {
 char **argv;
 
@@ -432,16 +454,14 @@ int pipefd[2];
 int envcount = 0;
 pipe_transport_options_block *ob =
   (pipe_transport_options_block *)(tblock->options_block);
-BOOL smtp_dots = FALSE;
+int timeout = ob->timeout;
 BOOL written_ok = FALSE;
-BOOL return_path_add = tblock->return_path_add;
-BOOL delivery_date_add = tblock->delivery_date_add;
-BOOL envelope_to_add = tblock->envelope_to_add;
 BOOL expand_arguments;
 char **argv;
-char *envp[20];
+char *envp[40];
+char *envlist = ob->environment;
 char *env_local_part;
-char *cmd;
+char *cmd, *ss;
 char *eol = (ob->use_crlf)? "\r\n" : "\n";
 
 DEBUG(2) debug_printf("%s transport entered\n", tblock->name);
@@ -489,8 +509,8 @@ there is an option to do that. */
 
 if (ob->use_shell)
   {
-  if (!set_up_shell_command(&argv, cmd, expand_arguments, addr, tblock->name,
-    ob)) return;
+  if (!set_up_shell_command(&argv, cmd, expand_arguments, addr, tblock->name))
+    return;
   }
 else if (!set_up_direct_command(&argv, cmd, expand_arguments, addr,
   tblock->name, ob)) return;
@@ -520,6 +540,30 @@ envp[envcount++] = "SHELL=/bin/sh";
 if (addr->host_list != NULL)
   envp[envcount++] = string_sprintf("HOST=%s", addr->host_list->name);
 
+/* Add any requested items */
+
+if (envlist != NULL)
+  {
+  envlist = expand_string(envlist);
+  if (envlist == NULL)
+    {
+    addr->transport_return = DEFER;
+    addr->message = string_sprintf("failed to expand string \"%s\" "
+      "for %s transport: %s", ob->environment, tblock->name,
+      expand_string_message);
+    return;
+    }
+  }
+
+while ((ss = string_nextinlist(&envlist, ':', big_buffer, big_buffer_size))
+       != NULL)
+   {
+   if (envcount > sizeof(envp)/sizeof(char *) - 2)
+     log_write(0, LOG_MAIN|LOG_PANIC_DIE, "Too many environment settings for "
+       "%s transport", tblock->name);
+   envp[envcount++] = string_copy(ss);
+   }
+
 envp[envcount] = NULL;
 
 
@@ -563,14 +607,16 @@ if (pipe(pipefd) < 0)
 
 /* As this is a local transport, we are already running with the required
 uid/gid and current directory, so pass NULL to child_open to indicate no
-change. */
+change. Request that the new process be a process group leader, so we
+can kill it and all its children on a timeout. */
 
 if ((pid = child_open(argv, envp, ob->umask, NULL, NULL, &fd,
-     pipefd[pipe_write], NULL)) < 0)
+     pipefd[pipe_write], NULL, TRUE)) < 0)
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf(
-    "Failed to create child process for %s transport", tblock->name);
+    "Failed to create child process for %s transport: %s", tblock->name,
+      strerror(errno));
   return;
   }
 
@@ -594,14 +640,14 @@ if ((outpid = fork()) < 0)
 /* This is the code for the output-handling subprocess. Read from the pipe
 in chunks, and write to the return file if one is provided. Keep track of
 the number of bytes handled. If the limit is exceeded, try to kill the
-subprocess, and in any case close the pipe and exit, which should cause the
-subprocess to fail. */
+subprocess group, and in any case close the pipe and exit, which should cause
+the subprocess to fail. */
 
 if (outpid == 0)
   {
   int count = 0;
   close(fd);
-  set_process_info("reading output from %s", cmd);
+  set_process_info("reading output from |%s", cmd);
   while ((rc = read(pipefd[pipe_read], big_buffer, big_buffer_size)) > 0)
     {
     if (addr->return_file >= 0)
@@ -613,7 +659,7 @@ if (outpid == 0)
       DEBUG(2) debug_printf("Too much output from pipe - killed\n");
       if (addr->return_file >= 0)
         write(addr->return_file, message, (int)strlen(message));
-      kill(pid, SIGKILL);
+      killpg(pid, SIGKILL);
       break;
       }
     }
@@ -633,6 +679,15 @@ ignore all writing errors. */
 
 DEBUG(2) debug_printf("Writing message to pipe\n");
 
+/* Arrange to time out writes if there is a timeout set. */
+
+if (timeout > 0)
+  {
+  sigalrm_seen = FALSE;
+  os_non_restarting_signal(SIGALRM, sigalrm_handler);
+  transport_write_timeout = timeout;
+  }
+
 /* If the local_smtp option is not unset, we need to write SMTP prefix
 information. The various different values for batching are handled outside; if
 there is more than one address available here, all must be included. Force
@@ -644,8 +699,6 @@ if (tblock->local_smtp != local_smtp_off)
   char *sender = (addr->errors_address != NULL)?
     addr->errors_address : sender_address;
 
-  smtp_dots = TRUE;
-
   if (tblock->bsmtp_helo &&
       !transport_write_string(fd, "HELO %s%s", primary_hostname, eol))
     goto END_WRITE;
@@ -655,18 +708,9 @@ if (tblock->local_smtp != local_smtp_off)
 
   for (a = addr; a != NULL; a = a->next)
     {
-    if (a->local_part[0] == ',' || a->local_part[0] == ':')
-      {
-      if (!transport_write_string(fd,
-          "RCPT TO:<@%s%s>%s", a->domain, a->local_part, eol))
-        goto END_WRITE;
-      }
-    else
-      {
-      if (!transport_write_string(fd,
-          "RCPT TO:<%s@%s>%s", a->local_part, a->domain, eol))
-        goto END_WRITE;
-      }
+    if (!transport_write_string(fd,
+        "RCPT TO:<%s@%s>%s", a->local_part, a->domain, eol))
+      goto END_WRITE;
     }
 
   if (!transport_write_string(fd, "DATA%s", eol)) goto END_WRITE;
@@ -687,17 +731,11 @@ if (ob->prefix != NULL)
   if (!transport_write_block(fd, prefix, (int)strlen(prefix))) goto END_WRITE;
   }
 
-/* Now the actual message */
+/* Now the actual message - the options were set at initialization time */
 
-if (!transport_write_message(addr, fd,
-    (return_path_add? topt_add_return_path : 0) |
-    (delivery_date_add? topt_add_delivery_date : 0) |
-    (envelope_to_add? topt_add_envelope_to : 0) |
-    (ob->from_hack? topt_from_hack : 0) |
-    (ob->use_crlf? topt_use_crlf : 0) |
-    (smtp_dots? topt_smtp_dots : 0),
-    addr->errors_address, 0, tblock->add_headers, tblock->remove_headers))
-  goto END_WRITE;
+if (!transport_write_message(addr, fd, ob->options, 0, tblock->add_headers,
+  tblock->remove_headers, ob->check_string, ob->escape_string))
+    goto END_WRITE;
 
 /* Now any configured suffix */
 
@@ -728,20 +766,25 @@ written_ok = TRUE;
 
 END_WRITE:
 
-/* OK, the writing is now all done. Close the pipe. */
+/* OK, the writing is now all done. Turn off the alarm handler and close the
+pipe. */
 
+signal(SIGALRM, SIG_IGN);
 (void) close(fd);
 
-/* Handle errors during writing. If the process at the far end of the pipe
-died without reading all of it, we expect an EPIPE error, which should be
-ignored. We used also to ignore WRITEINCOMPLETE but the writing function is
-now cleverer at handling OS where the death of a pipe doesn't give EPIPE
-immediately. See comments therein. This change made 04-Sep-98. Clean up this
-code in a year or so. */
+/* Handle errors during writing. For timeouts, set the timeout for waiting for
+the child process to 1 second. If the process at the far end of the pipe died
+without reading all of it, we expect an EPIPE error, which should be ignored.
+We used also to ignore WRITEINCOMPLETE but the writing function is now cleverer
+at handling OS where the death of a pipe doesn't give EPIPE immediately. See
+comments therein. This change made 04-Sep-98. Clean up this code in a year or
+so. */
 
 if (!written_ok)
   {
-  if (errno == EPIPE /* || errno == ERRNO_WRITEINCOMPLETE */ )
+  if (errno == ETIMEDOUT)
+    timeout = 1;
+  else if (errno == EPIPE /* || errno == ERRNO_WRITEINCOMPLETE */ )
     {
     debug_printf("transport error %s ignored\n",
       (errno == EPIPE)? "EPIPE" : "WRITEINCOMPLETE");
@@ -750,8 +793,10 @@ if (!written_ok)
     {
     addr->transport_return = PANIC;
     addr->basic_errno = errno;
-    if (errno == ERRNO_ADDHEADER_FAIL)
-      addr->message = string_sprintf("Failed to expand added headers");
+    if (errno == ERRNO_CHHEADER_FAIL)
+      addr->message =
+        string_sprintf("Failed to expand headers_add or headers_remove: %s",
+          expand_string_message);
     else if (errno == ERRNO_FILTER_FAIL)
       addr->message = string_sprintf("Filter process failure");
     else if (errno == ERRNO_WRITEINCOMPLETE)
@@ -763,15 +808,21 @@ if (!written_ok)
   }
 
 /* Wait for the child process to complete and take action if the returned
-status is nonzero or if something went wrong. */
+status is nonzero. The timeout will be just 1 second if any of the writes
+above timed out. */
 
-if ((rc = child_close(pid, ob->timeout)) != 0)
+if ((rc = child_close(pid, timeout)) != 0)
   {
-  /* The process did not complete in time; kill it and fail the delivery. */
+  /* The process did not complete in time; kill its process group and fail
+  the delivery. It appears to be necessary to kill the output process too, as
+  otherwise it hangs on for some time if the actual pipe process is sleeping.
+  (At least, that's what I observed on Solaris 2.5.1.) Since we are failing
+  the delivery, that shouldn't cause any problem. */
 
   if (rc == -256)
     {
-    kill(pid, SIGKILL);
+    killpg(pid, SIGKILL);
+    kill(outpid, SIGKILL);
     addr->transport_return = FAIL;
     addr->message = string_sprintf("pipe delivery process timed out",
       tblock->name);
@@ -797,7 +848,8 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
       {
       addr->transport_return = FAIL;
       addr->message = string_sprintf("Child process of %s transport (running "
-        "command \"%s\") was terminated by signal %d", tblock->name, cmd, -rc);
+        "command \"%s\") was terminated by signal %d (%s)", tblock->name, cmd,
+        -rc, os_strsignal(-rc));
       }
     }
 
@@ -852,7 +904,7 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
     else if (!ob->ignore_status)
       {
       char *s = ob->temp_errors;
-      char *p;
+      char *ss, *p;
       int size, ptr, i;
       char buffer[64];
 
@@ -861,9 +913,7 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
       /* Scan the list of temporary failure codes; if any match, the
       result becomes DEFER. */
 
-      for (p = string_nextinlist(&s, ':', buffer, sizeof(buffer));
-           p != NULL;
-           p = string_nextinlist(&s, ':', buffer, sizeof(buffer)))
+      while ((p = string_nextinlist(&s, ':', buffer, sizeof(buffer))) != NULL)
         {
         if (rc == atoi(p)) { addr->transport_return = DEFER; break; }
         }
@@ -872,9 +922,31 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
       doesn't have to be brilliantly efficient - it is an error situation. */
 
       addr->message = string_sprintf("Child process of %s transport returned "
-        "%d from command:", tblock->name, rc);
+        "%d", tblock->name, rc);
+
       ptr = (int)strlen(addr->message);
       size = ptr + 1;
+
+      /* If the return code is > 128, it often means that a shell command
+      was terminated by a signal. */
+
+      ss = (rc > 128)?
+        string_sprintf("(could mean shell command ended by signal %d (%s))",
+          rc-128, os_strsignal(rc-128)) :
+        os_strexit(rc);
+
+      if (*ss != 0)
+        {
+        addr->message = string_cat(addr->message, &size, &ptr, " ", 1);
+        addr->message = string_cat(addr->message, &size, &ptr,
+          ss, (int)strlen(ss));
+        }
+
+      /* Now add the command and arguments */
+
+      addr->message = string_cat(addr->message, &size, &ptr,
+        " from command:", 14);
+
       for (i = 0; i < sizeof(argv)/sizeof(int *) && argv[i] != NULL; i++)
         {
         BOOL quote = FALSE;
