@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -10,13 +10,17 @@
 typedef struct {
   char *cmd;
   char *allow_commands;
+  char *environment;
   char *path;
   char *prefix;
   char *suffix;
   char *temp_errors;
+  char *check_string;
+  char *escape_string;
   int   umask;
   int   max_output;
   int   timeout;
+  int   options;
   BOOL  freeze_exec_fail;
   BOOL  from_hack;
   BOOL  ignore_status;
