@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options and other private data. */
@@ -15,13 +15,8 @@ typedef struct {
   char *interface;
   char *service;
   char *mx_domains;
-  char *mx_domains_except;
-  re_block *re_mx_domains;
-  re_block *re_mx_domains_except;
   char *serialize_hosts;
-  re_block *re_serialize_hosts;
-  char *serialize_nets;
-  ip_net_item *serialize_netlist;
+  char *authenticate_hosts;
   int   batch_max;
   int   command_timeout;
   int   connect_timeout;
@@ -34,6 +29,8 @@ typedef struct {
   BOOL  dns_qualify_single;
   BOOL  dns_search_parents;
   BOOL  delay_after_cutoff;
+  BOOL  hosts_override;
+  BOOL  keepalive;
   BOOL  max_rcpt_serialize;
   BOOL  retry_include_ip_address;
 } smtp_transport_options_block;
