@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1998 */
+/* Copyright (c) University of Cambridge 1995 - 1999 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Function for setting up the version string. */
@@ -10,7 +10,7 @@
 #include "exim.h"
 
 
-#define THIS_VERSION  "2.05"
+#define THIS_VERSION  "3.12"
 
 
 /* The header file cnumber.h contains a single line containing the
@@ -21,24 +21,30 @@ we can't use #include inside a macro argument list */
 void
 version_init(void)
 {
-int i = 0;
 char today[20];
+char *version_cnumber_format;
 
 int cnumber =
 #include "cnumber.h"
 ;
 
-version_cnumber = store_malloc(8);
-sprintf(version_cnumber, "%d", cnumber);
-version_string = THIS_VERSION;
+/* The odd magic after each of these is so they can be easily found
+for automatic patching to standard values when running regression tests.
+The reason that version_cnumber_format isn't just written inline in the
+sprintf() call is the gcc -Wall warns about a \0 in a format string. */
+
+version_cnumber = store_malloc(24);
+version_cnumber_format = "%d\0<<eximcnumber>>";
+sprintf(version_cnumber, version_cnumber_format, cnumber);
+version_string = THIS_VERSION "\0<<eximversion>>";
 
 strcpy(today, __DATE__);
-if (today[4] == ' ') i = 1;
+if (today[4] == ' ') today[4] = '0';
 today[3] = today[6] = '-';
 
 version_date = (char *)store_malloc(32);
 version_date[0] = 0;
-strncat(version_date, today+4+i, 3-i);
+strncat(version_date, today+4, 3);
 strncat(version_date, today, 4);
 strncat(version_date, today+7, 4);
 strcat(version_date, " ");
